/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index$1 = require('./index-c591ecec.js');
const colorHelper = require('./color-helper-dc98a2ca.js');
require('./datetime-helper-eaf4fadb.js');
const generate = require('./generate-b43c3d3b.js');
require('./number-helper-a708fa79.js');
require('./string-helper-9cdfb273.js');
require('./validators-84d8ae28.js');
require('./element-dragger-a8562f82.js');
const bcm = require('./bcm-1d1fcf21.js');
const index = require('./index-5279dea2.js');
require('./colors-56282b00.js');
require('./utils-fc077139.js');
require('./colors-250d0982.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');

const avatarStyle = index.ce({
  slots: {
    base: "bcm-avatar bcm-ui-element bcm-ui-content-display tw-uppercase tw-inline-flex tw-items-center tw-justify-center tw-bg-[--bcm-avatar-bg] tw-text-[--bcm-avatar-text]",
    image: "tw-w-full tw-h-full",
    badge: "",
  },
  variants: {
    size: {
      small: "bcm-avatar__size-small tw-text-1 tw-w-6 tw-h-6",
      medium: "bcm-avatar__size-medium tw-text-2 tw-w-8 tw-h-8",
      large: "bcm-avatar__size-large tw-text-3 tw-w-10 tw-h-10",
    },
    shape: {
      ellipse: {
        base: "bcm-avatar__shape-ellipse tw-rounded-full",
        image: "tw-rounded-full",
        badge: "tw-top-0 tw-right-0",
      },
      square: {
        base: "bcm-avatar__shape-square tw-rounded",
        image: "tw-rounded",
        badge: "tw--top-0.5 tw--right-0.5",
      },
    },
    status: {
      true: {
        base: "bcm-avatar__status tw-relative",
        badge: "tw-absolute",
      },
    },
  },
  defaultVariants: {
    size: "medium",
    shape: "ellipse",
    status: false,
  },
});

const BcmAvatar = class {
  constructor(hostRef) {
    index$1.registerInstance(this, hostRef);
    this._id = generate.Generate.UID();
    this.image = undefined;
    this.name = undefined;
    this.alt = undefined;
    this.shape = bcm.Bcm.Shape.ellipse;
    this.size = bcm.Bcm.Size.medium;
    this.color = undefined;
    this.icon = "fas fa-user";
    this.status = undefined;
    this.blink = false;
    this.isFallback = false;
  }
  getFirstLetters(name) {
    const words = name.split(" ");
    const initials = words.map(word => word.charAt(0).toUpperCase()).join("");
    return initials.substring(0, 2);
  }
  renderDefaultIcon() {
    return index$1.h("bcm-icon", { icon: this.icon });
  }
  render() {
    const { shape, size, status, color, blink } = this;
    const { base, badge, image } = avatarStyle({ size, shape, status: !!status });
    const dc = generate.Generate.getTwColor({
      color: color,
      tones: {
        200: ["bg"],
        700: ["text"],
      },
      variable: "--bcm-avatar",
    });
    const customSize = !Object.keys(bcm.Bcm.BaseSize).includes(size)
      ? {
        width: `${size}px`,
        height: `${size}px`,
        fontSize: `${+size / 3}px`,
      }
      : {};
    const RenderContent = () => {
      if (this.isFallback) {
        if (this.name) {
          return this.getFirstLetters(this.name);
        }
        else {
          return this.renderDefaultIcon();
        }
      }
      else if (this.image) {
        return index$1.h("img", { class: image(), onError: () => (this.isFallback = true), src: this.image, alt: this.alt, "aria-label": this.alt || this.name || "Avatar" });
      }
      else if (this.name) {
        return this.getFirstLetters(this.name);
      }
      else {
        return this.renderDefaultIcon();
      }
    };
    return (index$1.h(index$1.Host, { role: "img", "aria-label": this.alt || this.name || "Avatar", class: base(), style: Object.assign(Object.assign({}, dc), customSize) }, index$1.h(RenderContent, null), status && index$1.h("bcm-badge", { class: badge(), status: status, blink: blink, "aria-hidden": "true" }), index$1.h("slot", null)));
  }
  get el() { return index$1.getElement(this); }
};

const badgeStyle = index.ce({
  slots: {
    host: "bcm-badge bcm-ui-element bcm-ui-content-display tw-inline-flex tw-items-center tw-justify-center tw-font-normal ",
    base: [
      "bcm-badge__dot tw-z-10",
      "tw-flex tw-items-center tw-justify-center tw-flex-row",
      "tw-rounded-full tw-text-center",
      "tw-bg-[--bcm-badge-bg] tw-text-[--bcm-badge-text]",
      "tw-whitespace-nowrap",
    ],
    dotText: "empty:tw-hidden",
  },
  variants: {
    size: {
      small: {
        host: "tw-text-2 tw-gap-1",
        base: "bcm-badge__dot-size-small tw-text-1 tw-min-w-1.5 tw-min-h-1.5",
        dotText: "tw-px-1.5 tw-min-w-4",
      },
      medium: {
        host: "tw-text-3 tw-gap-1.5",
        base: "bcm-badge__dot-size-medium tw-text-2 tw-min-w-2 tw-min-h-2",
        dotText: "tw-px-1.5 tw-min-w-5",
      },
      large: {
        host: "tw-text-4 tw-gap-2",
        base: "bcm-badge__dot-size-large tw-text-3 tw-min-w-2.5 tw-min-h-2.5",
        dotText: "tw-px-2 tw-min-w-6",
      },
    },
    blink: {
      true: {
        base: [
          "bcm-badge__dot-blink tw-relative",
          "after:tw-content-[''] after:tw-absolute after:tw-top-0 after:tw-left-0 after:tw-z-[-1]",
          "after:tw-w-full after:tw-h-full after:tw-bg-[--bcm-badge-bg] after:tw-opacity-75 after:tw-rounded-full",
          "after:tw-animate-blink after:tw-pointer-events-none",
        ],
      },
    },
  },
  defaultVariants: {
    size: "medium",
    blink: false,
  },
});

const BcmBadge = class {
  constructor(hostRef) {
    index$1.registerInstance(this, hostRef);
    this.statusColors = {
      info: "blue",
      error: "red",
      warning: "amber",
      success: "emerald",
    };
    this._id = generate.Generate.UID();
    this.blink = false;
    this.value = undefined;
    this.color = undefined;
    this.size = bcm.Bcm.Size.medium;
    this.status = undefined;
  }
  render() {
    const { blink, value, size, color, status } = this;
    const _color = this.statusColors[status] || color;
    const { tone, colorName } = colorHelper.ColorHelper.parseColor(_color, 700);
    const dc = generate.Generate.getTwColor({
      color: colorName,
      tones: {
        [tone]: ["bg"],
      },
      variable: "--bcm-badge",
    });
    const style = Object.assign(Object.assign({}, dc), (dc["--bcm-badge-bg"] && { "--bcm-badge-text": colorHelper.ColorHelper.reverseColor(dc["--bcm-badge-bg"]) }));
    const { base, dotText, host } = badgeStyle({ size, blink });
    return (index$1.h(index$1.Host, { class: host(), role: "status", "aria-live": "polite" }, index$1.h("div", { class: base(), style: style }, index$1.h("span", { class: dotText(), role: "presentation", "aria-hidden": "true" }, value), index$1.h("span", { class: "tw-sr-only" }, value)), index$1.h("slot", null)));
  }
  get el() { return index$1.getElement(this); }
};

exports.bcm_avatar = BcmAvatar;
exports.bcm_badge = BcmBadge;
