/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c591ecec.js');
require('./color-helper-dc98a2ca.js');
require('./datetime-helper-eaf4fadb.js');
const generate = require('./generate-b43c3d3b.js');
require('./number-helper-a708fa79.js');
const stringHelper = require('./string-helper-9cdfb273.js');
require('./validators-84d8ae28.js');
require('./element-dragger-a8562f82.js');
const utils = require('./utils-fc077139.js');
const bcm = require('./bcm-1d1fcf21.js');
const captionTemplate = require('./caption-template-85a18b6d.js');
const index$1 = require('./index-95ae50d0.js');
require('./types-cc4adee7.js');
require('./types-7523fd99.js');
const input_style = require('./input.style-b5676e6a.js');
require('./colors-56282b00.js');
require('./colors-250d0982.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');
require('./index-5279dea2.js');

const NumberInput = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.focus = index.createEvent(this, "bcm-focus", 3);
    this.blur = index.createEvent(this, "bcm-blur", 3);
    this.clear = index.createEvent(this, "bcm-clear", 3);
    this.change = index.createEvent(this, "bcm-change", 7);
    this.input = index.createEvent(this, "bcm-input", 3);
    this.beforeClear = index.createEvent(this, "bcm-input-before-clear", 3);
    this._onChange = index.createEvent(this, "bcm-onchange", 3);
    this.inValid = false;
    this.onInput = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("input", (e) => {
        this.input.emit(e);
      });
    };
    this.onChange = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("change", (e) => {
        this.change.emit(e);
      });
    };
    this.onFocus = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("focus", e => {
        this.isFocus = true;
        this.focus.emit(e);
      });
    };
    this.onBlur = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("blur", e => {
        this.isFocus = false;
        this.blur.emit(e);
      });
    };
    this.onKeydown = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("keydown", (e) => {
        if (e.key === "ArrowUp") {
          e.preventDefault();
          this.up();
        }
        if (e.key === "ArrowDown") {
          e.preventDefault();
          this.down();
        }
      });
    };
    this.listener = () => {
      this.onInput();
      this.onChange();
      this.onFocus();
      this.onBlur();
      this.onKeydown();
    };
    this._id = generate.Generate.UID();
    this.value = "";
    this.defaultValue = undefined;
    this._prefix = undefined;
    this._suffix = undefined;
    this.size = "medium";
    this.min = undefined;
    this.max = undefined;
    this.step = 1;
    this.label = undefined;
    this.name = undefined;
    this.fullWidth = false;
    this.clearable = false;
    this.readonly = false;
    this.disabled = false;
    this.required = false;
    this.mode = "none";
    this.placeholder = undefined;
    this.thousandSeparator = false;
    this.decimalSeparator = undefined;
    this.decimalScale = undefined;
    this.fixedDecimalScale = false;
    this.allowNegative = true;
    this.allowLeadingZeros = false;
    this.unitPrefix = undefined;
    this.unitSuffix = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = bcm.Bcm.Status.default;
    this.captionError = undefined;
    this.customErrorMessage = undefined;
    this.isFocus = false;
    this.captionCache = undefined;
    this.captionTypeCache = undefined;
    this.valueCache = "";
  }
  connectedCallback() {
    if (this.defaultValue)
      this.value = +this.defaultValue;
    this.valueCache = this.value;
    this.captionCache = this.caption;
    this.captionTypeCache = this.captionType;
  }
  componentDidLoad() {
    this.listener();
  }
  up() {
    if (this.disabled)
      return;
    if (!this.value)
      this.value = Number(0);
    this.value = +this.value + +this.step;
    if (this.mode === "strict") {
      if (this.max && +this.value > +this.max) {
        this.value = this.max;
      }
    }
  }
  down() {
    if (this.disabled)
      return;
    if (!this.value)
      this.value = Number(0);
    this.value = +this.value - +this.step;
    if (this.mode === "strict") {
      if (this.min && +this.value < +this.min) {
        this.value = this.min;
      }
    }
  }
  handleClear(e) {
    if (this.disabled)
      return;
    const event = this.beforeClear.emit(e);
    if (event.defaultPrevented)
      return;
    this.value = "";
    this.clear.emit(e);
  }
  checkLength() {
    const { min, max, value } = this;
    const minLength = min;
    const maxLength = max;
    let valuePoint = value === null || value === void 0 ? void 0 : value.toString();
    if (min && max) {
      if (+valuePoint < +min || +valuePoint > +max) {
        this.setCaption(stringHelper.StringHelper.getMessage("betweenlength", [minLength, maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (minLength) {
      if (+valuePoint < +minLength) {
        this.setCaption(stringHelper.StringHelper.getMessage("minlength", [minLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (maxLength) {
      if (valuePoint > maxLength) {
        this.setCaption(stringHelper.StringHelper.getMessage("maxlength", [maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
  }
  watchValue() {
    this.checkLength();
  }
  async onBcmFocus() {
    this.inputElement.focus();
  }
  async onBcmBlur() {
    this.inputElement.blur();
  }
  async onBcmSelect() {
    this.inputElement.select();
  }
  async set(data) {
    await this.setValue(data);
  }
  async get() {
    return this.value || null;
  }
  async setValue(value) {
    this.value = value;
  }
  async setClear() {
    this.value = null;
    await utils.delay(10);
    return Promise.resolve();
  }
  async resetCaption() {
    await utils.delay(10);
    this.caption = this.captionCache;
    this.captionType = this.captionTypeCache;
    this.captionError = null;
    this.inValid = false;
    await utils.delay(10);
    return Promise.resolve();
  }
  setCaption(caption = null, captionType = null) {
    this.caption = caption || this.captionCache;
    this.captionType = captionType || this.captionTypeCache;
    this.captionError = captionType == "error" ? caption : null;
    return Promise.resolve();
  }
  disconnectedCallback() {
    var _a, _b, _c, _d, _e;
    (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.removeEventListener("input", this.onInput.bind(this));
    (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.removeEventListener("change", this.onChange.bind(this));
    (_c = this.inputElement) === null || _c === void 0 ? void 0 : _c.removeEventListener("focus", this.onFocus.bind(this));
    (_d = this.inputElement) === null || _d === void 0 ? void 0 : _d.removeEventListener("blur", this.onBlur.bind(this));
    (_e = this.inputElement) === null || _e === void 0 ? void 0 : _e.removeEventListener("keydown", this.onKeydown.bind(this));
  }
  render() {
    const { _prefix, _suffix, noCaption, captionType, captionError, caption, required, _id, label, size, fullWidth, clearable, value, placeholder, thousandSeparator, decimalSeparator, decimalScale, fixedDecimalScale, allowNegative, allowLeadingZeros, unitPrefix, unitSuffix, min, max, step, defaultValue, disabled, mode, } = this;
    const isActiveClearable = (value === null || value === void 0 ? void 0 : value.toString().length) > 0 && clearable;
    const isInvalid = this.captionType == "error" || this.inValid;
    const { host, inputContainer, slotClass, clearButton, controls, controlsBtn, controlsBtnDown, controlsBtnUp, controlsDivider, input } = input_style.inputStyle({
      size,
      fullWidth,
      clearable: isActiveClearable,
      disabled,
      invalid: isInvalid,
    });
    const numericProps = {
      placeholder,
      thousandSeparator,
      decimalSeparator,
      decimalScale,
      fixedDecimalScale,
      allowNegative,
      allowLeadingZeros,
      prefix: unitPrefix,
      suffix: unitSuffix,
    };
    /* input  props */
    const inputProps = {
      min,
      max,
      step,
      value,
      defaultValue,
    };
    const generalProps = Object.assign(Object.assign(Object.assign({ "id": _id + "-base-input", "name": this.name, disabled,
      mode, "aria-label": label, "aria-labelledby": _id + "-label", "getInputRef": el => {
        this.inputElement = el;
      }, "className": input() }, numericProps), inputProps), { "onValueChange": (e) => {
        const { floatValue } = e;
        this.value = floatValue;
        this._onChange.emit(e);
      } });
    return (index.h(index.Host, { class: host() }, label && index.h("bcm-label", { size: size, type: captionType, value: label, required: required, htmlFor: _id + "-base-input", id: _id + "-label" }), index.h("div", { class: inputContainer() }, index.h("div", { class: slotClass() }, index.h("slot", { name: "prefix" }, _prefix && _prefix.length > 0 && index.h("span", { innerHTML: _prefix }))), index.h("bcm-react-number", { disabled: disabled, class: "tw-flex-1", options: generalProps }), index.h("div", { class: slotClass() }, !disabled && isActiveClearable && (index.h("button", { class: clearButton(), onClick: event => this.handleClear(event) }, index.h("i", { class: "far fa-times" }))), index.h("slot", { name: "suffix" }, _suffix && _suffix.length > 0 && index.h("span", { innerHTML: _suffix })), index.h("div", { class: controls() }, index.h("button", { type: "button", disabled: disabled, class: index$1.classnames(controlsBtn(), controlsBtnUp()), onClick: this.up.bind(this), "aria-label": "Increase value" }, index.h("i", { class: "fas fa-caret-up" })), index.h("div", { class: controlsDivider() }), index.h("button", { type: "button", disabled: disabled, class: index$1.classnames(controlsBtn(), controlsBtnDown()), onClick: this.down.bind(this), "aria-label": "Decrease value" }, index.h("i", { class: "fas fa-caret-down" }))))), index.h(captionTemplate.CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption, customErrorMessage: this.customErrorMessage })));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "value": ["watchValue"]
  }; }
};

exports.bcm_number_input = NumberInput;
