/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c591ecec.js');
const colorHelper = require('./color-helper-dc98a2ca.js');
require('./datetime-helper-eaf4fadb.js');
const generate = require('./generate-b43c3d3b.js');
require('./number-helper-a708fa79.js');
const stringHelper = require('./string-helper-9cdfb273.js');
require('./validators-84d8ae28.js');
require('./element-dragger-a8562f82.js');
const bcm = require('./bcm-1d1fcf21.js');
const utils = require('./utils-fc077139.js');
const floatingUi = require('./floating-ui-5b396838.js');
require('./colors-56282b00.js');
require('./colors-250d0982.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');

class Popover {
  constructor({ content, message, targetId, placement = "top", color = "slate-700", trigger = "none", openDelay = 0, closeDelay = 0, autoHide = false, emitShow = () => { }, emitHide = () => { } }) {
    this.destroy = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.el.remove();
        this.emitHide();
      }
      this.removeListener();
    };
    this.target = () => document.getElementById(this.targetId);
    this.init = () => {
      const { tone, colorName } = colorHelper.ColorHelper.parseColor(this.color, 700);
      const dc = generate.Generate.getTwColor({
        color: colorName,
        tones: {
          [tone]: ["bg"],
        },
        variable: "--bcm-popover",
      });
      const style = Object.assign(Object.assign({}, dc), (dc["--bcm-popover-bg"] && { "--bcm-popover-text": colorHelper.ColorHelper.reverseColor(dc["--bcm-popover-bg"]) }));
      const inner = `<div class="tw-z-[1] tw-rounded tw-py-2">
            ${this.message ? `<div class="tw-font-medium tw-w-full tw-pb-1 tw-px-4">${stringHelper.StringHelper.trim(this.message)}</div>` : ""}
            ${this.content ? `<div class="tw-font-normal tw-px-4">${stringHelper.StringHelper.trim(this.content)}</div>` : ""}
        </div>`;
      this.el = generate.Generate.createElement("div", {
        "id": `${this.targetId}-popover-box`,
        "class": `
            tw-whitespace-normal tw-break-words tw-block tw-font-sans tw-pointer-events-none tw-max-w-[256px] tw-rounded tw-bg-[--bcm-popover-bg] tw-text-[--bcm-popover-text] tw-text-3 tw-absolute tw-top-0 tw-left-0 tw-z-tooltip tw-opacity-0 tw-transition-opacity tw-duration-300
            tw-shadow`,
        "role": "popover",
        "aria-hidden": "true",
        "style": colorHelper.ColorHelper.generateStyleString(style),
        "innerHTML": inner,
      });
    };
    this.update = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
        this.show();
      }
    };
    this.listener = () => {
      window.addEventListener("resize", this.calculatePosition.bind(this));
      window.addEventListener("scroll", this.calculatePosition.bind(this), true);
      window.addEventListener("wheel", this.calculatePosition.bind(this));
      if (this.trigger && this.target()) {
        switch (this.trigger) {
          case "click":
            this.target().addEventListener("click", this.toggle.bind(this));
            this.autoHide && this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "hover":
            this.target().addEventListener("mouseenter", this.show.bind(this));
            this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "focus":
            this.target().addEventListener("focus", this.show.bind(this), true);
            this.target().addEventListener("blur", this.hide.bind(this), true);
            break;
        }
      }
      this.calculatePosition();
    };
    this.removeListener = () => {
      window.removeEventListener("resize", this.calculatePosition.bind(this));
      window.removeEventListener("scroll", this.calculatePosition.bind(this), true);
      window.removeEventListener("wheel", this.calculatePosition.bind(this));
      if (this.target()) {
        this.target().removeEventListener("click", this.toggle.bind(this));
        this.target().removeEventListener("mouseenter", this.show.bind(this));
        this.target().removeEventListener("mouseleave", this.hide.bind(this));
        this.target().removeEventListener("focus", this.show.bind(this), true);
        this.target().removeEventListener("blur", this.hide.bind(this), true);
      }
    };
    this.calculatePosition = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        floatingUi.FloatingUI({
          ref: this.target(),
          floating: this.el,
          placement: this.placement,
          offset: 8,
          arrow: true,
          arrowColor: `tw-bg-[--bcm-popover-bg]`,
        });
      }
    };
    this.show = async () => {
      if (this.message) {
        this.init();
        await utils.delay(this.openDelay);
        this.calculatePosition();
        this.emitShow();
      }
      else {
        this.hide();
      }
    };
    this.hide = async () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        await utils.delay(this.closeDelay);
        this.el.remove();
        this.emitHide();
      }
    };
    this.toggle = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
      }
      else {
        this.show();
      }
    };
    this.content = content;
    this.message = message;
    this.targetId = targetId;
    this.placement = placement;
    this.color = color;
    this.trigger = trigger;
    this.id = targetId + "-popover-box";
    this.openDelay = openDelay;
    this.closeDelay = closeDelay;
    this.autoHide = autoHide;
    this.emitShow = emitShow;
    this.emitHide = emitHide;
  }
}

const BcmPopover = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.bcmPopoverShow = index.createEvent(this, "bcmPopoverShow", 7);
    this.bcmPopoverHide = index.createEvent(this, "bcmPopoverHide", 7);
    this._id = generate.Generate.UID();
    this.message = "";
    this.content = "";
    this.placement = bcm.Bcm.Placement.top;
    this.trigger = bcm.Bcm.TriggerType.click;
    this.open = false;
    this.color = "white";
    this.openDelay = 0;
    this.closeDelay = 0;
    this.autoHide = false;
    this.targetId = undefined;
    this.popover = undefined;
  }
  emitShow() {
    this.bcmPopoverShow.emit();
  }
  emitHide() {
    this.bcmPopoverHide.emit();
  }
  connectedCallback() {
    this.popover = new Popover({
      targetId: this.targetId || this._id,
      content: this.content,
      message: this.message,
      placement: this.placement,
      color: this.color,
      trigger: this.trigger,
      openDelay: this.openDelay,
      closeDelay: this.closeDelay,
      autoHide: this.autoHide,
      emitShow: () => this.emitShow(),
      emitHide: () => this.emitHide(),
    });
  }
  componentDidLoad() {
    this.popover.listener();
  }
  componentDidRender() {
    const { content, message, placement, color, trigger } = this;
    this.popover.content = content;
    this.popover.message = message;
    this.popover.placement = placement;
    this.popover.color = color;
    this.popover.trigger = trigger;
    this.popover.openDelay = this.openDelay;
    this.popover.closeDelay = this.closeDelay;
    this.popover.autoHide = this.autoHide;
    this.popover.update();
  }
  disconnectedCallback() {
    this.popover.destroy();
  }
  async show() {
    this.popover.show();
  }
  async hide() {
    this.popover.hide();
  }
  async toggle() {
    this.popover.toggle();
  }
  render() {
    return (index.h(index.Host, { class: "tw-inline-flex tw-h-fit tw-w-fit", role: "tooltip", "aria-hidden": "true" }, index.h("slot", null)));
  }
  get el() { return index.getElement(this); }
};

exports.bcm_popover = BcmPopover;
