/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c591ecec.js');
const index$1 = require('./index-95ae50d0.js');
const generate = require('./generate-b43c3d3b.js');
const stepperStates = require('./stepper-states-e293bcd3.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');
require('./colors-250d0982.js');
require('./index-d321de96.js');

const BcmStep = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.dataIsLoaded = index.createEvent(this, "data-is-loaded", 7);
    this.bcmChangeHidden = index.createEvent(this, "bcm-change-hidden", 7);
    this.hidden = false;
    this._id = generate.Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = 'default';
    this.icon = undefined;
    this.loading = false;
    this.valid = true;
    this.index = undefined;
    this.active = false;
    this.htmlTemplate = undefined;
    this.isLast = false;
    this.disabled = false;
  }
  componentDidLoad() {
    if (this.isLast) {
      this.dataIsLoaded.emit(true);
    }
  }
  getStepper() {
    const stepper = this.el.closest('bcm-stepper');
    return stepper || null;
  }
  activeChanged() {
    if (this.active) {
      const stepper = this.getStepper();
      if (stepper) {
        stepperStates.stepperStates.changeStepDisabled(stepper.stepperId, this._id, false);
      }
    }
  }
  statusChange() {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.stepperStates.changeStepStatus(stepper.stepperId, this._id, this.status);
    }
  }
  hiddenChange(newValue, oldValue) {
    const stepper = this.getStepper();
    if (newValue !== oldValue && stepper && !this.active) {
      this.bcmChangeHidden.emit(newValue);
    }
  }
  disabledChange(newValue, _) {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.stepperStates.changeStepDisabled(stepper.stepperId, this._id, newValue);
    }
  }
  render() {
    const { hidden, valid, active } = this;
    const hostClasses = index$1.classnames('bcm-stepper__step', valid ? 'bcm-stepper__step-valid' : 'bcm-stepper__step-invalid', {
      'bcm-stepper__step-active': active,
      hidden,
    });
    return (index.h(index.Host, { class: hostClasses }, index.h("slot", null)));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "active": ["activeChanged"],
    "status": ["statusChange"],
    "hidden": ["hiddenChange"],
    "disabled": ["disabledChange"]
  }; }
};

exports.bcm_step = BcmStep;
