/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c591ecec.js');
require('./color-helper-dc98a2ca.js');
require('./datetime-helper-eaf4fadb.js');
const generate = require('./generate-b43c3d3b.js');
require('./number-helper-a708fa79.js');
require('./string-helper-9cdfb273.js');
require('./validators-84d8ae28.js');
require('./element-dragger-a8562f82.js');
const utils = require('./utils-fc077139.js');
const bcm = require('./bcm-1d1fcf21.js');
require('./colors-56282b00.js');
require('./colors-250d0982.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');

const BcmTabGroup = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.tabChange = index.createEvent(this, "bcm-tab-change", 7);
    this._id = generate.Generate.UID();
    this.active = "1";
    this.color = "blue";
    this.size = bcm.Bcm.Size.medium;
  }
  get tabItems() {
    return this.el.querySelectorAll("bcm-tab-item");
  }
  get tabItemHeaders() {
    return this.el.querySelectorAll("bcm-tab-item-header");
  }
  setTabAttributes() {
    this.tabItems.forEach((tab, index) => {
      const itemActive = tab.hasAttribute("active") ? true : false;
      const itemDisabled = tab.hasAttribute("disabled") ? true : false;
      const itemIndex = index + 1;
      tab.setAttribute("index", `${itemIndex}`);
      tab.setAttribute("size", this.size);
      tab.setAttribute("color", this.color);
      tab.removeAttribute("active");
      tab.removeAttribute("disabled");
      if (itemActive) {
        tab.setAttribute("active", "");
        this.active = itemIndex.toString();
      }
      if (itemDisabled) {
        tab.setAttribute("disabled", "");
      }
      // const headerElement = tab.querySelector("bcm-tab-item-header");
      // if (headerElement) {
      //     this.setHeaderTabAttributes(headerElement, tab);
      // }
    });
    const isHasActiveTab = Array.from(this.tabItems).some((tab) => tab.hasAttribute("active"));
    if (!isHasActiveTab) {
      this.tabItems[0].setAttribute("active", "");
    }
  }
  componentWillLoad() {
    this.setTabAttributes();
  }
  handleTabClick(index) {
    var _a, _b;
    if (index) {
      this.active = index.toString();
    }
    if ((_a = this.tabItems) === null || _a === void 0 ? void 0 : _a.length) {
      this.tabItems.forEach(tab => {
        tab.removeAttribute("active");
        tab.setAttribute("size", this.size);
        tab.setAttribute("color", this.color);
        if (tab.getAttribute("index") == this.active) {
          tab.setAttribute("active", "");
        }
      });
    }
    if ((_b = this.tabItemHeaders) === null || _b === void 0 ? void 0 : _b.length) {
      this.tabItemHeaders.forEach(tabHeader => {
        tabHeader.removeAttribute("active");
        tabHeader.setAttribute("size", this.size);
        tabHeader.setAttribute("color", this.color);
        if (tabHeader.getAttribute("index") == this.active) {
          tabHeader.setAttribute("active", "");
          // tabHeader.scrollIntoView({
          //     behavior: "smooth",
          //     block: "start",
          //     inline: "center",
          // });
        }
      });
    }
    this.tabChange.emit(this.active);
  }
  componentDidRender() {
    this.inkBarControl();
    this.handleTabClick();
  }
  componentDidLoad() {
    this.inkBarControl();
    this.observeTabItem();
  }
  async setActive(index) {
    if (index > 0 && index <= this.tabItems.length && index != +this.active) {
      this.handleTabClick(index);
    }
  }
  async refresh() {
    await utils.delay(100);
    index.forceUpdate(this);
  }
  inkBarControl() {
    var _a;
    const inkBar = this.el.querySelector("#ink-bar");
    if (!inkBar || !((_a = this.tabItemHeaders) === null || _a === void 0 ? void 0 : _a.length))
      return;
    for (let i = 0; i < this.tabItemHeaders.length; i++) {
      const tab = this.tabItemHeaders[i];
      const tabIndex = tab.getAttribute("index");
      if (tabIndex === this.active) {
        inkBar.style.width = `${tab.offsetWidth}px`;
        // inkBar.style.top = `${tab.offsetHeight}px`;
        inkBar.style.transform = `translateX(${tab.offsetLeft}px)`;
      }
    }
  }
  activeChanged(newVal, _oldVal) {
    var _a;
    if (!((_a = this.tabItems) === null || _a === void 0 ? void 0 : _a.length))
      return;
    const isHasActiveTab = Array.from(this.tabItems).some(tab => tab.getAttribute("index") == newVal);
    if (!isHasActiveTab) {
      this.active = "1";
    }
  }
  observeTabItem() {
    const callback = (mutationsList, _observer) => {
      for (const mutation of mutationsList) {
        if (mutation.type === "childList") {
          // for (const node of mutation.addedNodes) {
          //     if (node.nodeName === "BCM-TAB-ITEM") {
          //         // Do something when a bcm-tab-item is added
          //     }
          // }
          for (const node of mutation.removedNodes) {
            if (node.nodeName === "BCM-TAB-ITEM") {
              // Do something when a bcm-tab-item is removed
              // console.log("A bcm-tab-item was removed.", node);
              if (node === null || node === void 0 ? void 0 : node.active) {
                this.active = "1";
              }
              this.tabProcess();
            }
          }
        }
      }
    };
    this.observer = new MutationObserver(callback);
    this.observer.observe(this.el, { childList: true });
  }
  tabProcess() {
    this.updateTabHeaders();
    this.regenerateIndex();
    this.refresh();
    this.inkBarControl();
  }
  updateTabHeaders() {
    const tabIds = Array.from(this.tabItems).map(tab => tab.getAttribute("id"));
    const tabHeaderElements = Array.from(this.el.querySelector("#tab-header").querySelectorAll("bcm-tab-item-header"));
    if (tabIds.length !== tabHeaderElements.length) {
      tabHeaderElements.forEach(tabHeader => {
        const tabId = tabHeader.getAttribute("tab-id");
        if (!tabIds.includes(tabId)) {
          tabHeader.remove();
        }
      });
    }
  }
  regenerateIndex() {
    this.tabItems.forEach((tab, index) => {
      const itemIndex = index + 1;
      tab.setAttribute("index", `${itemIndex}`);
      const headerElement = this.el.querySelector(`bcm-tab-item-header[tab-id="${tab.getAttribute("id")}"]`);
      if (headerElement) {
        headerElement.setAttribute("index", `${itemIndex}`);
        headerElement.style.order = `${itemIndex}`;
      }
    });
  }
  render() {
    const { color } = this;
    const dc = generate.Generate.getTwColor({
      color: color,
      tones: [700]
    });
    return (index.h(index.Host, { class: "tw-flex tw-w-full tw-flex-col tw-items-center tw-font-sans" }, index.h("div", { role: "tablist", "aria-label": "Tab Group", class: `tw-relative tw-flex tw-w-full tw-flex-row tw-items-end tw-justify-between`, id: "tab-header", style: dc }, index.h("div", { id: "ink-bar", class: `tw-bg-default-700 tw-absolute tw-bottom-0 tw-left-0 tw-mt-[-2px] tw-h-[2px] tw-rounded-sm tw-transition-transform` })), index.h("slot", null)));
  }
  get el() { return index.getElement(this); }
  static get watchers() { return {
    "active": ["activeChanged"]
  }; }
};

exports.bcm_tab_group = BcmTabGroup;
