/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-c591ecec.js');
const index$1 = require('./index-95ae50d0.js');
const generate = require('./generate-b43c3d3b.js');
const stringHelper = require('./string-helper-9cdfb273.js');
const bcm = require('./bcm-1d1fcf21.js');
const utils = require('./utils-fc077139.js');
require('./_commonjsHelpers-ed84c3ca.js');
require('./package-033055fc.js');
require('./colors-250d0982.js');

const BcmToast = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.bcmToastClose = index.createEvent(this, "bcm-toast-close", 7);
    this.bcmToastOpen = index.createEvent(this, "bcm-toast-open", 7);
    this.bcmToastView = index.createEvent(this, "bcm-toast-view", 7);
    this._id = generate.Generate.UID();
    this.status = undefined;
    this.message = "";
    this.description = "";
    this.duration = 3000;
    this.position = bcm.Bcm$1.Placement.bottom;
    this.messageIcon = undefined;
    this.viewButton = false;
    this.closeButton = true;
    this.removeAfterClose = true;
    this.visible = false;
    this.visibleContainer = false;
  }
  async show() {
    const toast = document.getElementById(this._id + "-container");
    if (toast) {
      const container = this.createContainer(this.position);
      container.appendChild(toast);
    }
    this.visible = true;
    await utils.delay(50);
    this.visibleContainer = true;
    this.bcmToastOpen.emit();
    if (this.timer) {
      clearTimeout(this.timer);
    }
    this.timer = setTimeout(() => this.hide(), this.duration);
    return this.el;
  }
  async hide() {
    this.visibleContainer = false;
    clearTimeout(this.timer);
    this.bcmToastClose.emit();
    await utils.delay(100);
    this.visible = false;
    if (this.removeAfterClose) {
      await utils.delay(100);
      this.el.remove();
    }
  }
  handleMouseEnter() {
    clearTimeout(this.timer);
  }
  handleMouseLeave() {
    if (this.visible) {
      this.timer = setTimeout(() => this.hide(), 1000);
    }
  }
  createContainer(position) {
    const containerId = "bcm-toast-container-" + position;
    let container = document.getElementById(containerId);
    if (!container) {
      container = document.createElement("div");
      container.id = containerId;
      const classlist = index$1.classnames("tw-pointer-events-none tw-overflow-hidden tw-z-50", "tw-fixed tw-flex", "tw-p-4 tw-gap-4", {
        "tw-flex-col-reverse tw-inset-x-0 tw-top-4 tw-mx-auto": this.position === bcm.Bcm$1.Placement.top,
        "tw-flex-col tw-inset-x-0 tw-bottom-4 tw-mx-auto": this.position === bcm.Bcm$1.Placement.bottom,
        "tw-flex-col-reverse tw-top-4 tw-left-4": this.position === bcm.Bcm$1.Placement["top-start"],
        "tw-flex-col-reverse tw-top-4 tw-right-4": this.position === bcm.Bcm$1.Placement["top-end"],
        "tw-flex-col tw-bottom-4 tw-left-4": this.position === bcm.Bcm$1.Placement["bottom-start"],
        "tw-flex-col tw-bottom-4 tw-right-4": this.position === bcm.Bcm$1.Placement["bottom-end"],
      });
      container.classList.add(...classlist.split(" "));
      document.body.appendChild(container);
    }
    return container;
  }
  render() {
    return (index.h(index.Host, { class: "tw-hidden" }, index.h("div", { class: index$1.classnames({ "tw-hidden": !this.visible }), id: this._id + "-container", onMouseEnter: () => this.handleMouseEnter(), onMouseLeave: () => this.handleMouseLeave() }, index.h("div", { class: index$1.classnames("tw-pointer-events-auto tw-transform tw-transition tw-duration-300 tw-ease-in-out", "tw-flex tw-min-w-[256px] tw-max-w-[420px] tw-flex-col tw-gap-2 tw-rounded-lg tw-border tw-font-sans tw-font-normal tw-shadow-2", "tw-border-gray-300 tw-bg-white tw-text-slate-600", {
        "tw-p-4": !this.description,
        "tw-p-6": this.description,
      }, "tw-transform tw-transition tw-duration-300 tw-ease-in-out ", {
        "tw-translate-y-0 tw-opacity-100 ": this.visibleContainer,
        "tw-opacity-0": !this.visibleContainer,
        "tw-translate-y-[-100%]": !this.visibleContainer && this.position.split("-")[0] === "top",
        "tw-translate-y-[100%]": !this.visibleContainer && this.position.split("-")[0] === "bottom",
      }) }, index.h("div", { class: "tw-flex tw-items-center tw-justify-between tw-gap-5" }, index.h("div", { class: "tw-flex tw-items-center tw-gap-3 " }, this.status && (index.h("i", { class: index$1.classnames("tw-text-6", this.messageIcon ? this.messageIcon : "far", {
        "tw-text-emerald-500": this.status === "success",
        "tw-text-red-500": this.status === "error",
        "tw-text-amber-500": this.status === "warning",
        "tw-text-blue-500": this.status === "info",
      }, {
        "fa-check-circle": !this.messageIcon && this.status === "success",
        "fa-exclamation-circle": !this.messageIcon && this.status === "error",
        "fa-exclamation-triangle": !this.messageIcon && this.status === "warning",
        "fa-info-circle": !this.messageIcon && this.status === "info",
      }) })), index.h("span", { class: index$1.classnames({
        "tw-text-4": !this.description,
        "tw-text-6 tw-font-medium": this.description,
      }) }, !this.description && this.status && index.h("span", null, stringHelper.StringHelper.capitalize(this.status), ": "), this.message)), index.h("div", { class: "tw-flex tw-items-baseline tw-gap-2" }, this.viewButton && (index.h("button", { onClick: () => this.bcmToastView.emit(), title: "View details", class: index$1.classnames("tw-cursor-pointer tw-rounded tw-bg-transparent tw-p-0 tw-px-1 tw-text-4 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200", "tw-transition tw-duration-300 tw-ease-in-out", {
        "tw-font-medium": this.description,
      }) }, "View")), this.closeButton && (index.h("button", { title: "Close", onClick: () => this.hide(), class: index$1.classnames("tw-rounded tw-bg-transparent tw-p-0", "tw-flex tw-h-6 tw-w-6 tw-cursor-pointer tw-items-center tw-justify-center tw-text-3 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200") }, index.h("i", { class: "fal fa-times" }))))), this.description && index.h("div", { class: "tw-text-3 " }, this.description), index.h("slot", null)))));
  }
  get el() { return index.getElement(this); }
};

exports.bcm_toast = BcmToast;
