/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

const stringHelper = require('./string-helper-9cdfb273.js');

const opposites = {
  left: 'right',
  right: 'left',
  top: 'bottom',
  bottom: 'top'
};
const join = (places) => {
  if (places.length === 1) {
    return opposites[places[0]];
  }
  return [opposites[places[0]], places[1]].join('-');
};
const getPlacement = (box, host, placement, changePlacement) => {
  const { width, height } = box.getBoundingClientRect();
  const { left, right, top, bottom } = host.getBoundingClientRect();
  const places = stringHelper.StringHelper.textContains(placement, '-') ? placement.split('-') : placement.split(' ');
  const bodyWidth = document.documentElement.clientWidth;
  const bodyHeight = document.documentElement.clientHeight;
  switch (places[0]) {
    case 'left':
      if (left < (width + 8)) {
        changePlacement(join(places));
      }
      break;
    case 'right':
      if (bodyWidth < (right + width + 8)) {
        changePlacement(join(places));
      }
      break;
    case 'top':
      if (top < (height + 8)) {
        changePlacement(join(places));
      }
      break;
    case 'bottom':
      if (bodyHeight < (bottom + height + 8)) {
        changePlacement(join(places));
      }
      break;
  }
};
const setPosition = (placement, target, positions) => {
  const rect = target.getBoundingClientRect();
  const top = (positions === null || positions === void 0 ? void 0 : positions.top) || (rect.top + window.pageYOffset);
  const left = (positions === null || positions === void 0 ? void 0 : positions.left) || (rect.left + window.pageXOffset);
  const height = rect.height;
  const width = rect.width;
  const places = stringHelper.StringHelper.textContains(placement, '-') ? placement.split('-') : placement.split(' ');
  switch (places[0]) {
    case 'left':
      switch (places[1]) {
        case 'start':
          return [top, left];
        case 'end':
          return [top + height, left];
        default:
          return [top + (height / 2), left];
      }
    case 'right':
      switch (places[1]) {
        case 'start':
          return [top, left + width];
        case 'end':
          return [top + height, left + width];
        default:
          return [top + (height / 2), left + width];
      }
    case 'top':
      switch (places[1]) {
        case 'start':
          return [top, left];
        case 'end':
          return [top, left + width];
        default:
          return [top, left + width / 2];
      }
    case 'bottom':
      switch (places[1]) {
        case 'start':
          return [top + height, left];
        case 'end':
          return [top + height, left + width];
        default:
          return [top + height, left + width / 2];
      }
  }
};

exports.getPlacement = getPlacement;
exports.setPosition = setPosition;
