/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
'use strict';

require('./element-dragger-a8562f82.js');
const stringHelper = require('./string-helper-9cdfb273.js');
const utils = require('./utils-fc077139.js');
const floatingUi = require('./floating-ui-5b396838.js');
const colorHelper = require('./color-helper-dc98a2ca.js');
require('./datetime-helper-eaf4fadb.js');
const generate = require('./generate-b43c3d3b.js');
require('./number-helper-a708fa79.js');
require('./validators-84d8ae28.js');
const bcm = require('./bcm-1d1fcf21.js');

class Tooltip {
  constructor({ message, targetId, placement = "top", color = "slate-700", trigger = "none", size = "medium", openDelay = 0, closeDelay = 0, autoHide = false, emitShow = () => { }, emitHide = () => { }, }) {
    this.destroy = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.el.remove();
        this.emitHide();
      }
      this.removeListener();
    };
    this.target = () => document.getElementById(this.targetId);
    this.sizeClass = () => {
      switch (this.size) {
        case bcm.Bcm.Size.small:
          return "tw-text-1";
        case bcm.Bcm.Size.medium:
          return "tw-text-2";
        case bcm.Bcm.Size.large:
          return "tw-text-3";
        default:
          return "tw-text-2";
      }
    };
    this.init = () => {
      const { tone, colorName } = colorHelper.ColorHelper.parseColor(this.color, 700);
      const dc = generate.Generate.getTwColor({
        color: colorName,
        tones: {
          [tone]: ["bg"],
        },
        variable: "--bcm-tooltip",
      });
      const style = Object.assign(Object.assign({}, dc), (dc["--bcm-tooltip-bg"] && { "--bcm-tooltip-text": colorHelper.ColorHelper.reverseColor(dc["--bcm-tooltip-bg"]) }));
      this.el = generate.Generate.createElement("div", {
        "id": `${this.targetId}-tooltip-box`,
        "class": `
            tw-whitespace-normal tw-break-words tw-block tw-font-sans tw-pointer-events-none tw-max-w-xs tw-py-1 tw-px-2 tw-rounded tw-bg-[--bcm-tooltip-bg] tw-text-[--bcm-tooltip-text] ${this.sizeClass()} tw-absolute tw-top-0 tw-left-0 tw-z-tooltip tw-opacity-0 tw-transition-opacity tw-duration-300`,
        "role": "tooltip",
        "aria-hidden": "true",
        "style": colorHelper.ColorHelper.generateStyleString(style),
        "innerHTML": stringHelper.StringHelper.trim(this.message),
      });
    };
    this.update = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
        this.show();
      }
    };
    this.listener = () => {
      window.addEventListener("resize", this.calculatePosition.bind(this));
      window.addEventListener("scroll", this.calculatePosition.bind(this), true);
      window.addEventListener("wheel", this.calculatePosition.bind(this));
      if (this.trigger && this.target()) {
        switch (this.trigger) {
          case "click":
            this.target().addEventListener("click", this.toggle.bind(this));
            this.autoHide && this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "hover":
            this.target().addEventListener("mouseenter", this.show.bind(this));
            this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "focus":
            this.target().addEventListener("focus", this.show.bind(this), true);
            this.target().addEventListener("blur", this.hide.bind(this), true);
            break;
        }
      }
      this.calculatePosition();
    };
    this.removeListener = () => {
      window.removeEventListener("resize", this.calculatePosition.bind(this));
      window.removeEventListener("scroll", this.calculatePosition.bind(this), true);
      window.removeEventListener("wheel", this.calculatePosition.bind(this));
      if (this.target()) {
        this.target().removeEventListener("click", this.toggle.bind(this));
        this.target().removeEventListener("mouseenter", this.show.bind(this));
        this.target().removeEventListener("mouseleave", this.hide.bind(this));
        this.target().removeEventListener("focus", this.show.bind(this), true);
        this.target().removeEventListener("blur", this.hide.bind(this), true);
      }
    };
    this.calculatePosition = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        floatingUi.FloatingUI({
          ref: this.target(),
          floating: this.el,
          placement: this.placement,
          offset: 8,
          arrow: true,
          arrowColor: `tw-bg-[--bcm-tooltip-bg]`,
        });
      }
    };
    this.show = async () => {
      if (this.message) {
        this.init();
        await utils.delay(this.openDelay);
        this.calculatePosition();
        this.emitShow();
      }
      else {
        this.hide();
      }
    };
    this.hide = async () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        await utils.delay(this.closeDelay);
        this.el && this.el.remove();
        this.emitHide();
      }
    };
    this.toggle = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
      }
      else {
        this.show();
      }
    };
    this.message = message;
    this.targetId = targetId;
    this.placement = placement;
    this.color = color;
    this.id = targetId + "-tooltip-box";
    this.trigger = trigger;
    this.size = size;
    this.openDelay = openDelay;
    this.closeDelay = closeDelay;
    this.autoHide = autoHide;
    this.emitShow = emitShow;
    this.emitHide = emitHide;
  }
}

exports.Tooltip = Tooltip;
