/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { Host, h } from "@stencil/core";
import { avatarStyle } from "./avatar.style";
export class BcmAvatar {
  constructor() {
    this._id = Generate.UID();
    this.image = undefined;
    this.name = undefined;
    this.alt = undefined;
    this.shape = Bcm.Shape.ellipse;
    this.size = Bcm.Size.medium;
    this.color = undefined;
    this.icon = "fas fa-user";
    this.status = undefined;
    this.blink = false;
    this.isFallback = false;
  }
  getFirstLetters(name) {
    const words = name.split(" ");
    const initials = words.map(word => word.charAt(0).toUpperCase()).join("");
    return initials.substring(0, 2);
  }
  renderDefaultIcon() {
    return h("bcm-icon", { icon: this.icon });
  }
  render() {
    const { shape, size, status, color, blink } = this;
    const { base, badge, image } = avatarStyle({ size, shape, status: !!status });
    const dc = Generate.getTwColor({
      color: color,
      tones: {
        200: ["bg"],
        700: ["text"],
      },
      variable: "--bcm-avatar",
    });
    const customSize = !Object.keys(Bcm.BaseSize).includes(size)
      ? {
        width: `${size}px`,
        height: `${size}px`,
        fontSize: `${+size / 3}px`,
      }
      : {};
    const RenderContent = () => {
      if (this.isFallback) {
        if (this.name) {
          return this.getFirstLetters(this.name);
        }
        else {
          return this.renderDefaultIcon();
        }
      }
      else if (this.image) {
        return h("img", { class: image(), onError: () => (this.isFallback = true), src: this.image, alt: this.alt, "aria-label": this.alt || this.name || "Avatar" });
      }
      else if (this.name) {
        return this.getFirstLetters(this.name);
      }
      else {
        return this.renderDefaultIcon();
      }
    };
    return (h(Host, { role: "img", "aria-label": this.alt || this.name || "Avatar", class: base(), style: Object.assign(Object.assign({}, dc), customSize) }, h(RenderContent, null), status && h("bcm-badge", { class: badge(), status: status, blink: blink, "aria-hidden": "true" }), h("slot", null)));
  }
  static get is() { return "bcm-avatar"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "image": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "image",
        "reflect": false
      },
      "name": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "name",
        "reflect": false
      },
      "alt": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "alt",
        "reflect": false
      },
      "shape": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "ShapeTypes",
          "resolved": "\"ellipse\" | \"square\"",
          "references": {
            "ShapeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "shape",
        "reflect": false,
        "defaultValue": "Bcm.Shape.ellipse"
      },
      "size": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "color": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "icon": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false,
        "defaultValue": "\"fas fa-user\""
      },
      "status": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "StatusTypes",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false
      },
      "blink": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "blink",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "isFallback": {}
    };
  }
  static get elementRef() { return "el"; }
}
