/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { ColorHelper, Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { Host, h } from "@stencil/core";
import { badgeStyle } from "./badge.style";
export class BcmBadge {
  constructor() {
    this.statusColors = {
      info: "blue",
      error: "red",
      warning: "amber",
      success: "emerald",
    };
    this._id = Generate.UID();
    this.blink = false;
    this.value = undefined;
    this.color = undefined;
    this.size = Bcm.Size.medium;
    this.status = undefined;
  }
  render() {
    const { blink, value, size, color, status } = this;
    const _color = this.statusColors[status] || color;
    const { tone, colorName } = ColorHelper.parseColor(_color, 700);
    const dc = Generate.getTwColor({
      color: colorName,
      tones: {
        [tone]: ["bg"],
      },
      variable: "--bcm-badge",
    });
    const style = Object.assign(Object.assign({}, dc), (dc["--bcm-badge-bg"] && { "--bcm-badge-text": ColorHelper.reverseColor(dc["--bcm-badge-bg"]) }));
    const { base, dotText, host } = badgeStyle({ size, blink });
    return (h(Host, { class: host(), role: "status", "aria-live": "polite" }, h("div", { class: base(), style: style }, h("span", { class: dotText(), role: "presentation", "aria-hidden": "true" }, value), h("span", { class: "tw-sr-only" }, value)), h("slot", null)));
  }
  static get is() { return "bcm-badge"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "blink": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "blink",
        "reflect": false,
        "defaultValue": "false"
      },
      "value": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false
      },
      "color": {
        "type": "any",
        "mutable": false,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "size": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "status": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "StatusTypes",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false
      }
    };
  }
  static get elementRef() { return "el"; }
}
