/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
import cs from 'classnames';
import buttonState from "../states/button.states";
import { ButtonTemplate } from '../../../templates/button-template';
import { ColorHelper } from '../../../helper/color-helper';
export class BcmButton {
  constructor() {
    this._id = Generate.UID();
    this.hidden = false;
    this.type = Bcm.ButtonType.button;
    this.size = Bcm.Size.medium;
    this.disabled = false;
    this.kind = Bcm.Kind.solid;
    this.outline = false;
    this.icon = undefined;
    this.href = undefined;
    this.target = undefined;
    this.value = undefined;
    this.loading = false;
    this.variant = undefined;
    this.dataDismiss = false;
    this.fullWidth = false;
    this.iconPosition = Bcm.IconPosition.prefix;
    this.color = undefined;
    this.checked = false;
  }
  handleOnClick(e) {
    if (this.disabled) {
      e.preventDefault();
      e.stopPropagation();
      return false;
    }
  }
  componentWillRender() {
    const { size, kind, icon, iconPosition, disabled, outline, href, type, target, loading, variant, fullWidth, hidden, value } = this;
    buttonState.setValue({
      id: this._id,
      size,
      type,
      kind,
      icon,
      iconPosition,
      disabled,
      outline,
      href,
      target,
      loading,
      variant,
      fullWidth,
      hidden,
      value,
      isIconOnly: !!(this.isHaveChildNodes() && this.icon),
    });
  }
  getStateData() {
    return buttonState.getValue(this._id);
  }
  isHaveChildNodes() {
    const childNodes = Array.from(this.el.childNodes);
    return childNodes.find(node => node.nodeType === Node.TEXT_NODE) === undefined;
  }
  setColor(color = this.color) {
    if (!color)
      return;
    const { kind } = this;
    const prefix = `--bcm-button-${kind}`;
    if (kind === 'solid') {
      return {
        [`${prefix}-color-default`]: ColorHelper.dsColor('white'),
        [`${prefix}-color-hover`]: ColorHelper.dsColor('white'),
        [`${prefix}-color-focus-visible`]: ColorHelper.dsColor('white'),
        [`${prefix}-color-active`]: ColorHelper.dsColor('white'),
        [`${prefix}-color-disabled`]: ColorHelper.dsColor('white'),
        [`${prefix}-background-color-default`]: ColorHelper.dsColor(color + '-700'),
        [`${prefix}-background-color-hover`]: ColorHelper.dsColor(color + '-800'),
        [`${prefix}-background-color-focus-visible`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-background-color-active`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-background-color-disabled`]: ColorHelper.dsColor('slate-400'),
        [`${prefix}-border-default`]: ColorHelper.dsColor(color + '-700'),
        [`${prefix}-border-hover`]: ColorHelper.dsColor(color + '-800'),
        [`${prefix}-border-focus-visible`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-border-active`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-border-disabled`]: ColorHelper.dsColor('slate-400')
      };
    }
    else if (kind == 'link') {
      return {
        [`${prefix}-color-default`]: ColorHelper.dsColor(color + '-700'),
        [`${prefix}-color-hover`]: ColorHelper.dsColor(color + '-800'),
        [`${prefix}-color-focus-visible`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-color-active`]: ColorHelper.dsColor(color + '-900'),
        [`${prefix}-color-disabled`]: ColorHelper.dsColor('slate-400'),
        [`${prefix}-background-color-default`]: ColorHelper.dsColor('transparent'),
        [`${prefix}-background-color-hover`]: ColorHelper.dsColor('transparent'),
        [`${prefix}-background-color-focus-visible`]: ColorHelper.dsColor('slate-50'),
        [`${prefix}-background-color-active`]: ColorHelper.dsColor('slate-50'),
        [`${prefix}-background-color-disabled`]: ColorHelper.dsColor('transparent'),
        [`${prefix}-border-default`]: ColorHelper.dsColor('transparent'),
        [`${prefix}-border-hover`]: ColorHelper.dsColor('transparent'),
        [`${prefix}-border-focus-visible`]: ColorHelper.dsColor('slate-50'),
        [`${prefix}-border-active`]: ColorHelper.dsColor('slate-50'),
        [`${prefix}-border-disabled`]: ColorHelper.dsColor('transparent')
      };
    }
    else if (kind == 'ghost') {
      return {
        [`${prefix}-color-default`]: ColorHelper.dsColor(color + '-700'),
        [`${prefix}-color-hover`]: ColorHelper.dsColor(color + '-800'),
        [`${prefix}-color-focus-visible`]: ColorHelper.dsColor(color + '-600'),
        [`${prefix}-color-active`]: ColorHelper.dsColor(color + '-600'),
        [`${prefix}-color-disabled`]: ColorHelper.dsColor(color + '-300'),
        [`${prefix}-background-color-default`]: ColorHelper.dsColor(color + '-100'),
        [`${prefix}-background-color-hover`]: ColorHelper.dsColor(color + '-200'),
        [`${prefix}-background-color-focus-visible`]: ColorHelper.dsColor(color + '-400'),
        [`${prefix}-background-color-active`]: ColorHelper.dsColor(color + '-300'),
        [`${prefix}-background-color-disabled`]: ColorHelper.dsColor(color + '-50'),
        [`${prefix}-border-default`]: ColorHelper.dsColor(color + '-300'),
        [`${prefix}-border-hover`]: ColorHelper.dsColor(color + '-400'),
        [`${prefix}-border-focus-visible`]: ColorHelper.dsColor(color + '-600'),
        [`${prefix}-border-active`]: ColorHelper.dsColor(color + '-500'),
        [`${prefix}-border-disabled`]: ColorHelper.dsColor(color + '-200'),
      };
    }
    return;
  }
  render() {
    const { size, kind, disabled, outline, type, href, target, loading, variantColor, fullWidth, hidden, icon, iconPosition, value, isIconOnly, iconSize } = this.getStateData();
    const classPrefix = 'bcm-button';
    let componentClass = cs('bcm-button__container', `bcm-button__container-size-${size}`, `bcm-button__container-kind-${outline ? 'ghost' : kind}`, {
      'full-width': fullWidth,
      'checked': this.checked,
      'disabled': disabled,
    });
    const hostClasses = cs(`${classPrefix}`, {
      hidden,
      'full-width': fullWidth,
    });
    return (h(Host, { class: hostClasses, style: this.setColor(variantColor), onClick: () => !disabled && this.bcmClick.emit(this.el) }, h(ButtonTemplate, { type: type, value: value, componentClass: componentClass, disabled: disabled, href: href, target: target, icon: icon, loading: loading, iconSize: iconSize, isIconOnly: isIconOnly, iconPosition: iconPosition })));
  }
  static get is() { return "bcm-button"; }
  static get originalStyleUrls() {
    return {
      "$": ["button.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["button.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": true,
        "defaultValue": "false"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "ButtonTypes",
          "resolved": "\"button\" | \"reset\" | \"submit\"",
          "references": {
            "ButtonTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "Bcm.ButtonType.button"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": true,
        "defaultValue": "false"
      },
      "kind": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "KindTypes",
          "resolved": "\"ghost\" | \"ghost-link\" | \"link\" | \"solid\"",
          "references": {
            "KindTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "kind",
        "reflect": false,
        "defaultValue": "Bcm.Kind.solid"
      },
      "outline": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "outline",
        "reflect": false,
        "defaultValue": "false"
      },
      "icon": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false
      },
      "href": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "href",
        "reflect": false
      },
      "target": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target",
        "reflect": false
      },
      "value": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false
      },
      "loading": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "loading",
        "reflect": false,
        "defaultValue": "false"
      },
      "variant": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "VariantProp",
          "resolved": "\"filter\" | \"remove\" | \"cancel\" | \"close\" | \"reset\" | \"submit\" | \"apply\" | \"download\" | \"save\" | \"ok\" | \"new\" | \"add\" | \"create\" | \"prev\" | \"next\" | \"send\" | \"delete\" | \"decline\" | \"archive\" | \"edit\" | \"export\" | \"import\" | \"update\"",
          "references": {
            "VariantProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "variant",
        "reflect": false
      },
      "dataDismiss": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "data-dismiss",
        "reflect": false,
        "defaultValue": "false"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "iconPosition": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "IconPositionTypes",
          "resolved": "\"prefix\" | \"suffix\"",
          "references": {
            "IconPositionTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon-position",
        "reflect": false,
        "defaultValue": "Bcm.IconPosition.prefix"
      },
      "color": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "checked": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "checked",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmClick",
        "name": "bcm-click",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleOnClick",
        "target": undefined,
        "capture": true,
        "passive": false
      }];
  }
}
