/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { CaptionTemplate, LabelTemplate } from "@bcm/template";
import { Host, h } from "@stencil/core";
import cs from "classnames";
export class BcmChipGroup {
  constructor() {
    this._id = Generate.UID();
    this.name = undefined;
    this.value = undefined;
    this.required = false;
    this.size = "medium";
    this.label = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = Bcm.Status.default;
    this.captionError = undefined;
  }
  handleSelectItem(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      const data = e.detail;
      const chipItem = document.createElement("bcm-chip");
      // chipItem.setAttribute("type", "dismissable");
      chipItem.setAttribute("value", data.id);
      chipItem.innerHTML = data.text;
      target.parentNode.insertBefore(chipItem, target);
      this.setSelect(data.id);
    }
  }
  handleDismiss(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      target.remove();
      this.setSelect();
    }
  }
  handleSelect(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      if (target.type === "addable") {
        this.showListComponent(target.id);
      }
      else {
        this.setSelect(target.value);
      }
    }
  }
  showListComponent(id) {
    const listComponent = document.getElementById(id + "-list");
    if (listComponent) {
      listComponent.show();
    }
  }
  getSlots() {
    return this.el.querySelectorAll("bcm-chip");
  }
  setSelect(val = this.value, noEvent = false) {
    const slots = this.getSlots();
    if ((slots === null || slots === void 0 ? void 0 : slots.length) === 0)
      return;
    const slotArray = Array.from(slots);
    const selectedSlot = slotArray.find(slot => slot.value === val);
    if (!selectedSlot) {
      this.value = slotArray[0].value;
    }
    else {
      this.value = val;
    }
    slotArray.forEach(slot => {
      slot.selected = slot.value === this.value;
    });
    if (!noEvent) {
      this.change.emit(this.value);
    }
  }
  setAttr() {
    const slots = this.getSlots();
    if ((slots === null || slots === void 0 ? void 0 : slots.length) === 0)
      return;
    const slotArray = Array.from(slots);
    slotArray.forEach(slot => {
      slot.setAttribute("size", this.size);
    });
  }
  componentWillLoad() {
    this.setSelect();
  }
  componentDidRender() {
    this.setAttr();
  }
  render() {
    const { label, required, noCaption, captionType, captionError, caption } = this;
    const hostClasses = cs("bcm-chip-group");
    return (h(Host, { class: hostClasses }, label && (h("div", null, h(LabelTemplate, { type: captionType, value: label, required: required }))), h("div", { class: "tw-flex tw-flex-wrap tw-gap-1" }, h("slot", null)), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption })));
  }
  static get is() { return "bcm-chip-group"; }
  static get originalStyleUrls() {
    return {
      "$": ["chip-group.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["chip-group.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "name": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "name",
        "reflect": false
      },
      "value": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false
      },
      "required": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "required",
        "reflect": false,
        "defaultValue": "false"
      },
      "size": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "\"small\" | \"medium\" | \"large\"",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "\"medium\""
      },
      "label": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "label",
        "reflect": false
      },
      "caption": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption",
        "reflect": false
      },
      "noCaption": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-caption",
        "reflect": false,
        "defaultValue": "false"
      },
      "captionType": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "StatusTypes",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-type",
        "reflect": false,
        "defaultValue": "Bcm.Status.default"
      },
      "captionError": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-error",
        "reflect": false
      }
    };
  }
  static get events() {
    return [{
        "method": "change",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "bcm-chip-select-item",
        "method": "handleSelectItem",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-chip-dismiss",
        "method": "handleDismiss",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-chip-select",
        "method": "handleSelect",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
