/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from "@stencil/core";
import cs from "classnames";
import { ColorHelper } from "../../../helper/color-helper";
import { Generate } from "../../../helper/generate";
export class BcmChip {
  constructor() {
    this._id = Generate.UID();
    this.type = "default";
    this.size = "medium";
    this.avatar = undefined;
    this.icon = undefined;
    this.disabled = undefined;
    this.color = undefined;
    this.selected = false;
    this.value = undefined;
    this.data = [];
  }
  // @ts-ignore
  setColor() {
    if (!this.color && !this.selected)
      return;
    const color = this.selected ? "indigo" : this.color; // Important: Find a solution for this in V3! ( selected ? this.color : "indigo" )
    return {
      "--bcm-chip-color": ColorHelper.dsColor(color + "-700"),
      "--bcm-chip-color-hover": ColorHelper.dsColor(color + "-700"),
      "--bcm-chip-color-focus": ColorHelper.dsColor(color + "-700"),
      "--bcm-chip-color-active": ColorHelper.dsColor(color + "-700"),
      "--bcm-chip-color-disabled": ColorHelper.dsColor("slate" + "-300"),
      "--bcm-chip-background-color": ColorHelper.dsColor(color + "-100"),
      "--bcm-chip-background-color-hover": ColorHelper.dsColor(color + "-200"),
      "--bcm-chip-background-color-focus": ColorHelper.dsColor(color + "-300"),
      "--bcm-chip-background-color-active": ColorHelper.dsColor(color + "-200"),
      "--bcm-chip-background-color-disabled": ColorHelper.dsColor("slate" + "-100"),
    };
  }
  handleDismiss(e) {
    e.stopPropagation();
    e.preventDefault();
    this.bcmChipDismiss.emit(this.el);
  }
  handleAddable(e) {
    e.stopPropagation();
    e.preventDefault();
    this.bcmChipAddable.emit(this.el);
  }
  listSelectedItems(e) {
    // To be added to v3
    e.stopPropagation();
    e.preventDefault();
    if (e.detail) {
      this.bcmChipSelectItem.emit(e.detail);
      e.target.setClear();
      this.data = this.data.filter(item => item.id !== e.detail.id);
    }
  }
  render() {
    const { type, size, avatar, disabled, color } = this;
    const hostClasses = cs("bcm-chip", `bcm-chip__type-${type}`, `bcm-chip__size-${size}`, {
      "bcm-chip--disabled": disabled,
      "bcm-chip--avatar": avatar,
    });
    return (h(Host, { class: hostClasses, tabindex: "0", disabled: disabled, style: this.setColor(), onClick: e => this.bcmChipSelect.emit(e) }, h("div", { class: cs(this.selected ? "bcm-chip--selected-icon" : "tw-hidden tw-w-0") }, h("bcm-icon", { icon: "fal fa-check" })), avatar && h("bcm-avatar", Object.assign({ size: size }, (color && { color }), (avatar && { image: avatar }))), type === "addable" && (h("span", { class: "bcm-chip__left" }, h("bcm-icon", { onClick: e => this.handleAddable(e), icon: "far fa-plus" }))), h("slot", null), type === "dismissable" && (h("span", { class: "bcm-chip__type-dismissable-icon" }, h("bcm-icon", { onClick: e => this.handleDismiss(e), icon: "fal fa-times" }))), this.data &&
      this.data.length > 0 && ( // To be added to v3
    h("bcm-list", { "onBcm-change": e => this.listSelectedItems(e), id: this._id + "-list", data: this.data, linkedComponent: this._id, searchable: true, highlight: true }))));
  }
  static get is() { return "bcm-chip"; }
  static get originalStyleUrls() {
    return {
      "$": ["chip.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["chip.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "\"default\" | \"dismissable\" | \"addable\" | \"checkable\"",
          "resolved": "\"addable\" | \"checkable\" | \"default\" | \"dismissable\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "\"default\""
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "\"small\" | \"medium\" | \"large\"",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "\"medium\""
      },
      "avatar": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "avatar",
        "reflect": false
      },
      "icon": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "false",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false
      },
      "color": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "selected": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "selected",
        "reflect": false,
        "defaultValue": "false"
      },
      "value": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false
      },
      "data": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "data",
        "reflect": false,
        "defaultValue": "[]"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmChipDismiss",
        "name": "bcm-chip-dismiss",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmChipSelect",
        "name": "bcm-chip-select",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmChipAddable",
        "name": "bcm-chip-addable",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmChipSelectItem",
        "name": "bcm-chip-select-item",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
}
