/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { StatusProps, TypeProps } from './types';
import cs from 'classnames';
import alertStates from '../states/alert.states';
import { Generate } from '../../../helper/generate';
import { StringHelper } from '../../../helper/string-helper';
export class BcmAlert {
  constructor() {
    this.hidden = false;
    this._id = Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = StatusProps.info;
    this.type = TypeProps.basic;
    this.dismissible = undefined;
    this.icon = false;
    this.bigIcon = false;
    this.fullWidth = false;
  }
  componentWillRender() {
    if (this.dismissible != undefined) {
      if (this.dismissible == '') {
        this.dismissible = true;
      }
      else if (typeof this.dismissible === 'string' && this.dismissible == 'false' || this.dismissible == 'true') {
        this.dismissible = StringHelper.elementBooleanCheck(this.dismissible);
      }
      else {
        return;
      }
    }
    this.setData();
  }
  dismiss(e) {
    this.alertdismissible.emit(e);
    this.el.remove();
  }
  getStateData() {
    return alertStates.getValue(this._id);
  }
  setData() {
    const { status, type, bigIcon } = this;
    alertStates.setValue({
      id: this._id,
      status,
      type,
      bigIcon,
    });
  }
  render() {
    const { fullWidth, status, type, icon, bigIcon, _title, hidden, dismissible, description } = this;
    const hostClasses = cs('bcm-alert', 'bcm-alert__type-' + type, 'bcm-alert__status-' + status, {
      'full-width': fullWidth,
      hidden,
    });
    const headerMessage = cs('bcm-alert__header-message', !!description ? 'size-3' : 'size-2');
    const state = this.getStateData();
    return (h(Host, { class: hostClasses }, h("span", { hidden: true }, h("slot", null)), h("header", { class: "bcm-alert__header" }, h("span", { class: headerMessage }, (bigIcon || icon) && h("bcm-icon", { icon: state.icon }), _title), dismissible && (h("span", { class: "bcm-alert__header-dismiss", onClick: e => this.dismiss(e) }, typeof dismissible === 'string' ? dismissible : h("bcm-icon", { icon: "fal fa-times" })))), h("main", { class: "bcm-alert__content size-2" }, description && h("p", null, description), h("slot", null))));
  }
  static get is() { return "bcm-alert"; }
  static get originalStyleUrls() {
    return {
      "$": ["alert.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["alert.css"]
    };
  }
  static get properties() {
    return {
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_title": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_title",
        "reflect": false
      },
      "description": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "description",
        "reflect": false
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "StatusProp",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "StatusProps.info"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TypeProp",
          "resolved": "\"banner\" | \"basic\"",
          "references": {
            "TypeProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "TypeProps.basic"
      },
      "dismissible": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "dismissible",
        "reflect": false
      },
      "icon": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false,
        "defaultValue": "false"
      },
      "bigIcon": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "big-icon",
        "reflect": false,
        "defaultValue": "false"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "alertdismissible",
        "name": "bcm-alert-dismissible",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
}
