/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
import cs from 'classnames';
import snq from 'snq';
import { StringHelper } from '../../../helper/string-helper';
export class BcmButtonGroup {
  constructor() {
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.type = 'button';
    this.color = undefined;
    this.disabled = undefined;
    this.direction = Bcm.Direction.horizontal;
  }
  componentWillLoad() {
    const { size, color } = this;
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        let buttonDisabled = StringHelper.elementBooleanCheck(this.el.getAttribute('disabled')) || button.getAttribute('disabled');
        color && button.setAttribute('color', color);
        // type && button.setAttribute('type', type);
        size && button.setAttribute('size', size);
        buttonDisabled && (button.disabled = true);
      });
  }
  getButtons() {
    return snq(() => Array.from(this.el.querySelectorAll('bcm-button')), []);
  }
  radioControl(id) {
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        button.checked = button.id == id;
      });
  }
  checkboxControl(id) {
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        button.checked = button.id == id ? (button.checked = !button.checked) : button.checked;
      });
  }
  listenButton({ target }) {
    if (target.id) {
      switch (this.type) {
        case 'radio':
          this.radioControl(target.id);
          break;
        case 'checkbox':
          this.checkboxControl(target.id);
          break;
      }
      this.handleChange();
    }
  }
  handleChange() {
    const obj = this.getButtons()
      .filter(Boolean)
      .map(button => ({
      id: button.id,
      checked: button.checked,
    }));
    this.bcmChange.emit(obj);
  }
  render() {
    const hostClasses = cs('bcm-button-group', {
      'bcm-button-group__vertical': this.direction === Bcm.Direction.vertical,
    });
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  static get is() { return "bcm-button-group"; }
  static get originalStyleUrls() {
    return {
      "$": ["button-group.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["button-group.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'checkbox' | 'radio' | 'button'",
          "resolved": "\"button\" | \"checkbox\" | \"radio\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "'button'"
      },
      "color": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "disabled": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false
      },
      "direction": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "DirectionTypes",
          "resolved": "\"horizontal\" | \"vertical\"",
          "references": {
            "DirectionTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "direction",
        "reflect": false,
        "defaultValue": "Bcm.Direction.horizontal"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmChange",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "bcm-click",
        "method": "listenButton",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
