/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { ColorHelper } from '../../../helper/color-helper';
import { Generate } from '../../../helper/generate';
import { StringHelper } from '../../../helper/string-helper';
import { SlotTemplate } from '../../../templates/slot-template';
export class BcmCard {
  constructor() {
    this.modalSlots = {};
    this._id = Generate.UID();
    this.hidden = false;
    this.color = "slate";
    this.header = undefined;
    this.footer = undefined;
  }
  connectedCallback() {
    const bcmSlots = this.el.querySelectorAll('bcm-card-header, bcm-card-footer');
    if (bcmSlots) {
      bcmSlots.forEach(el => {
        let slotName = StringHelper.lowercase(StringHelper.split(el.nodeName, '-').pop());
        el.slot = slotName;
        if (slotName === 'header') {
          let color = el.getAttribute('color');
          if (color) {
            color = color == 'white' ? color : color + '-50';
            el.setAttribute('style', `background-color: ${ColorHelper.dsColor(color)}`);
          }
        }
        el.classList.add(`bcm-card__${slotName}`);
        this.modalSlots[slotName] = el;
      });
    }
  }
  render() {
    const hostClasses = cs('bcm-card', this.hidden ? 'hidden' : null);
    const FooterTemplate = () => {
      return (this.modalSlots['footer'] && (h("slot", { name: "footer" })));
    };
    const HeaderTemplate = () => {
      return (this.modalSlots['header'] && (h("slot", { name: "header" })));
    };
    return (h(Host, { class: hostClasses }, h(HeaderTemplate, null), h("div", { class: "bcm-card__body" }, h(SlotTemplate, null)), h(FooterTemplate, null)));
  }
  static get is() { return "bcm-card"; }
  static get originalStyleUrls() {
    return {
      "$": ["card.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["card.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "color": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false,
        "defaultValue": "\"slate\""
      }
    };
  }
  static get states() {
    return {
      "header": {},
      "footer": {}
    };
  }
  static get elementRef() { return "el"; }
}
