import { Generate } from '@bcm/helpers';
import { Bcm } from '@bcm/model';
import { h, Host } from '@stencil/core';
import cs from 'classnames';
export class BcmDrawer {
  constructor() {
    this._id = Generate.UID();
    this._title = undefined;
    this.position = Bcm.Position.right;
    this.fullWidth = false;
    this._open = false;
    this._close = false;
  }
  async show() {
    this.bcmBeforeOpen.emit(true);
  }
  async hide() {
    this.bcmBeforeClose.emit(true);
    this._close = true;
  }
  _beforeOpen(event) {
    var _a;
    event && ((_a = event.target) === null || _a === void 0 ? void 0 : _a.id) == this._id && (this._open = true);
  }
  _beforeClose(event) {
    var _a;
    event && ((_a = event.target) === null || _a === void 0 ? void 0 : _a.id) == this._id && (this._open = false);
  }
  _openHandler() {
    if (this._open) {
      document.body.classList.add('bcm-disable-scroll');
    }
    else {
      document.body.classList.remove('bcm-disable-scroll');
    }
  }
  keyboardNavigation(e) {
    const keyCode = e.keyCode || 0;
    if (keyCode == Bcm.KeyCode.escape) {
      e.preventDefault();
      this.hide();
    }
  }
  disconnectedCallback() {
    this._open = false;
    document.body.classList.remove('bcm-disable-scroll');
    this.el.remove();
  }
  render() {
    const hostClasses = cs('bcm-drawer', {
      'bcm-drawer--open': this._open,
    });
    const contentClasses = cs('bcm-drawer__content', `bcm-drawer__content--position-${this.position}`, {
      'bcm-drawer__content--open': this._open,
      'bcm-drawer__content--close': this._close,
      'full-width': this.fullWidth,
    });
    const overlay = cs('bcm-drawer__overlay', {
      'bcm-drawer__overlay--open': this._open,
      'bcm-drawer__overlay--close': this._close,
    });
    return (h(Host, { class: hostClasses }, h("div", { class: overlay, onClick: () => this.hide() }), h("div", { class: contentClasses }, h("div", { class: "bcm-drawer__content-header" }, h("bcm-text", { class: "bcm-drawer__content-header-title", "font-size": "6", weight: "600", color: "slate-800" }, this._title, " ", h("slot", { name: "header" })), h("div", { class: "bcm-drawer__content-header-close size-4", onClick: () => this.hide() }, h("bcm-icon", { icon: "fal fa-times" }))), h("div", { style: { 'overflow': 'auto', 'flex': '1' } }, h("div", { class: "bcm-drawer__content-body" }, h("slot", null))), h("div", { class: "bcm-drawer__content-footer" }, h("slot", { name: "footer" })))));
  }
  static get is() { return "bcm-drawer"; }
  static get originalStyleUrls() {
    return {
      "$": ["drawer.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["drawer.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_title": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_title",
        "reflect": false
      },
      "position": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "PositionTypes",
          "resolved": "\"bottom\" | \"left\" | \"right\" | \"top\"",
          "references": {
            "PositionTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "position",
        "reflect": false,
        "defaultValue": "Bcm.Position.right"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "_open": {},
      "_close": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmBeforeOpen",
        "name": "bcm-before-open",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmBeforeClose",
        "name": "bcm-before-close",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "_open",
        "methodName": "_openHandler"
      }];
  }
  static get listeners() {
    return [{
        "name": "bcm-before-open",
        "method": "_beforeOpen",
        "target": "window",
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-before-close",
        "method": "_beforeClose",
        "target": "window",
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "keyboardNavigation",
        "target": "document",
        "capture": false,
        "passive": false
      }];
  }
}
