/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import cs from 'classnames';
import { StringHelper } from '../../../helper/string-helper';
import { ColorHelper } from '../../../helper/color-helper';
export class BcmExpansionPanel {
  constructor() {
    this._id = Generate.UID();
    this.expanded = false;
    this.disabled = false;
    this.hidden = false;
    this.valid = true;
    this.hideToggle = false;
    this.expandIconPosition = 'right';
    this.headerColor = 'white';
    this.tooltip = undefined;
    this.tooltipDefaultValue = {
      message: '',
      placement: 'top',
      trigger: 'hover',
      icon: 'fas fa-question-circle',
      color: 'grey-8',
    };
  }
  componentWillRender() {
    this.restoreSlots();
  }
  componentDidLoad() {
    this.checkExpanded();
  }
  createTooltip() {
    let tool;
    if (this.tooltip) {
      const { tooltip } = this;
      try {
        tool = JSON.parse(tooltip);
        tool = Object.assign(Object.assign({}, this.tooltipDefaultValue), tool);
      }
      catch (error) {
        tool = Object.assign(Object.assign({}, this.tooltipDefaultValue), { message: tooltip });
      }
    }
    return tool;
  }
  checkExpanded() {
    this.expanded = !!this.el['expanded'];
  }
  restoreSlots() {
    const slots = this.el.querySelectorAll('bcm-expansion-panel-header, bcm-expansion-panel-description, bcm-expansion-panel-extra');
    if (slots) {
      slots.forEach(element => {
        let slotName = StringHelper.lowercase(StringHelper.split(element.nodeName, '-').pop());
        element.slot = slotName;
        element.classList.add(`bcm-expansion-panel__${slotName}-content`);
        let color = element.getAttribute('color');
        if (color) {
          color = color == 'white' ? color : color + '-50';
          this.headerColor = color;
        }
      });
    }
  }
  toggle() {
    if (this.disabled) {
      return;
    }
    this.expanded = !this.expanded;
    if (this.expanded) {
      this.bcmOpen.emit(this._id);
    }
    else {
      this.bcmClose.emit(this._id);
    }
  }
  clickExtra(e) {
    e.stopPropagation();
    this.bcmExtra.emit(e);
  }
  render() {
    const { _id, expanded, disabled, hidden, hideToggle, tooltip, valid } = this;
    const { message, placement, icon, trigger, color } = tooltip ? this.createTooltip() : {};
    const hostClasses = cs('bcm-expansion-panel', valid ? 'bcm-expansion-panel--valid' : 'bcm-expansion-panel--invalid', {
      'bcm-expansion-panel--expanded': expanded,
      'bcm-expansion-panel--disabled': disabled,
      'bcm-expansion-panel--hide-toggle': hideToggle,
      'bcm-expansion-panel--icon-right': this.expandIconPosition === 'right',
      hidden,
    });
    return (h(Host, { class: hostClasses }, h("div", { class: "bcm-expansion-panel__header", onClick: () => this.toggle(), style: { 'background-color': ColorHelper.dsColor(this.headerColor) } }, h("div", { class: "bcm-expansion-panel__header-title" }, h("span", { id: _id + '-header' }, h("slot", { name: "header" })), tooltip && (h("span", { class: "bcm-expansion-panel__header-tooltip", style: Object.assign({}, color && { color: ColorHelper.convertColorToHex(color) }) }, h("bcm-tooltip", { message: message, placement: placement, trigger: trigger, "aria-label": message }, h("bcm-icon", { color: color, icon: icon })))), h("span", { id: _id + '-description', class: "bcm-expansion-panel__header-title-description" }, h("slot", { name: "description" }))), h("span", { id: _id + '-extra', class: "bcm-expansion-panel__header-extra", onClick: (e) => this.clickExtra(e) }, h("slot", { name: "extra" })), !hideToggle && (h("div", { class: "bcm-expansion-panel__header-toggle" }, h("bcm-icon", { icon: "far fa-chevron-down" })))), h("div", { class: "bcm-expansion-panel__body", id: _id + '-body' }, h("slot", null))));
  }
  static get is() { return "bcm-expansion-panel"; }
  static get originalStyleUrls() {
    return {
      "$": ["expansion-panel.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["expansion-panel.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "expanded": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "expanded",
        "reflect": false,
        "defaultValue": "false"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "valid": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "valid",
        "reflect": false,
        "defaultValue": "true"
      },
      "hideToggle": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hide-toggle",
        "reflect": false,
        "defaultValue": "false"
      },
      "expandIconPosition": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'right' | 'left'",
          "resolved": "\"left\" | \"right\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "expand-icon-position",
        "reflect": false,
        "defaultValue": "'right'"
      },
      "tooltip": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "tooltip",
        "reflect": false
      }
    };
  }
  static get states() {
    return {
      "headerColor": {},
      "tooltipDefaultValue": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmOpen",
        "name": "bcm-open",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmClose",
        "name": "bcm-close",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmExtra",
        "name": "bcm-extra",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
}
