/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import { NumberHelper } from '../../../helper/number-helper';
import { StringHelper } from '../../../helper/string-helper';
import { Validators } from '../../../helper/validators';
import Bcm from '../../../models/bcm';
import { CaptionTemplate } from '../../../templates/caption-template';
import { InputTemplate } from "../../../templates/input-template";
import cs from 'classnames';
import { delay } from '../../../utils/utils';
import { IsLoad } from '../../../decorators/is-load-decorator';
export class BcmInput {
  constructor() {
    this.isBlur = false;
    this._id = Generate.UID();
    this._internal_id = Generate.UID();
    this.value = '';
    this.size = Bcm.Size.medium;
    this.label = undefined;
    this.pattern = undefined;
    this.tooltip = undefined;
    this.placeholder = undefined;
    this.disabled = false;
    this.readonly = false;
    this.clearable = false;
    this.autocomplete = false;
    this.name = undefined;
    this.type = Bcm.InputType.text;
    this.hidden = false;
    this.required = false;
    this.fullWidth = false;
    this.noDefaultIcon = undefined;
    this.disableWatchValue = false // datetimepicker
    ;
    this.passwordToggle = false // type password
    ;
    this.maxLength = null;
    this.minLength = null;
    this.unitPrefix = null;
    this.unit = undefined;
    this.decimal = true;
    this.min = undefined;
    this.max = undefined;
    this.step = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = Bcm.Status.default;
    this.captionError = undefined;
    this.customErrorMessage = undefined;
    this._prefix = undefined;
    this._suffix = undefined;
    this.captionCache = undefined;
    this.captionTypeCache = undefined;
    this.inValid = false;
    this.valueCache = '';
    this.focused = false;
    this.isPasswordVisible = false;
  }
  connectedCallback() {
    this.valueCache = this.value;
    this.captionCache = this.caption;
    this.captionTypeCache = this.captionType;
  }
  componentWillRender() {
    const { type, isBlur, value } = this;
    if (value) {
      this.value.length > 0 && (this.checkPattern());
      switch (type) {
        case "number":
          {
            !this.checkValid("number") && (this.validationRange());
          }
          break;
        case "email":
          {
            isBlur && (this.checkValid("email"));
          }
          break;
        case "tel":
          {
            isBlur && (this.checkValid("phone"));
          }
          break;
        case "text":
          {
            isBlur && (this.checkLength());
          }
          break;
      }
    }
    else {
      !this.required && (this.resetCaption());
    }
  }
  async onBcmFocus() {
    this.inputElement.focus();
  }
  async onBcmBlur() {
    this.inputElement.blur();
  }
  async onBcmSelect() {
    this.inputElement.select();
  }
  async set(data) {
    await this.setValue(data);
  }
  async get() {
    return this.value || null;
  }
  async setValue(value) {
    this.inputElement.value = value;
    this.handleChange();
    return value;
  }
  async setClear() {
    this.inputElement.value = '';
    this.value = '';
    await delay(10);
    return Promise.resolve();
  }
  async resetCaption() {
    await delay(10);
    this.caption = this.captionCache;
    this.captionType = this.captionTypeCache;
    this.captionError = null;
    this.inValid = false;
    await delay(10);
    return Promise.resolve();
  }
  async getValue() {
    return this.value;
  }
  watchValue(newValue) {
    this.inputElement.value = newValue;
    if (!this.disableWatchValue && this.type != "number") {
      this.handleChange();
    }
  }
  listenKeyDown(e) {
    if (!this.disabled && !this.readonly) {
      this.onBcmFocus();
      if (this.type == "number") {
        var which = e.which || 0;
        var key = e.key;
        if (which == 38) { // up arrow
          e.preventDefault(),
            this.stepUpOrDown('up');
        }
        if (which == 40) { // down arrow
          e.preventDefault(),
            this.stepUpOrDown('down');
        }
        if ((which == 190 || which == 110 || key == ".") && this.value.indexOf(".") > -1) { //.
          e.preventDefault();
        }
        if ((which == 189 || which == 109 || key == "-")) { //-
          if (this.value.charAt(0) == '-' || this.value.indexOf("-") > -1) {
            e.preventDefault();
          }
        }
      }
    }
  }
  handleFocus() {
    this.focused = true;
    this.focus.emit();
    this.whatchHasFocus();
  }
  handleBlur() {
    this.focused = false;
    this.isBlur = true;
    this.whatchHasFocus();
    this.blur.emit();
  }
  handleChange() {
    this.valueSync();
  }
  handleInput() {
    if (this.type == "number" && this.maxLength) {
      const inputValue = this.inputElement.value;
      if (inputValue && inputValue.length > 0 && inputValue.length > this.maxLength) {
        this.inputElement.value = inputValue.substring(0, this.maxLength);
      }
    }
    this.valueSync();
  }
  valueSync() {
    if (this.value != this.inputElement.value) {
      this.value = this.inputElement.value;
      this.change.emit(this.value);
      return true;
    }
    this.input.emit(this.value);
    return false;
  }
  handleClear(e) {
    this.inputElement.value = '';
    this.value = '';
    this.inputElement.focus();
    this.clear.emit(e);
    this.change.emit(this.value);
    e.stopPropagation();
  }
  changePasswordVisibility() {
    this.isPasswordVisible = !this.isPasswordVisible;
  }
  setCaption(caption = null, captionType = null) {
    this.caption = caption || this.captionCache;
    this.captionType = captionType || this.captionTypeCache;
    this.captionError = captionType == "error" ? caption : null;
  }
  stepUpOrDown(type) {
    if (!this.inValid && !this.disabled && !this.readonly) {
      let step = +(this.step) || 1;
      let val = +(this.value);
      if (type == 'up') {
        val = +(val + step);
      }
      else {
        val = +(val - step);
      }
      this.value = val.toString();
      this.change.emit(this.value);
    }
  }
  validationRange() {
    const { min, max, value } = this;
    if (value !== undefined) {
      if ((min !== undefined) && (max !== undefined)) {
        if (value < min || value > max) {
          this.setCaption(StringHelper.getMessage('between', [min, max]), "error");
          return false;
        }
      }
      else if (min !== undefined) {
        if (value < min) {
          this.setCaption(StringHelper.getMessage('min', [min]), "error");
          return false;
        }
      }
      else if (max !== undefined) {
        if (value > max) {
          this.setCaption(StringHelper.getMessage('max', [max]), "error");
          return false;
        }
      }
    }
    this.setCaption();
    return true;
  }
  checkLength() {
    const { minLength, maxLength, value } = this;
    if (minLength && maxLength) {
      if (value.length < minLength || value.length > maxLength) {
        this.setCaption(StringHelper.getMessage('betweenlength', [minLength, maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (minLength) {
      if (value.length < minLength) {
        this.setCaption(StringHelper.getMessage('minlength', [minLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (maxLength) {
      if (value.length > maxLength) {
        this.setCaption(StringHelper.getMessage('maxlength', [maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
  }
  checkPattern() {
    const { pattern, value } = this;
    if (pattern) {
      if (Validators.pattern(value, pattern)) {
        this.setCaption();
        this.inValid = false;
      }
      else {
        this.setCaption(StringHelper.getMessage('pattern'), "error");
        this.inValid = true;
      }
    }
  }
  checkValid(type) {
    const { value } = this;
    if (Validators.isValidate(value, type)) {
      this.setCaption();
      this.inValid = false;
    }
    else {
      this.setCaption(StringHelper.getMessage(type), "error");
      this.inValid = true;
    }
    return this.inValid;
  }
  whatchHasFocus() {
    const { type, focused, valueCache, inValid } = this;
    if (type == Bcm.InputType.number && inValid) {
      if (focused) {
        this.value = '';
      }
      else {
        if (!StringHelper.isNullOrEmpty(this.value)) {
          this.value = this.value ? this.fixToNumber(this.value) : '';
        }
        else {
          this.value = valueCache;
        }
      }
    }
  }
  fixToNumber(val) {
    if (val) {
      const decLen = NumberHelper.getDecimalLength(this.step);
      return decLen > 0 ? NumberHelper.parseFloatFixed(val, decLen) : val;
    }
    return val;
  }
  eyeDropper() {
    const eyeDropper = new window.EyeDropper();
    eyeDropper
      .open()
      .then((color) => {
      this.setValue(color.sRGBHex);
    })
      .catch((error) => {
      if (error.name === "NotAllowedError") {
        console.log("Permission to access sensor was denied.");
      }
      else if (error.name === "NotSupportedError") {
        console.log("Sensor not supported.");
      }
    });
  }
  isEyeDropperSupported() {
    return typeof window !== "undefined" && "EyeDropper" in window;
  }
  toggleLinked() {
    const linkedElement = document.getElementById(`${this._id}-linked`);
    if (linkedElement) {
      linkedElement.toggle();
    }
  }
  render() {
    const { value, size, label, type, captionType, caption, captionError, placeholder, fullWidth, disabled, readonly, clearable, passwordToggle, isPasswordVisible, noDefaultIcon, unit, inValid, focused, unitPrefix, noCaption, required, hidden, _id, autocomplete } = this;
    const isActiveClearable = (value === null || value === void 0 ? void 0 : value.length) > 0 && clearable;
    const hostClasses = cs('bcm-input', `bcm-input__container--${size}`, {
      hidden,
      // 'no-margin': noMargin,
      'full-width': fullWidth
    });
    return (h(Host, { onfocus: () => this.onBcmFocus(), class: hostClasses, onClick: () => this.onBcmFocus() }, label && h("bcm-label", { onClick: () => this.toggleLinked(), tooltip: this.tooltip, type: captionType, value: label, required: required, htmlFor: _id, id: _id + "-label" }), h(InputTemplate, { id: _id, type: type, value: value, pattern: this.pattern, placeholder: placeholder, disabled: disabled, readonly: readonly, clearable: isActiveClearable, passwordToggle: passwordToggle, isPasswordVisible: isPasswordVisible, noDefaultIcon: noDefaultIcon, focused: focused, inValid: inValid, unitPrefix: unitPrefix, maxLength: this.maxLength, internalId: this._internal_id, captionType: captionType, unit: unit, size: size, handleFocus: () => this.handleFocus(), handleBlur: () => this.handleBlur(), handleChange: () => this.handleChange(), handleInput: () => this.handleInput(), steps: (e) => this.stepUpOrDown(e), changePasswordVisibility: () => this.changePasswordVisibility(), handleClear: (e) => this.handleClear(e), eyeDropper: () => this.eyeDropper(), isEyeDropperSupported: () => this.isEyeDropperSupported(), ref: (el) => this.inputElement = el, _suffix: this._suffix, _prefix: this._prefix, autocomplete: autocomplete }), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption, customErrorMessage: this.customErrorMessage }), type == Bcm.InputType.color &&
      h("bcm-linked", { targetId: this._id + "-container", id: this._id + "-linked" }, h("bcm-colorful", { id: this._id + "-picker", color: this.value, onBcmChange: e => this.setValue(e.detail) }))));
  }
  static get is() { return "bcm-input"; }
  static get originalStyleUrls() {
    return {
      "$": ["input.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["input.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Props *"
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_internal_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_internal_id",
        "reflect": false,
        "defaultValue": "Generate.UID()"
      },
      "value": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false,
        "defaultValue": "''"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "label": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "label",
        "reflect": false
      },
      "pattern": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "pattern",
        "reflect": false
      },
      "tooltip": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "tooltip",
        "reflect": false
      },
      "placeholder": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placeholder",
        "reflect": false
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "readonly": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "readonly",
        "reflect": false,
        "defaultValue": "false"
      },
      "clearable": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "clearable",
        "reflect": false,
        "defaultValue": "false"
      },
      "autocomplete": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "autocomplete",
        "reflect": false,
        "defaultValue": "false"
      },
      "name": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "name",
        "reflect": false
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "InputTypes",
          "resolved": "\"color\" | \"email\" | \"hidden\" | \"number\" | \"password\" | \"search\" | \"tel\" | \"text\" | \"url\"",
          "references": {
            "InputTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "Bcm.InputType.text"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "required": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "required",
        "reflect": false,
        "defaultValue": "false"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "noDefaultIcon": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-default-icon",
        "reflect": false
      },
      "disableWatchValue": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disable-watch-value",
        "reflect": false,
        "defaultValue": "false"
      },
      "passwordToggle": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "password-toggle",
        "reflect": false,
        "defaultValue": "false"
      },
      "maxLength": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max-length",
        "reflect": false,
        "defaultValue": "null"
      },
      "minLength": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min-length",
        "reflect": false,
        "defaultValue": "null"
      },
      "unitPrefix": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "unit-prefix",
        "reflect": false,
        "defaultValue": "null"
      },
      "unit": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "unit",
        "reflect": false
      },
      "decimal": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "decimal",
        "reflect": false,
        "defaultValue": "true"
      },
      "min": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min",
        "reflect": false
      },
      "max": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max",
        "reflect": false
      },
      "step": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "step",
        "reflect": false
      },
      "caption": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption",
        "reflect": false
      },
      "noCaption": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-caption",
        "reflect": false,
        "defaultValue": "false"
      },
      "captionType": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "StatusTypes",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-type",
        "reflect": false,
        "defaultValue": "Bcm.Status.default"
      },
      "captionError": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-error",
        "reflect": false
      },
      "customErrorMessage": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "custom-error-message",
        "reflect": false
      },
      "_prefix": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_prefix",
        "reflect": false
      },
      "_suffix": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_suffix",
        "reflect": false
      }
    };
  }
  static get states() {
    return {
      "captionCache": {},
      "captionTypeCache": {},
      "inValid": {},
      "valueCache": {},
      "focused": {},
      "isPasswordVisible": {}
    };
  }
  static get events() {
    return [{
        "method": "focus",
        "name": "bcm-focus",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Events *"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "blur",
        "name": "bcm-blur",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "clear",
        "name": "bcm-clear",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "change",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "input",
        "name": "bcm-input",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "onBcmFocus": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "onBcmBlur": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "onBcmSelect": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "set": {
        "complexType": {
          "signature": "(data: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "get": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setValue": {
        "complexType": {
          "signature": "(value: any) => Promise<any>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setClear": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "resetCaption": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getValue": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get watchers() {
    return [{
        "propName": "value",
        "methodName": "watchValue"
      }];
  }
  static get listeners() {
    return [{
        "name": "keydown",
        "method": "listenKeyDown",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
__decorate([
  IsLoad()
], BcmInput.prototype, "inputElement", void 0);
