/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
import cs from 'classnames';
import { ColorHelper } from '../../../helper/color-helper';
export class BcmItem {
  constructor() {
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.color = undefined;
    this.static = false;
  }
  setColor(color = this.color) {
    if (!color)
      return;
    const prefix = `--bcm-item`;
    if (color == "white") {
      return {
        [`${prefix}-color-default`]: ColorHelper.dsColor('slate-600'),
        [`${prefix}-color-hover`]: ColorHelper.dsColor('slate-600'),
        [`${prefix}-color-focus-visible`]: ColorHelper.dsColor('slate-600'),
        [`${prefix}-color-active`]: ColorHelper.dsColor('slate-600'),
        [`${prefix}-color-disabled`]: ColorHelper.dsColor('white'),
        [`${prefix}-background-color-default`]: ColorHelper.dsColor(this.static ? 'white' : 'white'),
        [`${prefix}-background-color-hover`]: ColorHelper.dsColor(this.static ? 'white' : 'slate-100'),
        [`${prefix}-background-color-focus-visible`]: ColorHelper.dsColor(this.static ? 'white' : 'slate-200'),
        [`${prefix}-background-color-active`]: ColorHelper.dsColor(this.static ? 'white' : 'slate-300'),
        [`${prefix}-background-color-disabled`]: ColorHelper.dsColor('slate-400'),
      };
    }
    return {
      [`${prefix}-color-default`]: ColorHelper.dsColor('white'),
      [`${prefix}-color-hover`]: ColorHelper.dsColor('white'),
      [`${prefix}-color-focus-visible`]: ColorHelper.dsColor('white'),
      [`${prefix}-color-active`]: ColorHelper.dsColor('white'),
      [`${prefix}-color-disabled`]: ColorHelper.dsColor('white'),
      [`${prefix}-background-color-default`]: ColorHelper.dsColor(color + '-500'),
      [`${prefix}-background-color-hover`]: ColorHelper.dsColor(color + '-600'),
      [`${prefix}-background-color-focus-visible`]: ColorHelper.dsColor(color + '-700'),
      [`${prefix}-background-color-active`]: ColorHelper.dsColor(color + '-800'),
      [`${prefix}-background-color-disabled`]: ColorHelper.dsColor('slate-400'),
    };
  }
  disconnectedCallback() {
    //   this.el.remove()
  }
  render() {
    const hostClasses = cs('bcm-item', `bcm-item-size-${this.size}`);
    return (h(Host, { class: hostClasses, style: this.setColor() }, h("slot", null)));
  }
  static get is() { return "bcm-item"; }
  static get originalStyleUrls() {
    return {
      "$": ["item.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["item.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "color": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false
      },
      "static": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "static",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get elementRef() { return "el"; }
}
