/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { FloatingUI, delay } from "@bcm/utils";
import { Host, h } from "@stencil/core";
export class BcmLinked {
  constructor() {
    this._id = Generate.UID();
    this.placement = Bcm.Placement["bottom-start"];
    this.trigger = Bcm.TriggerType.click;
    this.targetId = "";
    this.height = undefined;
    this.maxHeight = undefined;
    this.minHeight = "50px";
    this.isOpen = false;
    this.hideDelay = 500;
    this.moveOverLayContainer = false;
    this.isMouseOverLink = false;
    window.addEventListener("scroll", () => this.calculatePosition(), true);
    window.addEventListener("wheel", () => this.calculatePosition(), true);
    window.addEventListener("resize", () => this.calculatePosition(), true);
  }
  target() {
    const target = document.querySelectorAll(`[id="${this.targetId}"]`);
    if (target.length > 0) {
      target.length > 1 && console.warn(`[bcm-linked] - Target id "${this.targetId}" is duplicated`);
      return target[0];
    }
    else {
      // console.warn(`[bcm-linked] - id="${this._id}" was removed from the dom because id="${this.targetId}" could not be found.`);
      this.el.remove();
      return null;
    }
  }
  detectClone() {
    const linked = document.querySelectorAll(`[target-id="${this.targetId}"]`);
    if (linked.length > 0) {
      linked.forEach((item) => {
        if (item.id !== this._id) {
          console.warn(`[bcm-linked] - id="${this._id}" was removed from the dom because id="${this.targetId}" is duplicated.`);
          item.remove();
        }
      });
    }
  }
  dismissEvent() {
    document.dispatchEvent(new CustomEvent("linked-data-dismiss", { detail: { id: this._id } }));
  }
  checkDataDismiss(e) {
    Generate.findEventPathHasAttribute(e, "data-dismiss") && this.dismissEvent();
  }
  hideChildLinked() {
    this.el.querySelectorAll("[is-linked][linked-is-open]").forEach(item => {
      var _a;
      (_a = document.querySelector(`#${item.getAttribute("is-linked")}[is-open]`)) === null || _a === void 0 ? void 0 : _a.hide(false);
    });
  }
  calculatePosition() {
    const el = this.el;
    if (el && this.target()) {
      if (this.isOpen) {
        FloatingUI({
          ref: this.target(),
          floating: el,
          placement: this.placement,
          offset: 4,
          autoHidden: el.getAttribute("main-linked") === "true" ? false : true,
          autoSize: true,
          style: {
            height: this.height,
            maxHeight: this.maxHeight,
            minHeight: this.minHeight,
          },
        });
      }
      else {
        el.style.visibility = "hidden";
        el.style.opacity = "0";
      }
    }
  }
  init() {
    const target = this.target();
    this.trigger === Bcm.TriggerType.hover && this.listenMouseOver();
    if (target) {
      target.setAttribute("is-linked", this.el.id);
      if (!Generate.getAncestor(target, "bcm-linked")) {
        this.el.setAttribute("main-linked", "true");
      }
      else {
        this.el.removeAttribute("main-linked");
      }
      if (this.moveOverLayContainer) {
        const bcmOverlayContainer = document.querySelector("#bcm-overlay-container");
        bcmOverlayContainer.appendChild(this.el);
      }
    }
  }
  componentDidLoad() {
    this.init();
  }
  triggerWatch(newValue) {
    if (newValue === Bcm.TriggerType.hover) {
      this.listenMouseOver();
    }
  }
  watchIsOpen() {
    this.bcmOpen.emit(this.isOpen);
  }
  onLinkedDataDismis() {
    this.hide(false);
  }
  onBcmFloatingIsHide(e) {
    if (e.detail.id === this._id) {
      this.hide(false);
    }
  }
  async show(wait = true) {
    wait && (await delay(50));
    this.detectClone();
    const target = this.target();
    if (target) {
      const bcmOverlayContainer = document.querySelector("#bcm-overlay-container");
      bcmOverlayContainer.appendChild(this.el);
      target.setAttribute("linked-is-open", "true");
      this.isOpen = true;
      this.calculatePosition();
    }
    else {
      this.hide(false);
    }
    return this.isOpen;
  }
  async hide(wait = true) {
    wait && (await delay(50));
    const target = this.target();
    this.isMouseOverLink = false;
    if (target) {
      target.removeAttribute("linked-is-open");
      this.hideChildLinked();
      this.isOpen = false;
    }
    return this.isOpen;
  }
  async toggle(wait = true) {
    return this.isOpen ? await this.hide(wait) : await this.show(wait);
  }
  clickListener(e) {
    if (!this.target())
      return;
    if (this.trigger === Bcm.TriggerType.hover || this.trigger === Bcm.TriggerType.custom)
      return;
    const isTarget = Generate.findEventPath(e, this.target());
    const isSelf = Generate.findEventPath(e, this.el);
    const isLinked = Generate.findEventPathWithNodeName(e, "BCM-LINKED");
    if (!isTarget && !isSelf && !isLinked) {
      this.hide();
      this.bcmOpen.emit(this.isOpen);
      return;
    }
    if (this.isOpen) {
      isTarget && this.hide(false);
      isSelf && this.checkDataDismiss(e);
      if (this.el.getAttribute("main-linked") === "true" && !isTarget && !isSelf && !isLinked) {
        this.dismissEvent();
      }
    }
    else {
      isTarget && this.show(false);
    }
  }
  listenMouseOver() {
    var _a, _b;
    this.el.addEventListener("mouseover", () => (this.isMouseOverLink = true));
    this.el.addEventListener("mouseleave", () => {
      this.isMouseOverLink = false;
      this.handleMouseLeave();
    });
    (_a = this.target()) === null || _a === void 0 ? void 0 : _a.addEventListener("mouseenter", this.handleMouseEnter.bind(this));
    (_b = this.target()) === null || _b === void 0 ? void 0 : _b.addEventListener("mouseleave", this.handleMouseLeave.bind(this));
  }
  handleMouseLeave() {
    setTimeout(() => {
      !this.isMouseOverLink && this.hide();
    }, this.hideDelay);
  }
  handleMouseEnter() {
    clearTimeout(this.timeoutId);
    this.isMouseOverLink = false;
    this.show();
  }
  render() {
    const openStyle = this.isOpen ? { visibility: "visible", opacity: "1" } : { visibility: "hidden", opacity: "0" };
    return (h(Host, { class: "tw-invisible tw-absolute tw-left-0 tw-top-0 tw-z-tooltip  tw-block tw-overscroll-contain  tw-scroll-smooth tw-rounded tw-bg-white  tw-opacity-0 tw-shadow tw-transition-opacity tw-duration-100", style: Object.assign(Object.assign({}, openStyle), { overflow: "auto" }) }, h("slot", null)));
  }
  static get is() { return "bcm-linked"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "placement": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "Bcm.Placement[\"bottom-start\"]"
      },
      "trigger": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "TrigglerTypes",
          "resolved": "\"click\" | \"custom\" | \"focus\" | \"hover\"",
          "references": {
            "TrigglerTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "trigger",
        "reflect": false,
        "defaultValue": "Bcm.TriggerType.click"
      },
      "targetId": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target-id",
        "reflect": false,
        "defaultValue": "\"\""
      },
      "height": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "height",
        "reflect": false
      },
      "maxHeight": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max-height",
        "reflect": false
      },
      "minHeight": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min-height",
        "reflect": false,
        "defaultValue": "\"50px\""
      },
      "isOpen": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-open",
        "reflect": true,
        "defaultValue": "false"
      },
      "hideDelay": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hide-delay",
        "reflect": false,
        "defaultValue": "500"
      },
      "moveOverLayContainer": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "move-overlay-container",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "isMouseOverLink": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmOpen",
        "name": "bcm-open",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "(wait?: boolean) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "(wait?: boolean) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "toggle": {
        "complexType": {
          "signature": "(wait?: boolean) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "trigger",
        "methodName": "triggerWatch"
      }, {
        "propName": "isOpen",
        "methodName": "watchIsOpen"
      }];
  }
  static get listeners() {
    return [{
        "name": "linked-data-dismiss",
        "method": "onLinkedDataDismis",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-floating-is-hide",
        "method": "onBcmFloatingIsHide",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "click",
        "method": "clickListener",
        "target": "document",
        "capture": true,
        "passive": false
      }];
  }
}
