/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { forceUpdate, h, Host } from '@stencil/core';
import cs from 'classnames';
import { Generate, Tooltip } from '@bcm/helpers';
import { IsLoad } from '@bcm/decorator';
export class BcmMenu {
  constructor() {
    this._id = Generate.UID();
    this.size = 'medium';
    this.collapsible = false;
    this.collapse = false;
    this.mode = 'inline';
    this.disabled = false;
    this.hidden = false;
    this.linkedComponent = undefined;
    this.height = 'auto';
    this.maxHeight = '500px';
    this.minHeight = '150px';
    this.selectable = true;
    this.gap = "16px";
    this.dropdown = false;
    this.placement = 'bottom-start';
    this.activeItemId = undefined;
    this.data = [];
  }
  async setActive(param) {
    this.activeItemId = null;
    const recursive = (data) => {
      data.forEach(item => {
        if (item.id == param) {
          item.active = true;
          this.activeItemId = item.id;
          this.bcmClick.emit(item);
        }
        else {
          item.active = false;
          if (item.items) {
            recursive(item.items);
          }
        }
      });
    };
    recursive(this.data);
  }
  async getActive() {
    return this.activeItem;
  }
  handleSlotItems({ detail }) {
    this.data = detail;
  }
  handleClickItem(item) {
    if (item.items && item.items.length > 0) {
      item.isOpen = !item.isOpen;
      forceUpdate(this);
    }
    else {
      this.activeItemId = item.id;
      document.dispatchEvent(new CustomEvent('linked-data-dismiss', { detail: { id: this._id } }));
    }
    // this.data = this.data.map(menuItem => ({ ...menuItem, active: menuItem.id === item.id }));
    this.bcmClick.emit(item);
  }
  getTooltip(id, text) {
    return new Tooltip({ message: text, targetId: 'bcm-menu__item-' + this._id + '-' + id, placement: 'right' });
  }
  render() {
    const { data, collapsible, collapse } = this;
    const hostClasses = cs('bcm-menu', `bcm-menu__size-${this.size}`, {
      'bcm-menu__collapse': collapse,
      'bcm-menu__dropdown': this.dropdown,
    });
    const MenuLinkTemplate = (item, index) => {
      if (!item.text)
        return;
      if (!item.id)
        item.id = Generate.UID();
      const subItems = !!(item.items && item.items.length);
      if (this.selectable && (this.activeItemId ? this.activeItemId === item.id : item.active)) {
        this.bcmMenuChange.emit(item);
        this.activeItem = item;
      }
      return (h("a", { id: 'bcm-menu__item-' + this._id + '-' + item.id, onMouseEnter: () => this.collapse && this.getTooltip(item.id, item.text).show(), onMouseLeave: () => this.collapse && this.getTooltip(item.id, item.text).hide(), key: index, href: !item.disabled ? item.link || 'javascript:void(0)' : 'javascript:void(0)', tabindex: item.disabled ? '-1' : '0', class: cs('bcm-menu__item', {
          'bcm-menu__item--is-sub': subItems && item.isOpen,
          'hidden tw-hidden': item.hidden,
        }) }, h("span", { class: cs('bcm-menu__item-link', `bcm-menu__item-link--${this.mode}`, {
          'bcm-menu__item-link--active': this.selectable && (this.activeItemId ? this.activeItemId === item.id : item.active),
          'bcm-menu__item-link--disabled': item.disabled,
        }), onClick: () => {
          !this.disabled && !item.disabled && this.handleClickItem(item);
        } }, item.icon && (h("span", { class: "bcm-menu__item-link-icon" }, h("i", { class: item.icon || 'far fa-angle-left' }))), h("span", { class: cs('bcm-menu__item-link-text', {
          'bcm-menu__item-link-text--collapse': collapse,
        }), innerHTML: item.text }), subItems && (h("div", { class: cs('bcm-menu__item-link-toggle', {
          'bcm-menu__item-link-toggle--collapse': item.isOpen,
        }) }, h("i", { class: "fas fa-chevron-down" })))), subItems && (h("div", { class: cs('bcm-menu__item-submenu', {
          'bcm-menu__item-submenu--collapse': collapse,
          'bcm-menu__item-submenu--open': item.isOpen,
        }) }, item.items.map((subItem, subIndex) => MenuLinkTemplate(subItem, subIndex))))));
    };
    return (h(Host, { class: hostClasses, disabled: this.disabled }, h("div", { class: "bcm-menu__container", style: { gap: this.gap } }, h("slot", null), (data === null || data === void 0 ? void 0 : data.length) > 0 && data.map((item, index) => MenuLinkTemplate(item, index)), collapsible && (h("div", { class: cs('bcm-menu__collapse-button', {
        'bcm-menu__collapse-button--collapsed': collapse,
      }), onClick: () => {
        this.collapse = !collapse;
        collapsible && this.bcmExpanded.emit(collapse);
      } }, h("i", { class: "far fa-angle-left" }))))));
  }
  static get is() { return "bcm-menu"; }
  static get originalStyleUrls() {
    return {
      "$": ["menu.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["menu.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'small' | 'medium' | 'large'",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "'medium'"
      },
      "collapsible": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "collapsible",
        "reflect": true,
        "defaultValue": "false"
      },
      "collapse": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "collapse",
        "reflect": true,
        "defaultValue": "false"
      },
      "mode": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'horizontal' | 'vertical' | 'inline'",
          "resolved": "\"horizontal\" | \"inline\" | \"vertical\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "mode",
        "reflect": false,
        "defaultValue": "'inline'"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "linkedComponent": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "linked-component",
        "reflect": false
      },
      "height": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "height",
        "reflect": false,
        "defaultValue": "'auto'"
      },
      "maxHeight": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max-height",
        "reflect": false,
        "defaultValue": "'500px'"
      },
      "minHeight": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min-height",
        "reflect": false,
        "defaultValue": "'150px'"
      },
      "selectable": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "selectable",
        "reflect": false,
        "defaultValue": "true"
      },
      "gap": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "gap",
        "reflect": false,
        "defaultValue": "\"16px\""
      },
      "dropdown": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "dropdown",
        "reflect": false,
        "defaultValue": "false"
      },
      "placement": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "'bottom-start'"
      },
      "data": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "data",
        "reflect": false,
        "defaultValue": "[]"
      }
    };
  }
  static get states() {
    return {
      "activeItemId": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmClick",
        "name": "bcm-click",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmExpanded",
        "name": "bcm-expanded",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmMenuChange",
        "name": "bcm-menu-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "setActive": {
        "complexType": {
          "signature": "(param: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getActive": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "bcm-items",
        "method": "handleSlotItems",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
__decorate([
  IsLoad()
], BcmMenu.prototype, "el", void 0);
