/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { StatusProps } from './types';
import cs from 'classnames';
export class BcmMessage {
  constructor() {
    this.status = StatusProps.info;
    this.fullWidth = false;
    this.loading = false;
    this.isShow = false;
    this.hidden = false;
  }
  /**
   * @desc
   */
  getIcon() {
    switch (this.status) {
      case StatusProps.info:
        return h("bcm-icon", { icon: "far fa-info-circle" });
      case StatusProps.error:
        return h("bcm-icon", { icon: "far fa-times-circle" });
      case StatusProps.warning:
        return h("bcm-icon", { icon: "far fa-exclamation-circle" });
      case StatusProps.success:
        return h("bcm-icon", { icon: "far fa-check-circle" });
    }
  }
  /**
   * @desc Removes component itself
   */
  dismiss() {
    this.isShow = false;
    // this.el.remove()
  }
  /**
   * @desc
   */
  async hide() {
    this.dismiss();
  }
  /**
   * @desc
   */
  async show() {
    this.isShow = true;
  }
  render() {
    const { fullWidth, status, isShow, loading } = this;
    const classes = cs('message', loading ? 'info' : status, {
      'full-width': fullWidth,
      'show': isShow
    });
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: classes }, loading && (h("bcm-icon", { icon: "far fa-spinner-third fa-spin" })), !loading && (this.getIcon()), h("slot", null))));
  }
  static get is() { return "bcm-message"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["message.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["message.css"]
    };
  }
  static get properties() {
    return {
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "StatusProp",
          "resolved": "\"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "StatusProps.info"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "loading": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "loading",
        "reflect": false,
        "defaultValue": "false"
      },
      "isShow": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": true,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get methods() {
    return {
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": [{
              "name": "desc",
              "text": undefined
            }]
        }
      },
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": [{
              "name": "desc",
              "text": undefined
            }]
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
