/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import Bcm from '../../../models/bcm';
import { StringHelper } from '../../../helper/string-helper';
import { SlotTemplate } from '../../../templates/slot-template';
import { Generate } from '../../../helper/generate';
export class BcmModal {
  constructor() {
    this.modalSlots = {};
    this._id = Generate.UID();
    this.isShow = false;
    this.hidden = false;
    this.size = Bcm.Size.medium;
    this.backdrop = true;
    this.width = undefined;
    this.height = undefined;
    this.fullWidth = false;
  }
  connectedCallback() {
    const bcmSlots = this.el.querySelectorAll('bcm-modal-header, bcm-modal-footer');
    if (bcmSlots) {
      bcmSlots.forEach(el => {
        let slotName = StringHelper.lowercase(StringHelper.split(el.nodeName, '-').pop());
        el.slot = slotName;
        el.classList.add(`bcm-modal__container-${slotName}-content`);
        this.modalSlots[slotName] = el;
      });
    }
    const disableScrollClass = `bcm-modal--disable-scroll-${this._id}`;
    const styleId = `bcm-modal-style-${this._id}`;
    let style = document.getElementById(styleId);
    if (!style) {
      style = document.createElement('style');
      style.id = styleId;
      style.innerHTML = `.${disableScrollClass} { overflow: hidden; }`;
      document.head.appendChild(style);
    }
  }
  componentWillLoad() {
    // document.body.appendChild(this.el);
  }
  handleSubmit(e) {
    const button = Generate.findEventPathWithNodeName(e, StringHelper.uppercase('bcm-button'));
    if (button && button.dataDismiss) {
      this.hide();
    }
  }
  handleBackdrop() {
    if (this.backdrop == true) {
      this.hide();
    }
  }
  async hide(obj = { force: false }) {
    if (obj === null || obj === void 0 ? void 0 : obj.force) {
      this.isShow = false;
      this.close.emit(); // Deprecated
      this.modalClose.emit();
      return;
    }
    this.modalBeforeClose.emit(this.el);
  }
  async show(obj = { force: false }) {
    if (obj === null || obj === void 0 ? void 0 : obj.force) {
      this.isShow = true;
      return;
    }
    this.modalBeforeOpen.emit(this.el);
  }
  keyboardNavigation(e) {
    if (this.backdrop == true) {
      const keyCode = e.keyCode || 0;
      if (keyCode == Bcm.KeyCode.escape) {
        e.preventDefault();
        this.hide();
      }
    }
  }
  async showModal(e) {
    if (e.defaultPrevented) {
      return;
    }
    if (e.target === this.el) {
      this.isShow = true;
      // document.body.appendChild(this.el); A solution must be found for this.
    }
  }
  async hideModal(e) {
    if (e.defaultPrevented) {
      return;
    }
    if (e.target === this.el) {
      this.isShow = false;
      this.close.emit(); // Deprecated
      this.modalClose.emit();
    }
  }
  _openHandler() {
    const disableScrollClass = `bcm-modal--disable-scroll-${this._id}`;
    if (this.isShow) {
      document.body.classList.add(disableScrollClass);
    }
    else {
      document.body.classList.remove(disableScrollClass);
    }
  }
  disconnectedCallback() {
    const disableScrollClass = `bcm-modal--disable-scroll-${this._id}`;
    document.body.classList.remove(disableScrollClass);
  }
  render() {
    const { isShow, size, fullWidth, hidden, width, height } = this;
    const bcmModalContainer = cs('bcm-modal__container', size, {
      'custom-size': width || height,
      'full-width': fullWidth,
      'bcm-modal__show': isShow,
    });
    const bcmModalHostClasses = cs('bcm-modal', {
      hidden,
      'bcm-modal__show': isShow,
    });
    const mainClasses = cs('bcm-modal__container-main', 'size-2');
    const FooterTemplate = () => {
      return (this.modalSlots['footer'] && (h("footer", { class: "bcm-modal__container-footer" }, h("slot", { name: "footer" }))));
    };
    const HeaderTemplate = () => {
      return (this.modalSlots['header'] && (h("header", { class: "bcm-modal__container-header" }, h(SlotTemplate, { name: "header", className: 'full-width' }), h("div", { class: "bcm-modal__container-header-close", onClick: () => this.hide() }, h("bcm-icon", { icon: "far fa-times" })))));
    };
    const customStyle = Object.assign(Object.assign({}, (width && { width: width })), (height && { height: height }));
    return (h(Host, { class: `tw-overscroll-contain  ${bcmModalHostClasses}` }, h("div", { class: "overlay", onClick: () => this.handleBackdrop() }), h("div", { class: bcmModalContainer, style: customStyle }, h(HeaderTemplate, null), h("main", { class: mainClasses }, h("article", null, h(SlotTemplate, null))), h(FooterTemplate, null))));
  }
  static get is() { return "bcm-modal"; }
  static get originalStyleUrls() {
    return {
      "$": ["modal.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["modal.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "isShow": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-show",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "size": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "backdrop": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "boolean | 'static'",
          "resolved": "\"static\" | boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "backdrop",
        "reflect": false,
        "defaultValue": "true"
      },
      "width": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "width",
        "reflect": false
      },
      "height": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "height",
        "reflect": false
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "close",
        "name": "bcm-close",
        "bubbles": false,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "modalClose",
        "name": "bcm-modal-close",
        "bubbles": false,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "modalBeforeClose",
        "name": "bcm-modal-before-close",
        "bubbles": false,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "modalBeforeOpen",
        "name": "bcm-modal-before-open",
        "bubbles": false,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "hide": {
        "complexType": {
          "signature": "(obj?: { force: boolean; }) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "show": {
        "complexType": {
          "signature": "(obj?: { force: boolean; }) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "isShow",
        "methodName": "_openHandler"
      }];
  }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleSubmit",
        "target": "window",
        "capture": false,
        "passive": false
      }, {
        "name": "keydown",
        "method": "keyboardNavigation",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-modal-before-open",
        "method": "showModal",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-modal-before-close",
        "method": "hideModal",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
