/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { TypeProps } from './types';
import cs from 'classnames';
export class BcmNotification {
  constructor() {
    this._title = undefined;
    this.status = TypeProps.default;
    this.confirmText = undefined;
    this.isShow = false;
    this.hidden = false;
  }
  /**
   * @desc
   * @returns {BcmOldIcon}
   */
  getIcon() {
    switch (this.status) {
      case TypeProps.info:
        return h("bcm-icon", { icon: "far fa-info-circle" });
      case TypeProps.error:
        return h("bcm-icon", { icon: "far fa-times-circle" });
      case TypeProps.warning:
        return h("bcm-icon", { icon: "far fa-exclamation-circle" });
      case TypeProps.success:
        return h("bcm-icon", { icon: "far fa-check-circle" });
    }
  }
  /**
   * @desc Removes component itself
   */
  dismiss() {
    this.isShow = false;
    // this.el.remove()
  }
  /**
   * @desc Removes component itself
   */
  handleConfirm() {
    this.isShow = false;
    this.confirm.emit();
  }
  /**
   * @desc
   */
  async hide() {
    this.dismiss();
  }
  /**
   * @desc
   */
  async show() {
    this.isShow = true;
  }
  render() {
    const { status, _title, confirmText, isShow } = this;
    const notification = cs('notification', {
      'show': isShow
    });
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: notification }, status !== TypeProps.default && (h("div", { class: "icon-content" }, this.getIcon())), h("div", { class: "text-content" }, h("span", { class: "title" }, _title, h("bcm-icon", { icon: "far fa-times", color: "grey-10", onClick: () => this.dismiss() })), h("p", null, h("slot", null)), confirmText != undefined && (h("div", { class: "actions" }, h("bcm-button", { size: "small", onClick: () => this.handleConfirm() }, confirmText ? confirmText : "Confirm")))))));
  }
  static get is() { return "bcm-notification"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["notification.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["notification.css"]
    };
  }
  static get properties() {
    return {
      "_title": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "title",
        "reflect": false
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TypeProp",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "TypeProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "TypeProps.default"
      },
      "confirmText": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "confirm",
        "reflect": false
      },
      "isShow": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": true,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "confirm",
        "name": "bcm-confirm",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Component Events"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": [{
              "name": "desc",
              "text": undefined
            }]
        }
      },
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": [{
              "name": "desc",
              "text": undefined
            }]
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
