/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { ColorHelper } from '../../../helper/color-helper';
import { Generate } from '../../../helper/generate';
import { StringHelper } from '../../../helper/string-helper';
import { setPosition } from '../../../utils/popover-placement';
const statuses = {
  error: ['fal fa-times-circle', 'red-6'],
  warning: ['fal fa-exclamation-circle', 'warmyellow-6'],
};
export class BcmPopconfirmBox {
  constructor() {
    this._id = Generate.UID();
    this._target_id = undefined;
    this.message = undefined;
    this.status = 'warning';
    this.cancelText = "No";
    this.okeyText = "Yes";
    this.placement = undefined;
    this.open = false;
    this.hidden = false;
    this._internal_id = undefined;
    this.placement_string = undefined;
  }
  getPopconfirm() {
    return document.querySelector(`bcm-popconfirm[id="${this._target_id}"]`);
  }
  handleClickOutside(e) {
    if (this.el.contains(e.target) || this.getPopconfirm().contains(e.target)) {
      return;
    }
    this.getPopconfirm().hide();
    this.hide();
  }
  showHandler(e) {
    if (e.detail != this._target_id) {
      this.getPopconfirm().hide();
      this.hide();
    }
  }
  async show() {
    this.open = true;
    this.onShow.emit(this._target_id);
  }
  async hide() {
    this.open = false;
  }
  componentWillRender() {
    this.calculateLocation();
  }
  calculateLocation() {
    if (this.open) {
      var placement = this.placement;
      const box = this.el.querySelector('.bcm-pop-box');
      const targetElement = document.getElementById(this._target_id).children[0];
      if (box && targetElement) {
        let pos = setPosition(placement, targetElement, { top: targetElement.offsetTop, left: targetElement.offsetLeft });
        this.setPlacementString(placement);
        this.el.style.top = pos[0] + 'px';
        this.el.style.left = pos[1] + 'px';
      }
    }
  }
  setPlacementString(placement) {
    const placements = StringHelper.textContains(placement, '-') ? placement.split('-') : placement.split(' ');
    if (placements.length === 1) {
      placements.push('center');
    }
    this.placement_string = placements.join(' ');
  }
  handleConfirm() {
    this.getPopconfirm().hide();
    this.hide();
    this.onConfirm.emit(this._target_id);
  }
  handleCancel() {
    this.getPopconfirm().hide();
    this.hide();
    this.onCancel.emit(this._target_id);
  }
  render() {
    const { message, cancelText, okeyText, open, placement_string, status } = this;
    const statusies = statuses[status];
    const boxClasses = cs('bcm-pop-box');
    const hostClasses = cs('bcm-popconfirm-box', placement_string, {
      'bcm-pop-open': open,
    });
    return (h(Host, { class: hostClasses }, h("div", { class: boxClasses }, h("div", { class: "bcm-pop-content" }, h("div", { class: "bcm-pop-content-icon size-3", style: { color: ColorHelper.convertColorToHex(statusies[1]) } }, h("bcm-icon", { icon: statusies[0] })), h("div", { class: "bcm-pop-content-text" }, h("bcm-text", { scale: "size-2", innerHTML: message }))), h("div", { class: "bcm-pop-buttons" }, h("bcm-button", { size: "small", onClick: () => this.handleCancel(), outline: true }, cancelText), h("bcm-button", { size: "small", onClick: () => this.handleConfirm() }, okeyText)))));
  }
  static get is() { return "bcm-popconfirm-box"; }
  static get originalStyleUrls() {
    return {
      "$": ["popconfirm-box.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["popconfirm-box.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_target_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target-id",
        "reflect": false
      },
      "message": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'warning' | 'error'",
          "resolved": "\"error\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "'warning'"
      },
      "cancelText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "cancel-text",
        "reflect": false,
        "defaultValue": "\"No\""
      },
      "okeyText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "okey-text",
        "reflect": false,
        "defaultValue": "\"Yes\""
      },
      "placement": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "_internal_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "internal-id",
        "reflect": true
      }
    };
  }
  static get states() {
    return {
      "placement_string": {}
    };
  }
  static get events() {
    return [{
        "method": "onConfirm",
        "name": "bcm-popconfirm-confirm",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onCancel",
        "name": "bcm-popconfirm-cancel",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onShow",
        "name": "bcm-popconfirm-show",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleClickOutside",
        "target": "window",
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-popconfirm-show",
        "method": "showHandler",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "resize",
        "method": "calculateLocation",
        "target": "window",
        "capture": false,
        "passive": true
      }];
  }
}
