/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
export class BcmPopconfirm {
  constructor() {
    this._id = Generate.UID();
    this.message = undefined;
    this.status = 'warning';
    this.cancelText = "No";
    this.okeyText = "Yes";
    this.placement = Bcm.Placement.top;
    this.trigger = Bcm.TriggerType.click;
    this.open = false;
    this.hidden = false;
  }
  async show() {
    // if (this.open) return
    this.open = true;
    this.onShow.emit();
    await this.createBox().then(() => this.showBox());
  }
  async hide() {
    // if (!this.open) return
    this.open = false;
    this.onHide.emit();
    // this.hideBox()
  }
  async createBox() {
    let element = this.getPopconfirmBox();
    if (!element) {
      const box = document.createElement('bcm-popconfirm-box');
      box.setAttribute('id', 'box-' + this._id);
      box.setAttribute('target-id', this._id);
      box.setAttribute('placement', this.placement);
      box.setAttribute('message', this.message);
      box.setAttribute('status', this.status);
      box.setAttribute('cancel-text', this.cancelText);
      box.setAttribute('okey-text', this.okeyText);
      this.el.parentElement.appendChild(box);
    }
    return Promise.resolve();
  }
  getPopconfirmBox() {
    return document.querySelector(`bcm-popconfirm-box[id="box-${this._id}"]`);
  }
  showBox() {
    this.getPopconfirmBox() && this.getPopconfirmBox().show();
  }
  hideBox() {
    this.getPopconfirmBox() && this.getPopconfirmBox().hide();
  }
  listenTrigger(event) {
    switch (this.trigger) {
      case 'click':
        event === 'click' && (this.open ? this.hide() : this.show());
        break;
      case 'hover':
        event === 'onMouseOver' && this.show();
        event === 'onMouseOut' && this.hide();
        break;
      case 'focus':
        event === 'onFocus' && this.show();
        event === 'onBlur' && this.hide();
        break;
      default: break;
    }
  }
  onBcmConfirm(e) {
    e.detail == this._id && (this.onConfirm.emit(this._id));
  }
  onBcmCancel(e) {
    e.detail == this._id && (this.onCancel.emit(this._id));
  }
  render() {
    const popconfirmClasses = cs('bcm-popconfirm');
    return (h(Host, { class: popconfirmClasses, onclick: () => this.listenTrigger('click') }, h("slot", null)));
  }
  static get is() { return "bcm-popconfirm"; }
  static get originalStyleUrls() {
    return {
      "$": ["popconfirm.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["popconfirm.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "message": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'warning' | 'error'",
          "resolved": "\"error\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "'warning'"
      },
      "cancelText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "cancel-text",
        "reflect": false,
        "defaultValue": "\"No\""
      },
      "okeyText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "okey-text",
        "reflect": false,
        "defaultValue": "\"Yes\""
      },
      "placement": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "Bcm.Placement.top"
      },
      "trigger": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TrigglerTypes",
          "resolved": "\"click\" | \"custom\" | \"focus\" | \"hover\"",
          "references": {
            "TrigglerTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "trigger",
        "reflect": false,
        "defaultValue": "Bcm.TriggerType.click"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "onShow",
        "name": "bcm-show",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onHide",
        "name": "bcm-hide",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onConfirm",
        "name": "bcm-confirm",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onCancel",
        "name": "bcm-cancel",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "bcm-popconfirm-confirm",
        "method": "onBcmConfirm",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-popconfirm-cancel",
        "method": "onBcmCancel",
        "target": "document",
        "capture": false,
        "passive": false
      }];
  }
}
