/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from "@stencil/core";
import cs from "classnames";
import { ColorHelper } from "../../../helper/color-helper";
import { Generate } from "../../../helper/generate";
import Bcm from "../../../models/bcm";
import { circleSizes } from "./types";
export class BcmProgress {
  constructor() {
    this._id = Generate.UID();
    this.value = 0;
    this.max = 100;
    this.size = Bcm.Size.medium;
    this.type = "line-rounded";
    this.info = null;
    this.error = false;
    this.hidden = false;
    this.fullWidth = false;
    this.color = "blue";
    this.isCompleted = false;
  }
  connectedCallback() {
    this.maxPercent(this.value);
  }
  componentDidRender() {
    const circle = this.el.querySelector("#circle");
    if (circle) {
      const radius = circle.r.baseVal.value;
      const circumference = radius * 2 * Math.PI;
      const offset = circumference - (this.value / this.max) * circumference;
      circle.style.strokeDasharray = `${circumference} ${circumference}`;
      circle.style.strokeDashoffset = `${circumference}`;
      circle.style.strokeDashoffset = `${offset}`;
    }
  }
  componentShouldUpdate(newVal, oldVal, propName) {
    if (propName === "value" && newVal !== oldVal) {
      this.handlePercent(newVal);
    }
    if (propName === "max" && newVal !== oldVal) {
      this.value > newVal ? this.value = newVal : this.value;
    }
  }
  maxPercent(newValue) {
    if (newValue >= this.max) {
      this.value = this.max;
      this.isCompleted = true;
    }
    if (newValue < this.max) {
      this.isCompleted = false;
    }
    if (newValue <= 0) {
      this.value = 0;
    }
  }
  async increment(percent) {
    this.handlePercent(+percent, "increment");
  }
  async decrement(percent) {
    this.handlePercent(+percent, "decrement");
  }
  async set(percent) {
    this.handlePercent(+percent, "set");
  }
  handlePercent(percent, process = "set") {
    if ((percent || percent == 0) && typeof percent === "number") {
      switch (process) {
        case "increment":
          this.value += percent;
          break;
        case "decrement":
          this.value -= percent;
          break;
        case "set":
          this.value = percent;
          break;
      }
      this.maxPercent(this.value);
      this.error = false;
    }
    else {
      this.error = true;
    }
  }
  getIcon(type = "line") {
    let icon = "far fa-" + (this.error ? "times" : "check") + (type === "line" ? "-circle" : "");
    if (this.error || (this.info === "icon" && this.isCompleted)) {
      return h("bcm-icon", { icon: icon });
    }
  }
  render() {
    const { type, hidden, error, size, isCompleted, value, fullWidth } = this;
    const isIcon = this.info === "icon";
    const { circleSize, radius, textSize } = circleSizes[this.size];
    const centerOfCircle = +circleSize / 2;
    const completeState = cs({
      completed: isIcon && isCompleted,
      error,
    });
    const lineClasses = cs("bcm-progress__line-progress", size, type, completeState);
    const circleClasses = cs("bcm-progress__circle-circle", "bar", completeState);
    const containerStyle = {
      width: `${circleSize}px`,
      height: `${circleSize}px`,
    };
    const hostClasses = cs("bcm-progress", {
      hidden,
      "full-width": fullWidth,
    });
    const CircleTemplate = () => (h("div", { class: "bcm-progress__circle", style: containerStyle }, h("svg", { width: circleSize, height: circleSize }, h("circle", { r: radius, cx: centerOfCircle, cy: centerOfCircle, class: "bcm-progress__circle-circle" }), h("circle", { r: radius, cx: centerOfCircle, cy: centerOfCircle, class: circleClasses, id: "circle" }), (this.info && this.info != "icon") && !this.error && (h("text", { x: centerOfCircle, y: centerOfCircle, "text-anchor": "middle", class: textSize, "alignment-baseline": "middle" }, this.info === "value-max" && (value + "/" + this.max), this.info === "value" && { value }, this.info === "percent" &&
      (100 / this.max) * +(value) + "%"))), this.getIcon("circle")));
    const LineTemplate = () => (h("div", { class: "bcm-progress__line" }, h("progress", { class: lineClasses, value: value, max: this.max }), this.info === "value-max" && h("span", { class: "value" }, value, "/", this.max, " "), this.info === "value" && h("span", { class: "value" }, value, " "), this.info === "percent" &&
      h("span", { class: "value" }, (100 / this.max) * +(value), "%"), this.info === "icon" && h("span", { class: "value" }, this.getIcon("line"))));
    return (h(Host, { class: hostClasses, style: {
        [`--bcm-progress-color`]: ColorHelper.dsColor(this.color),
      } }, type === "circle" ? h(CircleTemplate, null) : h(LineTemplate, null)));
  }
  static get is() { return "bcm-progress"; }
  static get originalStyleUrls() {
    return {
      "$": ["progress.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["progress.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "value": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false,
        "defaultValue": "0"
      },
      "max": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max",
        "reflect": false,
        "defaultValue": "100"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "\"small\" | \"medium\"",
          "resolved": "\"medium\" | \"small\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "\"line\" | \"line-rounded\" | \"circle\"",
          "resolved": "\"circle\" | \"line\" | \"line-rounded\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "\"line-rounded\""
      },
      "info": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "\"value-max\" | \"value\" | \"percent\" | \"icon\"",
          "resolved": "\"icon\" | \"percent\" | \"value\" | \"value-max\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "info",
        "reflect": false,
        "defaultValue": "null"
      },
      "error": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "error",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "color": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false,
        "defaultValue": "\"blue\""
      }
    };
  }
  static get states() {
    return {
      "isCompleted": {}
    };
  }
  static get methods() {
    return {
      "increment": {
        "complexType": {
          "signature": "(percent: number) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "decrement": {
        "complexType": {
          "signature": "(percent: number) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "set": {
        "complexType": {
          "signature": "(percent: number) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
