/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { statuses } from './types';
export class BcmResult {
  constructor() {
    this.status = null;
    this.heading = undefined;
    this.description = undefined;
    this.hidden = false;
    this.footerArea = undefined;
    this.footerHtml = undefined;
  }
  getIcon() {
    if (this.status != null) {
      const icon = statuses[this.status].icon;
      return (h("div", { class: "icon" }, h("bcm-icon", { icon: icon })));
    }
  }
  componentDidRender() {
    const slot = this.el.shadowRoot.querySelector('slot').assignedElements();
    slot.map(el => {
      if (String(el.tagName).toLowerCase() === 'bcm-result-footer') {
        el.slot = 'footer';
        this.footerArea = true;
        this.footerHtml = el.innerHTML;
        el.innerHTML = "";
      }
    });
  }
  render() {
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: "result" }, this.getIcon(), h("div", { class: "content" }, h("span", { class: "heading size-5 weight-semibold" }, this.heading), h("span", { class: "desc size-2" }, this.description), h("div", { class: "extra" }, h("slot", null)), (this.footerArea && (h("div", { class: "buttons", innerHTML: this.footerHtml })))))));
  }
  static get is() { return "bcm-result"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["result.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["result.css"]
    };
  }
  static get properties() {
    return {
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "statusType",
          "resolved": "\"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "statusType": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "null"
      },
      "heading": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "heading",
        "reflect": false
      },
      "description": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "description",
        "reflect": false
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "footerArea": {},
      "footerHtml": {}
    };
  }
  static get elementRef() { return "el"; }
}
