/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
import { InputTemplate } from '../../../templates/input-template';
import { delay } from '../../../utils/utils';
import searchState from '../states/search.states';
export class BcmSearch {
  constructor() {
    this.debounceHandler = Generate.debounceInput((event) => this.onSearchValue(event, this), 300, false, () => {
      var _a, _b;
      this.isActiveClearable = ((_b = (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.length) > 0 && this.clearable;
    });
    this._id = Generate.UID();
    this.placeholder = "Search";
    this.disabled = false;
    this.readonly = false;
    this.clearable = true;
    this.size = Bcm.Size.medium;
    this.limit = 10;
    this.searchIsOnlyChilds = false;
    this.minSearchLength = 2;
    this.data = [];
    this.tick = {};
    this.isActiveClearable = false;
  }
  async componentDidLoad() {
    await this.initState();
  }
  async dataChanged(newValue, oldValue) {
    // console.log('data changed', newValue, oldValue)
    if (newValue !== oldValue) {
      await this.initState();
    }
  }
  async initState() {
    await searchState.setValue({
      id: this._id + "-input",
      dataSource: this.data,
      searchIsOnlyChilds: this.searchIsOnlyChilds,
    });
    await this.markForCheck();
  }
  markForCheck() {
    this.tick = Generate.UID();
  }
  handleInput() {
    // console.log("aheyya")
  }
  async onBcmFocus() {
    this.inputElement.focus();
  }
  async onBcmBlur() {
    this.inputElement.blur();
  }
  async onBcmSelect() {
    this.inputElement.select();
  }
  async setValue(newValue) {
    this.inputElement.value = newValue;
  }
  handleFocus() { }
  handleBlur() { }
  handleChange() { }
  async handleClear(eventEmit = true) {
    this.inputElement.value = '';
    const { value, id } = this.inputElement;
    searchState.searchData(id, value);
    this.isActiveClearable = false;
    this.bcmOnSearch.emit({ result: null, value: null, matching: null, parentIndex: null });
    this.bcmOnSearchClear.emit(eventEmit);
    await delay(10);
    return Promise.resolve();
  }
  onSearchValue(event, instance) {
    const { value, id } = event.target;
    const searchResult = (value === null || value === void 0 ? void 0 : value.length) >= this.minSearchLength ? searchState.searchData(id, value, null) : null;
    const { bcmOnSearch, inputElement } = instance;
    bcmOnSearch.emit({ result: searchResult === null || searchResult === void 0 ? void 0 : searchResult.result, value: inputElement === null || inputElement === void 0 ? void 0 : inputElement.value, matching: searchResult === null || searchResult === void 0 ? void 0 : searchResult.matching, parentIndex: searchResult === null || searchResult === void 0 ? void 0 : searchResult.parentIndex, nestedResult: searchResult === null || searchResult === void 0 ? void 0 : searchResult.nestedResult });
  }
  render() {
    const { _id, placeholder, disabled, readonly } = this;
    return h(Host, { onClick: () => { this.onBcmFocus(); this.onBcmSelect(); } }, h(InputTemplate, { size: this.size, id: _id, type: "search", placeholder: placeholder, disabled: disabled, "full-width": true, readonly: readonly, clearable: this.isActiveClearable, handleFocus: () => this.handleFocus(), handleBlur: () => this.handleBlur(), handleChange: () => this.handleChange(), handleClear: () => this.handleClear(), handleInput: this.debounceHandler, ref: (el) => this.inputElement = el }));
  }
  static get is() { return "bcm-search"; }
  static get originalStyleUrls() {
    return {
      "$": ["search.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["search.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "placeholder": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placeholder",
        "reflect": false,
        "defaultValue": "\"Search\""
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "readonly": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "readonly",
        "reflect": false,
        "defaultValue": "false"
      },
      "clearable": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "clearable",
        "reflect": false,
        "defaultValue": "true"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "limit": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "limit",
        "reflect": false,
        "defaultValue": "10"
      },
      "searchIsOnlyChilds": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "search-is-only-childs",
        "reflect": false,
        "defaultValue": "false"
      },
      "minSearchLength": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min-search-length",
        "reflect": false,
        "defaultValue": "2"
      },
      "data": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "data",
        "reflect": false,
        "defaultValue": "[]"
      }
    };
  }
  static get states() {
    return {
      "tick": {},
      "isActiveClearable": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmOnSearch",
        "name": "bcm-on-search",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmOnSearchClear",
        "name": "bcm-on-search-clear",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "onBcmFocus": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "onBcmBlur": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "onBcmSelect": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setValue": {
        "complexType": {
          "signature": "(newValue: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "handleClear": {
        "complexType": {
          "signature": "(eventEmit?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "data",
        "methodName": "dataChanged"
      }];
  }
}
