/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { createStore } from '@stencil/store';
import snq from 'snq';
import { Generate } from '../../../helper/generate';
const { state } = createStore({
  group: new Map(),
  allChecked: true,
  allUnchecked: true,
  changeInderminate: {},
});
const setValue = async ({ fullWidth, optionType, buttonStyle, size, defaultCheckedItems, el, id, label, required, captionType, hidden, direction, childs, indeterminate, gap, noCaption, }) => {
  if (el) {
    state.group.set(id, {
      fullWidth,
      optionType,
      buttonStyle,
      size,
      defaultCheckedItems,
      label,
      required,
      captionType,
      hidden,
      direction,
      indeterminate,
      gap,
      noCaption,
      childs: childs === null || childs === void 0 ? void 0 : childs.map(child => {
        child.checked && (state.allUnchecked = false);
        !child.checked && (state.allChecked = false);
        // fullWidth && (fullWidth == 'flex' ? (child.style.flex = 'auto') : (child.style.width = '100%'));
        optionType && (child.optionType = optionType);
        buttonStyle && (child.buttonStyle = buttonStyle);
        size && (child.size = size);
        defaultCheckedItems && defaultCheckedItems.find(e => e === child.name) && (child.checked = true);
        child.noCaption = noCaption;
        return {
          id: child.id || Generate.UID(),
          checked: child.checked || false,
          name: child.name,
          size: child.size,
          buttonStyle: child.buttonStyle,
          optionType: child.optionType,
          style: child.style,
          noCaption: child.noCaption,
          color: child.color,
          disabled: child.disabled,
          readonly: child.readonly,
          label: child.label || child.name,
          slot: child.slot || snq(() => child.querySelector('.slot').textContent, child.textContent),
          required: child.required || required,
          el: child,
          captionType,
          hidden,
        };
      }),
      indeterminateState: !state.allChecked && !state.allUnchecked ? 'indeterminate' : state.allChecked ? 'determinate' : 'uncheck',
    });
  }
};
const getProperty = id => {
  return snq(() => state.group.get(id), {});
};
const getValue = (id, name) => {
  var _a;
  const group = getProperty(id);
  let value = {};
  (_a = group.childs) === null || _a === void 0 ? void 0 : _a.map(child => {
    if (name) {
      if (child.name === name) {
        value[child.name] = {
          value: !!child.checked,
          name: child.name,
        };
      }
    }
    else {
      value[child.name] = {
        value: !!child.checked,
        name: child.name,
      };
    }
  });
  return value;
};
const getCheckedList = id => {
  var _a;
  const group = getProperty(id);
  let value = {};
  (_a = group.childs) === null || _a === void 0 ? void 0 : _a.map(child => {
    if (child.checked === true) {
      value[child.name] = {
        value: !!child.checked,
        name: child.name,
      };
    }
  });
  return value;
};
const setClear = id => {
  var _a;
  (_a = getProperty(id).childs) === null || _a === void 0 ? void 0 : _a.forEach(child => {
    child.checked = false;
  });
  checkInderminate(id);
};
const setCheckedItems = (id, items, checked, reset) => {
  var _a;
  const checkedItems = items && (typeof items == 'string' ? JSON.parse(items) : items);
  (_a = getProperty(id).childs) === null || _a === void 0 ? void 0 : _a.forEach(child => {
    if (checkedItems) {
      const checkedItem = checkedItems.find(e => e === child.name || e.name === child.name);
      if (checkedItem) {
        if (typeof checkedItem == 'object') {
          Object.keys(checkedItem).map(key => {
            if (key === 'checked') {
              child.checked = checkedItem[key];
            }
            else {
              child.checked = checked;
            }
          });
        }
        else {
          child.checked = checked;
        }
      }
      else {
        reset && (child.checked = false);
      }
    }
    else {
      child.checked = false;
    }
  });
  checkInderminate(id);
};
const resetCaption = id => {
  var _a;
  // const noCaption = getProperty(id).noCaption;
  (_a = getProperty(id).childs) === null || _a === void 0 ? void 0 : _a.forEach(child => {
    child.resetCaption();
    // child.noCaption = noCaption;
  });
};
const changeIndeterminate = id => {
  var _a;
  const group = getProperty(id);
  if (!group.disabled) {
    let checkAll = true;
    if (group.indeterminateState === 'determinate') {
      checkAll = false;
    }
    (_a = getProperty(id).childs) === null || _a === void 0 ? void 0 : _a.map(child => {
      child.checked = checkAll;
    });
    checkInderminate(id);
  }
};
const checkInderminate = id => {
  var _a;
  const group = getProperty(id);
  let allChecked = true;
  let allUnchecked = true;
  (_a = group.childs) === null || _a === void 0 ? void 0 : _a.map(child => {
    child.checked && (allUnchecked = false);
    !child.checked && (allChecked = false);
  });
  group.indeterminateState = !allChecked && !allUnchecked ? 'indeterminate' : allChecked ? 'determinate' : 'uncheck';
};
const onChange = (obj) => {
  var _a;
  (_a = getProperty(obj.id).childs) === null || _a === void 0 ? void 0 : _a.map(child => {
    child.checked = child.id === obj.childId ? !child.checked : child.checked;
  });
  checkInderminate(obj.id);
};
const removeState = id => {
  return Promise.resolve(state.group.delete(id));
};
const getStatesKeys = id => {
  return Object.keys(state.group.get(id));
};
export default {
  setValue,
  getValue,
  setCheckedItems,
  getCheckedList,
  getProperty,
  setClear,
  resetCaption,
  onChange,
  getStatesKeys,
  state,
  removeState,
  changeIndeterminate,
};
