/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { createStore } from "@stencil/store";
import snq from "snq";
import { Generate } from "../../../helper/generate";
import { StringHelper } from "../../../helper/string-helper";
const { state } = createStore({
  value: new Map(),
});
const setValue = val => {
  const { id, dataSource } = val;
  const data = {
    id: snq(() => id, Generate.UID),
    dataSource: dataSource && dataSource.length > 0 && recursive(dataSource),
    searchIsOnlyChilds: val.searchIsOnlyChilds,
  };
  state.value.set(data.id, data);
};
const recursive = (data, parentId) => {
  let recData = [];
  if (data) {
    recData = data.map(item => (Object.assign(Object.assign(Object.assign({}, item), { parentId: snq(() => parentId, null) }), (item.items && item.items.length > 0 && { items: snq(() => recursive(item.items, item.id), []) }))));
  }
  return recData;
};
const removeValue = _id => {
  state.value.delete(_id);
};
const getValue = id => snq(() => state.value.get(id), {});
const getDataPagination = (id, limit = 3) => {
  let data = [];
  const value = getValue(id);
  if (value) {
    const { dataSource } = value;
    data = dataSource.slice(0, limit);
  }
  return data;
};
// //search treeview data by text
// const searchData = (id, search, items?) => {
//     let dataSource
//     if (items && items.length > 0) {
//         dataSource = items
//     } else {
//         const value = getValue(id);
//         dataSource = value.dataSource;
//     }
//     let result = []
//     if (dataSource) {
//         dataSource.forEach(child => {
//             if (child.text.toLowerCase().includes(search.toLowerCase())) {
//                 result.push(child)
//             }
//             result = result.concat(searchData(id, search, child.items))
//         })
//     };
//     return result
// };
//nested search
const searchData = (id, searchText, items) => {
  let dataSource;
  if (items && items.length > 0) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  let results = [];
  if (dataSource) {
    results.push(nestedSearch(id, dataSource, searchText));
    //return with their parents
    // dataSource.forEach(child => {
    //     if (child.text.toLowerCase().includes(searchText.toLowerCase())) {
    //         results.push(child)
    //     } else if (child.items && child.items.length > 0) {
    //         child.items = nestedSearch(child.items, searchText)
    //         if (child.items.length > 0) {
    //             results.push(child)
    //         }
    //     }
    // })
  }
  return {
    result: snq(() => Generate.flatArray(results[0].result), []),
    matching: snq(() => results[0].matching, []),
    parentIndex: snq(() => results[0].parentIndex, []),
    nestedResult: snq(() => results[0].nestedResult, []),
  };
};
const nestedSearch = (id, items, searchText) => {
  let matching = [];
  let parentIndex = [];
  let result = [];
  let nestedResult = [];
  const { searchIsOnlyChilds } = getValue(id);
  if (items && items.length > 0) {
    items.forEach(child => {
      let isChild = searchIsOnlyChilds && child.parentId !== null;
      let isMatching = (child.text && StringHelper.slugify(child.text).includes(StringHelper.slugify(searchText))) ||
        (child.keywords && StringHelper.slugify(child.keywords).includes(StringHelper.slugify(searchText)));
      if (isMatching && (!searchIsOnlyChilds || (searchIsOnlyChilds && isChild))) {
        result.push(child.id);
        matching.push(child.id);
        parentIndex.push(child.id);
        nestedResult.push({ id: child.id, text: child.text, parentId: child.parentId });
        if (child.items && child.items.length > 0) {
          const found = nestedSearch(id, child.items, searchText);
          if (found.matching.length > 0) {
            found.matching.forEach(chi => {
              matching.push(chi);
            });
          }
        }
      }
      else if (child.items && child.items.length > 0) {
        const found = nestedSearch(id, child.items, searchText);
        if (found.result.length > 0) {
          found.result.forEach(chi => {
            result.push(chi);
          });
          result.push(child.id);
          parentIndex.push(child.id);
          nestedResult.push({ id: child.id, text: child.text, parentId: child.parentId, items: found.nestedResult });
        }
        if (found.matching.length > 0) {
          matching.push(...found.matching);
        }
      }
    });
  }
  return { result, matching, parentIndex, nestedResult };
};
export default {
  setValue,
  removeValue,
  getValue,
  getDataPagination,
  searchData,
};
