/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { createStore } from '@stencil/store';
import snq from 'snq';
import { Generate } from '../../../helper/generate';
import { forceUpdate } from '@stencil/core';
const { state } = createStore({
  value: new Map(),
  setData: {},
});
const setValue = val => {
  const data = Object.assign(Object.assign({}, val), { steps: val.steps &&
      val.steps.length > 0 &&
      val.steps
        .filter(data => !data.hidden)
        .map((item, index) => ({
        id: item.id || Generate.UID(),
        title: item.title || item._title,
        _title: item.title || item._title || 'Step ' + (index + 1),
        description: item.description || '',
        status: item.status || 'default',
        disabled: val.linear ? (index == 0 ? false : true) : item.disabled || false,
        icon: item.icon || undefined,
        valid: item.valid || true,
        loading: item.loading || false,
        index: index + 1,
        active: val.linear ? (index == 0 ? true : false) : val.active === index + 1,
        isLast: index === val.steps.length - 1,
        statusIcon: val.icon,
        headerHidden: item.hidden,
      })) });
  state.value.set(val.id, data);
};
const removeState = _id => {
  state.value.delete(_id);
};
const changeActiveStep = (id, index) => {
  const value = state.value.get(id);
  if (value) {
    value.active = index;
    value.steps = value.steps.map((step, i) => (Object.assign(Object.assign({}, step), { active: i === index - 1 })));
  }
};
const getActiveIndex = id => snq(() => state.value.get(id).active, 0);
const changeStepStatus = (id, stepId, status) => {
  const value = state.value.get(id);
  if (value && value.steps) {
    value.steps = value.steps.map(step => (Object.assign(Object.assign({}, step), { status: step.id === stepId ? status : step.status })));
    forceUpdate(value.el);
  }
};
const changeStepDisabled = (id, stepId, disabled) => {
  const value = state.value.get(id);
  if (value && value.steps) {
    value.steps = value.steps.map(step => (Object.assign(Object.assign({}, step), { disabled: step.id === stepId ? disabled : step.disabled })));
    forceUpdate(value.el);
  }
};
const setActiveIndex = (id, index) => state.value.get(id).active = index;
const stepperReset = id => {
  const value = state.value.get(id);
  if (value) {
    value.active = 1;
    value.steps = value.steps.map((step, i) => (Object.assign(Object.assign({}, step), { active: i === 0, status: 'default', disabled: i !== 0, loading: false })));
    forceUpdate(value.el);
  }
};
const getValue = id => snq(() => state.value.get(id), {});
export default {
  removeState,
  getValue,
  setValue,
  changeActiveStep,
  getActiveIndex,
  changeStepStatus,
  changeStepDisabled,
  stepperReset,
  setActiveIndex
};
