/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { Generate } from '../../../helper/generate';
import stepperStates from '../states/stepper-states';
export class BcmStep {
  constructor() {
    this.hidden = false;
    this._id = Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = 'default';
    this.icon = undefined;
    this.loading = false;
    this.valid = true;
    this.index = undefined;
    this.active = false;
    this.htmlTemplate = undefined;
    this.isLast = false;
    this.disabled = false;
  }
  componentDidLoad() {
    if (this.isLast) {
      this.dataIsLoaded.emit(true);
    }
  }
  getStepper() {
    const stepper = this.el.closest('bcm-stepper');
    return stepper || null;
  }
  activeChanged() {
    if (this.active) {
      const stepper = this.getStepper();
      if (stepper) {
        stepperStates.changeStepDisabled(stepper.stepperId, this._id, false);
      }
    }
  }
  statusChange() {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepStatus(stepper.stepperId, this._id, this.status);
    }
  }
  hiddenChange(newValue, oldValue) {
    const stepper = this.getStepper();
    if (newValue !== oldValue && stepper && !this.active) {
      this.bcmChangeHidden.emit(newValue);
    }
  }
  disabledChange(newValue, _) {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepDisabled(stepper.stepperId, this._id, newValue);
    }
  }
  render() {
    const { hidden, valid, active } = this;
    const hostClasses = cs('bcm-stepper__step', valid ? 'bcm-stepper__step-valid' : 'bcm-stepper__step-invalid', {
      'bcm-stepper__step-active': active,
      hidden,
    });
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  static get is() { return "bcm-step"; }
  static get properties() {
    return {
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_title": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_title",
        "reflect": false
      },
      "description": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "description",
        "reflect": false
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'default' | 'success' | 'warning' | 'error'",
          "resolved": "\"default\" | \"error\" | \"success\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "'default'"
      },
      "icon": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false
      },
      "loading": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "loading",
        "reflect": false,
        "defaultValue": "false"
      },
      "valid": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "valid",
        "reflect": false,
        "defaultValue": "true"
      },
      "index": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "index",
        "reflect": false
      },
      "active": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "active",
        "reflect": true,
        "defaultValue": "false"
      },
      "htmlTemplate": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "html-template",
        "reflect": false
      },
      "isLast": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-last",
        "reflect": false,
        "defaultValue": "false"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "dataIsLoaded",
        "name": "data-is-loaded",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmChangeHidden",
        "name": "bcm-change-hidden",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "active",
        "methodName": "activeChanged"
      }, {
        "propName": "status",
        "methodName": "statusChange"
      }, {
        "propName": "hidden",
        "methodName": "hiddenChange"
      }, {
        "propName": "disabled",
        "methodName": "disabledChange"
      }];
  }
}
