import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { StepperHeaderTemplate } from "@bcm/template";
import { getChildElements } from "@bcm/utils";
import { Host, forceUpdate, h } from "@stencil/core";
import cs from "classnames";
import snq from "snq";
import stepperStates from "../states/stepper-states";
export class BcmStepper {
  constructor() {
    this.hidden = false;
    this._id = Generate.UID();
    this.direction = Bcm.Direction.horizontal;
    this.linear = false;
    this.size = Bcm.Size.medium;
    this.icon = false;
    this.active = 1;
    this.stepperId = Generate.UID();
    this.data = [];
    this.tick = {};
  }
  async dataChanged() {
    this.initState();
  }
  componentDidLoad() {
    this.initState();
  }
  disconnectedCallback() {
    // stepperStates.removeState(this.stepperId);
    this.reset();
  }
  async initState() {
    const { stepperId, data, direction, linear, size, icon, active, el } = this;
    const steps = data.length > 0 ? data : this.getSlotElements();
    await stepperStates.setValue({
      id: stepperId,
      el,
      direction,
      linear,
      size,
      icon,
      active,
      steps,
    });
    this.markForCheck();
  }
  componentDidRender() {
    this.setSteps(this.getItems());
  }
  setSteps(steps) {
    const slots = getChildElements(this.el, "bcm-step");
    if (slots.length > 0) {
      slots.forEach((item, index) => {
        const step = steps.find(step => !item.headerHidden && step.id === item.id);
        if (step) {
          Generate.setAttributes(item, Object.assign({ title: "", _title: step.title || step._title || "", status: step.status || "default", disabled: step.disabled || false, valid: step.valid || true, loading: step.loading || false, index: step.index || index, active: step.active || false, innerHtml: step.innerHtml || "" }, (item.hidden == true && { headerHidden: true })));
          if ((item.hidden = false)) {
            item.removeAttribute("hidden");
          }
        }
      });
    }
  }
  markForCheck() {
    this.tick = Generate.UID();
  }
  handleChangeBcmHidden() {
    this.initState();
  }
  getActiveIndex() {
    return stepperStates.getActiveIndex(this.stepperId);
  }
  getSlotElements() {
    return snq(() => getChildElements(this.el, "bcm-step"), []);
  }
  async onStepChange(index) {
    if (index) {
      stepperStates.changeActiveStep(this.stepperId, index);
      this.bcmStepChange.emit(index);
    }
    forceUpdate(this.el);
    return Promise.resolve(index);
  }
  async prev() {
    const index = this.getActiveIndex() - 1;
    if (index >= 1) {
      if (!this.linear) {
        const previosStep = this.getItems()[index - 1];
        if (previosStep.disabled)
          return;
        await this.onStepChange(index);
      }
      else {
        await this.onStepChange(index);
      }
    }
    forceUpdate(this.el);
    return Promise.resolve(this.getActiveIndex());
  }
  async next(idx) {
    let index;
    idx == 0 ? (index = 0) : index == 1 ? (index = 1) : (index = idx || this.getActiveIndex());
    const step = await this.getCurrentStep();
    const lastStep = this.getItems()[this.getItems().length - 1];
    if (step.valid) {
      this.linear && lastStep.id != step.id && (step.status = "success");
      if (index < this.getItems().length) {
        await this.onStepChange(index + 1);
      }
    }
    else {
      step.status = "error";
    }
    forceUpdate(this.el);
    return Promise.resolve(this.getActiveIndex());
  }
  async setActive(index) {
    if (!index)
      return;
    if (typeof index != "number") {
      console.warn("index must be a number");
      return;
    }
    if (this.linear) {
      const prevSteps = this.getItems().filter(item => item.index < index);
      const prevStepIsValid = prevSteps === null || prevSteps === void 0 ? void 0 : prevSteps.map(item => ({
        id: item.id,
        status: item.status,
        disabled: item.disabled,
      })).every(item => item.status == "success" || item.status == "default");
      if (prevStepIsValid) {
        prevSteps === null || prevSteps === void 0 ? void 0 : prevSteps.forEach(item => {
          item.status = "success";
          item.disabled = false;
        });
        await this.next(index - 1);
      }
    }
    else {
      stepperStates.setActiveIndex(this.stepperId, index);
      await this.next(index - 1);
    }
  }
  async reset() {
    await this.onStepChange(1);
    stepperStates.stepperReset(this.stepperId);
    return Promise.resolve(this.getActiveIndex());
  }
  async getCurrentStep() {
    var _a;
    return snq(() => this.getItems().filter(item => item.active == true)[0], (_a = this.getItems()[0]) !== null && _a !== void 0 ? _a : null);
  }
  async getSteps() {
    return Promise.resolve(this.getItems());
  }
  getItems() {
    return snq(() => { var _a; return (_a = stepperStates.getValue(this.stepperId).steps) === null || _a === void 0 ? void 0 : _a.filter(item => !item.headerHidden); }, []);
  }
  headerClick(index) {
    this.bcmStepperBeforeChange.emit({
      current: this.getActiveIndex() - 1,
      next: index,
    });
  }
  async handleBcmStepBeforeChange(e) {
    if (e.defaultPrevented) {
      return;
    }
    if (e.target === this.el) {
      const { next } = e.detail;
      this.next(next);
    }
  }
  render() {
    const { hidden, direction, linear, size } = this;
    const hostClasses = cs("bcm-ui-element tw-font-sans tw-antialiased", "bcm-stepper", "bcm-stepper__direction-" + direction, "bcm-stepper__size-" + size, {
      "bcm-stepper__linear": linear,
      hidden,
    });
    const lineClasses = item => cs("bcm-stepper__header-line", `bcm-stepper__header-line--${(item === null || item === void 0 ? void 0 : item.disabled) ? "disabled" : (item === null || item === void 0 ? void 0 : item.status) === "error" ? "default" : item === null || item === void 0 ? void 0 : item.status}`);
    const items = this.getItems();
    return (h(Host, { class: hostClasses }, h("span", { hidden: true, id: `bcm-stepper__steps-${this._id}` }, h("slot", null)), h("main", null, h("header", { class: "bcm-stepper__header" }, items === null || items === void 0 ? void 0 : items.map((item, index) => [
      h(StepperHeaderTemplate, Object.assign({}, item, { stepChange: step => this.headerClick(step - 1) })),
      items.length > index + 1 && h("div", { class: lineClasses(item) }),
    ])), h("section", { class: "bcm-stepper__body" }, this.data && this.data.length > 0 ? items === null || items === void 0 ? void 0 : items.map(item => h("bcm-step", Object.assign({}, item))) : h("slot", null)))));
  }
  static get is() { return "bcm-stepper"; }
  static get originalStyleUrls() {
    return {
      "$": ["stepper.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["stepper.css"]
    };
  }
  static get properties() {
    return {
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "direction": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "DirectionTypes",
          "resolved": "\"horizontal\" | \"vertical\"",
          "references": {
            "DirectionTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "direction",
        "reflect": false,
        "defaultValue": "Bcm.Direction.horizontal"
      },
      "linear": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "linear",
        "reflect": false,
        "defaultValue": "false"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "icon": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "icon",
        "reflect": false,
        "defaultValue": "false"
      },
      "active": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "active",
        "reflect": false,
        "defaultValue": "1"
      },
      "stepperId": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "stepper-id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "data": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "any[]",
          "resolved": "any[]",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "[]"
      },
      "tick": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "{}",
          "resolved": "{}",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "{}"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmStepChange",
        "name": "bcm-step-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmStepperBeforeChange",
        "name": "bcm-step-before-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "prev": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "next": {
        "complexType": {
          "signature": "(idx?: number) => Promise<any>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setActive": {
        "complexType": {
          "signature": "(index: number) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "reset": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getCurrentStep": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getSteps": {
        "complexType": {
          "signature": "() => Promise<any[]>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any[]>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "data",
        "methodName": "dataChanged"
      }];
  }
  static get listeners() {
    return [{
        "name": "bcm-change-hidden",
        "method": "handleChangeBcmHidden",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-step-before-change",
        "method": "handleBcmStepBeforeChange",
        "target": "document",
        "capture": false,
        "passive": false
      }];
  }
}
