/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { IsLoad } from '../../../decorators/is-load-decorator';
import { Generate } from '../../../helper/generate';
import Bcm from '../../../models/bcm';
import { CaptionTemplate } from '../../../templates/caption-template';
import { delay } from '../../../utils/utils';
export class BcmSwitch {
  constructor() {
    this._id = Generate.UID();
    this._internal_id = Generate.UID();
    this.checked = false;
    this.hidden = false;
    this.value = false;
    this.name = undefined;
    this.disabled = false;
    this.required = false;
    this.size = Bcm.Size.medium;
    this.readonly = false;
    this.activeIcon = undefined;
    this.activeText = undefined;
    this.inactiveIcon = undefined;
    this.inactiveText = undefined;
    this.pending = false;
    this.label = undefined;
    this.labelPosition = 'top';
    this.tooltip = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionError = undefined;
    this.captionType = Bcm.Status.default;
    this.captionCache = undefined;
    this.captionTypeCache = undefined;
  }
  connectedCallback() {
    this.captionCache = this.caption;
    this.captionTypeCache = this.captionType;
    this.value = this.checked;
  }
  handleChange() {
    this.checked = this.inputElement.checked;
    this.value = this.checked;
    this.change.emit(this.checked);
  }
  componentWillRender() {
    if (this.pending) {
      this.disabled = true;
    }
  }
  async onBcmFocus() {
    this.el.focus();
  }
  async onBcmBlur() {
    this.el.blur();
  }
  async set(data) {
    if (typeof data === 'boolean') {
      await this.check(data);
    }
  }
  async check(uncheck = true) {
    this.inputElement.checked = uncheck;
    this.handleChange();
  }
  async get() {
    return this.checked || false;
  }
  async setClear() {
    if (this.inputElement.checked == true) {
      this.inputElement.checked = false;
      this.checked = this.inputElement.checked;
      this.value = this.checked;
      this.change.emit(this.checked);
    }
    await delay(10);
    return Promise.resolve();
  }
  async resetCaption() {
    await delay(10);
    this.caption = this.captionCache;
    this.captionType = this.captionTypeCache;
    this.captionError = null;
    await delay(10);
    return Promise.resolve();
  }
  handleKeyUp(ev) {
    if (ev.keyCode == Bcm.KeyCode.space || ev.keyCode == Bcm.KeyCode.enter) {
      if (!this.disabled && !this.readonly) {
        this.inputElement.checked = !this.inputElement.checked;
        this.handleChange();
      }
      ev.stopPropagation();
      ev.preventDefault();
      return false;
    }
  }
  render() {
    const { hidden, _internal_id, checked, size, activeIcon, inactiveIcon, activeText, inactiveText, pending, disabled, readonly, label, required, noCaption, captionType, captionError, caption } = this;
    const classes = cs('bcm-switch__toggle', 'bcm-switch__size-' + size, {
      'switch-checked': checked,
      'disabled': disabled,
      'readonly': readonly
    });
    const bcmSwitchClasses = cs('bcm-switch', 'bcm-switch__size-' + size, {
      hidden,
      checked,
      disabled,
      readonly
    });
    const bcmSwitchLabel = cs('bcm-switch__label', 'input-size-' + Bcm.Size[size]);
    const bcmSwitchContainer = cs('bcm-switch__container', `bcm-switch__container--${this.labelPosition}`);
    const switchText = (text) => (size != Bcm.Size.small && (activeText || inactiveText)) && (h("span", { class: text + '-text' }, text == 'active' ? activeText : inactiveText));
    const switchIcon = (text) => (size != Bcm.Size.small && (activeIcon || inactiveIcon)) && h("bcm-icon", { icon: text == 'active' ? activeIcon : inactiveIcon, class: ' icon-' + text });
    return (h(Host, { tabindex: "0", onFocus: () => this.focus.emit(this.el), onBlur: () => this.blur.emit(this.el), class: bcmSwitchClasses }, h("span", { hidden: true }, h("slot", null)), h("div", { class: bcmSwitchContainer }, label && h("bcm-label", { htmlFor: _internal_id, tooltip: this.tooltip, type: captionType, value: label, required: required }), h("input", { id: _internal_id, onChange: () => this.handleChange(), ref: el => (this.inputElement = el), checked: checked, type: "checkbox", class: "switch", disabled: disabled || readonly }), h("label", { htmlFor: _internal_id, class: bcmSwitchLabel }, switchText('inactive'), h("div", { class: classes }, switchIcon('active'), h("span", { class: "handle" }, pending && h("span", { class: "switch-spinner" })), switchIcon('inactive')), switchText('active'))), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption })));
  }
  static get is() { return "bcm-switch"; }
  static get originalStyleUrls() {
    return {
      "$": ["switch.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["switch.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Props *"
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_internal_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_internal_id",
        "reflect": false,
        "defaultValue": "Generate.UID()"
      },
      "checked": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "checked",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "value": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false,
        "defaultValue": "false"
      },
      "name": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "name",
        "reflect": false
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "required": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "required",
        "reflect": false,
        "defaultValue": "false"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "readonly": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "readonly",
        "reflect": false,
        "defaultValue": "false"
      },
      "activeIcon": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "active-icon",
        "reflect": false
      },
      "activeText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "active-text",
        "reflect": false
      },
      "inactiveIcon": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "inactive-icon",
        "reflect": false
      },
      "inactiveText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "inactive-text",
        "reflect": false
      },
      "pending": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "pending",
        "reflect": false,
        "defaultValue": "false"
      },
      "label": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "label",
        "reflect": false
      },
      "labelPosition": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'top' | 'left' | 'right'",
          "resolved": "\"left\" | \"right\" | \"top\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "label-position",
        "reflect": false,
        "defaultValue": "'top'"
      },
      "tooltip": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "tooltip",
        "reflect": false
      },
      "caption": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption",
        "reflect": false
      },
      "noCaption": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-caption",
        "reflect": false,
        "defaultValue": "false"
      },
      "captionError": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-error",
        "reflect": false
      },
      "captionType": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "StatusTypes",
          "resolved": "\"default\" | \"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {
            "StatusTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-type",
        "reflect": false,
        "defaultValue": "Bcm.Status.default"
      }
    };
  }
  static get states() {
    return {
      "captionCache": {},
      "captionTypeCache": {}
    };
  }
  static get events() {
    return [{
        "method": "change",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "focus",
        "name": "bcm-focus",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "blur",
        "name": "bcm-blur",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "onBcmFocus": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "onBcmBlur": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "set": {
        "complexType": {
          "signature": "(data: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "check": {
        "complexType": {
          "signature": "(uncheck?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "get": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setClear": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "resetCaption": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "keydown",
        "method": "handleKeyUp",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
__decorate([
  IsLoad()
], BcmSwitch.prototype, "el", void 0);
