/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
export class BcmTimeline {
  constructor() {
    this.icon = undefined;
    this.type = 'left';
    this.hidden = false;
    this.items = [];
  }
  /**
   * @StencilMethod
   */
  componentDidRender() {
    let slotElements;
    slotElements = this.el.shadowRoot.querySelector('slot').assignedElements();
    // Get only bcm-timeline-item elements from 
    // slot childs
    slotElements.map((element, idx) => {
      if (String(element.tagName).toLowerCase() == 'bcm-timeline-item') {
        element.timelineType = this.type;
        element.even = idx % 2 === 0;
        element.first = idx === 0;
        element.last = idx === slotElements.length - 1;
        element.icon = element.icon ? element.icon : this.icon;
        this.items.push(element);
      }
    });
  }
  /**
  * @StencilMethod
  */
  componentWillRender() {
    console.log(this.el.shadowRoot);
  }
  render() {
    const classes = cs('timeline', [this.type]);
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: classes }, h("slot", null))));
  }
  static get is() { return "bcm-timeline"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["timeline.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["timeline.css"]
    };
  }
  static get properties() {
    return {
      "icon": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "icon",
        "reflect": false
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'left' | 'right' | 'alternate'",
          "resolved": "\"alternate\" | \"left\" | \"right\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "'left'"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get states() {
    return {
      "items": {}
    };
  }
  static get elementRef() { return "el"; }
}
