/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate, Tooltip } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { Host, h } from "@stencil/core";
export class BcmTooltip {
  constructor() {
    this._id = Generate.UID();
    this.message = "";
    this.placement = Bcm.Placement.top;
    this.trigger = Bcm.TriggerType.click;
    this.open = false;
    this.color = "slate";
    this.size = Bcm.Size.medium;
    this.openDelay = 0;
    this.closeDelay = 0;
    this.autoHide = false;
    this.targetId = undefined;
    this.tooltip = undefined;
  }
  emitShow() {
    this.bcmTooltipShow.emit();
  }
  emitHide() {
    this.bcmTooltipHide.emit();
  }
  connectedCallback() {
    this.tooltip = new Tooltip({
      targetId: this.targetId || this._id,
      message: this.message,
      placement: this.placement,
      color: this.color,
      trigger: this.trigger,
      size: this.size,
      openDelay: this.openDelay,
      closeDelay: this.closeDelay,
      autoHide: this.autoHide,
      emitShow: () => this.emitShow(),
      emitHide: () => this.emitHide(),
    });
  }
  componentDidLoad() {
    this.tooltip.listener();
  }
  componentDidRender() {
    this.tooltip.message = this.message;
    this.tooltip.placement = this.placement;
    this.tooltip.color = this.color;
    this.tooltip.trigger = this.trigger;
    this.tooltip.size = this.size;
    this.tooltip.openDelay = this.openDelay;
    this.tooltip.closeDelay = this.closeDelay;
    this.tooltip.autoHide = this.autoHide;
    this.tooltip.update();
  }
  disconnectedCallback() {
    this.tooltip.destroy();
  }
  async show() {
    this.tooltip.show();
  }
  async hide() {
    this.tooltip.hide();
  }
  async toggle() {
    this.tooltip.toggle();
  }
  render() {
    return (h(Host, { class: "tw-inline-flex tw-h-fit tw-w-fit", role: "tooltip" }, h("slot", null)));
  }
  static get is() { return "bcm-tooltip"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "message": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false,
        "defaultValue": "\"\""
      },
      "placement": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "Bcm.Placement.top"
      },
      "trigger": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "TrigglerTypes",
          "resolved": "\"click\" | \"custom\" | \"focus\" | \"hover\"",
          "references": {
            "TrigglerTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "trigger",
        "reflect": false,
        "defaultValue": "Bcm.TriggerType.click"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "color": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false,
        "defaultValue": "\"slate\""
      },
      "size": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "openDelay": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open-delay",
        "reflect": false,
        "defaultValue": "0"
      },
      "closeDelay": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "close-delay",
        "reflect": false,
        "defaultValue": "0"
      },
      "autoHide": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "auto-hide",
        "reflect": false,
        "defaultValue": "false"
      },
      "targetId": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target-id",
        "reflect": false
      }
    };
  }
  static get states() {
    return {
      "tooltip": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmTooltipShow",
        "name": "bcmTooltipShow",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmTooltipHide",
        "name": "bcmTooltipHide",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "toggle": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
