/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
export class BcmAutocomplete {
  constructor() {
    this.objectMapping = {
      'id': "id",
      'text': "text",
      'value': "value",
    };
    this.searchFields = ['text', 'secondary_text', 'tooltip_text'];
    this._items = [];
    this.fullWidth = false;
    this.hidden = false;
    this.searchText = null;
    this.returnField = "id";
    this.searchSub = "items";
    this.searchPlaceholder = "Search";
    this.inputLength = 0;
    this.isOpen = false;
    this.resultFound = false;
    this.searchData = undefined;
    this.searchResults = [];
    this.items = [];
  }
  async config(_config) {
    await customElements.whenDefined("bcm-listbox");
    this.bcmListbox = this.el.shadowRoot.querySelector("bcm-listbox");
    await this.bcmListbox.config(_config);
    if (_config['searchFields']) {
      if (Array.isArray(_config['searchFields'])) {
        this.searchFields = _config['searchFields'];
      }
    }
    if (_config['searchSub']) {
      typeof _config['searchSub'] == 'string'
        ? this.searchSub = _config['searchSub']
        : this.searchSub = null;
    }
    if (_config['fullWidth']) {
      typeof _config['fullWidth'] == 'boolean'
        ? this.fullWidth = _config['fullWidth']
        : this.fullWidth = false;
    }
    if (_config['searchPlaceholder']) {
      typeof _config['searchPlaceholder'] == 'string'
        ? this.searchPlaceholder = _config['searchPlaceholder']
        : this.searchPlaceholder = null;
    }
    if (_config['mapping']) {
      if (_config['mapping']['id']) {
        this.objectMapping['id'] = _config['mapping']['id'];
        this.returnField = this.objectMapping['id'];
      }
      if (_config['mapping']['text']) {
        this.objectMapping['text'] = _config['mapping']['text'];
      }
      if (_config['mapping']['value']) {
        this.objectMapping['value'] = _config['mapping']['value'];
      }
      if (_config['mapping']['items']) {
        this.objectMapping['items'] = _config['mapping']['items'];
        this.searchSub = this.objectMapping['items'];
      }
    }
  }
  async setData(_data = []) {
    await customElements.whenDefined("bcm-listbox");
    this.bcmListbox = this.el.shadowRoot.querySelector("bcm-listbox");
    await this.bcmListbox.setData(_data);
    this.searchData = JSON.parse(_data);
  }
  /**
   * @desc
   */
  async handleListboxClearSelected(e) {
    const optionValue = e.detail[this.objectMapping['text']];
    const search = this.el.shadowRoot.querySelector("bcm-old-search");
    this.autocompleteItemSelected.emit(e.detail);
    await search.setValue(optionValue);
    this.isOpen = false;
  }
  listenSearchValue(e) {
    this.inputLength = e.detail.length;
    if (this.inputLength > 0) {
      this.isOpen = true;
    }
    else {
      this.isOpen = false;
    }
  }
  async listenSearchResult(e) {
    this.searchResults = e.detail;
    const bcmListboxx = this.el.shadowRoot.querySelector("bcm-listbox");
    await bcmListboxx.searchResultMethod(this.searchResults);
  }
  handleClickOutside(e) {
    if (!this.el.contains(e.target)) {
      if (this.isOpen)
        this.isOpen = false;
    }
    else {
      if (this.inputLength > 0) {
        this.isOpen = true;
      }
      else {
        this.isOpen = false;
      }
    }
  }
  render() {
    const { searchText, searchData, searchFields, searchSub, returnField, isOpen, fullWidth } = this;
    const classes = cs('bcm-autocomplete', {
      'open': isOpen
    });
    const classesSearch = cs('bcm-autocomplete-search');
    const classesResult = cs('bcm-autocomplete-result', {
      'open': isOpen
    });
    const hostClasses = cs(this.hidden ? 'hidden' : null, {
      'full-width': fullWidth
    });
    return (h(Host, { class: hostClasses }, h("div", { class: classes }, h("div", { class: classesSearch }, h("bcm-old-search", { searchPlaceholder: this.searchPlaceholder, id: "autocomplete", searchValue: searchText, data: searchData, returnField: returnField, searchFields: searchFields, searchSub: searchSub, clearable: true })), h("div", { class: classesResult }, h("div", { class: "viewport" }, h("bcm-listbox", { searchText: searchText, _flex: "true" }))))));
  }
  static get is() { return "bcm-autocomplete"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["autocomplete.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["autocomplete.css"]
    };
  }
  static get properties() {
    return {
      "searchFields": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "Array<string>",
          "resolved": "string[]",
          "references": {
            "Array": {
              "location": "global"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "['text', 'secondary_text', 'tooltip_text']"
      },
      "_items": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "Array<object> | string",
          "resolved": "object[] | string",
          "references": {
            "Array": {
              "location": "global"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "data",
        "reflect": false,
        "defaultValue": "[]"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "searchText": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "searchtext",
        "reflect": false,
        "defaultValue": "null"
      },
      "returnField": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "return-field",
        "reflect": false,
        "defaultValue": "\"id\""
      },
      "searchSub": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "search-sub",
        "reflect": false,
        "defaultValue": "\"items\""
      },
      "searchPlaceholder": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "searchplaceholder",
        "reflect": false,
        "defaultValue": "\"Search\""
      }
    };
  }
  static get states() {
    return {
      "inputLength": {},
      "isOpen": {},
      "resultFound": {},
      "searchData": {},
      "searchResults": {},
      "items": {}
    };
  }
  static get events() {
    return [{
        "method": "autocompleteItemSelected",
        "name": "bcm-autocomplete-item-selected",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "config": {
        "complexType": {
          "signature": "(_config: Array<object>) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            },
            "Array": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setData": {
        "complexType": {
          "signature": "(_data?: Array<object> | string) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            },
            "Array": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "bcm-listbox-item-selected",
        "method": "handleListboxClearSelected",
        "target": undefined,
        "capture": true,
        "passive": false
      }, {
        "name": "bcm-on-search-value",
        "method": "listenSearchValue",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-on-search",
        "method": "listenSearchResult",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "click",
        "method": "handleClickOutside",
        "target": "window",
        "capture": true,
        "passive": false
      }];
  }
}
