/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { IsLoad } from '../../../decorators/is-load-decorator';
import { StringHelper } from '../../../helper/string-helper';
import { delay, getChildElements } from '../../../utils/utils';
import { validate } from '../../../utils/validation';
export class BcmForm {
  constructor() {
    this.formData = {};
    this.inputs = [];
    this.duplicateInput = [];
    this.isValid = true;
    this.returnCaption = [];
    this.Bromcom = {
      errors: {
        targetFormNotExist: 'target form is not exist --> ',
      },
      inputComponents: {
        CHIP_GROUP: 'chip-group',
        // Birbirinin aynısı
        INPUT: 'input',
        TEXTAREA: 'textarea',
        //çok benzer
        SWITCH: 'switch',
        CHECKBOX: 'checkbox',
        RADIO_GROUP: 'radio-group',
        CHECKBOX_GROUP: 'checkbox-group',
        RANGE: 'range',
        // sadece value fırlatıyor
        FORM: 'form',
        FORM_GROUP: 'form-group',
        // *En vakit alacak => içinde select, autocomplete, listbox, treeview özellikleri var
        LIST: 'list',
        //
        COLORPICKER: 'colorpicker',
        DATE: 'date',
        // picker aslında tek komponent hepsi datetimepicker
        DATE_PICKER: 'date-picker',
        TIME_PICKER: 'time-picker',
        DATETIME_PICKER: 'datetime-picker',
        // silinecek komponentler
        SELECT: 'select',
        LISTBOX: 'listbox',
        NUMBER_INPUT: 'number-input',
      },
      helpers: {
        randomId: function () {
          return Math.random().toString(16).slice(2, -1);
        },
      },
    };
    this._id = (Math.random() * 4).toString(16).replace('.', '');
    this.hidden = false;
    this.validation = false;
    this.name = "";
    this.value = undefined;
    this.autocomplete = false;
  }
  handleSubmit() {
    // if (lowercase(e.target.nodeName) === 'bcm-button') {
    //     if (e.target.type === 'submit') {
    //         this.submitFunction()
    //     }
    // }
  }
  async submitFunction() {
    this.formData = {};
    this.inputs = [];
    this.selector = this.el;
    await this.getInputs();
    await delay(10);
    const usrFrmData = await this.userFormData();
    var formData = { isValid: true, formData: usrFrmData };
    if (this.name) {
      formData = Object.assign(Object.assign({}, formData), { formName: this.name });
    }
    if (this.isValid) {
      this.bcmForm.emit(formData);
      return formData;
    }
    else {
      this.returnCaption = { isValid: false, 'message': 'Please fill in all inputs', 'errorDetails': this.returnCaption, formData: usrFrmData };
      if (this.name) {
        this.returnCaption = Object.assign(Object.assign({}, this.returnCaption), { formName: this.name });
      }
      this.bcmForm.emit(this.returnCaption);
      return this.returnCaption;
    }
  }
  async submit() {
    return Promise.resolve(this.submitFunction());
  }
  async set(data) {
    var _a;
    const validate = this.validation;
    this.validation = false;
    this.selector = this.el;
    await this.getInputs();
    if (data && ((_a = this.inputs) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      for (const key of Object.keys(data)) {
        const item = this.inputs.find(item => item.name === key);
        if (item) {
          try {
            await item.set(data[key]);
          }
          catch (error) {
            console.log(item.nodeName, "set", error);
          }
        }
      }
    }
    this.validation = validate;
  }
  async get() {
    var _a;
    let formData = {};
    this.selector = this.el;
    const inputsCache = this.inputs;
    await this.getInputs();
    ((_a = this.inputs) === null || _a === void 0 ? void 0 : _a.length) > 0 && this.inputs.forEach(async (item) => {
      try {
        const value = await item.get();
        formData = Object.assign(Object.assign({}, formData), { [item.name]: value });
      }
      catch (error) {
        console.log(`${item.nodeName} get error`);
      }
    });
    this.inputs = inputsCache;
    await delay(10);
    return Promise.resolve(formData);
  }
  async setClear() {
    this.formData = {};
    this.inputs = [];
    this.selector = this.el;
    let clear = [];
    await this.getInputs();
    this.inputs.forEach(async (item) => {
      try {
        clear.push(await item.setClear());
      }
      catch (error) {
        console.log(error);
      }
    });
    await Promise.all(clear).then(() => {
      this.formData = {};
      this.inputs = [];
      this.duplicateInput = [];
      this.selector;
      this.isValid = true;
      this.returnCaption = [];
    });
    await delay(10);
    return Promise.resolve(true);
  }
  async resetCaption() {
    await delay(10);
    this.formData = {};
    this.inputs = [];
    this.selector = this.el;
    await this.getInputs();
    for (const item of this.inputs) {
      try {
        await item.resetCaption();
      }
      catch (error) {
        console.log(item.nodeName, "rc p");
      }
    }
    await delay(10);
    return Promise.resolve();
  }
  async config(_config) {
    if (_config['validation']) {
      typeof _config['validation'] == 'boolean'
        ? this.validation = _config['validation']
        : this.validation = false;
    }
    return true;
  }
  async getInputs() {
    var form = this.selector;
    if (!form) {
      console.warn(this.Bromcom.errors.targetFormNotExist + this.selector);
      return;
    }
    const inputs = Array.from(form.querySelectorAll('bcm-' + Object.values(this.Bromcom.inputComponents).join(',bcm-')));
    this.duplicateInput = [];
    // s-hn: Stencil private flag that contains host web component
    // https://github.com/ionic-team/stencil/blob/45388e95edb46ef357eb9ae37cd32bbb5bc1ed23/src/runtime/vdom/vdom-render.ts#L116
    const rootInputs = inputs.filter((input) => { var _a; return !input['s-hn'] || ((_a = input['s-hn']) === null || _a === void 0 ? void 0 : _a.nodeName) !== 'BcmForm'; });
    // debugger;
    for (const item of rootInputs) {
      if (StringHelper.lowercase(item.nodeName) == "bcm-checkbox-group") {
        let checkboxs = item.querySelectorAll("bcm-checkbox");
        if (checkboxs) {
          for (const checkbox of checkboxs) {
            this.duplicateInput.push(checkbox.name);
          }
        }
      }
      if (StringHelper.lowercase(item.nodeName) == "bcm-form") {
        const groupInputs = Array.from(item.querySelectorAll('bcm-' + Object.values(this.Bromcom.inputComponents).join(',bcm-')));
        let itemObject = {};
        for (const element of groupInputs) {
          var name = element.name;
          var value = element.value;
          itemObject = Object.assign(Object.assign({}, itemObject), { [name]: value });
          this.duplicateInput.push(element.name);
        }
        item.value = itemObject;
        let checkboxs = item.querySelectorAll("bcm-checkbox");
        if (checkboxs) {
          for (const checkbox of checkboxs) {
            this.duplicateInput.push(checkbox.name);
          }
        }
      }
    }
    await delay(10);
    this.inputs = rootInputs;
  }
  getItemValue(item) {
    const tagName = item.tagName.toLowerCase().replace('bcm-', '');
    switch (tagName) {
      /*   case 'date-picker':
        case 'datetime-picker':
            return item.value.timestamp
        case 'time-picker':
            return item.value.time */
      default:
        return item.value;
    }
  }
  isWrapperComponent(item) {
    return ['bcm-datetime-picker', 'bcm-date-picker', 'bcm-time-picker'].includes(item.toLowerCase());
  }
  isGroupComponent(item) {
    return ['bcm-checkbox-group'].includes(item.toLowerCase());
  }
  isListComponent(item) {
    return ['bcm-list'].includes(item.toLowerCase());
  }
  async userFormData() {
    this.formData = {};
    this.isValid = true;
    this.returnCaption = [];
    for (const item of this.inputs) {
      // const isListComponent = this.isListComponent(item.tagName);
      const isGroupComponent = this.isGroupComponent(item.tagName);
      const isWrapperComponent = this.isWrapperComponent(item.tagName);
      const wrapperComponent = isWrapperComponent && item;
      const input = isWrapperComponent
        ? await item.getInput() && item
        : item;
      // input.caption = ''
      // input.captionType = ''
      if (this.validation) {
        let valid = null;
        // if (isListComponent) {
        //     valid = !(input.value && input.value.length > 0) && validate.required(input.value)
        // } else
        if (isWrapperComponent) {
          const isPickerValid = await wrapperComponent.isValid();
          valid = validate.required(input.value, isPickerValid);
          if (valid) {
            if (input.value instanceof Object) {
              if (input.value["isValid"] == false) {
                valid = validate.required("", isPickerValid);
              }
            }
            else {
              valid = validate.required(input.value, isPickerValid);
            }
          }
        }
        else if (isGroupComponent) {
          const childsCheckbox = getChildElements(input, 'bcm-checkbox:not(#bcm-indeterminate-element)');
          if (childsCheckbox && childsCheckbox.length > 0) {
            for (const element of childsCheckbox) {
              if (element && element.required === true) {
                valid = validate.required(element.checked);
                if (!valid['valid']) {
                  element.noCaption = false;
                  element.captionError = valid['message'];
                  element.captionType = valid['type'];
                }
                else {
                  element.noCaption = input.noCaption;
                  element.captionError = null;
                  element.captionType = "default";
                }
              }
            }
          }
        }
        else {
          valid = validate.required(input.value);
        }
        if (input.required) {
          if (!valid['valid']) {
            input.captionError = valid['message'];
            input.captionType = valid['type'];
          }
          else {
            input.captionError = null;
            input.captionType = "default";
          }
          await delay(10);
          const selfValidation = validate.selfValidation(input);
          if (valid['valid']) {
            if (!selfValidation['valid']) {
              input.captionError = selfValidation['message'];
              input.captionType = selfValidation['type'];
            }
          }
        }
        let name = input.name;
        let value = input.captionError;
        !input.hidden && input.captionType == 'error' && (this.returnCaption = [...Array.from(this.returnCaption), { [name]: value }],
          this.isValid = false);
      }
      if (input.name && !this.duplicateInput.includes(input.name)) {
        var itemValue = this.getItemValue(input);
        if (!itemValue || itemValue == undefined || itemValue == null) {
          itemValue = null;
        }
        this.formData[input.name] = itemValue;
      }
    }
    return this.formData;
  }
  async handleChange(e) {
    const isWrapperComponent = this.isWrapperComponent(e.target.tagName);
    const isGroupComponent = this.isGroupComponent(e.target.tagName);
    let target;
    if (isWrapperComponent) {
      target = e.target.getInput && await e.target.getInput() || e.target;
    }
    else if (isGroupComponent) {
      let req = false;
      const childsCheckbox = getChildElements(e.target, 'bcm-checkbox:not(#bcm-indeterminate-element)');
      childsCheckbox.length > 0 && (childsCheckbox.forEach((element) => {
        if (element && element.required === true) {
          req = true;
        }
      }));
      target = Object.assign(Object.assign({}, e.target), { required: req });
    }
    else {
      target = e.target;
    }
    const value = e.target.value;
    const wrapperComponent = isWrapperComponent && e.target;
    var isPickerValid;
    if (isWrapperComponent) {
      isPickerValid = await wrapperComponent.isValid();
    }
    const tick = setTimeout(() => {
      if (target.required) {
        let valid = null;
        if (isWrapperComponent) {
          valid = validate.required(value, isPickerValid);
          if (valid) {
            if (value instanceof Object) {
              if (value["isValid"] == false) {
                valid = validate.required("", isPickerValid);
              }
            }
            else {
              valid = validate.required(value, isPickerValid);
            }
          }
        }
        else if (isGroupComponent) {
          const childsCheckbox = getChildElements(e.target, 'bcm-checkbox:not(#bcm-indeterminate-element)');
          childsCheckbox.length > 0 && (childsCheckbox.forEach((element) => {
            if (element && element.required === true) {
              valid = validate.required(element.checked);
              if (!valid['valid']) {
                element.noCaption = false;
                element.captionError = valid['message'];
                element.captionType = valid['type'];
              }
              else {
                element.noCaption = e.target.noCaption;
                element.captionError = null;
                element.captionType = "default";
              }
            }
          }));
        }
        else {
          valid = validate.required(value);
        }
        if (valid['valid']) {
          target.captionError = null;
          target.captionType = null;
        }
        else {
          target.captionError = valid['message'];
          target.captionType = valid['type'];
        }
      }
      clearTimeout(tick);
    });
    this.value = await this.userFormData();
  }
  async handleSubmitForm(e) {
    e.preventDefault();
  }
  render() {
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("form", { class: "bcm-form", autocomplete: this.autocomplete ? "on" : "off" }, h("slot", null))));
  }
  static get is() { return "bcm-form"; }
  static get originalStyleUrls() {
    return {
      "$": ["form.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["form.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "(Math.random() * 4).toString(16).replace('.', '')"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "validation": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "validation",
        "reflect": false,
        "defaultValue": "false"
      },
      "name": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "name",
        "reflect": false,
        "defaultValue": "\"\""
      },
      "value": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false
      },
      "autocomplete": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "autocomplete",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmForm",
        "name": "bcm-form",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "submit": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "set": {
        "complexType": {
          "signature": "(data: object) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "get": {
        "complexType": {
          "signature": "() => Promise<{}>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<{}>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setClear": {
        "complexType": {
          "signature": "() => Promise<boolean>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "resetCaption": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "config": {
        "complexType": {
          "signature": "(_config: Array<object>) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            },
            "Array": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleSubmit",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "bcm-change",
        "method": "handleChange",
        "target": undefined,
        "capture": false,
        "passive": false
      }, {
        "name": "submit",
        "method": "handleSubmitForm",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
__decorate([
  IsLoad()
], BcmForm.prototype, "el", void 0);
