/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { createStore } from "@stencil/store";
import snq from "snq";
import { Generate } from "../../../helper/generate";
const { state } = createStore({
  value: new Map(),
});
const setValue = val => {
  const { id, dataSource, options } = val;
  const { type, treeview, checkboxes, searchable, value, name, label, size, disabled, readonly, highlight, required, hidden, fullWidth, height, maxHeight, minHeight, width, infoFooter, variableText, template, allOpen, keyControl, checkAll, linkedComponent, placeholder, searchPlaceholder, searchIsOnlyChilds, clearable, caption, noCaption, captionType, captionError, placement, } = options;
  const itemOptions = { size, checkboxes, template, allOpen, highlight, treeview };
  const data = {
    id: snq(() => id, Generate.UID),
    dataSource: dataSource && dataSource.length > 0 && recursive(dataSource, itemOptions),
    options: {
      type,
      treeview,
      checkboxes,
      searchable,
      value,
      name,
      label,
      size,
      disabled,
      readonly,
      highlight,
      required,
      hidden,
      fullWidth,
      height,
      maxHeight,
      minHeight,
      width,
      infoFooter,
      variableText,
      template,
      allOpen,
      keyControl,
      checkAll,
      linkedComponent,
      placeholder,
      searchPlaceholder,
      searchIsOnlyChilds,
      clearable,
      caption,
      noCaption,
      captionType,
      captionError,
      placement,
    },
    totalItems: (dataSource === null || dataSource === void 0 ? void 0 : dataSource.length) > 0 && calculateTotalItems(dataSource, options.treeview),
    totalRoots: dataSource === null || dataSource === void 0 ? void 0 : dataSource.length,
  };
  state.value.set(data.id, data);
};
const recursive = (data, options, parentId, parentIndex = "0.") => {
  let recData = [];
  if (data && data.length > 0) {
    recData = data.map((item, index) => (Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({ itemObject: item }, item), options), { parentId: snq(() => parentId, null), open: options.allOpen || item.open || false }), ((options.treeview || (!options.treeview && item.type == 'group')) && item.items && item.items.length > 0
      ? { items: snq(() => recursive(item.items, options, item.id, `${parentIndex}${index}`), []) }
      : { items: [] })), { ct: `${parentIndex}${index}` })));
  }
  return recData;
};
const removeValue = _id => {
  state.value.delete(_id);
};
const getValue = id => snq(() => state.value.get(id), {});
const calculateTotalItems = (dataSource, treeview) => {
  let total = 0;
  let selectableItems = 0;
  dataSource.forEach(item => {
    if (treeview || (!treeview && item.type != "group")) {
      total += 1;
      if (!item.disabled) {
        selectableItems += 1;
      }
    }
    if ((treeview || (!treeview && item.type == "group")) && item.items) {
      const result = calculateTotalItems(item.items, treeview);
      total += result.total;
      selectableItems += result.selectableItems;
    }
  });
  return { total, selectableItems };
};
const getData = id => {
  let data = [];
  const value = getValue(id);
  if (value) {
    const { dataSource } = value;
    data = dataSource;
  }
  return data;
};
const getDataPagination = (id, limit = 3, ids) => {
  let data = [];
  const value = getValue(id);
  if (value) {
    const { dataSource } = value;
    data = !ids ? dataSource.slice(0, limit) : dataSource.filter(item => ids.indexOf(item.id) > -1).slice(0, limit);
  }
  return data;
};
const openGroup = (id, groupId, items, status) => {
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (status != undefined) {
        item.open = status;
      }
      else if (item.id == groupId) {
        item.open = !item.open;
        item.isSearching = item.open;
        if (!item.open && item.items) {
          openGroup(id, groupId, item.items, false);
        }
      }
      else if (item.items) {
        openGroup(id, groupId, item.items);
      }
    });
  }
};
const removeOtherTags = (id, ids, items) => {
  ids = Array.isArray(ids) ? ids : [ids];
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    ids.forEach(otherTagIds => {
      dataSource.forEach(child => {
        if (child.id == otherTagIds) {
          child.checked = false;
          child.itemObject.checked = false;
        }
        else if (child.items) {
          onRemoveItems(id, otherTagIds, child.items);
        }
      });
    });
  }
};
const onRemoveItems = (id, itemId, items) => {
  if (items) {
    items.forEach(item => {
      if (item.id == itemId) {
        item.checked = false;
        item.itemObject.checked = false;
        indeterminateStatus(id, item.parentId);
      }
      else if (item.items) {
        onRemoveItems(id, itemId, item.items);
      }
    });
  }
};
const addChecked = (id, ids) => {
  const value = getValue(id);
  if (value) {
    ids.forEach(item => {
      checkedItem(id, item);
    });
  }
};
const checkedItem = (id, itemId) => {
  findItem(id, itemId);
};
const findItem = (id, itemId, items) => {
  itemId = Array.isArray(itemId) ? itemId : [itemId];
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (itemId.includes(item.id)) {
        checkedProcess(id, item);
        if (item.parentId) {
          indeterminateStatus(id, item.parentId);
        }
      }
      else if (item.items) {
        findItem(id, itemId, item.items);
      }
    });
  }
};
const checkedProcess = (id, item, status) => {
  const value = getValue(id);
  if (value) {
    const treeview = value.options.treeview;
    if (item) {
      // && !item.hidden => select all bastığımızda hidden olanları seçmemesi içindi
      if (treeview || (!treeview && item.type != "group")) {
        if (!item.disabled) {
          if (status != undefined) {
            item.checked = status;
          }
          else {
            item.checked = !item.checked;
          }
          item.itemObject.checked = item.checked;
          item.indeterminate = item.checked ? "determinate" : "uncheck";
        }
      }
      if (item.items) {
        item.items.forEach(child => {
          checkedProcess(id, child, status == undefined ? item.checked : status);
        });
      }
    }
  }
};
const indeterminateStatus = (id, itemId, items) => {
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (item.id == itemId) {
        let allChecked = true;
        let allUnchecked = true;
        if (item.items) {
          item.items.map(child => {
            (child.checked || child.indeterminate === "indeterminate") && (allUnchecked = false);
            !child.checked && (allChecked = false);
          });
        }
        const indeterminateState = !allChecked && !allUnchecked ? "indeterminate" : allChecked ? "determinate" : "uncheck";
        item.indeterminate = indeterminateState;
        item.checked = indeterminateState == "determinate" ? true : false;
        item.itemObject.checked = item.checked;
        if (item.parentId) {
          indeterminateStatus(id, item.parentId);
        }
      }
      else if (item.items) {
        indeterminateStatus(id, itemId, item.items);
      }
    });
  }
};
const filterData = (id, ids, items, searchOtherTag) => {
  let checkeds = [];
  let visibled = [];
  getValue(id).isSearchOtherTag = searchOtherTag;
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (ids) {
        item.hidden = true;
        // const filter = ids.find((find) => find.id === item.id)
        var filter;
        for (let index = 0; index < ids.length; index++) {
          if (ids[index].id === item.id) {
            filter = ids[index];
            break;
          }
        }
        if (filter) {
          item.hidden = false;
          if (!getValue(id).isSearchOtherTag) {
            item.isSearching = true;
          }
          visibled.push(item);
          if (item.checked) {
            checkeds.push(item);
          }
          if (filter.items) {
            const { checkedItems, visibledItems } = filterData(id, filter.items, item.items, searchOtherTag);
            if (checkedItems) {
              checkeds = [...new Set([...checkeds, ...checkedItems])];
            }
            if (visibledItems) {
              visibled = [...new Set([...visibled, ...visibledItems])];
            }
          }
        }
      }
      else {
        item.hidden = false;
        if (item.items) {
          filterData(id, null, item.items);
        }
        visibled.push(item);
        item.isSearching = false;
      }
    });
  }
  return { checkedItems: checkeds !== null && checkeds !== void 0 ? checkeds : [], visibledItems: visibled !== null && visibled !== void 0 ? visibled : [] };
};
const getParents = (id, itemId, items) => {
  let checkeds = [];
  let visibled = [];
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (item.id == itemId) {
        if (!getValue(id).isSearchOtherTag) {
          item.isSearching = true;
        }
        item.hidden = false;
        visibled.push(item);
        if (item.checked) {
          checkeds.push(item);
        }
        if (item.parentId) {
          const { checkedItems, visibledItems } = getParents(id, item.parentId);
          if (checkedItems) {
            checkeds = [...new Set([...checkeds, ...checkedItems])];
          }
          if (visibledItems) {
            visibled = [...new Set([...visibled, ...visibledItems])];
          }
        }
      }
      if (item.items) {
        const { checkedItems, visibledItems } = getParents(id, itemId, item.items);
        if (checkedItems) {
          checkeds = [...new Set([...checkeds, ...checkedItems])];
        }
        if (visibledItems) {
          visibled = [...new Set([...visibled, ...visibledItems])];
        }
      }
    });
  }
  return { checkedItems: checkeds !== null && checkeds !== void 0 ? checkeds : [], visibledItems: visibled !== null && visibled !== void 0 ? visibled : [] };
};
const getSelected = (id, items) => {
  let dataSource;
  let selected = null;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (item.selected) {
        selected = item.itemObject;
      }
      else if (item.items) {
        const found = getSelected(id, item.items);
        if (found) {
          selected = found;
        }
      }
    });
  }
  return selected;
};
const getCheckedList = (id, items) => {
  let dataSource;
  let checkedList = [];
  let nestedResult = [];
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      var _a;
      const newItem = Object.assign(Object.assign({}, item.itemObject), { ct: item.ct });
      let foundChild = [];
      let foundItem = [];
      if (item.checked) {
        checkedList.push(newItem);
        foundItem.push(Object.assign(Object.assign({}, newItem), { items: [] }));
      }
      if (item.items) {
        const found = getCheckedList(id, item.items);
        if (((_a = found === null || found === void 0 ? void 0 : found.checkedList) === null || _a === void 0 ? void 0 : _a.length) > 0) {
          checkedList = [...checkedList, ...found.checkedList];
        }
        if (found.nestedResult.length > 0) {
          foundChild = found.nestedResult;
        }
      }
      if (foundChild.length > 0) {
        if (foundItem.length == 0) {
          foundItem.push(Object.assign(Object.assign({}, newItem), { items: [] }));
        }
        foundItem[0].items = foundChild;
        nestedResult.push(foundItem[0]);
      }
      else {
        if (foundItem.length > 0) {
          foundItem[0].items = foundChild;
          nestedResult.push(foundItem[0]);
        }
      }
    });
  }
  return { checkedList: (checkedList === null || checkedList === void 0 ? void 0 : checkedList.length) > 0 ? checkedList : null, nestedResult: nestedResult };
};
const selectedItem = (id, itemId, items) => {
  let dataSource;
  let selected = [];
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      if (itemId) {
        if (item.id == itemId) {
          // if (!item.selected){
          item.selected = true;
          item.itemObject.selected = item.selected;
          // }
        }
        else {
          if (item.selected) {
            item.selected = false;
            item.itemObject.selected = item.selected;
          }
        }
      }
      if (item.items) {
        const found = selectedItem(id, itemId, item.items);
        if ((found === null || found === void 0 ? void 0 : found.length) > 0) {
          selected = [...selected, ...found];
        }
      }
      if (item.selected) {
        selected.push(item.itemObject);
      }
    });
  }
  return selected && selected.length > 0 ? selected : null;
};
const setClear = (id, items) => {
  let dataSource;
  if (items) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  if (dataSource) {
    dataSource.forEach(item => {
      var _a, _b, _c;
      item.checked && (item.checked = false);
      item.selected && (item.selected = false);
      item.indeterminate && (item.indeterminate = "uncheck");
      ((_a = item.itemObject) === null || _a === void 0 ? void 0 : _a.checked) && (item.itemObject.checked = false);
      ((_b = item.itemObject) === null || _b === void 0 ? void 0 : _b.selected) && (item.itemObject.selected = false);
      ((_c = item.itemObject) === null || _c === void 0 ? void 0 : _c.indeterminate) && (item.itemObject.indeterminate = "uncheck");
      if (item.items) {
        setClear(id, item.items);
      }
    });
  }
};
const selectAll = (id, status) => {
  var _a, _b;
  if (status == undefined) {
    const totalItems = getValue(id).totalItems;
    const checkedItems = ((_b = (_a = getCheckedList(id)) === null || _a === void 0 ? void 0 : _a.checkedList) === null || _b === void 0 ? void 0 : _b.length) || 0;
    status = totalItems.selectableItems !== checkedItems;
  }
  const dataSource = getValue(id).dataSource;
  if (dataSource) {
    dataSource.forEach(item => {
      checkedProcess(id, item, status);
    });
  }
};
const updateOptions = (id, options) => {
  Object.keys(getValue(id).options).forEach(key => {
    getValue(id).options[key] = options[key];
  });
};
const removeState = id => {
  return Promise.resolve(state.value.delete(id));
};
export default {
  setValue,
  removeValue,
  getValue,
  getData,
  getDataPagination,
  openGroup,
  checkedItem,
  filterData,
  getSelected,
  getCheckedList,
  selectedItem,
  addChecked,
  setClear,
  selectAll,
  removeOtherTags,
  onRemoveItems,
  updateOptions,
  removeState,
};
