/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { FloatingUI, delay } from "@bcm/utils";
import { Host, h } from "@stencil/core";
import { attendanceActionCommentStyle } from "./attendance-actions-comment.style";
export class BcmAttendanceActionsComment {
  constructor() {
    this.lateMinutes = null;
    this.comment = null;
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.disabled = false;
    this.selectedKey = null;
    this.targetId = null;
    this.isShow = false;
    this.isLate = false;
    this.selectedItem = null;
  }
  handleSave(lateMinutes, comment) {
    this.change.emit({ lateMinutes, comment });
    this.hide();
  }
  async show() {
    const targetElement = document.getElementById(this.targetId);
    FloatingUI({
      ref: targetElement,
      floating: this.el,
      placement: "bottom-end",
      offset: 8,
      arrow: true,
      arrowColor: "tw-bg-white",
    });
    this.isShow = true;
    await delay(100);
    if (this.isLate) {
      this.focusInput();
    }
    else {
      this.focusTextarea();
    }
  }
  focusInput() {
    const input = this.el.querySelector("bcm-input");
    input === null || input === void 0 ? void 0 : input.onBcmSelect();
  }
  focusTextarea() {
    const textarea = this.el.querySelector("bcm-textarea");
    textarea === null || textarea === void 0 ? void 0 : textarea.onBcmSelect();
  }
  async hide() {
    if (this.isShow) {
      this.el.remove();
    }
  }
  handleClickOutside(event) {
    if (!Generate.findEventPath(event, this.el)) {
      this.hide();
    }
  }
  async formSubmit() {
    var _a, _b, _c;
    const { lateMinutes, comment, isLate } = this;
    if (isLate) {
      if (lateMinutes) {
        if (!(lateMinutes >= 0 && lateMinutes <= 200)) {
          return;
        }
      }
    }
    const { isValid } = await ((_a = this.el.querySelector("bcm-form")) === null || _a === void 0 ? void 0 : _a.submit());
    if (!isValid)
      return;
    this.handleSave(lateMinutes || ((_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.lateMin), comment || ((_c = this.selectedItem) === null || _c === void 0 ? void 0 : _c.comment));
  }
  render() {
    var _a, _b;
    const { size } = this;
    const { host, footer } = attendanceActionCommentStyle({
      size,
    });
    return (h(Host, { class: host(), onKeyDown: e => {
        if (e.key === "Escape")
          this.hide();
      } }, h("bcm-form", { validation: true, style: { width: "256px" } }, this.isLate ? (h("bcm-input", { label: "Minus Late", required: true, name: "lateMin", id: this._id + "-input-dropdown-late-content-input", type: "number", size: this.size, autocomplete: false, "no-margin": true, min: 0, max: 200, value: (_a = this.selectedItem) === null || _a === void 0 ? void 0 : _a.lateMin, placeholder: "Enter Minus Late", "onBcm-change": ({ detail }) => (this.lateMinutes = detail), onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }, h("i", { slot: "prefix", class: "far fa-clock" }))) : null, h("bcm-textarea", { name: "comment", value: (_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.comment, size: this.size, "onBcm-change": ({ detail }) => (this.comment = detail), label: "Description", "no-caption": true, rows: 4, onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }), h("footer", { class: footer() }, h("bcm-button", { class: "tw-order-2", icon: "fas fa-save", "icon-position": "prefix", size: this.size, onClick: () => this.formSubmit() }, "Save"), h("bcm-button", { class: "tw-order-1", icon: "fas fa-times", kind: "ghost", size: this.size, "icon-position": "prefix", onClick: () => this.hide() }, "Close")))));
  }
  static get is() { return "bcm-attendance-actions-comment"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.medium"
      },
      "disabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "selectedKey": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "selected-key",
        "reflect": false,
        "defaultValue": "null"
      },
      "targetId": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target-id",
        "reflect": false,
        "defaultValue": "null"
      },
      "isShow": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-show",
        "reflect": false,
        "defaultValue": "false"
      },
      "isLate": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "is-late",
        "reflect": false,
        "defaultValue": "false"
      },
      "selectedItem": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "Bcm.InputDropdownData",
          "resolved": "InputDropdownData",
          "references": {
            "Bcm": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "null"
      }
    };
  }
  static get events() {
    return [{
        "method": "change",
        "name": "bcm-attendance-actions-comment-change",
        "bubbles": false,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleClickOutside",
        "target": "window",
        "capture": false,
        "passive": false
      }];
  }
}
