/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Generate } from "@bcm/helpers";
import { Bcm } from "@bcm/model";
import { Host, h } from "@stencil/core";
import cs from "classnames";
const osKeys = {
  ctrl: "⌘",
  alt: "⌥",
  shift: "⇧",
  del: "⌫",
  delete: "⌫",
  backspace: "⌦",
  enter: "⏎",
  return: "⏎",
};
export class BcmShortcut {
  constructor() {
    this.isMac = /Mac/.test(window.navigator.userAgent);
    this.hotkey = undefined;
    this.color = "slate";
    this.size = Bcm.Size.small;
  }
  getHotkey(key) {
    if (!key) {
      return "";
    }
    if (this.isMac && (key.includes(" ") || key.includes("+"))) {
      const keys = key.split(/[\s+]+/);
      return keys.map(k => osKeys[k] || k).join("");
    }
    else {
      return key;
    }
  }
  render() {
    const { size, color, hotkey } = this;
    return (h(Host, { class: cs("bcm-ui-element bcm-ui-content-display", "tw-block tw-h-fit tw-w-fit tw-rounded tw-border tw-border-solid tw-px-1", "tw-border-default-300 tw-bg-default-100 tw-text-default-700", "tw-text-center tw-font-medium tw-capitalize", {
        "tw-text-2": size === Bcm.Size.small,
        "tw-text-3": size === Bcm.Size.medium,
        "tw-text-4": size === Bcm.Size.large,
      }), style: Object.assign({}, Generate.getTwColor({ color })) }, this.getHotkey(hotkey)));
  }
  static get is() { return "bcm-shortcut"; }
  static get properties() {
    return {
      "hotkey": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hotkey",
        "reflect": false
      },
      "color": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "color",
        "reflect": false,
        "defaultValue": "\"slate\""
      },
      "size": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "SizeTypes",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeTypes": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "Bcm.Size.small"
      }
    };
  }
}
