/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Host, h } from "@stencil/core";
import cs from "classnames";
import { Generate } from "../../../helper/generate";
import { StringHelper } from "../../../helper/string-helper";
import Bcm from "../../../models/bcm";
import { delay } from "../../../utils/utils";
export class BcmToast {
  constructor() {
    this._id = Generate.UID();
    this.status = undefined;
    this.message = "";
    this.description = "";
    this.duration = 3000;
    this.position = Bcm.Placement.bottom;
    this.messageIcon = undefined;
    this.viewButton = false;
    this.closeButton = true;
    this.removeAfterClose = true;
    this.visible = false;
    this.visibleContainer = false;
  }
  async show() {
    const toast = document.getElementById(this._id + "-container");
    if (toast) {
      const container = this.createContainer(this.position);
      container.appendChild(toast);
    }
    this.visible = true;
    await delay(50);
    this.visibleContainer = true;
    this.bcmToastOpen.emit();
    if (this.timer) {
      clearTimeout(this.timer);
    }
    this.timer = setTimeout(() => this.hide(), this.duration);
    return this.el;
  }
  async hide() {
    this.visibleContainer = false;
    clearTimeout(this.timer);
    this.bcmToastClose.emit();
    await delay(100);
    this.visible = false;
    if (this.removeAfterClose) {
      await delay(100);
      this.el.remove();
    }
  }
  handleMouseEnter() {
    clearTimeout(this.timer);
  }
  handleMouseLeave() {
    if (this.visible) {
      this.timer = setTimeout(() => this.hide(), 1000);
    }
  }
  createContainer(position) {
    const containerId = "bcm-toast-container-" + position;
    let container = document.getElementById(containerId);
    if (!container) {
      container = document.createElement("div");
      container.id = containerId;
      const classlist = cs("tw-pointer-events-none tw-overflow-hidden tw-z-50", "tw-fixed tw-flex", "tw-p-4 tw-gap-4", {
        "tw-flex-col-reverse tw-inset-x-0 tw-top-4 tw-mx-auto": this.position === Bcm.Placement.top,
        "tw-flex-col tw-inset-x-0 tw-bottom-4 tw-mx-auto": this.position === Bcm.Placement.bottom,
        "tw-flex-col-reverse tw-top-4 tw-left-4": this.position === Bcm.Placement["top-start"],
        "tw-flex-col-reverse tw-top-4 tw-right-4": this.position === Bcm.Placement["top-end"],
        "tw-flex-col tw-bottom-4 tw-left-4": this.position === Bcm.Placement["bottom-start"],
        "tw-flex-col tw-bottom-4 tw-right-4": this.position === Bcm.Placement["bottom-end"],
      });
      container.classList.add(...classlist.split(" "));
      document.body.appendChild(container);
    }
    return container;
  }
  render() {
    return (h(Host, { class: "tw-hidden" }, h("div", { class: cs({ "tw-hidden": !this.visible }), id: this._id + "-container", onMouseEnter: () => this.handleMouseEnter(), onMouseLeave: () => this.handleMouseLeave() }, h("div", { class: cs("tw-pointer-events-auto tw-transform tw-transition tw-duration-300 tw-ease-in-out", "tw-flex tw-min-w-[256px] tw-max-w-[420px] tw-flex-col tw-gap-2 tw-rounded-lg tw-border tw-font-sans tw-font-normal tw-shadow-2", "tw-border-gray-300 tw-bg-white tw-text-slate-600", {
        "tw-p-4": !this.description,
        "tw-p-6": this.description,
      }, "tw-transform tw-transition tw-duration-300 tw-ease-in-out ", {
        "tw-translate-y-0 tw-opacity-100 ": this.visibleContainer,
        "tw-opacity-0": !this.visibleContainer,
        "tw-translate-y-[-100%]": !this.visibleContainer && this.position.split("-")[0] === "top",
        "tw-translate-y-[100%]": !this.visibleContainer && this.position.split("-")[0] === "bottom",
      }) }, h("div", { class: "tw-flex tw-items-center tw-justify-between tw-gap-5" }, h("div", { class: "tw-flex tw-items-center tw-gap-3 " }, this.status && (h("i", { class: cs("tw-text-6", this.messageIcon ? this.messageIcon : "far", {
        "tw-text-emerald-500": this.status === "success",
        "tw-text-red-500": this.status === "error",
        "tw-text-amber-500": this.status === "warning",
        "tw-text-blue-500": this.status === "info",
      }, {
        "fa-check-circle": !this.messageIcon && this.status === "success",
        "fa-exclamation-circle": !this.messageIcon && this.status === "error",
        "fa-exclamation-triangle": !this.messageIcon && this.status === "warning",
        "fa-info-circle": !this.messageIcon && this.status === "info",
      }) })), h("span", { class: cs({
        "tw-text-4": !this.description,
        "tw-text-6 tw-font-medium": this.description,
      }) }, !this.description && this.status && h("span", null, StringHelper.capitalize(this.status), ": "), this.message)), h("div", { class: "tw-flex tw-items-baseline tw-gap-2" }, this.viewButton && (h("button", { onClick: () => this.bcmToastView.emit(), title: "View details", class: cs("tw-cursor-pointer tw-rounded tw-bg-transparent tw-p-0 tw-px-1 tw-text-4 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200", "tw-transition tw-duration-300 tw-ease-in-out", {
        "tw-font-medium": this.description,
      }) }, "View")), this.closeButton && (h("button", { title: "Close", onClick: () => this.hide(), class: cs("tw-rounded tw-bg-transparent tw-p-0", "tw-flex tw-h-6 tw-w-6 tw-cursor-pointer tw-items-center tw-justify-center tw-text-3 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200") }, h("i", { class: "fal fa-times" }))))), this.description && h("div", { class: "tw-text-3 " }, this.description), h("slot", null)))));
  }
  static get is() { return "bcm-toast"; }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "status": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "\"success\" | \"error\" | \"warning\" | \"info\"",
          "resolved": "\"error\" | \"info\" | \"success\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false
      },
      "message": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false,
        "defaultValue": "\"\""
      },
      "description": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "description",
        "reflect": false,
        "defaultValue": "\"\""
      },
      "duration": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "duration",
        "reflect": false,
        "defaultValue": "3000"
      },
      "position": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "position",
        "reflect": false,
        "defaultValue": "Bcm.Placement.bottom"
      },
      "messageIcon": {
        "type": "any",
        "mutable": false,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message-icon",
        "reflect": false
      },
      "viewButton": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "view-button",
        "reflect": false,
        "defaultValue": "false"
      },
      "closeButton": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "close-button",
        "reflect": false,
        "defaultValue": "true"
      },
      "removeAfterClose": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "remove-after-close",
        "reflect": false,
        "defaultValue": "true"
      }
    };
  }
  static get states() {
    return {
      "visible": {},
      "visibleContainer": {}
    };
  }
  static get events() {
    return [{
        "method": "bcmToastClose",
        "name": "bcm-toast-close",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmToastOpen",
        "name": "bcm-toast-open",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmToastView",
        "name": "bcm-toast-view",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<HTMLElement>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            },
            "HTMLElement": {
              "location": "global"
            }
          },
          "return": "Promise<HTMLElement>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
