/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Host, h } from "@stencil/core";
import cs from "classnames";
import { trim } from "lodash";
import { Generate } from "../../../helper/generate";
export class BcmCollapse {
  constructor() {
    this._id = Generate.UID();
    this.borderless = false;
    this.open = false;
    this.noBorder = false;
    this.noPadding = false;
    this.hidden = false;
  }
  componentDidRender() {
    this.open && this.bcmCollapseOpen.emit(this._id);
  }
  componentDidLoad() {
    this.checkSlotEmpty();
  }
  checkSlotEmpty() {
    var _a;
    const textContent = (_a = this.el.querySelector('[slot="body"]')) === null || _a === void 0 ? void 0 : _a.textContent;
    if (trim(textContent) !== "") {
      this.el.removeAttribute("data-slot-empty");
    }
    else {
      this.el.setAttribute("data-slot-empty", "true");
    }
  }
  render() {
    const { hidden, borderless, open, noBorder, _id, noPadding } = this;
    const classes = cs("bcm-collapse", {
      "bcm-collapse--borderless": borderless,
      "bcm-collapse--open": open,
      "bcm-collapse--no-border": noBorder,
      "bcm-collapse--no-padding": noPadding,
    });
    const caretDown = '<svg style="fill: #595959; width: 14px; height: 14px" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M31.3 192h257.3c17.8 0 26.7 21.5 14.1 34.1L174.1 354.8c-7.8 7.8-20.5 7.8-28.3 0L17.2 226.1C4.6 213.5 13.5 192 31.3 192z"/></svg>';
    const caretRight = '<svg style="fill: #595959; width: 14px; height: 14px" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 192 512"><path d="M0 384.662V127.338c0-17.818 21.543-26.741 34.142-14.142l128.662 128.662c7.81 7.81 7.81 20.474 0 28.284L34.142 398.804C21.543 411.404 0 402.48 0 384.662z"/></svg>';
    return (h(Host, { class: { hidden }, id: _id }, h("details", { class: classes, open: open }, h("summary", { class: "bcm-collapse__header size-2", onClick: () => (this.open = !open) }, h("span", { class: "collapse-icon", innerHTML: open ? caretDown : caretRight }), h("slot", { name: "title" })), h("div", { class: "bcm-collapse__body" }, h("slot", { name: "body", onSlotchange: () => this.checkSlotEmpty() })))));
  }
  static get is() { return "bcm-collapse"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["collapse.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["collapse.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "borderless": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "borderless",
        "reflect": false,
        "defaultValue": "false"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "noBorder": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-border",
        "reflect": false,
        "defaultValue": "false"
      },
      "noPadding": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-padding",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "bcmCollapseOpen",
        "name": "bcm-collapse-open",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
}
