/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host, getAssetPath } from '@stencil/core';
import { ElementDragger, DragDirections } from '../../../utils/element-dragger';
import cs from 'classnames';
export class BcmColorPicker {
  constructor() {
    this.initialized = false;
    this.setColorBeforeInitialized = false;
    this._color = '#4293CF' // default color
    ;
    this.open = false;
    this.value = null;
    this.label = undefined;
    this.size = "medium";
    this.noCaption = false;
    this.caption = undefined;
    this.captionError = undefined;
    this.captionType = 'default';
    this.fullWidth = false;
    this.disabled = false;
    this.hidden = false;
    this.required = false;
    this.tooltip = undefined;
    this.h = -1;
    this.s = 1;
    this.v = 1;
    this.a = 1;
    this.selectedH = -1;
    this.selectedS = 1;
    this.selectedV = 1;
    this.selectedA = 1;
    this.color = { rgba: '', hex: '' };
    this.inputValue = '';
    this.mouseState = 'up';
  }
  /**
   * @ComponentMethod
   */
  componentWillLoad() {
    const initialColor = this.hextohsv(this._color);
    this.s = initialColor.s;
    this.v = initialColor.v;
    this.h = initialColor.h;
  }
  /**
   * @ComponentMethod
   */
  componentDidRender() {
    if (this.initialized)
      return;
    Promise.all([
      this.initColorDragger(),
      this.initColorsDragger(),
      this.initTransparencyDragger()
    ]).then(() => {
      !this.setColorBeforeInitialized && this._setColor(this._color);
      this.updateValue();
      this.initialized = true;
    });
  }
  /**
   * @desc
   */
  initColorDragger() {
    if (this.colorDragger)
      return;
    const limit = {
      x: {
        min: 0,
        max: this.colorElement.offsetWidth
      },
      y: {
        min: 0,
        max: this.colorElement.offsetHeight
      }
    };
    this.colorDragger = new ElementDragger({
      target: this.colorIndicatorElement,
      trigger: this.colorElement,
      direction: DragDirections.both,
      limit,
      snapOnMouseDown: true,
      center: {
        x: this.colorIndicatorElement.offsetWidth / 2,
        y: this.colorIndicatorElement.offsetHeight / 2
      }
    });
    /**
     * [ ElementDragger Event: drag ]
     */
    this.colorDragger.on('drag', (pos) => this.colorDrag(pos));
    return Promise.resolve();
  }
  /**
   * @desc
   */
  initColorsDragger() {
    if (this.colorsDragger)
      return;
    const limit = {
      x: {
        min: 0,
        max: this.colorsElement.offsetWidth
      }
    };
    this.colorsDragger = new ElementDragger({
      target: this.colorsIndicatorElement,
      trigger: this.colorsElement,
      direction: DragDirections.x,
      limit,
      snapOnMouseDown: true,
      center: {
        x: this.colorsIndicatorElement.offsetWidth / 2
      }
    });
    /**
     * [ ElementDragger Event: drag ]
     */
    this.colorsDragger.on('drag', (pos) => this.colorsDrag(pos));
    return Promise.resolve();
  }
  /**
   * @desc
   */
  initTransparencyDragger() {
    return;
    /* this.transparencyDragger = new ElementDragger({
        target: this.transparencyIndicatorElement,
        trigger: this.transparencyIndicatorElement,
        direction: DragDirections.x,
        limit: {
            x: {
                    min: 0,
                    max: this.transparencyElement.offsetWidth - this.transparencyIndicatorElement.offsetWidth
            }
        }
    })

    // Set initial position
    // #
    this.transparencyDragger.setPosition({
        x: this.transparencyElement.offsetWidth - indicatorSize
    })

    
    this.transparencyDragger.on('drag', (pos: any) => this.transparencyDrag(pos))
    this.setTransparencyElementColor() */
  }
  /**
   * @desc
   * @param hsv
   */
  hsvtorgb(hsv) {
    let R;
    let G;
    let B;
    let X;
    let C;
    let h = (hsv.h % 360) / 60;
    C = hsv.v * hsv.s;
    X = C * (1 - Math.abs(h % 2 - 1));
    R = G = B = hsv.v - C;
    h = ~~h;
    R += [C, X, 0, 0, X, C][h];
    G += [X, C, C, X, 0, 0][h];
    B += [0, 0, X, C, C, X][h];
    const rgb = {
      r: Math.round(R * 255),
      g: Math.round(G * 255),
      b: Math.round(B * 255)
    };
    return Object.assign(Object.assign({}, rgb), { hex: "#" + (16777216 | rgb.b | (rgb.g << 8) | (rgb.r << 16)).toString(16).slice(1) });
  }
  /**
   * @desc
   * @param rgb
   */
  rgbtohsv(rgb) {
    let H;
    let S;
    let V;
    let C;
    let r = rgb.r;
    let g = rgb.g;
    let b = rgb.b;
    if (rgb.r > 1 || rgb.g > 1 || rgb.b > 1) {
      r /= 255;
      g /= 255;
      b /= 255;
    }
    V = Math.max(r, g, b);
    C = V - Math.min(r, g, b);
    H = (C == 0 ? null :
      V == r ? (g - b) / C + (g < b ? 6 : 0) :
        V == g ? (b - r) / C + 2 :
          (r - g) / C + 4);
    H = (H % 6) * 60;
    S = C == 0 ? 0 : C / V;
    return {
      h: H,
      s: S,
      v: V
    };
  }
  /**
   * @desc
   * @param hex
   */
  hextohsv(hex) {
    return this.rgbtohsv(this.hextorgb(hex));
  }
  /**
   * @desc
   * @param hex
   */
  hextorgb(hex) {
    return {
      r: parseInt(hex.substr(1, 2), 16),
      g: parseInt(hex.substr(3, 2), 16),
      b: parseInt(hex.substr(5, 2), 16)
    };
  }
  /**
   * @desc
   * @param pos
   */
  colorDrag(pos) {
    const width = this.colorElement.offsetWidth;
    const height = this.colorElement.offsetHeight;
    this.selectedS = (pos.x) / width;
    this.selectedV = (height - (pos.y)) / height;
    this.updateValue();
  }
  /**
   * @desc
   * @param pos
   */
  colorsDrag(pos) {
    const width = this.colorsElement.offsetWidth;
    this.h = (pos.x) / width * 360;
    this.s = 1;
    this.v = 1;
    this.selectedH = this.h;
    this.updateValue();
  }
  /**
   * @desc
   * @param pos
   */
  /* transparencyDrag(pos: any) {
      const sliderWidth: number = this.transparencyElement.offsetWidth
      const actualX: number = pos.x + indicatorSize / 2
      const result: number = actualX / sliderWidth
      let opacity: string | number = result.toFixed(1)

      opacity === '1.0' && (opacity = '1')
      opacity === '0.0' && (opacity = '0')
      opacity = parseFloat(opacity)

      this.a = opacity
  } */
  /**
   * @desc
   */
  setHsvColor() {
    const rgb = this.hsvtorgb({
      h: this.selectedH,
      s: this.selectedS,
      v: this.selectedV
    });
    this.color = {
      rgba: `rgba(${rgb.r}, ${rgb.g}, ${rgb.b}, ${this.a})`,
      hex: rgb.hex
    };
    return this.color;
  }
  /**
   * @desc
   * @param hexColor
   */
  _setColor(color, preventInputUpdate = false) {
    const regHex = /^#+([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/;
    const regHexCheck = regHex.test(color);
    if (regHexCheck) {
      if (color.length === 4) {
        color = color.split('').map(c => c !== '#' ? c + c : c).join('');
      }
      const hsvColor = this.hextohsv(color);
      this.h = hsvColor.h;
      this.s = 1;
      this.v = 1;
      this.selectedH = hsvColor.h;
      this.selectedS = hsvColor.s;
      this.selectedV = hsvColor.v;
      this.updateIndicators();
      this.updateValue(preventInputUpdate);
    }
  }
  /**
   * @desc
   */
  updateIndicators() {
    const { colorsElement: colors, colorElement: color, colorsIndicatorElement: colorsIndicator } = this;
    this.colorDragger.setPosition({
      x: this.selectedS * color.offsetWidth,
      y: color.offsetHeight - this.selectedV * color.offsetHeight
    });
    this.colorsDragger.setPosition({
      x: (this.selectedH * (colors.offsetWidth - colorsIndicator.offsetWidth * 2)) / 360
    });
  }
  /**
   * @desc
   * @param preventInputUpdate
   */
  updateValue(preventInputUpdate = false) {
    const { selectedH: h, selectedS: s, selectedV: v } = this;
    const hex = this.hsvtorgb({ h, s, v }).hex;
    this.value = hex;
    if (!preventInputUpdate) {
      this.updateInputValue(hex);
    }
  }
  /**
   * @desc
   * @param value
   */
  updateInputValue(value) {
    if (!this.inputElement)
      return;
    this.inputElement.value = value;
  }
  /**
   * @desc
   */
  /* setTransparencyElementColor() {
      this.transparencyImageElement1.setAttribute('stop-color', this.color.hex)
      this.transparencyImageElement2.setAttribute('stop-color', this.color.hex)
  } */
  /**
   * @desc
   */
  handleClick() {
    this.open = true;
  }
  /**
   * @desc
   * @param event
   */
  handleColorInputKeyUp(event) {
    var path = event.path || (event.composedPath && event.composedPath());
    const val = path[0].value;
    this._setColor(val, true);
  }
  /**
   * @desc
   * @param event
   */
  handleWindowClick(event) {
    const target = event.target;
    const classes = target.classList;
    const tagName = target.tagName.toLowerCase();
    const isNotbyClass = !classes.contains('sc-bcm-colorpicker');
    const isNotbyTagName = tagName !== 'bcm-colorpicker';
    if ((isNotbyClass && isNotbyTagName && this.mouseState === 'up')) {
      this.open = false;
      this.updateInputValue(this.value);
    }
    this.mouseState = 'up';
  }
  /**
   * @desc
   */
  valueChange() {
    if (!this.initialized)
      return;
    this.change.emit(this.value);
  }
  async getValue() {
    return this.value;
  }
  async setColor(color) {
    this.setColorBeforeInitialized = true;
    this._setColor(color);
  }
  render() {
    const { label, required, noCaption, caption, captionError, captionType } = this;
    const classes = cs('color-picker', {
      open: this.open
    });
    const captionClasses = cs('size-1', 'input-caption', 'caption-' + captionType);
    return (h(Host, null, h("div", { class: classes, onClick: () => this.handleClick() }, label && h("div", null, h("bcm-label", { tooltip: this.tooltip, type: captionType, value: label, required: required })), h("div", { class: "input" }, h("div", { class: "active-color" }, h("div", { class: "color-thumb", style: { backgroundColor: this.value } })), h("span", { class: "icon", slot: "suffix" }, h("svg", { style: { 'fill': '#595959', 'width': ' 14px', 'height': '14px' }, xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 512 512" }, h("path", { d: "M483.89 28.14l-.02-.02-.03-.03c-37.47-37.47-98.26-37.46-135.72.03l-77.09 77.09-13.1-13.1c-9.44-9.44-24.65-9.31-33.94 0l-63.6 63.6c-9.37 9.37-9.37 24.57 0 33.94l16.98 16.98L50.75 333.25c-12 12-18.75 28.28-18.75 45.26V424L0 480l32 32 56-32h45.49c16.97 0 33.25-6.74 45.25-18.74l126.64-126.62 16.96 16.96c9.44 9.44 24.65 9.31 33.94 0l63.6-63.6c9.37-9.37 9.37-24.57 0-33.94l-13.1-13.1 77.09-77.09c37.5-37.47 37.5-98.25.02-135.73zM144.8 427.32a15.892 15.892 0 0 1-11.31 4.68H80v-53.49c0-4.27 1.66-8.29 4.69-11.31l126.63-126.62 60.12 60.12L144.8 427.32zm305.14-297.38l-77.09 77.09-33.94 33.94 30.07 30.06-29.66 29.66-128-128 29.66-29.65 30.06 30.07L382.08 62.05c9.05-9.06 21.1-14.05 33.91-14.05 12.82 0 24.86 4.98 33.91 14.04l.04.04C459.01 71.14 464 83.19 464 96.01c0 12.81-5 24.86-14.06 33.93z" })))), !noCaption && (h("div", { class: "caption-area" }, captionType == "error"
      ? h("span", { class: captionClasses }, " ", captionError, " ")
      : caption && h("span", { class: captionClasses }, " ", caption, " "))), h("div", { class: "picker" }, h("div", { class: "input" }, h("input", { type: "text", class: "color-input", maxlength: "7", ref: el => (this.inputElement = el), onKeyUp: (e) => this.handleColorInputKeyUp(e) })), h("div", { class: "color", onMouseDown: () => this.mouseState = 'down', style: { backgroundColor: this.hsvtorgb({ h: this.h, s: this.s, v: this.v }).hex }, ref: el => (this.colorElement = el) }, h("img", { src: getAssetPath('./static/color.svg') }), h("div", { class: "color-indicator", ref: el => (this.colorIndicatorElement = el) })), h("div", { class: "colors-transparency" }, h("div", { class: "colors", onMouseDown: () => this.mouseState = 'down', ref: el => (this.colorsElement = el) }, h("div", { class: "colors-holder" }, h("img", { src: getAssetPath('./static/colors.svg') })), h("div", { class: "colors-indicator", ref: el => (this.colorsIndicatorElement = el) })))))));
  }
  static get is() { return "bcm-colorpicker"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["colorpicker.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["colorpicker.css"]
    };
  }
  static get assetsDirs() { return ["static"]; }
  static get properties() {
    return {
      "_color": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "color",
        "reflect": false,
        "defaultValue": "'#4293CF'"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "value": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "value",
        "reflect": false,
        "defaultValue": "null"
      },
      "label": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "label",
        "reflect": false
      },
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'small' | 'medium' | 'large'",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "\"medium\""
      },
      "noCaption": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "no-caption",
        "reflect": false,
        "defaultValue": "false"
      },
      "caption": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption",
        "reflect": false
      },
      "captionError": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-error",
        "reflect": false
      },
      "captionType": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'primary' | 'success' | 'warning' | 'error' | 'default'",
          "resolved": "\"default\" | \"error\" | \"primary\" | \"success\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "caption-type",
        "reflect": false,
        "defaultValue": "'default'"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "required": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "required",
        "reflect": false,
        "defaultValue": "false"
      },
      "tooltip": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "tooltip",
        "reflect": false
      }
    };
  }
  static get states() {
    return {
      "h": {},
      "s": {},
      "v": {},
      "a": {},
      "selectedH": {},
      "selectedS": {},
      "selectedV": {},
      "selectedA": {},
      "color": {},
      "inputValue": {},
      "mouseState": {}
    };
  }
  static get events() {
    return [{
        "method": "change",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Component Events"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "getValue": {
        "complexType": {
          "signature": "() => Promise<string>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<string>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "setColor": {
        "complexType": {
          "signature": "(color: string) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "value",
        "methodName": "valueChange"
      }];
  }
  static get listeners() {
    return [{
        "name": "click",
        "method": "handleWindowClick",
        "target": "window",
        "capture": false,
        "passive": false
      }];
  }
}
