/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { FloatingUI } from '@bcm/utils';
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import { Generate } from '../../../helper/generate';
import { NumberHelper } from '../../../helper/number-helper';
import { StringHelper } from '../../../helper/string-helper';
import { getPlacement, setPosition } from '../../../utils/popover-placement';
export class OldBcmPopoverBox {
  constructor() {
    this._id = Generate.UID();
    this._target_id = undefined;
    this.message = undefined;
    this._title = undefined;
    this.placement = 'bottom';
    this.open = false;
    this.hidden = false;
    this.htmlContent = undefined;
    this.linkedComponent = undefined;
    this.height = '256px';
    this.maxHeight = '500px';
    this.minHeight = '150px';
    this.placement_string = undefined;
  }
  async show() {
    this.open = true;
    this.calculateLocation();
  }
  async hide() {
    this.open = false;
  }
  // @Listen('click', { target: 'window' })
  // handleOnClick(event) {
  //   if (!this.linkedComponent) return;
  //   const linkedId = document.getElementById(this.linkedComponent);
  //   const isClickTarget = linkedId && Generate.findEventPath(event, linkedId);
  //   if (isClickTarget) {
  //     this.open = !this.open;
  //   } else {
  //     !Generate.findEventPath(event, this.el) && this.hide();
  //   }
  // }
  getParentPopoverBox(parent) {
    if (parent) {
      const parentPopoverBox = parent.closest('old-bcm-popover-box');
      if (parentPopoverBox) {
        return parentPopoverBox;
      }
    }
    return null;
  }
  componentDidRender() {
    this.calculateLocation();
  }
  handleWheel() {
    this.calculateLocation();
  }
  handleScroll() {
    this.calculateLocation();
  }
  calculateLocation() {
    if (this.open) {
      var placement = this.placement;
      const box = this.el.querySelector('.bcm-pop-box');
      const linked = document.getElementById(this.linkedComponent);
      const targetElement = linked ? linked : document.getElementById(this._target_id);
      if (box && targetElement) {
        FloatingUI({
          ref: targetElement,
          floating: box,
          placement: this.placement,
          offset: 2,
          arrow: false,
          arrowColor: ``,
        });
        getPlacement(box, targetElement, placement, (place) => (placement = place));
        let rect = targetElement.getBoundingClientRect();
        let rectTop = rect.top;
        let pos = setPosition(placement, targetElement, { rectTop });
        if (this.height.indexOf('%') > -1) {
          box.style.height = NumberHelper.toNumber(this.height) * 0.01 * window.innerHeight - pos[0] - 50 + 'px';
        }
        else {
          box.style.height = this.height;
        }
        box.style.minHeight = this.minHeight;
        // const calculatedMaxHeight = window.innerHeight - pos[0] - 50;
        const calculatedMaxHeight = placement.split('-')[0] == 'top' ? window.innerHeight - (window.innerHeight - rectTop) - 50 : window.innerHeight - rectTop - 50;
        if (this.maxHeight.indexOf('%') > -1) {
          box.style.maxHeight = NumberHelper.toNumber(this.maxHeight) * 0.01 * calculatedMaxHeight + 'px';
        }
        else {
          box.style.maxHeight = this.maxHeight != 'auto' && NumberHelper.toNumber(this.maxHeight) < calculatedMaxHeight ? this.maxHeight : calculatedMaxHeight + 'px';
        }
        box.style.overflowY = 'auto';
        // getPlacement(box, targetElement, placement, (place: PlacementTypes) => placement = place)
        // rect = targetElement.getBoundingClientRect()
        // rectTop = rect.top;
        // pos = setPosition(placement, targetElement, { rectTop })
        // this.setPlacementString(placement)
        // this.el.style.top = pos[0] + "px"
        // this.el.style.left = pos[1] + "px"
        // this.el.style.zIndex = "10700";
        box.style.minWidth = targetElement.offsetWidth + 'px';
        const parentPopoverBox = this.getParentPopoverBox(targetElement);
        if (parentPopoverBox) {
          const zindex = parentPopoverBox.style.zIndex ? NumberHelper.toNumber(parentPopoverBox.style.zIndex) : 10900;
          this.el.style.zIndex = (+zindex + 1).toString();
        }
        if (this.message || this._title) {
          box.style.minHeight = '0px';
          box.style.height = 'auto';
          box.style.minWidth = '230px';
        }
      }
    }
  }
  setPlacementString(placement) {
    const placements = StringHelper.textContains(placement, '-') ? placement.split('-') : placement.split(' ');
    if (placements.length === 1) {
      placements.push('center');
    }
    this.placement_string = placements.join(' ');
  }
  render() {
    const { message, _title, open, placement_string } = this;
    const classes = cs('bcm-popover-box', placement_string, {
      'bcm-pop-open': open,
      'bcm-pop-linked': this.linkedComponent,
    });
    const boxClasses = cs('bcm-pop-box');
    return (h(Host, { class: classes }, h("div", { class: boxClasses }, _title && h("span", { class: "bcm-pop-heading size-2 weight-medium", innerHTML: _title }), h("span", { class: "bcm-pop-content size-2", innerHTML: message }), h("slot", null))));
  }
  static get is() { return "old-bcm-popover-box"; }
  static get originalStyleUrls() {
    return {
      "$": ["old-popover-box.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["old-popover-box.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "_target_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "target-id",
        "reflect": false
      },
      "message": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false
      },
      "_title": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_title",
        "reflect": false
      },
      "placement": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "'bottom'"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "htmlContent": {
        "type": "any",
        "mutable": true,
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "html-content",
        "reflect": false
      },
      "linkedComponent": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "linked-component",
        "reflect": false
      },
      "height": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "height",
        "reflect": false,
        "defaultValue": "'256px'"
      },
      "maxHeight": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "max-height",
        "reflect": false,
        "defaultValue": "'500px'"
      },
      "minHeight": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "min-height",
        "reflect": false,
        "defaultValue": "'150px'"
      }
    };
  }
  static get states() {
    return {
      "placement_string": {}
    };
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get listeners() {
    return [{
        "name": "wheel",
        "method": "handleWheel",
        "target": "document",
        "capture": false,
        "passive": false
      }, {
        "name": "scroll",
        "method": "handleScroll",
        "target": "window",
        "capture": false,
        "passive": false
      }, {
        "name": "resize",
        "method": "calculateLocation",
        "target": "window",
        "capture": false,
        "passive": true
      }];
  }
}
