/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import cs from 'classnames';
import Bcm from '../../../models/bcm';
import { Generate } from '../../../helper/generate';
export class OldBcmPopover {
  constructor() {
    this._title = undefined;
    this._id = Generate.UID();
    this.message = undefined;
    this.placement = Bcm.Placement.top;
    this.trigger = Bcm.TriggerType.click;
    this.open = false;
    this.linkedComponent = undefined;
  }
  async show() {
    if (this.open)
      return;
    this.open = true;
    this.onShow.emit();
    this.createBox();
    this.showBox();
  }
  async hide() {
    if (!this.open)
      return;
    this.open = false;
    this.onHide.emit();
    this.hideBox();
  }
  createBox() {
    if (!this.getPopoverBox()) {
      const box = document.createElement('old-bcm-popover-box');
      box.setAttribute('id', 'box-' + this._id);
      box.setAttribute('target-id', this._id);
      box.setAttribute('placement', this.placement);
      box.setAttribute('message', this.message);
      box.setAttribute('_title', this._title);
      box.setAttribute('linked-component', this.linkedComponent);
      document.body.appendChild(box);
    }
  }
  getPopoverBox() {
    return document.querySelector(`old-bcm-popover-box[id="box-${this._id}"]`);
  }
  showBox() {
    this.getPopoverBox() && this.getPopoverBox().show();
  }
  hideBox() {
    this.getPopoverBox() && this.getPopoverBox().hide();
  }
  listenTrigger(event) {
    switch (this.trigger) {
      case 'click':
        event === 'click' && (this.open ? this.hide() : this.show());
        break;
      case 'hover':
        event === 'onMouseOver' && this.show();
        event === 'onMouseOut' && this.hide();
        break;
      case 'focus':
        event === 'onFocus' && this.show();
        event === 'onBlur' && this.hide();
        break;
      default:
        break;
    }
  }
  render() {
    const popoverClasses = cs('bcm-popover');
    return (h(Host, { class: popoverClasses, onMouseOver: () => this.listenTrigger('onMouseOver'), onMouseOut: () => this.listenTrigger('onMouseOut'), onclick: () => this.listenTrigger('click'), onFocus: () => this.listenTrigger('onFocus'), onBlur: () => this.listenTrigger('onBlur'), tabindex: 0 }, h("slot", null)));
  }
  static get is() { return "old-bcm-popover"; }
  static get originalStyleUrls() {
    return {
      "$": ["old-popover.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["old-popover.css"]
    };
  }
  static get properties() {
    return {
      "_title": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_title",
        "reflect": true
      },
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "message": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "message",
        "reflect": false
      },
      "placement": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "PlacementTypes",
          "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
          "references": {
            "PlacementTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "placement",
        "reflect": false,
        "defaultValue": "Bcm.Placement.top"
      },
      "trigger": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TrigglerTypes",
          "resolved": "\"click\" | \"custom\" | \"focus\" | \"hover\"",
          "references": {
            "TrigglerTypes": {
              "location": "import",
              "path": "../../../models/bcm-types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "trigger",
        "reflect": false,
        "defaultValue": "Bcm.TriggerType.click"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "open",
        "reflect": false,
        "defaultValue": "false"
      },
      "linkedComponent": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "linked-component",
        "reflect": false
      }
    };
  }
  static get events() {
    return [{
        "method": "onShow",
        "name": "bcm-show",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "onHide",
        "name": "bcm-hide",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "show": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "hide": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
}
