/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, Host } from '@stencil/core';
import { TypeProps } from './types';
import cs from 'classnames';
export class BcmTab {
  constructor() {
    this.type = TypeProps.normal;
    this.active = false;
    this.disabled = false;
    this.closable = false;
    this.tabId = (Math.random() * 4).toString(16).replace('.', '');
    this.formControl = null;
    this.hidden = false;
    this.status = 'default';
    this.badge = false;
    this.blink = false;
  }
  /**
   * @desc
   */
  handleClose() {
    if (this.disabled)
      return;
    this.close.emit();
  }
  /**
   * @desc
   * @param event
   */
  handleClick(event) {
    if (this.disabled)
      return;
    const targetElement = event.target;
    // Detect close icon clicked
    // #
    if (String(targetElement.tagName).toLowerCase() == 'bcm-icon' && targetElement.classList.contains('close')) {
      this.close.emit(event);
    }
    else {
      this.click.emit(event);
    }
  }
  render() {
    const { active, disabled, closable } = this;
    const classes = cs('tab', [this.type], {
      'active': active,
      'disabled': disabled,
      'closable': closable
    });
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    const badgeClasses = cs('badge');
    return (h(Host, { class: hostClasses }, h("div", { class: classes, onClick: (event) => this.handleClick(event) }, h("slot", { name: "icon-prefix" }), h("slot", null), h("slot", { name: "icon-suffix" }), this.closable && (h("bcm-icon", { class: "close", icon: "far fa-times" })), this.badge && (h("bcm-badge", { blink: true, class: badgeClasses, size: "small", status: this.status })))));
  }
  static get is() { return "bcm-tab"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["tab.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["tab.css"]
    };
  }
  static get properties() {
    return {
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TypeProp",
          "resolved": "\"normal\" | \"pane\" | \"stepper\"",
          "references": {
            "TypeProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "TypeProps.normal"
      },
      "active": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "active",
        "reflect": false,
        "defaultValue": "false"
      },
      "disabled": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "disabled",
        "reflect": false,
        "defaultValue": "false"
      },
      "closable": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "closable",
        "reflect": false,
        "defaultValue": "false"
      },
      "tabId": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "tab-id",
        "reflect": false,
        "defaultValue": "(Math.random() * 4).toString(16).replace('.', '')"
      },
      "formControl": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "form-control",
        "reflect": false,
        "defaultValue": "null"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "status": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "'processing' | 'error' | 'warning' | 'success' | 'default'",
          "resolved": "\"default\" | \"error\" | \"processing\" | \"success\" | \"warning\"",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "status",
        "reflect": false,
        "defaultValue": "'default'"
      },
      "badge": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "badge",
        "reflect": false,
        "defaultValue": "false"
      },
      "blink": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "blink",
        "reflect": false,
        "defaultValue": "false"
      }
    };
  }
  static get events() {
    return [{
        "method": "click",
        "name": "bcm-click",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Component Events"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "close",
        "name": "bcm-close",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get elementRef() { return "el"; }
}
