/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, forceUpdate, Host } from '@stencil/core';
import { DirectionProps, SizeProps, TypeProps } from './types';
import { delay, getChilds } from '../../../utils/utils';
import cs from 'classnames';
export class BcmTabs {
  constructor() {
    /**
     * Component Private variables
     */
    this.tabs = [];
    this.firstInit = true;
    this.count = 0;
    this.size = SizeProps.large;
    this.type = TypeProps.normal;
    this.direction = DirectionProps.horizontal;
    this.hidden = false;
    this.fullWidth = false;
    this.underline = true;
    this.activeChild = 1;
    this.lineOpts = { top: '0px', left: '0px', right: '0px', width: '0px', height: '0px' };
    this.prevButton = null;
    this.nextButton = null;
    this.completeButton = null;
    this.prevButtonState = 'disabled';
    this.nextButtonState = 'disabled';
    this.completeButtonState = 'disabled';
  }
  async config(_config) {
    if (_config['prevButton']) {
      if (_config['prevButton']['id']) {
        var prevButtonId = "#" + _config['prevButton']['id'];
        this.prevButton = document.querySelector(prevButtonId);
      }
      if (_config['prevButton']['state']) {
        this.prevButtonState = _config['prevButton']['state'];
      }
    }
    if (_config['nextButton']) {
      if (_config['nextButton']['id']) {
        var nextButtonId = "#" + _config['nextButton']['id'];
        this.nextButton = document.querySelector(nextButtonId);
      }
      if (_config['nextButton']['state']) {
        this.nextButtonState = _config['nextButton']['state'];
      }
    }
    if (_config['completeButton']) {
      if (_config['completeButton']['id']) {
        var completeButtonId = "#" + _config['completeButton']['id'];
        this.completeButton = document.querySelector(completeButtonId);
      }
      if (_config['completeButton']['state']) {
        this.completeButtonState = _config['completeButton']['state'];
      }
    }
    this.buttonControl();
    return true;
  }
  async next() {
    var getFormId = "#" + this.tabs[this.getActive() - 1]["formControl"];
    var getForm = document.querySelector(getFormId);
    var formData = null;
    if (getForm) {
      formData = await getForm.submit();
    }
    if ((getForm && formData.isValid) || !getForm) {
      this.activeChild = this.getActive() + 1;
      this.active(this.tabs[this.getActive() - 1]["tabId"]);
    }
    return formData;
  }
  async prev() {
    this.activeChild = this.getActive() - 1;
    this.active(this.tabs[this.getActive() - 1]["tabId"]);
    return this.activeChild;
  }
  async complete() {
    const tabStatuses = [];
    var isValid = true;
    this.tabs.forEach(element => {
      if (element['formControl']) {
        const getFormId = "#" + element["formControl"];
        const getForm = document.querySelector(getFormId);
        if (getForm) {
          tabStatuses.push(getForm.submit());
        }
      }
    });
    const allStatuses = await Promise.all(tabStatuses);
    allStatuses.forEach(item => {
      if (item.isValid === false) {
        isValid = false;
        if (item.formName) {
          const formId = this.tabs.filter((input) => input['formControl'] == item.formName)[0]['tabId'];
          this.active(formId);
        }
      }
    });
    return { isValid: isValid, Forms: allStatuses };
  }
  async active(id) {
    var i = 0;
    this.activeChild = -1,
      await delay(100);
    getChilds(this.el, 'bcm-tab').forEach(items => {
      i++,
        items.tabId == id && (items.disabled = false,
          items.active = true,
          this.bcmTabsActiveItem.emit(items.tabId),
          this.activeChild = i);
      items.tabId != id && (items.active = false);
    });
  }
  async getActiveTab() {
    return this.tabs[this.getActive() - 1]["tabId"];
  }
  async getActiveIndex() {
    return this.getActive();
  }
  async error(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "error");
    });
  }
  async info(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "processing");
    });
  }
  async warning(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "warning");
    });
  }
  async success(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "success");
    });
  }
  async statusReset(id) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = false,
        items.blink = false,
        items.status = "default");
    });
  }
  buttonControl() {
    if (this.prevButton) {
      if (this.activeChild == 1) {
        this.prevButton.setAttribute(this.prevButtonState, "true");
      }
      else {
        this.prevButton.removeAttribute(this.prevButtonState);
      }
    }
    if (this.nextButton) {
      if (this.getActive() == this.count) {
        this.nextButton.setAttribute(this.nextButtonState, "true");
      }
      else {
        this.nextButton.removeAttribute(this.nextButtonState);
      }
    }
    if (this.completeButton) {
      if (this.getActive() == this.count) {
        this.completeButton.removeAttribute(this.completeButtonState);
      }
      else {
        this.completeButton.setAttribute(this.completeButtonState, "true");
      }
    }
  }
  /**
   * @ComponentMethod
   */
  componentDidRender() {
    if (!this.firstInit)
      return;
    let slotElements;
    slotElements = this.el.shadowRoot.querySelector('slot').assignedElements();
    this.tabs = [];
    this.activeChild = this.getActive();
    var isActive = false;
    var firstId = null;
    var i = 0;
    // Get only bcm-tab elements from 
    // slot childs
    slotElements.map((element) => {
      i++;
      this.type == "stepper" && (element.disabled = true);
      if (!firstId) {
        firstId = element.tabId;
      }
      if (String(element.tagName).toLowerCase() == 'bcm-tab') {
        element.type = this.type;
        // Activate default tab
        // #
        if (this.tabs.push(element) - 1 == this.activeChild) {
          this.activateChild(this.activeChild);
          isActive = true;
        }
      }
      this.count = i;
    });
    if (!isActive) {
      this.active(firstId);
    }
    setTimeout(() => {
      this.el.shadowRoot.querySelector('.tabs').classList.add('animatable');
    }, 500);
  }
  componentDidLoad() {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.active && (this.bcmTabsActiveItem.emit(items.tabId), items.disabled = false);
    });
  }
  getActive() {
    const childs = getChilds(this.el, 'bcm-tab');
    const childIndex = childs.findIndex(child => child.active);
    return childIndex >= 0 ? childIndex + 1 : 1;
  }
  /**
   * @desc
   */
  getChildIdx(child) {
    let i = 1;
    while (child.previousElementSibling) {
      child = child.previousElementSibling;
      i++;
    }
    return i;
  }
  /**
   * @desc
   * @param idx
   */
  activateChild(idx) {
    idx = idx - 1;
    const child = this.tabs[idx];
    if (child) {
      this.tabs.forEach((tab) => tab.active = false);
      child.active = true;
      child.disabled = false;
      this.transformLine(idx);
      this.firstInit && forceUpdate(this.el);
      this.firstInit = false;
    }
  }
  /**
   * @desc
   * @param idx
   */
  transformLine(idx) {
    const tab = this.tabs[idx];
    const rect = tab.getBoundingClientRect();
    const top = (rect.top + window.pageYOffset);
    const left = (rect.left + window.pageXOffset);
    const width = rect.width;
    const height = rect.height;
    if (this.direction == 'horizontal') {
      // this.lineOpts.top = (tab.offsetTop + tab.offsetHeight) + 'px'
      this.lineOpts.top = top + height + 'px';
      // this.lineOpts.left = tab.offsetLeft + 'px'
      this.lineOpts.left = left + 'px';
      // this.lineOpts.width = tab.offsetWidth + 'px'
      this.lineOpts.width = width + 'px';
      this.lineOpts.right = 'unset';
      this.lineOpts.height = '2px';
    }
    else {
      // this.lineOpts.top = tab.offsetTop + 'px'
      this.lineOpts.top = top + 'px';
      this.lineOpts.left = 'unset';
      this.lineOpts.right = '0';
      this.lineOpts.width = '2px';
      // this.lineOpts.height = tab.offsetHeight + 'px'
      this.lineOpts.height = height + 'px';
    }
  }
  /**
   * @desc
   */
  getLineTransform() {
    return {
      '--bcm-tabs-line-top': this.lineOpts.top,
      '--bcm-tabs-line-left': this.lineOpts.left,
      '--bcm-tabs-line-width': this.lineOpts.width,
      '--bcm-tabs-line-right': this.lineOpts.right,
      '--bcm-tabs-line-height': this.lineOpts.height
    };
  }
  /**
   * @desc
   * @param event
   */
  handleTabClick(event) {
    const targetElement = event.target;
    const childIdx = this.getChildIdx(targetElement);
    this.activeChild = null;
    this.activeChild = childIdx;
  }
  /**
   * @desc
   * @param event
   */
  handleTabClose(event) {
    const targetElement = event.target;
    const childIdx = this.getChildIdx(targetElement);
    // Remove element
    // #
    targetElement.remove();
    this.tabs.splice(childIdx - 1, 1);
    if (childIdx === this.activeChild) {
      this.activeChild = 1;
      this.activateChild(1);
    }
  }
  /**
   * @desc
   * @param newValue
   */
  activeChildChange(newValue) {
    this.activateChild(newValue);
    this.change.emit(newValue);
    this.buttonControl();
  }
  /**
   * @desc
   */
  windowResize() {
    this.transformLine(this.activeChild - 1);
    forceUpdate(this.el);
  }
  render() {
    const classes = cs('tabs', [this.direction], [this.size], [this.type], this.fullWidth ? 'full-width' : null, this.underline ? 'underline' : null);
    const hostClasses = cs(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: classes, style: this.getLineTransform(), "on-bcm-click": (event) => this.handleTabClick(event), "on-bcm-close": (event) => this.handleTabClose(event) }, h("slot", null))));
  }
  static get is() { return "bcm-tabs"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["tabs.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["tabs.css"]
    };
  }
  static get properties() {
    return {
      "size": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "SizeProp",
          "resolved": "\"large\" | \"medium\" | \"small\"",
          "references": {
            "SizeProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Component Properties"
        },
        "attribute": "size",
        "reflect": false,
        "defaultValue": "SizeProps.large"
      },
      "type": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "TypeProp",
          "resolved": "\"normal\" | \"pane\" | \"stepper\"",
          "references": {
            "TypeProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "type",
        "reflect": false,
        "defaultValue": "TypeProps.normal"
      },
      "direction": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "DirectionProp",
          "resolved": "\"horizontal\" | \"vertical\"",
          "references": {
            "DirectionProp": {
              "location": "import",
              "path": "./types"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "direction",
        "reflect": false,
        "defaultValue": "DirectionProps.horizontal"
      },
      "hidden": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "hidden",
        "reflect": false,
        "defaultValue": "false"
      },
      "fullWidth": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "full-width",
        "reflect": false,
        "defaultValue": "false"
      },
      "underline": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "underline",
        "reflect": false,
        "defaultValue": "true"
      }
    };
  }
  static get states() {
    return {
      "activeChild": {},
      "lineOpts": {},
      "prevButton": {},
      "nextButton": {},
      "completeButton": {},
      "prevButtonState": {},
      "nextButtonState": {},
      "completeButtonState": {}
    };
  }
  static get events() {
    return [{
        "method": "click",
        "name": "bcm-click",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Component Events"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "change",
        "name": "bcm-change",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "bcmTabsActiveItem",
        "name": "bcm-tabs-active-item",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": ""
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "config": {
        "complexType": {
          "signature": "(_config: Array<object>) => Promise<boolean>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            },
            "Array": {
              "location": "global"
            }
          },
          "return": "Promise<boolean>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "next": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "prev": {
        "complexType": {
          "signature": "() => Promise<number>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<number>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "complete": {
        "complexType": {
          "signature": "() => Promise<{ isValid: Boolean; Forms: any[]; }>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            },
            "Boolean": {
              "location": "global"
            },
            "HTMLElement": {
              "location": "global"
            }
          },
          "return": "Promise<{ isValid: Boolean; Forms: any[]; }>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "active": {
        "complexType": {
          "signature": "(id: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getActiveTab": {
        "complexType": {
          "signature": "() => Promise<any>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<any>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "getActiveIndex": {
        "complexType": {
          "signature": "() => Promise<number>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<number>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "error": {
        "complexType": {
          "signature": "(id: any, blink?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }, {
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "info": {
        "complexType": {
          "signature": "(id: any, blink?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }, {
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "warning": {
        "complexType": {
          "signature": "(id: any, blink?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }, {
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "success": {
        "complexType": {
          "signature": "(id: any, blink?: boolean) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }, {
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      },
      "statusReset": {
        "complexType": {
          "signature": "(id: any) => Promise<void>",
          "parameters": [{
              "tags": [],
              "text": ""
            }],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "activeChild",
        "methodName": "activeChildChange"
      }];
  }
  static get listeners() {
    return [{
        "name": "resize",
        "method": "windowResize",
        "target": "window",
        "capture": false,
        "passive": true
      }];
  }
}
