/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { ClientSideRowModelModule } from "@ag-grid-community/client-side-row-model";
import { Grid, ModuleRegistry } from "@ag-grid-community/core";
import { ExcelExportModule } from "@ag-grid-enterprise/excel-export";
import { StatusBarModule } from "@ag-grid-enterprise/status-bar";
import { Generate } from "@bcm/helpers";
import { ToolbarTemplate } from "@bcm/template";
import { Host, h } from "@stencil/core";
import cs from "classnames";
import snq from "snq";
import { ExportTypes, tableOptions } from "./table-definations";
ModuleRegistry.registerModules([StatusBarModule, ClientSideRowModelModule, ExcelExportModule]);
export class BcmTable {
  constructor() {
    this._id = Generate.UID();
    this.gridOptions = {};
    this.extensions = "csv";
    this.height = "250px";
    this.showToolbar = false;
    this.showFooter = false;
    this.toolBarConfig = {
      showNewButton: false,
      showExportButton: true,
      showColumnButton: true,
      showFilterButton: false,
      showSearchInput: true,
    };
    this.agGrid = null;
    this.columns = undefined;
    this.fileTypes = [];
    this.isReady = false;
    this.footerOptions = {
      statusBar: {
        statusPanels: [
          { statusPanel: "agTotalAndFilteredRowCountComponent", align: "left" },
          { statusPanel: "agTotalRowCountComponent", align: "right" },
          { statusPanel: "agFilteredRowCountComponent" },
          { statusPanel: "agSelectedRowCountComponent" },
          { statusPanel: "agAggregationComponent" },
        ],
      },
    };
  }
  get grid() {
    var _a;
    return (_a = this.agGrid) === null || _a === void 0 ? void 0 : _a.gridOptions;
  }
  componentDidLoad() {
    this.initGrid();
  }
  initGrid() {
    var _a;
    const agGridDiv = this.el.querySelector(`#bcm-table-${this._id}`);
    if (!agGridDiv)
      return;
    const options = Object.assign(Object.assign(Object.assign({}, tableOptions), Object.assign({}, (this.showFooter && this.footerOptions))), this.gridOptions);
    (_a = this.agGrid) === null || _a === void 0 ? void 0 : _a.destroy();
    this.agGrid = new Grid(agGridDiv, Object.assign(Object.assign({}, options), { onGridReady: params => this.initToolbar(params) }));
  }
  initToolbar(params) {
    this.columns = snq(() => { var _a; return (_a = params === null || params === void 0 ? void 0 : params.api) === null || _a === void 0 ? void 0 : _a.getColumnDefs(); }, [])
      .filter(Boolean)
      .filter((col) => !col.hide)
      .map((column) => {
      var _a, _b, _c, _d, _e;
      return ({
        id: column.colId,
        type: (_c = (_b = (_a = column === null || column === void 0 ? void 0 : column.filterParams) === null || _a === void 0 ? void 0 : _a.type) === null || _b === void 0 ? void 0 : _b.toLowerCase()) !== null && _c !== void 0 ? _c : "text",
        disabled: (_e = (_d = column === null || column === void 0 ? void 0 : column.filterParams) === null || _d === void 0 ? void 0 : _d.disabledColumn) !== null && _e !== void 0 ? _e : false,
      });
    });
    this.fileTypes = snq(() => {
      var _a;
      return (_a = this.extensions
        .toLowerCase()
        .split(",")) === null || _a === void 0 ? void 0 : _a.map((extention) => extention.trim()).map((extention) => ExportTypes[extention]);
    }, []);
    this.isReady = true;
  }
  handleColumnVisible({ checked, id }) {
    if (!id)
      return;
    this.grid.columnApi.setColumnVisible(id, checked);
  }
  handleFileType({ text }) {
    if (!text)
      return;
    switch (text.toLowerCase()) {
      case "csv":
        this.grid.api.exportDataAsCsv();
        break;
      case "excel":
        this.grid.api.exportDataAsExcel();
        break;
      case "pdf":
        alert("PDF export is not supported yet");
        break;
    }
  }
  handleSearch(text) {
    if (text === undefined)
      return;
    this.grid.api.setQuickFilter(text);
    if (this.grid.api.getDisplayedRowCount() === 0) {
      this.grid.api.showNoRowsOverlay();
    }
    else {
      this.grid.api.hideOverlay();
    }
  }
  watchHandler(newValue, oldValue) {
    if (newValue !== oldValue) {
      this.initGrid();
    }
  }
  render() {
    const { _id, height, toolBarConfig, columns } = this;
    return (h(Host, null, h("div", { class: cs("bcm-ui-element bcm-ui-table bcm-table", "tw-flex tw-w-full tw-flex-col tw-gap-2", {
        "tw-hidden": !this.isReady,
      }) }, this.showToolbar && (h(ToolbarTemplate, { tableId: _id, config: toolBarConfig, columns: columns, onHideColumn: e => this.handleColumnVisible(e), onSearch: e => this.handleSearch(e), onSelectExportType: e => this.handleFileType(e), extensions: this.fileTypes })), h("div", { id: `bcm-table-${this._id}`, class: "ag-theme-bcm", style: { height } }))));
  }
  static get is() { return "bcm-table"; }
  static get originalStyleUrls() {
    return {
      "$": ["table.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["table.css"]
    };
  }
  static get properties() {
    return {
      "_id": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "_id",
        "reflect": true,
        "defaultValue": "Generate.UID()"
      },
      "gridOptions": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "GridOptions",
          "resolved": "GridOptions<any>",
          "references": {
            "GridOptions": {
              "location": "import",
              "path": "@ag-grid-community/core"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "{}"
      },
      "extensions": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "extensions",
        "reflect": true,
        "defaultValue": "\"csv\""
      },
      "height": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "height",
        "reflect": true,
        "defaultValue": "\"250px\""
      },
      "showToolbar": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "show-toolbar",
        "reflect": true,
        "defaultValue": "false"
      },
      "showFooter": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "show-footer",
        "reflect": true,
        "defaultValue": "false"
      },
      "toolBarConfig": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "Bcm.ToolBarButtonProps",
          "resolved": "ToolBarButtonProps",
          "references": {
            "Bcm": {
              "location": "import",
              "path": "@bcm/model"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "defaultValue": "{\n        showNewButton: false,\n        showExportButton: true,\n        showColumnButton: true,\n        showFilterButton: false,\n        showSearchInput: true,\n    }"
      }
    };
  }
  static get states() {
    return {
      "agGrid": {},
      "columns": {},
      "fileTypes": {},
      "isReady": {},
      "footerOptions": {}
    };
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "gridOptions",
        "methodName": "watchHandler"
      }];
  }
}
