import { version } from "../../package.json";
import { LicenseManager } from "@ag-grid-enterprise/core";
import { Env } from "@stencil/core";

window.bcm = (function () {
    return {
        version: version,
    };
})();

function addHeadScript(obj) {
    const head = document.head;
    const script = document.createElement("script");
    Object.keys(obj).forEach(key => {
        script[key] = obj[key];
    });
    head.appendChild(script);
}

addHeadScript({ src: "https://kit.fontawesome.com/5e86bea963.js", crossOrigin: "anonymous" });

(() => {
    LicenseManager.setLicenseKey(Env["AG_GRID_LICENSE_KEY"]);
})();

async function fetchJSON(url = null) {
    const response = await fetch(url);
    return await response.json();
}

async function componentInit(component, componentId, componentContainerId = "0") {
    await removeComponent(componentId);
    const element = await createComponent(component, componentId, componentContainerId);
    if (component.substring(0, 4) === "bcm-") {
        await customElements.whenDefined(component);
        await element.componentOnReady();
    }
    return Promise.resolve(element);
}

async function createComponent(tag, id, target) {
    var elem = document.createElement(tag);
    elem.id = id;
    if (target !== "0") {
        document.getElementById(target).appendChild(elem);
    }

    return Promise.resolve(elem);
}

async function removeComponent(id) {
    var elem = document.getElementById(id);
    if (elem) {
        elem.remove();
    }
    return Promise.resolve();
}
