/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { delay, FloatingUI } from "@bcm/utils";
import { ColorHelper, Generate, StringHelper } from "@bcm/helpers";
export class Popover {
  constructor({ content, message, targetId, placement = "top", color = "slate-700", trigger = "none", openDelay = 0, closeDelay = 0, autoHide = false, emitShow = () => { }, emitHide = () => { } }) {
    this.destroy = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.el.remove();
        this.emitHide();
      }
      this.removeListener();
    };
    this.target = () => document.getElementById(this.targetId);
    this.init = () => {
      const { tone, colorName } = ColorHelper.parseColor(this.color, 700);
      const dc = Generate.getTwColor({
        color: colorName,
        tones: {
          [tone]: ["bg"],
        },
        variable: "--bcm-popover",
      });
      const style = Object.assign(Object.assign({}, dc), (dc["--bcm-popover-bg"] && { "--bcm-popover-text": ColorHelper.reverseColor(dc["--bcm-popover-bg"]) }));
      const inner = `<div class="tw-z-[1] tw-rounded tw-py-2">
            ${this.message ? `<div class="tw-font-medium tw-w-full tw-pb-1 tw-px-4">${StringHelper.trim(this.message)}</div>` : ""}
            ${this.content ? `<div class="tw-font-normal tw-px-4">${StringHelper.trim(this.content)}</div>` : ""}
        </div>`;
      this.el = Generate.createElement("div", {
        "id": `${this.targetId}-popover-box`,
        "class": `
            tw-whitespace-normal tw-break-words tw-block tw-font-sans tw-pointer-events-none tw-max-w-[256px] tw-rounded tw-bg-[--bcm-popover-bg] tw-text-[--bcm-popover-text] tw-text-3 tw-absolute tw-top-0 tw-left-0 tw-z-tooltip tw-opacity-0 tw-transition-opacity tw-duration-300
            tw-shadow`,
        "role": "popover",
        "aria-hidden": "true",
        "style": ColorHelper.generateStyleString(style),
        "innerHTML": inner,
      });
    };
    this.update = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
        this.show();
      }
    };
    this.listener = () => {
      window.addEventListener("resize", this.calculatePosition.bind(this));
      window.addEventListener("scroll", this.calculatePosition.bind(this), true);
      window.addEventListener("wheel", this.calculatePosition.bind(this));
      if (this.trigger && this.target()) {
        switch (this.trigger) {
          case "click":
            this.target().addEventListener("click", this.toggle.bind(this));
            this.autoHide && this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "hover":
            this.target().addEventListener("mouseenter", this.show.bind(this));
            this.target().addEventListener("mouseleave", this.hide.bind(this));
            break;
          case "focus":
            this.target().addEventListener("focus", this.show.bind(this), true);
            this.target().addEventListener("blur", this.hide.bind(this), true);
            break;
          default:
            break;
        }
      }
      this.calculatePosition();
    };
    this.removeListener = () => {
      window.removeEventListener("resize", this.calculatePosition.bind(this));
      window.removeEventListener("scroll", this.calculatePosition.bind(this), true);
      window.removeEventListener("wheel", this.calculatePosition.bind(this));
      if (this.target()) {
        this.target().removeEventListener("click", this.toggle.bind(this));
        this.target().removeEventListener("mouseenter", this.show.bind(this));
        this.target().removeEventListener("mouseleave", this.hide.bind(this));
        this.target().removeEventListener("focus", this.show.bind(this), true);
        this.target().removeEventListener("blur", this.hide.bind(this), true);
      }
    };
    this.calculatePosition = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        FloatingUI({
          ref: this.target(),
          floating: this.el,
          placement: this.placement,
          offset: 8,
          arrow: true,
          arrowColor: `tw-bg-[--bcm-popover-bg]`,
        });
      }
    };
    this.show = async () => {
      if (this.message) {
        this.init();
        await delay(this.openDelay);
        this.calculatePosition();
        this.emitShow();
      }
      else {
        this.hide();
      }
    };
    this.hide = async () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        await delay(this.closeDelay);
        this.el.remove();
        this.emitHide();
      }
    };
    this.toggle = () => {
      this.el = document.getElementById(this.id);
      if (this.el) {
        this.hide();
      }
      else {
        this.show();
      }
    };
    this.content = content;
    this.message = message;
    this.targetId = targetId;
    this.placement = placement;
    this.color = color;
    this.trigger = trigger;
    this.id = targetId + "-popover-box";
    this.openDelay = openDelay;
    this.closeDelay = closeDelay;
    this.autoHide = autoHide;
    this.emitShow = emitShow;
    this.emitHide = emitHide;
  }
}
