/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import Bcm from '../models/bcm';
import slugify from 'slugify';
export class StringHelper {
}
StringHelper.lowercase = (str) => str.toLowerCase();
StringHelper.uppercase = (str) => str.toUpperCase();
StringHelper.capitalize = (str) => str.charAt(0).toUpperCase() + str.slice(1);
StringHelper.camelCase = (str) => str
  .replace(/\s(.)/g, function ($1) {
  return $1.toUpperCase();
})
  .replace(/\s/g, '');
StringHelper.kebabCase = (str) => str.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
StringHelper.snakeCase = (str) => str.replace(/([a-z])([A-Z])/g, '$1_$2').toLowerCase();
StringHelper.titleCase = (str) => str.replace(/\w\S*/g, function (txt) {
  return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
});
StringHelper.trim = (str) => str && str.trim();
StringHelper.trimLeft = (str) => str.trimLeft();
StringHelper.trimRight = (str) => str.trimRight();
StringHelper.trimAll = (str) => str.trim().replace(/\s+/g, ' ');
StringHelper.replace = (str, search, replace) => str.replace(search, replace);
StringHelper.remove = (str, search) => str.replace(search, '');
StringHelper.isNullOrEmpty = (str) => str === null || str === undefined || str === '';
StringHelper.localizationMessage = (str, args) => str.replace(/{(\d+)}/g, (match, number) => (typeof args[number] != 'undefined' ? args[number] : match));
StringHelper.getMessage = (type, args) => StringHelper.localizationMessage(Bcm.FormErrorMessages[type], args);
StringHelper.split = (str, separator) => str.split(separator);
StringHelper.textContains = (str, search) => str && str.indexOf(search) !== -1;
StringHelper.textLimitter = (str, limit, newLine = false) => (str && str.length > limit ? `${str.substring(0, limit)}...` : str) + `${newLine ? ' <br /> ' : ''}`;
StringHelper.slugify = (str) => slugify(str, {
  replacement: ' ',
  remove: undefined,
  lower: true,
  strict: false,
  locale: 'vi',
  trim: false, // trim leading and trailing replacement chars, defaults to `true`
});
StringHelper.highlight = (str, search) => {
  let text = str;
  text = text.replace(/(<mark>|<\/mark>)/gim, '');
  const escapedSearch = search.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'); // Özel karakterleri kaçış dizileri ile kaçır
  const regex = new RegExp(escapedSearch, 'gi');
  const newText = text.replace(regex, '<mark>$&</mark>');
  return newText;
};
StringHelper.templateParser = (str, data) => str.replace(/\{{(\w+)\}}/g, (_, key) => data[key] || '');
StringHelper.parseStringToBoolean = (str) => str === 'true';
StringHelper.elementBooleanCheck = (obj) => {
  const item = JSON.parse(JSON.stringify(obj));
  if (item != undefined) {
    if (item.length > 0) {
      if (item == 'true')
        return true;
      if (item == 'false')
        return false;
    }
    return true;
  }
  return false;
};
StringHelper.htmlEntities = (str) => String(str).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
