export const argTypesCommon = {
    "value": {
        description: "The value of the date picker. This is a read-only field. The value is stored as an object.",
        table: {
            category: "Common Properties",
            type: { summary: "object" },
        },
    },
    "label": {
        control: "text",
        defaultValue: "Label",
        description: "The label of the date picker.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: null },
            type: { summary: "string" },
        },
    },
    "name": {
        control: "text",
        description: "The name of the date picker. When used in a form, the name is submitted along with the form value.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: null },
            type: { summary: "string" },
        },
    },
    "placeholder": {
        control: "text",
        description: "The placeholder text of the date picker.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: null },
            type: { summary: "string" },
        },
    },
    "clearable": {
        control: "boolean",
        description: "Specifies whether the date picker is clearable.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "caption": {
        control: "text",
        description: "The caption text of the date picker.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: null },
            type: { summary: "string" },
        },
    },
    "caption-type": {
        control: { type: "select", options: ["info", "error", "warning", "success", "default"] },
        description: "The caption type of the date picker.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: null },
            type: { summary: "string" },
        },
    },
    "full-width": {
        control: "boolean",
        description: "Date picker will take up the full width of its container.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "required": {
        control: "boolean",
        description: "Specifies whether the date picker is required.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "disabled": {
        control: "boolean",
        description: "Specifies whether the date picker is disabled.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "readonly": {
        control: "boolean",
        description: "Specifies whether the date picker is read-only.",
        table: {
            category: "Common Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "set": {
        description: "Sets the value of the date picker.",
        table: {
            category: "Methods",
            type: { summary: "void" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
// new Date() Or moment().toDate() can be used
bcmDate.set(new Date(2023, 11, 1));
// bcmDate.set(moment().endOf('month').toDate());`,
            },
        },
    },
    "get": {
        // timestamp döner 1633824000000 gibi
        description: "Gets the Timestamp of the date picker.",
        table: {
            category: "Methods",
            type: { summary: "string" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
await bcmDate.get(); // 1633824000000 (timestamp)`,
            },
        },
    },
    "setValue": {
        description: "Sets the value of the date picker. Same as set() method.",
        table: {
            category: "Methods",
            type: { summary: "void" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
// new Date() Or moment().toDate() can be used
await bcmDate.setValue(new Date(2023, 11, 1));
// bcmDate.setValue(moment().endOf('month').toDate());`,
            },
        },
    },
    "getValue": {
        description: "Gets the value object of the date picker. Date object contains the following Common Properties: isValid, date, timestamp, dateObject, value.",
        table: {
            category: "Methods",
            type: { summary: "object" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
await bcmDate.getValue();
// Returns the following object
{/* {
"isValid": true,
"date": "2021-10-10T00:00:00.000Z",
"timestamp": 1633824000000,
"dateObject": {
"year": 2021,
"day": 10,
"month": 9,
"dayOfWeek": 0,
"monthFirstDay": 5
},
"value": "10/10/2021"
} */}`,
            },
        },
    },
    "setClear": {
        description: "Sets the value of the date picker to null.",
        table: {
            category: "Methods",
            type: { summary: "void" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
bcmDate.setClear();`,
            },
        },
    },
    "resetCaption": {
        description: "Resets the caption of the date picker.",
        table: {
            category: "Methods",
            type: { summary: "void" },
            defaultValue: {
                summary: "Example",
                detail: `
const bcmDate = document.querySelector("bcm-date");
bcmDate.resetCaption();`,
            },
        },
    },
    "bcm-change": {
        description: "",
        table: {
            category: "Events",
            type: { summary: "void" },
        },
    },
};
