export const argTypesDateTime = {
    "range": {
        control: "boolean",
        description: "Specifies whether the date picker is in range mode.",
        table: {
            category: "Date Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "disabled-dates": {
        control: "object",
        description: "The dates that are disabled in the date picker. The value is stored as an object.",
        table: {
            category: "Date Properties",
            defaultValue: {
                summary: "Disabled Date Object",
                detail: `const bcmDate = document.querySelector("bcm-date");
bcmDate.disabledDates = {
    min: new Date(2023, 11, 1), // disable dates before 
    max: moment().endOf('month').toDate(), // disable dates after
    dates: [
        moment("2023-11-12").toDate(), 
        moment("2023-11-15").toDate(), 
        moment("2023-11-20").toDate(),
        new Date(2023, 11, 25)
    ],
    custom: (current) => {
        // return current <= moment().endOf('day') // disable dates before today
        // return current >= moment().startOf('day') // disable dates after today
        return current < moment().startOf("week") || current > moment().endOf("week"); // disable dates outside of the current week
    },
}`,
            },
            type: { summary: "object" },
        },
    },
    "show-second": {
        control: "boolean",
        description: "Specifies whether the second is shown.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "use-12-hours": {
        control: "boolean",
        description: "Specifies whether the time picker is shown in 12-hour format.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "hour-step": {
        control: "number",
        description: "Specifies the hour step.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "1" },
            type: { summary: "number" },
        },
    },
    "minute-step": {
        control: "number",
        description: "Specifies the minute step.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "1" },
            type: { summary: "number" },
        },
    },
    "second-step": {
        control: "number",
        description: "Specifies the second step.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "1" },
            type: { summary: "number" },
        },
    },
    "hide-disabled-options": {
        control: "boolean",
        description: "Specifies whether the disabled options are hidden.",
        table: {
            category: "Time Properties",
            defaultValue: { summary: "false" },
            type: { summary: "boolean" },
        },
    },
    "disabled-time": {
        description: "The times that are disabled in the time picker. The value is stored as an object.",
        table: {
            category: "Time Properties",
            defaultValue: {
                summary: "Disabled Time Object",
                detail: `
const bcmDate = document.querySelector("bcm-date");
bcmDate.disabledTime = {
disabledHours: [1, 2, 3, 4, 5, 6, 7, 10, 11],
disabledMinutes: [1, 2, 3, 4, 5, 6, 7, 8, 9],
disabledSeconds: [1, 2, 3, 4, 5, 6, 7, 8, 9],
};`,
            },
            type: { summary: "object" },
        },
    },
};
