/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { Bcm } from '@bcm/model';
import { computePosition, flip, shift, offset, arrow, hide, size } from '@floating-ui/dom';
export const FloatingUI = (
// el,
// component,
// options = {
//     placement: Bcm.Placement.bottom as PlacementTypes,
//     offset: 0,
//     arrow: false,
//     arrowColor: 'tw-bg-slate-700',
// },
// autoHidden = true,
// autoSize = false,
// style: any = {},
args) => {
  const { ref, floating, placement: _placement = Bcm.Placement.bottom, offset: _offset = 0, arrow: _arrow = false, arrowColor = 'tw-bg-slate-700', autoHidden = true, autoSize = false, style = {}, } = args;
  // const { placement: _placement, offset: _offset, arrow: _arrow, arrowColor } = options;
  // let _middleware = [offset(_offset), flip(), shift({ padding: 5 }), hide({ padding: 10 })];
  let _middleware = [offset(_offset), flip(), shift({ padding: 5 })];
  if (autoHidden) {
    _middleware.push(hide({ padding: 10 }));
  }
  style &&
    Object.assign(floating.style, Object.assign(Object.assign({}, (style.height && { height: style.height })), (style.minHeight && { minHeight: style.minHeight })));
  const heightCheck = ['auto', '100%', '100vh', '100vw', 'inherit', 'initial', 'unset', undefined].some(item => item == (style === null || style === void 0 ? void 0 : style.height));
  if (autoSize && heightCheck) {
    _middleware.push(size({
      apply({ availableHeight }) {
        const maxHeight = `${availableHeight > window.innerHeight ? window.innerHeight - 10 : availableHeight}px`;
        Object.assign(floating.style, {
          maxHeight: `${style.maxHeight && maxHeight >= style.maxHeight ? style.maxHeight : maxHeight}`,
        });
      },
      padding: 5,
    }));
  }
  let arrowElement = document.getElementById(`${floating.id}-arrow`);
  if (_arrow) {
    if (!arrowElement) {
      arrowElement = document.createElement('div');
      arrowElement.setAttribute('id', `${floating.id}-arrow`);
    }
    arrowElement.setAttribute('class', `tw-absolute tw-w-2 tw-h-2 ${arrowColor} tw-transform tw-z-[-1] tw-opacity-0 tw-transition-opacity tw-duration-300 
            tw-shadow 
        `);
    floating.prepend(arrowElement);
    _middleware.push(arrow({ element: arrowElement, padding: 4 }));
  }
  computePosition(ref, floating, { placement: _placement, middleware: _middleware }).then(({ x, y, placement, middlewareData }) => {
    const { referenceHidden } = autoHidden ? middlewareData.hide : { referenceHidden: false };
    Object.assign(floating.style, {
      top: '0',
      left: '0',
      transform: `translate(${Math.round(x)}px,${Math.round(y)}px)`,
      opacity: referenceHidden ? '0' : '1',
      visibility: referenceHidden ? 'hidden' : 'visible',
    });
    if (referenceHidden) {
      const isHideEvent = new CustomEvent('bcm-floating-is-hide', { detail: { id: floating.id } });
      document.dispatchEvent(isHideEvent);
      floating.getAttribute('outside-of-the-screen') !== 'true' && floating.setAttribute('outside-of-the-screen', 'true');
    }
    else {
      floating.getAttribute('outside-of-the-screen') === 'true' && floating.removeAttribute('outside-of-the-screen');
    }
    if (_arrow) {
      const { x: arrowX, y: arrowY } = middlewareData.arrow;
      const staticSide = {
        top: 'bottom',
        right: 'left',
        bottom: 'top',
        left: 'right',
      }[placement.split('-')[0]];
      const arrowStyle = {
        top: 'rotate(45deg)',
        right: 'rotate(135deg)',
        bottom: 'rotate(-135deg)',
        left: 'rotate(-45deg)',
      }[placement.split('-')[0]];
      Object.assign(arrowElement.style, {
        left: arrowX != null ? `${arrowX}px` : '',
        top: arrowY != null ? `${arrowY}px` : '',
        right: '',
        bottom: '',
        [staticSide]: '-4px',
        opacity: 1,
        visibility: referenceHidden ? 'hidden' : 'visible',
        transform: arrowStyle,
      });
    }
  });
};
