/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { ColorPalette } from '../global/variables/colors';
export const delay = ms => new Promise(res => setTimeout(res, ms));
/**
 * @desc
 * @param nodeMap
 */
export const getElementAttributes = (nodeMap) => {
  if (nodeMap) {
    const length = nodeMap.length;
    return Object.keys(nodeMap).reduce((props, current) => {
      try {
        const numCurrent = parseInt(current);
        if (numCurrent < length) {
          const property = nodeMap[current];
          return Object.assign(Object.assign({}, props), { [property.name]: property.value });
        }
      }
      catch (err) { }
    }, {});
  }
  return {};
};
/**
 * @desc
 * @param colors
 * @param colorName
 */
export const extractColor = (colors, colorName) => {
  const splitted = colorName.split('-');
  const grade = splitted[splitted.length - 1];
  const swatch = splitted.slice(0, splitted.length - 1).join('-');
  return colors[swatch] && colors[swatch][grade] ? colors[swatch][grade] : colors['prime-blue']['6'];
};
/**
 * @desc
 */
export const generateCssVariables = () => {
  let rootColors = '';
  Object.keys(ColorPalette).forEach((color) => {
    Object.keys(ColorPalette[color]).forEach((key) => {
      let variable = `--bcm-color-${color}-${key}: #\{$${color}-${key}\};\n`;
      rootColors += variable;
    });
  });
  return rootColors;
};
/**
 * @desc
 * @param component
 * @param component2
 */
export const isTargetComponent = (component, component2) => {
  return component.tagName && String(component.tagName).toLowerCase() === component2;
};
/**
 *
 * @param type
 */
export const getChilds = (el, component) => {
  let elements = [];
  let allElements = [];
  // Get @component's from 
  // slot childs
  const slot = Array.from(el.shadowRoot.querySelectorAll('slot')).find(slot => !slot.name);
  elements = (slot && slot.assignedElements()) || [];
  elements.map(element => {
    String(element.tagName).toLowerCase() == component
      && allElements.push(element);
  });
  // Get @component's from 
  // childs
  elements = el.shadowRoot.querySelectorAll(component);
  allElements = [
    ...allElements,
    ...elements
  ];
  return allElements;
};
export const getChildElements = (el, component) => {
  return [...el.querySelectorAll(component)];
};
/**
 * @desc
 */
export function createError(
// @ts-ignore
template, 
// @ts-ignore
arg1, 
// @ts-ignore
arg2, 
// @ts-ignore
arg3, 
// @ts-ignore
arg4, 
// @ts-ignore
arg5, 
// @ts-ignore
arg6, 
// @ts-ignore
arg7, 
// @ts-ignore
arg8, 
// @ts-ignore
arg9) {
  let str = arguments[0];
  let curr, pb = false, ts = '', tstr = str, ca = 1;
  for (let i = 0; i < str.length; i++) {
    curr = str[i];
    if (pb)
      if (curr == '}') {
        tstr = tstr.replace('{' + ts + '}', arguments[ca]);
        pb = false;
        ts = '';
        ca++;
      }
      else
        ts += curr;
    if (curr == '{')
      pb = true;
  }
  return tstr;
}
/**
 * @desc
 * @param element
 */
export const getRect = (element) => {
  return element.getBoundingClientRect();
};
/**
 * @desc
 * @param
 */
export const dropInComponent = (followingComponent, targetComponent) => {
  const el = followingComponent;
  const linkedComponent = targetComponent;
  const bodyWidth = document.documentElement.clientWidth;
  const bodyHeight = document.documentElement.clientHeight;
  // window.onload = () => {
  // document.querySelector(targetComponent).onload = () => {
  var linkedId = document.querySelector(linkedComponent);
  if (linkedId) {
    var captionAreaHeight = 0;
    if (linkedId.shadowRoot && linkedId.shadowRoot.querySelector(".caption-area") && linkedId.shadowRoot.querySelector(".caption-area").clientHeight) {
      captionAreaHeight = linkedId.shadowRoot.querySelector(".caption-area").clientHeight + 8;
    }
    var labelAreaHeight = 0;
    if (linkedId.shadowRoot && linkedId.shadowRoot.querySelector(".label") && linkedId.shadowRoot.querySelector(".label").clientHeight) {
      labelAreaHeight = linkedId.shadowRoot.querySelector(".label").clientHeight + 2;
    }
    var captionHeight = linkedId["noCaption"] ? 0 : captionAreaHeight - 8;
    var linkedPosition = getRect(linkedId);
    el.style.position = "fixed";
    el.style.top = linkedPosition.top + linkedPosition.height - captionHeight + "px";
    el.style.left = linkedPosition.left + "px";
    if (bodyHeight < (el.clientHeight + linkedPosition.top + linkedPosition.height - captionHeight)) {
      el.style.top = linkedPosition.top - el.clientHeight + labelAreaHeight + "px";
    }
    if (bodyWidth < el.clientWidth + linkedPosition.left) {
      el.style.left = linkedPosition.left - el.clientWidth + linkedPosition.width - 2 + "px";
    }
  }
  // }
  // };
};
/**
 * @desc
 * @param element
 */
export const getTransform = (element) => {
  let transform = getComputedStyle(element).getPropertyValue('transform');
  let values;
  values = transform.replace(/(matrix|\(|\)|px)/g, '');
  values = values.split(', ');
  return { x: parseInt(values[4]), y: parseInt(values[5]) };
};
/**
 * @desc
 * @param mouseEvent
 * @param element
 */
export const getDragPoint = (event, element) => {
  // const rectElement: DOMRect = getRect(element)
  const transformElement = getTransform(element);
  console.log(event.pageY, transformElement.y, event.pageY - transformElement.y);
  return {
    x: event.pageX - element.offsetLeft,
    y: event.pageY - transformElement.y
  };
  /* return {
      x: Math.floor(event.clientX) - (rectElement.x + (rectElement.width / 2)),
      y: Math.floor(event.clientY) - (rectElement.y + (rectElement.height / 2))
  } */
};
export const pluralize = (function () {
  // var vowels = "aeiou";
  var irregulars = { "addendum": "addenda", "aircraft": "aircraft", "alumna": "alumnae", "alumnus": "alumni", "analysis": "analyses", "antenna": "antennae", "antithesis": "antitheses", "apex": "apices", "appendix": "appendices", "axis": "axes", "bacillus": "bacilli", "bacterium": "bacteria", "basis": "bases", "beau": "beaux", "bison": "bison", "bureau": "bureaux", "cactus": "cacti", "château": "châteaux", "child": "children", "codex": "codices", "concerto": "concerti", "corpus": "corpora", "crisis": "crises", "criterion": "criteria", "curriculum": "curricula", "datum": "data", "deer": "deer", "diagnosis": "diagnoses", "die": "dice", "dwarf": "dwarves", "ellipsis": "ellipses", "erratum": "errata", "faux pas": "faux pas", "fez": "fezzes", "fish": "fish", "focus": "foci", "foot": "feet", "formula": "formulae", "fungus": "fungi", "genus": "genera", "goose": "geese", "graffito": "graffiti", "grouse": "grouse", "half": "halves", "hoof": "hooves", "hypothesis": "hypotheses", "index": "indices", "larva": "larvae", "libretto": "libretti", "loaf": "loaves", "locus": "loci", "louse": "lice", "man": "men", "matrix": "matrices", "medium": "media", "memorandum": "memoranda", "minutia": "minutiae", "moose": "moose", "mouse": "mice", "nebula": "nebulae", "nucleus": "nuclei", "oasis": "oases", "offspring": "offspring", "opus": "opera", "ovum": "ova", "ox": "oxen", "parenthesis": "parentheses", "phenomenon": "phenomena", "phylum": "phyla", "quiz": "quizzes", "radius": "radii", "referendum": "referenda", "salmon": "salmon", "scarf": "scarves", "self": "selves", "series": "series", "sheep": "sheep", "shrimp": "shrimp", "species": "species", "stimulus": "stimuli", "stratum": "strata", "swine": "swine", "syllabus": "syllabi", "symposium": "symposia", "synopsis": "synopses", "tableau": "tableaux", "thesis": "theses", "thief": "thieves", "tooth": "teeth", "trout": "trout", "tuna": "tuna", "vertebra": "vertebrae", "vertex": "vertices", "vita": "vitae", "vortex": "vortices", "wharf": "wharves", "wife": "wives", "wolf": "wolves", "woman": "women", "guy": "guys", "buy": "buys", "person": "people" };
  function text(word, length = 1) {
    if (length > 1) {
      if (irregulars[word]) {
        return irregulars[word];
      }
      else if (word[word.length - 1] === "s" || word.endsWith("sh") || word.endsWith("ch") || word[word.length - 1] === "x" || word[word.length - 1] === "z") {
        return word + "es";
      }
      else if (word[word.length - 1] === "y") {
        return word.substring(0, word.length - 1) + "ies";
      }
      else {
        return word + "s";
      }
    }
    else {
      return word;
    }
  }
  return text;
})();
