/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';
import { c as ce } from './index5.js';
import { d as defineCustomElement$2 } from './badge.js';
import { d as defineCustomElement$1 } from './icon.js';

const avatarStyle = ce({
  slots: {
    base: "bcm-avatar bcm-ui-element bcm-ui-content-display tw-uppercase tw-inline-flex tw-items-center tw-justify-center tw-bg-[--bcm-avatar-bg] tw-text-[--bcm-avatar-text]",
    image: "tw-w-full tw-h-full",
    badge: "",
  },
  variants: {
    size: {
      small: "bcm-avatar__size-small tw-text-1 tw-w-6 tw-h-6",
      medium: "bcm-avatar__size-medium tw-text-2 tw-w-8 tw-h-8",
      large: "bcm-avatar__size-large tw-text-3 tw-w-10 tw-h-10",
    },
    shape: {
      ellipse: {
        base: "bcm-avatar__shape-ellipse tw-rounded-full",
        image: "tw-rounded-full",
        badge: "tw-top-0 tw-right-0",
      },
      square: {
        base: "bcm-avatar__shape-square tw-rounded",
        image: "tw-rounded",
        badge: "tw--top-0.5 tw--right-0.5",
      },
    },
    status: {
      true: {
        base: "bcm-avatar__status tw-relative",
        badge: "tw-absolute",
      },
    },
  },
  defaultVariants: {
    size: "medium",
    shape: "ellipse",
    status: false,
  },
});

const BcmAvatar = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this._id = Generate.UID();
    this.image = undefined;
    this.name = undefined;
    this.alt = undefined;
    this.shape = Bcm.Shape.ellipse;
    this.size = Bcm.Size.medium;
    this.color = undefined;
    this.icon = "fas fa-user";
    this.status = undefined;
    this.blink = false;
    this.isFallback = false;
  }
  getFirstLetters(name) {
    const words = name.split(" ");
    const initials = words.map(word => word.charAt(0).toUpperCase()).join("");
    return initials.substring(0, 2);
  }
  renderDefaultIcon() {
    return h("bcm-icon", { icon: this.icon });
  }
  render() {
    const { shape, size, status, color, blink } = this;
    const { base, badge, image } = avatarStyle({ size, shape, status: !!status });
    const dc = Generate.getTwColor({
      color: color,
      tones: {
        200: ["bg"],
        700: ["text"],
      },
      variable: "--bcm-avatar",
    });
    const customSize = !Object.keys(Bcm.BaseSize).includes(size)
      ? {
        width: `${size}px`,
        height: `${size}px`,
        fontSize: `${+size / 3}px`,
      }
      : {};
    const RenderContent = () => {
      if (this.isFallback) {
        if (this.name) {
          return this.getFirstLetters(this.name);
        }
        else {
          return this.renderDefaultIcon();
        }
      }
      else if (this.image) {
        return h("img", { class: image(), onError: () => (this.isFallback = true), src: this.image, alt: this.alt, "aria-label": this.alt || this.name || "Avatar" });
      }
      else if (this.name) {
        return this.getFirstLetters(this.name);
      }
      else {
        return this.renderDefaultIcon();
      }
    };
    return (h(Host, { role: "img", "aria-label": this.alt || this.name || "Avatar", class: base(), style: Object.assign(Object.assign({}, dc), customSize) }, h(RenderContent, null), status && h("bcm-badge", { class: badge(), status: status, blink: blink, "aria-hidden": "true" }), h("slot", null)));
  }
  get el() { return this; }
}, [4, "bcm-avatar", {
    "_id": [513, "id"],
    "image": [1],
    "name": [1],
    "alt": [1],
    "shape": [1],
    "size": [1],
    "color": [1],
    "icon": [1],
    "status": [1],
    "blink": [4],
    "isFallback": [32]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-avatar", "bcm-badge", "bcm-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-avatar":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmAvatar);
      }
      break;
    case "bcm-badge":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { BcmAvatar as B, defineCustomElement as d };
