/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { C as ColorHelper } from './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';
import { c as ce } from './index5.js';

const badgeStyle = ce({
  slots: {
    host: "bcm-badge bcm-ui-element bcm-ui-content-display tw-inline-flex tw-items-center tw-justify-center tw-font-normal ",
    base: [
      "bcm-badge__dot tw-z-10",
      "tw-flex tw-items-center tw-justify-center tw-flex-row",
      "tw-rounded-full tw-text-center",
      "tw-bg-[--bcm-badge-bg] tw-text-[--bcm-badge-text]",
      "tw-whitespace-nowrap",
    ],
    dotText: "empty:tw-hidden",
  },
  variants: {
    size: {
      small: {
        host: "tw-text-2 tw-gap-1",
        base: "bcm-badge__dot-size-small tw-text-1 tw-min-w-1.5 tw-min-h-1.5",
        dotText: "tw-px-1.5 tw-min-w-4",
      },
      medium: {
        host: "tw-text-3 tw-gap-1.5",
        base: "bcm-badge__dot-size-medium tw-text-2 tw-min-w-2 tw-min-h-2",
        dotText: "tw-px-1.5 tw-min-w-5",
      },
      large: {
        host: "tw-text-4 tw-gap-2",
        base: "bcm-badge__dot-size-large tw-text-3 tw-min-w-2.5 tw-min-h-2.5",
        dotText: "tw-px-2 tw-min-w-6",
      },
    },
    blink: {
      true: {
        base: [
          "bcm-badge__dot-blink tw-relative",
          "after:tw-content-[''] after:tw-absolute after:tw-top-0 after:tw-left-0 after:tw-z-[-1]",
          "after:tw-w-full after:tw-h-full after:tw-bg-[--bcm-badge-bg] after:tw-opacity-75 after:tw-rounded-full",
          "after:tw-animate-blink after:tw-pointer-events-none",
        ],
      },
    },
  },
  defaultVariants: {
    size: "medium",
    blink: false,
  },
});

const BcmBadge = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.statusColors = {
      info: "blue",
      error: "red",
      warning: "amber",
      success: "emerald",
    };
    this._id = Generate.UID();
    this.blink = false;
    this.value = undefined;
    this.color = undefined;
    this.size = Bcm.Size.medium;
    this.status = undefined;
  }
  render() {
    const { blink, value, size, color, status } = this;
    const _color = this.statusColors[status] || color;
    const { tone, colorName } = ColorHelper.parseColor(_color, 700);
    const dc = Generate.getTwColor({
      color: colorName,
      tones: {
        [tone]: ["bg"],
      },
      variable: "--bcm-badge",
    });
    const style = Object.assign(Object.assign({}, dc), (dc["--bcm-badge-bg"] && { "--bcm-badge-text": ColorHelper.reverseColor(dc["--bcm-badge-bg"]) }));
    const { base, dotText, host } = badgeStyle({ size, blink });
    return (h(Host, { class: host(), role: "status", "aria-live": "polite" }, h("div", { class: base(), style: style }, h("span", { class: dotText(), role: "presentation", "aria-hidden": "true" }, value), h("span", { class: "tw-sr-only" }, value)), h("slot", null)));
  }
  get el() { return this; }
}, [4, "bcm-badge", {
    "_id": [513, "id"],
    "blink": [4],
    "value": [1],
    "color": [8],
    "size": [1],
    "status": [1]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-badge"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-badge":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmBadge);
      }
      break;
  } });
}

export { BcmBadge as B, defineCustomElement as d };
