/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { G as Generate, s as snq, S as StatusProps, T as TypeProps } from './generate.js';
import { c as classnames } from './index2.js';
import { c as createStore } from './index4.js';
import { S as StringHelper } from './string-helper.js';
import { d as defineCustomElement$2 } from './icon.js';

const { state } = createStore({
  value: new Map(),
  setData: {},
  removeData: {}
});
const setValue = (val) => {
  const data = Object.assign(Object.assign(Object.assign({}, val), { size: val.bigIcon ? 'medium' : 14, iconType: val.bigIcon ? 'outlined' : 'fill' }), Generate.getIconAttrWithStatusType(val.status));
  state.value.set(data.id, data);
};
const removeValue = (_id) => {
  state.value.delete(_id);
};
const getValue = (id) => snq(() => state.value.get(id), {});
const alertStates = {
  setValue,
  removeValue,
  getValue
};

const alertCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}.bcm-alert{display:block;width:100%;max-width:400px;box-sizing:border-box;padding:8px 16px;box-shadow:0px 2px 4px rgba(0, 0, 0, 0.15);border-radius:2px;position:relative;grid-gap:8px}.bcm-alert.hidden{display:none}.bcm-alert.full-width{max-width:100%}.bcm-alert__status-info{border:1px solid var(--bcm-color-blue-3);background-color:var(--bcm-color-blue-1)}.bcm-alert__status-error{border:1px solid var(--bcm-color-red-3);background-color:var(--bcm-color-red-1)}.bcm-alert__status-warning{border:1px solid var(--bcm-color-warmyellow-3);background-color:var(--bcm-color-warmyellow-1)}.bcm-alert__status-success{border:1px solid var(--bcm-color-green-3);background-color:var(--bcm-color-green-1)}.bcm-alert__type-banner{border:none}.bcm-alert__header{display:flex;align-items:center;justify-content:space-between;padding:4px 0}.bcm-alert__header-message{display:flex;align-items:center;grid-gap:8px;color:var(--bcm-color-grey-8)}.bcm-alert__header-dismiss{display:flex;position:absolute;top:-1px;right:0;padding:16px;color:var(--bcm-color-grey-8)}.bcm-alert__header-dismiss:hover{cursor:pointer;color:var(--bcm-color-grey-9)}.bcm-alert__content{color:var(--bcm-color-grey-8)}.bcm-alert__content p{margin:0;padding:0}";

const BcmAlert$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.alertdismissible = createEvent(this, "bcm-alert-dismissible", 7);
    this.hidden = false;
    this._id = Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = StatusProps.info;
    this.type = TypeProps.basic;
    this.dismissible = undefined;
    this.icon = false;
    this.bigIcon = false;
    this.fullWidth = false;
  }
  componentWillRender() {
    if (this.dismissible != undefined) {
      if (this.dismissible == '') {
        this.dismissible = true;
      }
      else if (typeof this.dismissible === 'string' && this.dismissible == 'false' || this.dismissible == 'true') {
        this.dismissible = StringHelper.elementBooleanCheck(this.dismissible);
      }
      else {
        return;
      }
    }
    this.setData();
  }
  dismiss(e) {
    this.alertdismissible.emit(e);
    this.el.remove();
  }
  getStateData() {
    return alertStates.getValue(this._id);
  }
  setData() {
    const { status, type, bigIcon } = this;
    alertStates.setValue({
      id: this._id,
      status,
      type,
      bigIcon,
    });
  }
  render() {
    const { fullWidth, status, type, icon, bigIcon, _title, hidden, dismissible, description } = this;
    const hostClasses = classnames('bcm-alert', 'bcm-alert__type-' + type, 'bcm-alert__status-' + status, {
      'full-width': fullWidth,
      hidden,
    });
    const headerMessage = classnames('bcm-alert__header-message', !!description ? 'size-3' : 'size-2');
    const state = this.getStateData();
    return (h(Host, { class: hostClasses }, h("span", { hidden: true }, h("slot", null)), h("header", { class: "bcm-alert__header" }, h("span", { class: headerMessage }, (bigIcon || icon) && h("bcm-icon", { icon: state.icon }), _title), dismissible && (h("span", { class: "bcm-alert__header-dismiss", onClick: e => this.dismiss(e) }, typeof dismissible === 'string' ? dismissible : h("bcm-icon", { icon: "fal fa-times" })))), h("main", { class: "bcm-alert__content size-2" }, description && h("p", null, description), h("slot", null))));
  }
  get el() { return this; }
  static get style() { return alertCss; }
}, [4, "bcm-alert", {
    "hidden": [1028],
    "_id": [1537, "id"],
    "_title": [1032],
    "description": [1032],
    "status": [1025],
    "type": [1025],
    "dismissible": [1032],
    "icon": [1028],
    "bigIcon": [1028, "big-icon"],
    "fullWidth": [1028, "full-width"]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-alert", "bcm-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-alert":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmAlert$1);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmAlert = BcmAlert$1;
const defineCustomElement = defineCustomElement$1;

export { BcmAlert, defineCustomElement };
