/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { d as delay } from './utils.js';
import { F as FloatingUI } from './floating-ui.js';
import { B as Bcm } from './bcm.js';
import { c as ce } from './index5.js';
import { d as defineCustomElement$a } from './button.js';
import { d as defineCustomElement$9 } from './colorful.js';
import { d as defineCustomElement$8 } from './form.js';
import { d as defineCustomElement$7 } from './icon.js';
import { d as defineCustomElement$6 } from './input.js';
import { d as defineCustomElement$5 } from './label.js';
import { d as defineCustomElement$4 } from './linked.js';
import { d as defineCustomElement$3 } from './textarea.js';
import { d as defineCustomElement$2 } from './tooltip.js';

const attendanceActionCommentStyle = ce({
  slots: {
    host: [
      // "tw-flex tw-h-fit tw-w-full tw-min-w-[288px] tw-flex-col tw-items-start tw-rounded tw-p-4 tw-shadow-2",
      "tw-w-fit",
      "tw-absolute tw-left-0 tw-right-0 tw-z-tooltip tw-opacity-100 tw-transition-opacity tw-duration-300",
      "tw-rounded tw-bg-white tw-shadow",
      "bcm-ui-element bcm-attendance-actions tw-flex tw-flex-col tw-items-start tw-justify-start tw-p-4 tw-font-medium tw-text-slate-700",
    ],
    footer: ["tw-flex tw-w-full tw-flex-row tw-items-center tw-justify-end tw-gap-2 "],
  },
  variants: {
    size: {
      small: {},
      medium: {},
      large: {},
    },
  },
  defaultVariants: {
    size: "medium",
  },
});

const BcmAttendanceActionsComment$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-attendance-actions-comment-change", 3);
    this.lateMinutes = null;
    this.comment = null;
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.disabled = false;
    this.selectedKey = null;
    this.targetId = null;
    this.isShow = false;
    this.isLate = false;
    this.selectedItem = null;
  }
  handleSave(lateMinutes, comment) {
    this.change.emit({ lateMinutes, comment });
    this.hide();
  }
  async show() {
    const targetElement = document.getElementById(this.targetId);
    FloatingUI({
      ref: targetElement,
      floating: this.el,
      placement: "bottom-end",
      offset: 8,
      arrow: true,
      arrowColor: "tw-bg-white",
    });
    this.isShow = true;
    await delay(100);
    if (this.isLate) {
      this.focusInput();
    }
    else {
      this.focusTextarea();
    }
  }
  focusInput() {
    const input = this.el.querySelector("bcm-input");
    input === null || input === void 0 ? void 0 : input.onBcmSelect();
  }
  focusTextarea() {
    const textarea = this.el.querySelector("bcm-textarea");
    textarea === null || textarea === void 0 ? void 0 : textarea.onBcmSelect();
  }
  async hide() {
    if (this.isShow) {
      this.el.remove();
    }
  }
  handleClickOutside(event) {
    if (!Generate.findEventPath(event, this.el)) {
      this.hide();
    }
  }
  async formSubmit() {
    var _a, _b, _c;
    const { lateMinutes, comment, isLate } = this;
    if (isLate) {
      if (lateMinutes) {
        if (!(lateMinutes >= 0 && lateMinutes <= 200)) {
          return;
        }
      }
    }
    const { isValid } = await ((_a = this.el.querySelector("bcm-form")) === null || _a === void 0 ? void 0 : _a.submit());
    if (!isValid)
      return;
    this.handleSave(lateMinutes || ((_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.lateMin), comment || ((_c = this.selectedItem) === null || _c === void 0 ? void 0 : _c.comment));
  }
  render() {
    var _a, _b;
    const { size } = this;
    const { host, footer } = attendanceActionCommentStyle({
      size,
    });
    return (h(Host, { class: host(), onKeyDown: e => {
        if (e.key === "Escape")
          this.hide();
      } }, h("bcm-form", { validation: true, style: { width: "256px" } }, this.isLate ? (h("bcm-input", { label: "Minus Late", required: true, name: "lateMin", id: this._id + "-input-dropdown-late-content-input", type: "number", size: this.size, autocomplete: false, "no-margin": true, min: 0, max: 200, value: (_a = this.selectedItem) === null || _a === void 0 ? void 0 : _a.lateMin, placeholder: "Enter Minus Late", "onBcm-change": ({ detail }) => (this.lateMinutes = detail), onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }, h("i", { slot: "prefix", class: "far fa-clock" }))) : null, h("bcm-textarea", { name: "comment", value: (_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.comment, size: this.size, "onBcm-change": ({ detail }) => (this.comment = detail), label: "Description", "no-caption": true, rows: 4, onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }), h("footer", { class: footer() }, h("bcm-button", { class: "tw-order-2", icon: "fas fa-save", "icon-position": "prefix", size: this.size, onClick: () => this.formSubmit() }, "Save"), h("bcm-button", { class: "tw-order-1", icon: "fas fa-times", kind: "ghost", size: this.size, "icon-position": "prefix", onClick: () => this.hide() }, "Close")))));
  }
  get el() { return this; }
}, [0, "bcm-attendance-actions-comment", {
    "_id": [1537, "id"],
    "size": [1025],
    "disabled": [4],
    "selectedKey": [1025, "selected-key"],
    "targetId": [1, "target-id"],
    "isShow": [1028, "is-show"],
    "isLate": [4, "is-late"],
    "selectedItem": [1040],
    "show": [64],
    "hide": [64]
  }, [[8, "click", "handleClickOutside"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-attendance-actions-comment", "bcm-button", "bcm-colorful", "bcm-form", "bcm-icon", "bcm-input", "bcm-label", "bcm-linked", "bcm-textarea", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-attendance-actions-comment":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmAttendanceActionsComment$1);
      }
      break;
    case "bcm-button":
      if (!customElements.get(tagName)) {
        defineCustomElement$a();
      }
      break;
    case "bcm-colorful":
      if (!customElements.get(tagName)) {
        defineCustomElement$9();
      }
      break;
    case "bcm-form":
      if (!customElements.get(tagName)) {
        defineCustomElement$8();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$7();
      }
      break;
    case "bcm-input":
      if (!customElements.get(tagName)) {
        defineCustomElement$6();
      }
      break;
    case "bcm-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-linked":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-textarea":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmAttendanceActionsComment = BcmAttendanceActionsComment$1;
const defineCustomElement = defineCustomElement$1;

export { BcmAttendanceActionsComment, defineCustomElement };
