/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { F as FloatingUI } from './floating-ui.js';
import { B as Bcm } from './bcm.js';
import { c as classnames } from './index2.js';
import { c as ce } from './index5.js';

const attendanceActionDropdownStyle = ce({
  slots: {
    host: [
      "tw-w-fit",
      "tw-absolute tw-left-0 tw-right-0 tw-z-tooltip tw-opacity-100 tw-transition-opacity tw-duration-300",
      "tw-overscroll-contain  tw-scroll-smooth tw-rounded tw-bg-white tw-shadow tw-max-h-64 tw-overflow-y-auto",
      "bcm-ui-element bcm-attendance-actions tw-flex tw-flex-col tw-items-start tw-justify-start tw-p-2 tw-font-medium tw-text-slate-700"
    ],
    dataItem: ["tw-flex tw-w-full tw-cursor-pointer tw-flex-row tw-items-start tw-justify-start tw-gap-2 tw-rounded  hover:tw-bg-blue-100"],
    key: ["tw-font-mono tw-text-slate-600"],
    selectedItem: ["tw-bg-blue-100"],
  },
  variants: {
    size: {
      small: {
        dataItem: ["tw-text-2.5 tw-px-2.5 tw-py-2"],
      },
      medium: {
        dataItem: ["tw-text-3 tw-px-3 tw-py-2"],
      },
      large: {
        dataItem: ["tw-text-4 tw-px-4 tw-py-2"],
      },
    }
  },
  defaultVariants: {
    size: "medium"
  },
});

const BcmAttendanceActionsDropdown$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-attendance-actions-dropdown-change", 3);
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.disabled = false;
    this.selectedKey = null;
    this.targetId = null;
    this.isShow = false;
    this.data = [];
  }
  handleSelectItem(item) {
    this.selectedKey = item.key;
    this.change.emit(item);
  }
  async show() {
    const targetElement = document.getElementById(this.targetId);
    FloatingUI({
      ref: targetElement,
      floating: this.el,
      placement: "bottom-end",
      offset: 4,
    });
    this.isShow = true;
  }
  async hide() {
    if (this.isShow) {
      this.el.remove();
    }
  }
  handleClickOutside() {
    this.hide();
  }
  render() {
    const { size } = this;
    const { host, dataItem, selectedItem, key } = attendanceActionDropdownStyle({
      size,
    });
    return (h(Host, { class: host() }, this.data.map((item) => (h("div", { class: classnames(dataItem(), this.selectedKey === item.key && selectedItem()), onClick: () => this.handleSelectItem(item) }, h("span", { class: key() }, item.key, " - "), h("span", null, item.value))))));
  }
  get el() { return this; }
}, [0, "bcm-attendance-actions-dropdown", {
    "_id": [1537, "id"],
    "size": [1025],
    "disabled": [4],
    "selectedKey": [1025, "selected-key"],
    "targetId": [1, "target-id"],
    "isShow": [1028, "is-show"],
    "data": [1040],
    "show": [64],
    "hide": [64]
  }, [[8, "click", "handleClickOutside"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-attendance-actions-dropdown"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-attendance-actions-dropdown":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmAttendanceActionsDropdown$1);
      }
      break;
  } });
}

const BcmAttendanceActionsDropdown = BcmAttendanceActionsDropdown$1;
const defineCustomElement = defineCustomElement$1;

export { BcmAttendanceActionsDropdown, defineCustomElement };
