/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { G as Generate, s as snq } from './generate.js';
import { a as Bcm } from './bcm.js';
import { c as classnames } from './index2.js';
import { S as StringHelper } from './string-helper.js';

const buttonGroupCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");@import url(\"https://fonts.googleapis.com/css2?family=Inter:wght@200;400;500;600&display=swap\"); .size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}*{box-sizing:border-box}.bcm-button-group{font-family:\"Inter\", sans-serif;font-size:14px;line-height:calc(14px + 8px);font-weight:400;display:flex;flex-direction:row;align-items:center;justify-content:flex-start;flex-wrap:nowrap;width:fit-content;background-color:var(--bcm-new-ds-color-slate-300);border:1px solid var(--bcm-new-ds-color-slate-300);gap:1px;border-radius:4px}.bcm-button-group .bcm-button__container{border-radius:0;border:none}.bcm-button-group .bcm-button:first-child .bcm-button__container{border-top-left-radius:4px;border-bottom-left-radius:4px}.bcm-button-group .bcm-button:last-child .bcm-button__container{border-top-right-radius:4px;border-bottom-right-radius:4px}.bcm-button-group__vertical{display:flex;flex-direction:column;align-items:center;justify-content:flex-start;flex-wrap:nowrap}.bcm-button-group__vertical .bcm-button{width:100%}.bcm-button-group__vertical .bcm-button__container{width:100%;border-radius:0}.bcm-button-group__vertical .bcm-button:first-child .bcm-button__container{border-radius:4px 4px 0 0}.bcm-button-group__vertical .bcm-button:last-child .bcm-button__container{border-radius:0 0 4px 4px}";

const BcmButtonGroup$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.bcmChange = createEvent(this, "bcm-change", 7);
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.type = 'button';
    this.color = undefined;
    this.disabled = undefined;
    this.direction = Bcm.Direction.horizontal;
  }
  componentWillLoad() {
    const { size, color } = this;
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        let buttonDisabled = StringHelper.elementBooleanCheck(this.el.getAttribute('disabled')) || button.getAttribute('disabled');
        color && button.setAttribute('color', color);
        // type && button.setAttribute('type', type);
        size && button.setAttribute('size', size);
        buttonDisabled && (button.disabled = true);
      });
  }
  getButtons() {
    return snq(() => Array.from(this.el.querySelectorAll('bcm-button')), []);
  }
  radioControl(id) {
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        button.checked = button.id == id;
      });
  }
  checkboxControl(id) {
    const buttons = this.getButtons();
    (buttons === null || buttons === void 0 ? void 0 : buttons.length) &&
      buttons.forEach(button => {
        button.checked = button.id == id ? (button.checked = !button.checked) : button.checked;
      });
  }
  listenButton({ target }) {
    if (target.id) {
      switch (this.type) {
        case 'radio':
          this.radioControl(target.id);
          break;
        case 'checkbox':
          this.checkboxControl(target.id);
          break;
      }
      this.handleChange();
    }
  }
  handleChange() {
    const obj = this.getButtons()
      .filter(Boolean)
      .map(button => ({
      id: button.id,
      checked: button.checked,
    }));
    this.bcmChange.emit(obj);
  }
  render() {
    const hostClasses = classnames('bcm-button-group', {
      'bcm-button-group__vertical': this.direction === Bcm.Direction.vertical,
    });
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  get el() { return this; }
  static get style() { return buttonGroupCss; }
}, [4, "bcm-button-group", {
    "_id": [1537, "id"],
    "size": [1025],
    "type": [1025],
    "color": [1032],
    "disabled": [1032],
    "direction": [1025]
  }, [[0, "bcm-click", "listenButton"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-button-group"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-button-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmButtonGroup$1);
      }
      break;
  } });
}

const BcmButtonGroup = BcmButtonGroup$1;
const defineCustomElement = defineCustomElement$1;

export { BcmButtonGroup, defineCustomElement };
