/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';
import { C as CaptionTemplate } from './caption-template.js';
import { c as classnames } from './index2.js';
import './types.js';
import { L as LabelTemplate, d as defineCustomElement$6 } from './label.js';
import './types2.js';
import { d as defineCustomElement$a } from './avatar.js';
import { d as defineCustomElement$9 } from './badge.js';
import { d as defineCustomElement$8 } from './chip.js';
import { d as defineCustomElement$7 } from './icon.js';
import { d as defineCustomElement$5 } from './list.js';
import { d as defineCustomElement$4 } from './rc-overflow.js';
import { d as defineCustomElement$3 } from './search.js';
import { d as defineCustomElement$2 } from './tooltip.js';

const chipGroupCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}.bcm-chip-group .bcm-chip{cursor:pointer}";

const BcmChipGroup$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-change", 7);
    this._id = Generate.UID();
    this.name = undefined;
    this.value = undefined;
    this.required = false;
    this.size = "medium";
    this.label = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = Bcm.Status.default;
    this.captionError = undefined;
  }
  handleSelectItem(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      const data = e.detail;
      const chipItem = document.createElement("bcm-chip");
      // chipItem.setAttribute("type", "dismissable");
      chipItem.setAttribute("value", data.id);
      chipItem.innerHTML = data.text;
      target.parentNode.insertBefore(chipItem, target);
      this.setSelect(data.id);
    }
  }
  handleDismiss(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      target.remove();
      this.setSelect();
    }
  }
  handleSelect(e) {
    const target = e === null || e === void 0 ? void 0 : e.target;
    if (target instanceof HTMLElement) {
      if (target.type === "addable") {
        this.showListComponent(target.id);
      }
      else {
        this.setSelect(target.value);
      }
    }
  }
  showListComponent(id) {
    const listComponent = document.getElementById(id + "-list");
    if (listComponent) {
      listComponent.show();
    }
  }
  getSlots() {
    return this.el.querySelectorAll("bcm-chip");
  }
  setSelect(val = this.value, noEvent = false) {
    const slots = this.getSlots();
    if ((slots === null || slots === void 0 ? void 0 : slots.length) === 0)
      return;
    const slotArray = Array.from(slots);
    const selectedSlot = slotArray.find(slot => slot.value === val);
    if (!selectedSlot) {
      this.value = slotArray[0].value;
    }
    else {
      this.value = val;
    }
    slotArray.forEach(slot => {
      slot.selected = slot.value === this.value;
    });
    if (!noEvent) {
      this.change.emit(this.value);
    }
  }
  setAttr() {
    const slots = this.getSlots();
    if ((slots === null || slots === void 0 ? void 0 : slots.length) === 0)
      return;
    const slotArray = Array.from(slots);
    slotArray.forEach(slot => {
      slot.setAttribute("size", this.size);
    });
  }
  componentWillLoad() {
    this.setSelect();
  }
  componentDidRender() {
    this.setAttr();
  }
  render() {
    const { label, required, noCaption, captionType, captionError, caption } = this;
    const hostClasses = classnames("bcm-chip-group");
    return (h(Host, { class: hostClasses }, label && (h("div", null, h(LabelTemplate, { type: captionType, value: label, required: required }))), h("div", { class: "tw-flex tw-flex-wrap tw-gap-1" }, h("slot", null)), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption })));
  }
  get el() { return this; }
  static get style() { return chipGroupCss; }
}, [4, "bcm-chip-group", {
    "_id": [1537, "id"],
    "name": [1],
    "value": [1032],
    "required": [4],
    "size": [1],
    "label": [1],
    "caption": [1],
    "noCaption": [4, "no-caption"],
    "captionType": [1, "caption-type"],
    "captionError": [1, "caption-error"]
  }, [[0, "bcm-chip-select-item", "handleSelectItem"], [0, "bcm-chip-dismiss", "handleDismiss"], [0, "bcm-chip-select", "handleSelect"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-chip-group", "bcm-avatar", "bcm-badge", "bcm-chip", "bcm-icon", "bcm-label", "bcm-list", "bcm-rc-overflow", "bcm-search", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-chip-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmChipGroup$1);
      }
      break;
    case "bcm-avatar":
      if (!customElements.get(tagName)) {
        defineCustomElement$a();
      }
      break;
    case "bcm-badge":
      if (!customElements.get(tagName)) {
        defineCustomElement$9();
      }
      break;
    case "bcm-chip":
      if (!customElements.get(tagName)) {
        defineCustomElement$8();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$7();
      }
      break;
    case "bcm-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$6();
      }
      break;
    case "bcm-list":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-rc-overflow":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-search":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmChipGroup = BcmChipGroup$1;
const defineCustomElement = defineCustomElement$1;

export { BcmChipGroup, defineCustomElement };
