/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, forceUpdate, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import './bcm.js';
import { d as defineCustomElement$7 } from './colorful.js';
import { d as defineCustomElement$6 } from './icon.js';
import { d as defineCustomElement$5 } from './input.js';
import { d as defineCustomElement$4 } from './label.js';
import { d as defineCustomElement$3 } from './linked.js';
import { d as defineCustomElement$2 } from './tooltip.js';

const BcmColorInput$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this._id = Generate.UID();
    this.label = "";
    this.placeholder = "";
    this.value = "";
    this.required = false;
    this.disabled = false;
    this.readonly = false;
    this.fullWidth = false;
  }
  handleChange(color) {
    this.value = color;
    this.getInputElement().setValue(color);
    forceUpdate(this.el);
  }
  getInputElement() {
    return this.el.querySelector(`bcm-input`);
  }
  getPickerElement() {
    return document.querySelector("#" + this._id + "-picker");
  }
  handleInputChanged(e) {
    this.getPickerElement().color = e.detail;
  }
  eyeDropper() {
    const eyeDropper = new window.EyeDropper();
    eyeDropper
      .open()
      .then((color) => {
      this.handleChange(color.sRGBHex);
    })
      .catch((error) => {
      if (error.name === "NotAllowedError") {
        console.log("Permission to access sensor was denied.");
      }
      else if (error.name === "NotSupportedError") {
        console.log("Sensor not supported.");
      }
    });
  }
  isEyeDropperSupported() {
    return typeof window !== "undefined" && "EyeDropper" in window;
  }
  render() {
    return (h(Host, { class: "tw-block" }, h("bcm-input", { id: this._id + "-color", fullWidth: this.fullWidth, label: this.label, placeholder: this.placeholder, type: "text", "onBcm-change": e => this.handleInputChanged(e), value: this.value }, h("i", { slot: "prefix", class: "tw-flex tw-h-4 tw-w-4 tw-items-center tw-justify-center tw-rounded-full tw-border tw-border-gray-100", style: {
        backgroundColor: this.value,
      } }), this.isEyeDropperSupported() && h("i", { slot: "suffix", class: "far fa-eye-dropper", onClick: () => this.eyeDropper() })), h("bcm-linked", { targetId: this._id + "-color-container" }, h("bcm-colorful", { id: this._id + "-picker", color: this.value, onBcmChange: e => this.handleChange(e.detail) }))));
  }
  get el() { return this; }
}, [0, "bcm-color-input", {
    "_id": [1537, "id"],
    "label": [1],
    "placeholder": [1],
    "value": [1025],
    "required": [4],
    "disabled": [4],
    "readonly": [4],
    "fullWidth": [4, "full-width"]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-color-input", "bcm-colorful", "bcm-icon", "bcm-input", "bcm-label", "bcm-linked", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-color-input":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmColorInput$1);
      }
      break;
    case "bcm-colorful":
      if (!customElements.get(tagName)) {
        defineCustomElement$7();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$6();
      }
      break;
    case "bcm-input":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-linked":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmColorInput = BcmColorInput$1;
const defineCustomElement = defineCustomElement$1;

export { BcmColorInput, defineCustomElement };
