/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { I as IsLoad } from './is-load-decorator.js';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';
import { C as CaptionTemplate } from './caption-template.js';
import { c as classnames } from './index2.js';
import './types.js';
import './types2.js';
import { h as hooks, d as defineCustomElement$3 } from './rc-picker.js';
import { d as defineCustomElement$5 } from './icon.js';
import { d as defineCustomElement$4 } from './label.js';
import { d as defineCustomElement$2 } from './tooltip.js';

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const BcmDate$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-change", 7);
    this.onChange = (e) => {
      this.value = this.getValueObject(e.detail);
      this.change.emit(this.value);
    };
    // type a göre format belirleme işlemi
    this.getFormat = () => {
      if (!this.format) {
        const hourFormat = this.use12Hours ? "hh" : "HH";
        const timeFormat = `${this.showSecond ? `${hourFormat}:mm:ss` : this.showMinute ? `${hourFormat}:mm` : `${hourFormat}`}${this.use12Hours ? " a" : ""}`;
        switch (this.type) {
          case "time":
            return `${timeFormat}`;
          case "date":
            return "DD/MM/YYYY";
          case "datetime":
            return `DD/MM/YYYY ${timeFormat}`;
          case "week":
            return "YYYY-wo";
          case "month":
            return "MM/YYYY";
          case "year":
            return "YYYY";
          default:
            return "DD/MM/YYYY";
        }
      }
      return this.format;
    };
    this.getValueObject = (value) => {
      if (this.range) {
        return this.getRangeValueObject(value);
      }
      else {
        return this.createValueObject(value);
      }
    };
    this.getRangeValueObject = (value) => {
      const [start, end] = value || [null, null];
      const startObject = this.createValueObject(start);
      const endObject = this.createValueObject(end);
      return Object.assign({ isValid: startObject.isValid && endObject.isValid, start: startObject, end: endObject, date: [startObject.date, endObject.date], timestamp: [startObject.timestamp, endObject.timestamp], value: `${startObject.value} - ${endObject.value}` }, (this.type === "time" && {
        time: `${startObject.time} - ${endObject.time}`,
      }));
    };
    this.createValueObject = (value) => {
      if (value && hooks(value).isValid()) {
        return Object.assign({ isValid: hooks(value).isValid(), date: hooks(value).toDate(), timestamp: hooks(value).valueOf(), dateObject: {
            year: hooks(value).year(),
            day: hooks(value).date(),
            month: hooks(value).month(),
            dayOfWeek: hooks(value).day(),
            monthFirstDay: hooks(value).startOf("month").day(),
          }, value: hooks(value).format(this.getFormat()) }, (this.type === "time" && {
          time: hooks(value).format(this.getFormat()),
          timeObject: {
            hour: hooks(value).hour(),
            minute: hooks(value).minute(),
            second: hooks(value).second(),
          },
        }));
      }
      else {
        return null;
      }
    };
    this._id = Generate.UID();
    this.label = undefined;
    this.placeholder = undefined;
    this.size = Bcm.Size.medium;
    this.fullWidth = false;
    this.required = false;
    this.clearable = false;
    this.disabled = false;
    this.noMargin = false;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = Bcm.Status.default;
    this.captionError = undefined;
    this.name = undefined;
    this.value = undefined;
    this.type = "date";
    this.format = undefined;
    this.range = false;
    this.showTime = false;
    this.showNow = true;
    this.showHour = true;
    this.showMinute = true;
    this.showSecond = false;
    this.use12Hours = false;
    this.hourStep = 1;
    this.minuteStep = 1;
    this.secondStep = 1;
    this.hideDisabledOptions = false;
    this.disabledTime = undefined;
    this.showToday = true;
    this.disabledDates = undefined;
  }
  async getValue() {
    return this.value;
  }
  async get() {
    var _a;
    return ((_a = this.value) === null || _a === void 0 ? void 0 : _a.timestamp) || null;
  }
  async set(value) {
    // this.value = value;
    this.getPickerElement().set(value);
  }
  async setClear() {
    this.getPickerElement().set(null);
  }
  async resetCaption() {
    this.captionType = "default";
    this.captionError = null;
  }
  async getInput() {
    return this.el.querySelector("input");
  }
  async isValid() {
    var _a;
    return ((_a = this.getValueObject(this.value)) === null || _a === void 0 ? void 0 : _a.isValid) || false;
  }
  getPickerElement() {
    return this.el.querySelector("bcm-rc-picker");
  }
  render() {
    const { label, size, captionType, required, _id } = this;
    return (h(Host, { class: classnames("bcm-ui-element bcm-ui-input", "tw-relative tw-m-0 tw-inline-flex tw-w-full tw-flex-col", {
        "tw-max-w-[256px]": !this.fullWidth,
        "tw-mb-2": !this.noMargin,
      }) }, label && h("bcm-label", { size: size, type: captionType, value: label, required: required, htmlFor: _id }), h("bcm-rc-picker", { id: _id + "-picker", class: classnames("w-full tw-flex"), onBcmRCPickerChange: e => this.onChange(e), disabled: this.disabled, clearable: this.clearable, size: size, placeholder: this.placeholder, error: this.captionType === "error", type: this.type, format: this.getFormat(), range: this.range, showToday: this.showToday, disabledDates: this.disabledDates, showTime: this.showTime, showNow: this.showNow, showHour: this.showHour, showMinute: this.showMinute, showSecond: this.showSecond, use12Hours: this.use12Hours, hourStep: this.hourStep, minuteStep: this.minuteStep, secondStep: this.secondStep, hideDisabledOptions: this.hideDisabledOptions, disabledTime: this.disabledTime, ariaLabel: label || this.placeholder || `${this.type} picker` }), h(CaptionTemplate, { noCaption: this.noCaption, captionType: this.captionType, captionError: this.captionError, caption: this.caption })));
  }
  get el() { return this; }
}, [0, "bcm-date", {
    "_id": [513, "id"],
    "label": [1],
    "placeholder": [1],
    "size": [1],
    "fullWidth": [4, "full-width"],
    "required": [4],
    "clearable": [4],
    "disabled": [4],
    "noMargin": [4, "no-margin"],
    "caption": [1],
    "noCaption": [4, "no-caption"],
    "captionType": [1025, "caption-type"],
    "captionError": [1025, "caption-error"],
    "name": [1],
    "value": [1032],
    "type": [1],
    "format": [8],
    "range": [4],
    "showTime": [4, "show-time"],
    "showNow": [4, "show-now"],
    "showHour": [4, "show-hour"],
    "showMinute": [4, "show-minute"],
    "showSecond": [4, "show-second"],
    "use12Hours": [4, "use-12-hours"],
    "hourStep": [2, "hour-step"],
    "minuteStep": [2, "minute-step"],
    "secondStep": [2, "second-step"],
    "hideDisabledOptions": [4, "hide-disabled-options"],
    "disabledTime": [8, "disabled-time"],
    "showToday": [4, "show-today"],
    "disabledDates": [8, "disabled-dates"],
    "getValue": [64],
    "get": [64],
    "set": [64],
    "setClear": [64],
    "resetCaption": [64],
    "getInput": [64],
    "isValid": [64]
  }]);
__decorate([
  IsLoad()
], BcmDate$1.prototype, "el", void 0);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-date", "bcm-icon", "bcm-label", "bcm-rc-picker", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-date":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmDate$1);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-rc-picker":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmDate = BcmDate$1;
const defineCustomElement = defineCustomElement$1;

export { BcmDate, defineCustomElement };
