/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { G as Generate } from './generate.js';
import { c as classnames } from './index2.js';
import { S as StringHelper } from './string-helper.js';
import { C as ColorHelper } from './color-helper.js';
import { d as defineCustomElement$3 } from './icon.js';
import { d as defineCustomElement$2 } from './tooltip.js';

const expansionPanelCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");@import url(\"https://fonts.googleapis.com/css2?family=Inter:wght@200;400;500;600&display=swap\"); .size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}*{box-sizing:border-box}.bcm-expansion-panel{font-family:\"Inter\", sans-serif;font-size:14px;line-height:calc(14px + 8px);font-weight:400;display:block;width:100%;background-color:var(--bcm-new-ds-color-white);border-radius:8px;border:1px solid var(--bcm-new-ds-color-slate-300);box-sizing:border-box}.bcm-expansion-panel+.bcm-expansion-panel{margin-top:4px}.bcm-expansion-panel__header{display:flex;align-items:center;justify-content:flex-start;user-select:none;min-height:50px;border-radius:8px;padding:4px 24px;color:var(--bcm-new-ds-color-slate-700);font-weight:600}.bcm-expansion-panel__header-content{display:flex;flex-direction:row;align-items:center;justify-content:flex-start;flex-wrap:nowrap;gap:8px}.bcm-expansion-panel__header:hover{cursor:pointer}.bcm-expansion-panel__header-title{order:1;user-select:none;width:100%;display:flex;flex-direction:row;align-items:center;justify-content:flex-start;gap:4px}.bcm-expansion-panel__header-title-description{margin-left:calc(4px / 2);color:var(--bcm-new-ds-color-slate-500);font-size:12px;line-height:calc(12px + 8px)}.bcm-expansion-panel__header-tooltip{line-height:1;color:var(--bcm-new-ds-color-slate-800)}.bcm-expansion-panel__header-toggle{height:100%;order:0;width:20px;display:flex;align-items:center;justify-content:center;transform:rotate(0deg);transition:transform 0.2s ease-in-out;font-size:12px}.bcm-expansion-panel__header-extra{display:none}.bcm-expansion-panel__header-extra:not(:empty){cursor:default;display:flex;align-items:center;order:5;line-height:1;padding:0 8px}.bcm-expansion-panel__header-extra .bcm-expansion-panel__extra-content{display:flex;align-items:center}.bcm-expansion-panel__body{display:none;padding:12px 24px}.bcm-expansion-panel--icon-right .bcm-expansion-panel__header{justify-content:space-between;padding-left:24px}.bcm-expansion-panel--icon-right .bcm-expansion-panel__header-toggle{order:6}.bcm-expansion-panel--invalid .bcm-expansion-panel__header-toggle,.bcm-expansion-panel--invalid .bcm-expansion-panel__header-content{color:var(--bcm-new-ds-color-red-500)}.bcm-expansion-panel--expanded{border-bottom:1px solid var(--bcm-color-grey-5)}.bcm-expansion-panel--expanded .bcm-expansion-panel__header{border-radius:8px 8px 0 0}.bcm-expansion-panel--expanded .bcm-expansion-panel__header-toggle{transform:rotate(180deg)}.bcm-expansion-panel--expanded .bcm-expansion-panel__body{display:block}";

const BcmExpansionPanel$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.bcmOpen = createEvent(this, "bcm-open", 7);
    this.bcmClose = createEvent(this, "bcm-close", 7);
    this.bcmExtra = createEvent(this, "bcm-extra", 7);
    this._id = Generate.UID();
    this.expanded = false;
    this.disabled = false;
    this.hidden = false;
    this.valid = true;
    this.hideToggle = false;
    this.expandIconPosition = 'right';
    this.headerColor = 'white';
    this.tooltip = undefined;
    this.tooltipDefaultValue = {
      message: '',
      placement: 'top',
      trigger: 'hover',
      icon: 'fas fa-question-circle',
      color: 'grey-8',
    };
  }
  componentWillRender() {
    this.restoreSlots();
  }
  componentDidLoad() {
    this.checkExpanded();
  }
  createTooltip() {
    let tool;
    if (this.tooltip) {
      const { tooltip } = this;
      try {
        tool = JSON.parse(tooltip);
        tool = Object.assign(Object.assign({}, this.tooltipDefaultValue), tool);
      }
      catch (error) {
        tool = Object.assign(Object.assign({}, this.tooltipDefaultValue), { message: tooltip });
      }
    }
    return tool;
  }
  checkExpanded() {
    this.expanded = !!this.el['expanded'];
  }
  restoreSlots() {
    const slots = this.el.querySelectorAll('bcm-expansion-panel-header, bcm-expansion-panel-description, bcm-expansion-panel-extra');
    if (slots) {
      slots.forEach(element => {
        let slotName = StringHelper.lowercase(StringHelper.split(element.nodeName, '-').pop());
        element.slot = slotName;
        element.classList.add(`bcm-expansion-panel__${slotName}-content`);
        let color = element.getAttribute('color');
        if (color) {
          color = color == 'white' ? color : color + '-50';
          this.headerColor = color;
        }
      });
    }
  }
  toggle() {
    if (this.disabled) {
      return;
    }
    this.expanded = !this.expanded;
    if (this.expanded) {
      this.bcmOpen.emit(this._id);
    }
    else {
      this.bcmClose.emit(this._id);
    }
  }
  clickExtra(e) {
    e.stopPropagation();
    this.bcmExtra.emit(e);
  }
  render() {
    const { _id, expanded, disabled, hidden, hideToggle, tooltip, valid } = this;
    const { message, placement, icon, trigger, color } = tooltip ? this.createTooltip() : {};
    const hostClasses = classnames('bcm-expansion-panel', valid ? 'bcm-expansion-panel--valid' : 'bcm-expansion-panel--invalid', {
      'bcm-expansion-panel--expanded': expanded,
      'bcm-expansion-panel--disabled': disabled,
      'bcm-expansion-panel--hide-toggle': hideToggle,
      'bcm-expansion-panel--icon-right': this.expandIconPosition === 'right',
      hidden,
    });
    return (h(Host, { class: hostClasses }, h("div", { class: "bcm-expansion-panel__header", onClick: () => this.toggle(), style: { 'background-color': ColorHelper.dsColor(this.headerColor) } }, h("div", { class: "bcm-expansion-panel__header-title" }, h("span", { id: _id + '-header' }, h("slot", { name: "header" })), tooltip && (h("span", { class: "bcm-expansion-panel__header-tooltip", style: Object.assign({}, color && { color: ColorHelper.convertColorToHex(color) }) }, h("bcm-tooltip", { message: message, placement: placement, trigger: trigger, "aria-label": message }, h("bcm-icon", { color: color, icon: icon })))), h("span", { id: _id + '-description', class: "bcm-expansion-panel__header-title-description" }, h("slot", { name: "description" }))), h("span", { id: _id + '-extra', class: "bcm-expansion-panel__header-extra", onClick: (e) => this.clickExtra(e) }, h("slot", { name: "extra" })), !hideToggle && (h("div", { class: "bcm-expansion-panel__header-toggle" }, h("bcm-icon", { icon: "far fa-chevron-down" })))), h("div", { class: "bcm-expansion-panel__body", id: _id + '-body' }, h("slot", null))));
  }
  get el() { return this; }
  static get style() { return expansionPanelCss; }
}, [4, "bcm-expansion-panel", {
    "_id": [1537, "id"],
    "expanded": [1028],
    "disabled": [1028],
    "hidden": [1028],
    "valid": [1028],
    "hideToggle": [1028, "hide-toggle"],
    "expandIconPosition": [1025, "expand-icon-position"],
    "tooltip": [1032],
    "headerColor": [32],
    "tooltipDefaultValue": [32]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-expansion-panel", "bcm-icon", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-expansion-panel":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmExpansionPanel$1);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmExpansionPanel = BcmExpansionPanel$1;
const defineCustomElement = defineCustomElement$1;

export { BcmExpansionPanel, defineCustomElement };
