/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as classnames } from './index2.js';
import { d as defineCustomElement$3 } from './button.js';
import { d as defineCustomElement$2 } from './icon.js';

/**
 * 'type' prop predefined values
 */
var TypeProps;
(function (TypeProps) {
  TypeProps["info"] = "info";
  TypeProps["error"] = "error";
  TypeProps["warning"] = "warning";
  TypeProps["success"] = "success";
  TypeProps["default"] = "default";
})(TypeProps || (TypeProps = {}));

const notificationCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}:host{display:block}:host(.hidden){display:none}.notification{display:none;width:100%;max-width:384px;padding:16px 24px;box-shadow:0px 4px 12px rgba(0, 0, 0, 0.15);border-radius:2px}.notification.show{display:flex}.text-content{width:100%}.title{position:relative;display:block;padding-right:24px;padding-top:8px;padding-bottom:8px;font-size:16px;color:var(--bcm-color-grey-9)}.title bcm-icon{position:absolute;right:0;top:10px;cursor:pointer}p{margin-top:8px;margin-bottom:0;font-size:14px;line-height:22px;color:var(--bcm-color-grey-8)}.icon-content{padding-top:8px;padding-right:16px}.actions{display:flex;align-items:center;justify-content:flex-end;margin-top:16px}";

const BcmNotification$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.confirm = createEvent(this, "bcm-confirm", 7);
    this._title = undefined;
    this.status = TypeProps.default;
    this.confirmText = undefined;
    this.isShow = false;
    this.hidden = false;
  }
  /**
   * @desc
   * @returns {BcmOldIcon}
   */
  getIcon() {
    switch (this.status) {
      case TypeProps.info:
        return h("bcm-icon", { icon: "far fa-info-circle" });
      case TypeProps.error:
        return h("bcm-icon", { icon: "far fa-times-circle" });
      case TypeProps.warning:
        return h("bcm-icon", { icon: "far fa-exclamation-circle" });
      case TypeProps.success:
        return h("bcm-icon", { icon: "far fa-check-circle" });
    }
  }
  /**
   * @desc Removes component itself
   */
  dismiss() {
    this.isShow = false;
    // this.el.remove()
  }
  /**
   * @desc Removes component itself
   */
  handleConfirm() {
    this.isShow = false;
    this.confirm.emit();
  }
  /**
   * @desc
   */
  async hide() {
    this.dismiss();
  }
  /**
   * @desc
   */
  async show() {
    this.isShow = true;
  }
  render() {
    const { status, _title, confirmText, isShow } = this;
    const notification = classnames('notification', {
      'show': isShow
    });
    const hostClasses = classnames(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: notification }, status !== TypeProps.default && (h("div", { class: "icon-content" }, this.getIcon())), h("div", { class: "text-content" }, h("span", { class: "title" }, _title, h("bcm-icon", { icon: "far fa-times", color: "grey-10", onClick: () => this.dismiss() })), h("p", null, h("slot", null)), confirmText != undefined && (h("div", { class: "actions" }, h("bcm-button", { size: "small", onClick: () => this.handleConfirm() }, confirmText ? confirmText : "Confirm")))))));
  }
  get el() { return this; }
  static get style() { return notificationCss; }
}, [1, "bcm-notification", {
    "_title": [1025, "title"],
    "status": [1025],
    "confirmText": [1032, "confirm"],
    "isShow": [1540, "open"],
    "hidden": [1028],
    "hide": [64],
    "show": [64]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-notification", "bcm-button", "bcm-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-notification":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmNotification$1);
      }
      break;
    case "bcm-button":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmNotification = BcmNotification$1;
const defineCustomElement = defineCustomElement$1;

export { BcmNotification, defineCustomElement };
