/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as classnames } from './index2.js';
import { G as Generate } from './generate.js';
import { a as Bcm } from './bcm.js';
import { d as defineCustomElement$5 } from './button.js';
import { d as defineCustomElement$4 } from './icon.js';
import { d as defineCustomElement$3 } from './popconfirm-box.js';
import { d as defineCustomElement$2 } from './text.js';

const popconfirmCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}.bcm-popconfirm{display:inline-flex}";

const BcmPopconfirm$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.onShow = createEvent(this, "bcm-show", 7);
    this.onHide = createEvent(this, "bcm-hide", 7);
    this.onConfirm = createEvent(this, "bcm-confirm", 7);
    this.onCancel = createEvent(this, "bcm-cancel", 7);
    this._id = Generate.UID();
    this.message = undefined;
    this.status = 'warning';
    this.cancelText = "No";
    this.okeyText = "Yes";
    this.placement = Bcm.Placement.top;
    this.trigger = Bcm.TriggerType.click;
    this.open = false;
    this.hidden = false;
  }
  async show() {
    // if (this.open) return
    this.open = true;
    this.onShow.emit();
    await this.createBox().then(() => this.showBox());
  }
  async hide() {
    // if (!this.open) return
    this.open = false;
    this.onHide.emit();
    // this.hideBox()
  }
  async createBox() {
    let element = this.getPopconfirmBox();
    if (!element) {
      const box = document.createElement('bcm-popconfirm-box');
      box.setAttribute('id', 'box-' + this._id);
      box.setAttribute('target-id', this._id);
      box.setAttribute('placement', this.placement);
      box.setAttribute('message', this.message);
      box.setAttribute('status', this.status);
      box.setAttribute('cancel-text', this.cancelText);
      box.setAttribute('okey-text', this.okeyText);
      this.el.parentElement.appendChild(box);
    }
    return Promise.resolve();
  }
  getPopconfirmBox() {
    return document.querySelector(`bcm-popconfirm-box[id="box-${this._id}"]`);
  }
  showBox() {
    this.getPopconfirmBox() && this.getPopconfirmBox().show();
  }
  hideBox() {
    this.getPopconfirmBox() && this.getPopconfirmBox().hide();
  }
  listenTrigger(event) {
    switch (this.trigger) {
      case 'click':
        event === 'click' && (this.open ? this.hide() : this.show());
        break;
      case 'hover':
        event === 'onMouseOver' && this.show();
        event === 'onMouseOut' && this.hide();
        break;
      case 'focus':
        event === 'onFocus' && this.show();
        event === 'onBlur' && this.hide();
        break;
    }
  }
  onBcmConfirm(e) {
    e.detail == this._id && (this.onConfirm.emit(this._id));
  }
  onBcmCancel(e) {
    e.detail == this._id && (this.onCancel.emit(this._id));
  }
  render() {
    const popconfirmClasses = classnames('bcm-popconfirm');
    return (h(Host, { class: popconfirmClasses, onclick: () => this.listenTrigger('click') }, h("slot", null)));
  }
  get el() { return this; }
  static get style() { return popconfirmCss; }
}, [4, "bcm-popconfirm", {
    "_id": [1537, "id"],
    "message": [1025],
    "status": [1025],
    "cancelText": [1025, "cancel-text"],
    "okeyText": [1025, "okey-text"],
    "placement": [1025],
    "trigger": [1025],
    "open": [1028],
    "hidden": [1028],
    "show": [64],
    "hide": [64]
  }, [[4, "bcm-popconfirm-confirm", "onBcmConfirm"], [4, "bcm-popconfirm-cancel", "onBcmCancel"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-popconfirm", "bcm-button", "bcm-icon", "bcm-popconfirm-box", "bcm-text"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-popconfirm":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmPopconfirm$1);
      }
      break;
    case "bcm-button":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-popconfirm-box":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-text":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmPopconfirm = BcmPopconfirm$1;
const defineCustomElement = defineCustomElement$1;

export { BcmPopconfirm, defineCustomElement };
