/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { d as delay } from './utils.js';
import { B as Bcm } from './bcm.js';
import { C as CaptionTemplate } from './caption-template.js';
import { c as classnames } from './index2.js';
import './types.js';
import './types2.js';
import { I as IsLoad } from './is-load-decorator.js';
import { d as defineCustomElement$5 } from './icon.js';
import { d as defineCustomElement$4 } from './label.js';
import { d as defineCustomElement$3 } from './radio.js';
import { d as defineCustomElement$2 } from './tooltip.js';

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const BcmRadioGroup$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-change", 7);
    this._id = Generate.UID();
    this.label = undefined;
    this.name = undefined;
    this.value = null;
    this.defaultValue = undefined;
    this.size = Bcm.Size.medium;
    this.disabled = undefined;
    this.hidden = false;
    this.required = false;
    this.gap = undefined;
    this.direction = Bcm.Direction.horizontal;
    this.fullWidth = false;
    this.items = [];
    this.caption = undefined;
    this.noCaption = false;
    this.captionError = undefined;
    this.captionType = Bcm.Status.default;
    this.captionCache = undefined;
    this.captionTypeCache = undefined;
  }
  get radios() {
    return Array.from(this.el.querySelectorAll("bcm-radio"));
  }
  get Items() {
    return this.items;
  }
  connectedCallback() {
    this.captionCache = this.caption;
    this.captionTypeCache = this.captionType;
  }
  updateValue() {
    var _a, _b, _c;
    this.value = (_c = (_b = (_a = this.radios) === null || _a === void 0 ? void 0 : _a.find(radio => radio.checked)) === null || _b === void 0 ? void 0 : _b.value) !== null && _c !== void 0 ? _c : null;
  }
  setRadioUpdate() {
    var _a;
    (_a = this.radios) === null || _a === void 0 ? void 0 : _a.forEach(radio => {
      radio.disabled = this.disabled || radio.disabled;
      radio.size = this.size;
      if (this.defaultValue) {
        radio.checked = radio.value == this.defaultValue;
      }
    });
  }
  watchDisabled(newVal, _oldVal) {
    var _a;
    (_a = this.radios) === null || _a === void 0 ? void 0 : _a.forEach(radio => radio.disabled = newVal);
  }
  watchDefaultValue(newVal, _oldVal) {
    var _a;
    (_a = this.radios) === null || _a === void 0 ? void 0 : _a.forEach(radio => radio.checked = radio.value == newVal);
    this.updateValue();
    this.change.emit(this.value);
  }
  componentWillLoad() {
    this.setRadioUpdate();
    this.updateValue();
  }
  handleChange(e) {
    var _a, _b;
    e.stopPropagation();
    e.preventDefault();
    const checked = (_a = e.target.checked) !== null && _a !== void 0 ? _a : false;
    (_b = this.radios) === null || _b === void 0 ? void 0 : _b.forEach(radio => {
      radio.checked = radio.id == e.target.id ? !checked : false;
    });
    this.updateValue();
    this.change.emit(this.value);
  }
  ;
  async setClear() {
    var _a;
    this.value = null;
    (_a = this.radios) === null || _a === void 0 ? void 0 : _a.forEach(radio => {
      radio.checked = false;
    });
  }
  async resetCaption() {
    await delay(10);
    this.caption = this.captionCache;
    this.captionType = this.captionTypeCache;
    this.captionError = null;
    await delay(10);
    return Promise.resolve();
  }
  async check(itemId, checked = true) {
    var _a;
    (_a = this.radios) === null || _a === void 0 ? void 0 : _a.forEach(radio => {
      radio.checked = radio.id == itemId || radio.value == itemId ? checked : false;
    });
    this.updateValue();
    this.change.emit(this.value);
  }
  async get() {
    return this.value;
  }
  async set(data) {
    await this.check(data);
  }
  async getValue() {
    return this.value;
  }
  render() {
    var _a, _b;
    const { direction, fullWidth, gap, label, size, required, disabled, caption, captionError, captionType, noCaption } = this;
    return (h(Host, { class: "tw-mb-2 tw-flex tw-flex-col", role: "radiogroup", "aria-labelledby": "radiogroup" }, label && h("bcm-label", { type: captionType, value: label, required: required }), h("div", { class: classnames("tw-flex tw-flex-wrap", direction == Bcm.Direction.horizontal ? "tw-flex-row" : "tw-flex-col", fullWidth && "tw-w-full"), style: { "gap": gap } }, h("slot", null), ((_a = this.Items) === null || _a === void 0 ? void 0 : _a.length) > 0 &&
      ((_b = this.Items) === null || _b === void 0 ? void 0 : _b.map(item => (h("bcm-radio", { id: item.id, value: item.value, disabled: disabled || item.disabled, checked: item.checked, color: item.color, size: size }, item.label))))), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption })));
  }
  get el() { return this; }
  static get watchers() { return {
    "disabled": ["watchDisabled"],
    "defaultValue": ["watchDefaultValue"]
  }; }
}, [4, "bcm-radio-group", {
    "_id": [513, "id"],
    "label": [1],
    "name": [1],
    "value": [1032],
    "defaultValue": [1032, "default-value"],
    "size": [1],
    "disabled": [4],
    "hidden": [4],
    "required": [4],
    "gap": [1],
    "direction": [1],
    "fullWidth": [4, "full-width"],
    "items": [1],
    "caption": [1],
    "noCaption": [4, "no-caption"],
    "captionError": [1, "caption-error"],
    "captionType": [1, "caption-type"],
    "captionCache": [32],
    "captionTypeCache": [32],
    "setClear": [64],
    "resetCaption": [64],
    "check": [64],
    "get": [64],
    "set": [64],
    "getValue": [64]
  }, [[0, "bcm-radio-change", "handleChange"]]]);
__decorate([
  IsLoad()
], BcmRadioGroup$1.prototype, "el", void 0);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-radio-group", "bcm-icon", "bcm-label", "bcm-radio", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-radio-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmRadioGroup$1);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$5();
      }
      break;
    case "bcm-label":
      if (!customElements.get(tagName)) {
        defineCustomElement$4();
      }
      break;
    case "bcm-radio":
      if (!customElements.get(tagName)) {
        defineCustomElement$3();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmRadioGroup = BcmRadioGroup$1;
const defineCustomElement = defineCustomElement$1;

export { BcmRadioGroup, defineCustomElement };
