/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';
import { c as classnames } from './index2.js';

const osKeys = {
  ctrl: "⌘",
  alt: "⌥",
  shift: "⇧",
  del: "⌫",
  delete: "⌫",
  backspace: "⌦",
  enter: "⏎",
  return: "⏎",
};
const BcmShortcut$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.isMac = /Mac/.test(window.navigator.userAgent);
    this.hotkey = undefined;
    this.color = "slate";
    this.size = Bcm.Size.small;
  }
  getHotkey(key) {
    if (!key) {
      return "";
    }
    if (this.isMac && (key.includes(" ") || key.includes("+"))) {
      const keys = key.split(/[\s+]+/);
      return keys.map(k => osKeys[k] || k).join("");
    }
    else {
      return key;
    }
  }
  render() {
    const { size, color, hotkey } = this;
    return (h(Host, { class: classnames("bcm-ui-element bcm-ui-content-display", "tw-block tw-h-fit tw-w-fit tw-rounded tw-border tw-border-solid tw-px-1", "tw-border-default-300 tw-bg-default-100 tw-text-default-700", "tw-text-center tw-font-medium tw-capitalize", {
        "tw-text-2": size === Bcm.Size.small,
        "tw-text-3": size === Bcm.Size.medium,
        "tw-text-4": size === Bcm.Size.large,
      }), style: Object.assign({}, Generate.getTwColor({ color })) }, this.getHotkey(hotkey)));
  }
}, [0, "bcm-shortcut", {
    "hotkey": [1],
    "color": [1],
    "size": [1]
  }]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-shortcut"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-shortcut":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmShortcut$1);
      }
      break;
  } });
}

const BcmShortcut = BcmShortcut$1;
const defineCustomElement = defineCustomElement$1;

export { BcmShortcut, defineCustomElement };
