/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, proxyCustomElement, HTMLElement, createEvent } from '@stencil/core/internal/client';
import { G as Generate } from './generate.js';
import { a as Bcm } from './bcm.js';
import { c as classnames } from './index2.js';
import { N as NumberHelper } from './number-helper.js';
import { e as extractColor } from './utils.js';
import { C as ColorPalette } from './colors.js';
import { C as ColorProps } from './types2.js';
import { d as defineCustomElement$2 } from './icon.js';

const TagTemplate = ({ _id, color, customStyle, size, checked, hidden, value, type, onDismiss, onSelect }) => {
  !size && (size = Bcm.FontSize['size-1']);
  const getColor = (color) => {
    if ((type === Bcm.TagType.basic || type == Bcm.TagType.dismissable) && ColorProps[color])
      return {
        color: extractColor(ColorPalette, ColorProps[color] + '-6'),
        borderColor: extractColor(ColorPalette, ColorProps[color] + '-3'),
        backgroundColor: extractColor(ColorPalette, ColorProps[color] + '-1')
      };
  };
  const getSize = (size, returnNumber = false) => {
    if (NumberHelper.isNumber(size)) {
      size = Bcm.FontSize["size-" + size] || Bcm.FontSize["size-4"];
    }
    if (returnNumber) {
      return +(Bcm.FontSizeValue[size] || Bcm.FontSizeValue['size-4']) + 2;
    }
    return size;
  };
  const classes = classnames(getSize(size), 'bcm-tag__content', type);
  const hostClasses = classnames('bcm-tag', {
    hidden
  });
  return (h("div", { id: _id, class: classnames(hostClasses, { checked }) },
    h("span", { hidden: true },
      h("slot", null)),
    h("span", { class: classes, style: getColor(color), onClick: (e) => onSelect(e) },
      type === Bcm.TagType.add && (h("span", { class: "left" },
        h("bcm-icon", { icon: "far fa-plus" }))),
      type === Bcm.TagType.checkable && h("input", { type: "checkbox", checked: checked }),
      h("span", { class: "value", style: customStyle }, value),
      type === Bcm.TagType.dismissable && (h("span", { class: "right close-button", onClick: (e) => onDismiss(e) },
        h("bcm-icon", { icon: "fal fa-times" }))))));
};

const BcmTag$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.change = createEvent(this, "bcm-change", 7);
    this.tagDismissable = createEvent(this, "bcm-tag-dismissable", 7);
    this.bcmTagSelect = createEvent(this, "bcm-tag-select", 7);
    this._id = Generate.UID();
    this._internal_id = Generate.UID();
    this.hidden = false;
    this.value = '';
    this.type = Bcm.TagType.basic;
    this.checked = false;
    this.color = undefined;
    this.customStyle = undefined;
    this.itemObject = undefined;
    this.size = Bcm.FontSize['size-1'];
    this.closeFunc = (e) => e;
    this.selectFunc = (e) => e;
  }
  handleClick() {
    this.type === Bcm.TagType.checkable && this.handleChecked();
  }
  handleClose() {
    this.tagDismissable.emit(this.el);
  }
  handleChecked() {
    this.checked = !this.checked;
    this.change.emit(this.checked);
  }
  render() {
    const { type, color, value, customStyle, size, hidden, checked } = this;
    return (h(TagTemplate, { size: size, hidden: hidden, color: color, customStyle: customStyle, checked: checked, value: value, type: Bcm.TagType[type], onSelect: (e) => { this.bcmTagSelect.emit(e); this.selectFunc({ event: e, id: this._id }); }, onDismiss: (e) => { this.handleClose(); this.closeFunc(e); } }));
  }
  get el() { return this; }
}, [4, "bcm-tag", {
    "_id": [1537, "id"],
    "_internal_id": [1537],
    "hidden": [1540],
    "value": [1025],
    "type": [1025],
    "checked": [1028],
    "color": [1537],
    "customStyle": [1032, "custom-style"],
    "itemObject": [1537, "item-object"],
    "size": [1032],
    "closeFunc": [1040],
    "selectFunc": [1040]
  }, [[0, "click", "handleClick"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-tag", "bcm-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-tag":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmTag$1);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
  } });
}

const BcmTag = BcmTag$1;
const defineCustomElement = defineCustomElement$1;

export { BcmTag, defineCustomElement };
