/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { B as Bcm } from './bcm.js';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import { c as classnames } from './index2.js';

const BcmRadio = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.radioChange = createEvent(this, "bcm-radio-change", 7);
    this.focus = createEvent(this, "bcm-focus", 7);
    this.blur = createEvent(this, "bcm-blur", 7);
    this._id = Generate.UID();
    this.checked = undefined;
    this.value = undefined;
    this.disabled = false;
    this.size = Bcm.Size.medium;
    this.color = undefined;
  }
  render() {
    const { value, size, disabled, checked, _id, color } = this;
    return (h(Host, { class: classnames("tw-cursor-pointer tw-font-sans tw-mb-1.5 tw-mt-1", {
        "tw-cursor-pointer tw-text-slate-600": !disabled,
        "tw-cursor-not-allowed tw-text-slate-400": disabled,
        "tw-text-1": size === Bcm.Size.small,
        "tw-text-2.5": size === Bcm.Size.medium,
        "tw-text-4": size === Bcm.Size.large,
      }), style: (Generate.getTwColor(color)) }, h("label", { class: classnames("tw-inline-flex tw-flex-row tw-gap-2 tw-mr-2 tw-items-center", disabled ? "tw-cursor-not-allowed" : "tw-cursor-pointer"), htmlFor: _id + '-input' }, h("input", { role: "radio", "aria-label": "radio", id: _id + '-input', type: "radio", value: value, checked: checked, disabled: disabled, class: classnames("tw-flex tw-items-center tw-justify-center", "tw-relative tw-appearance-none tw-border tw-border-solid tw-rounded-full tw-cursor-pointer tw-transition tw-ease-in-out ", "tw-m-0", "before:tw-content-[''] before:tw-absolute before:tw-rounded-full", {
        "tw-w-4 tw-h-4 before:tw-w-2.5 before:tw-h-2.5": size === Bcm.Size.medium,
        "tw-w-3.5 tw-h-3.5 before:tw-w-2 before:tw-h-2": size === Bcm.Size.small,
        "tw-w-4.5 tw-h-4.5 before:tw-w-3 before:tw-h-3": size === Bcm.Size.large,
      }, "focus:tw-outline-none focus:tw-shadow-outline", 
      // default
      `tw-border-slate-400 tw-bg-slate-50`, `checked:tw-border-default-500`, `checked:before:tw-bg-default-500`, 
      // disabled
      `disabled:tw-border-slate-400 disabled:tw-bg-slate-200 disabled:tw-cursor-not-allowed`, `disabled:checked:tw-border-slate-400`, `disabled:checked:before:tw-bg-slate-400`, !disabled && classnames(
      // hover
      `hover:tw-border-default-600`, `hover:checked:tw-border-default-600`, `hover:checked:before:tw-bg-default-600`, 
      // focus-visible 
      `focus-visible:tw-border-default-800 focus-visible:tw-shadow-sm`, `focus-visible:checked:tw-border-default-800`, `focus-visible:checked:before:tw-bg-default-800`, 
      // active
      `active:tw-border-default-700`, `active:checked:tw-border-default-700`, `active:checked:before:tw-bg-default-700`)), onChange: () => this.radioChange.emit(), onFocus: () => this.focus.emit(), onBlur: () => this.blur.emit() }), h("slot", null))));
  }
  get el() { return this; }
}, [4, "bcm-radio", {
    "_id": [513, "id"],
    "checked": [4],
    "value": [8],
    "disabled": [4],
    "size": [1],
    "color": [8]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-radio"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-radio":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmRadio);
      }
      break;
  } });
}

export { BcmRadio as B, defineCustomElement as d };
