/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import { S as StringHelper } from './string-helper.js';
import './element-dragger.js';
import './bcm.js';
import { F as ForwardOverflow } from './Overflow.js';
import { r as react, a as reactDom } from './index3.js';
import { d as defineCustomElement$2 } from './icon.js';
import { d as defineCustomElement$1 } from './tooltip.js';

const rcOverflowCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}.rc-overflow{display:flex;flex-direction:row;flex-wrap:nowrap;width:100%}.rc-overflow-item{transition:all 0.3s ease;padding:0 2px;display:flex;align-items:center;justify-content:center}";

const BcmRcOverflow = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.selectOtherTag = createEvent(this, "bcm-select-other-tag", 7);
    this.selectTag = createEvent(this, "bcm-select-tag", 7);
    this.deleteTag = createEvent(this, "bcm-delete-tag", 7);
    this.renderItem = item => {
      return react.createElement("bcm-tooltip", {
        placement: "bottom",
        message: item.text,
        trigger: "hover",
      }, react.createElement("div", {
        id: item.value + "-tag",
        className: "bcm-tag",
        onClick: ({ target }) => this.onClickControl(target, item, false),
      }, react.createElement("span", { className: "bcm-tag__content dismissable" }, 
      // .bcm-tag__content span {
      //     display: block;
      //     max-width: 38px;
      //     white-space: nowrap;
      //     overflow: hidden;
      //     text-overflow: ellipsis;
      react.createElement("div", { className: "value tw-block tw-max-w-[38px] tw-overflow-hidden tw-overflow-ellipsis tw-whitespace-nowrap" }, item.text), react.createElement("span", {
        className: "right close-button",
      }, react.createElement("bcm-icon", {
        icon: "fa fa-times",
      })))));
    };
    this.renderRest = items => {
      const titles = (items === null || items === void 0 ? void 0 : items.sort((a, b) => +a.ct - +b.ct).slice(0, 15).map(item => StringHelper.textLimitter(item.text, 40, true)).toString().split(",").join("\r\n")) + (items.length > 15 ? "\r\n..." : "");
      return react.createElement("bcm-tooltip", {
        placement: "bottom",
        message: titles,
        trigger: "hover",
      }, react.createElement("div", {
        className: "bcm-tag bcm-tag--other",
        onClick: ({ target }) => this.onClickControl(target, items, true),
      }, react.createElement("span", { className: `bcm-tag__content dismissable ${this.checked ? "checkable checked" : ""}` }, react.createElement("span", { className: "value" }, `+${items === null || items === void 0 ? void 0 : items.length}`), react.createElement("span", { className: "right close-button" }, react.createElement("bcm-icon", {
        icon: "fa fa-times",
      })))));
    };
    this._id = Generate.UID();
    this.maxCount = 1;
    this.data = [];
    this.checked = false;
  }
  isClickDismiss(e) {
    return e["s-hn"] === "BCM-ICON";
  }
  onClickControl(e, item, isOtherTag = false) {
    const isClickClose = this.isClickDismiss(e);
    isClickClose && this.deleteTag.emit({ id: Array.isArray(item) ? item.map(i => i.value) : item.value }) && this.checked && (this.checked = false);
    !isOtherTag && this.selectTag.emit({ id: item.value }) && this.checked && (this.checked = false);
    if (isOtherTag && !isClickClose) {
      this.checked = !this.checked;
      this.selectOtherTag.emit({ id: item === null || item === void 0 ? void 0 : item.map(i => i.value), checked: this.checked });
    }
  }
  componentWillRender() {
    const { data, renderItem, renderRest, maxCount } = this;
    this.overflowInstance = react.createElement(ForwardOverflow, {
      data,
      renderItem,
      renderRest,
      maxCount,
    });
    reactDom.render(this.overflowInstance, this.el);
  }
  get el() { return this; }
  static get style() { return rcOverflowCss; }
}, [0, "bcm-rc-overflow", {
    "_id": [513, "id"],
    "maxCount": [8, "max-count"],
    "data": [16],
    "checked": [32]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-rc-overflow", "bcm-icon", "bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-rc-overflow":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmRcOverflow);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$2();
      }
      break;
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { BcmRcOverflow as B, defineCustomElement as d };
