/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { s as snq, G as Generate } from './generate.js';
import { a as Bcm } from './bcm.js';
import { I as InputTemplate } from './input-template.js';
import { d as delay } from './utils.js';
import { c as createStore } from './index4.js';
import { S as StringHelper } from './string-helper.js';
import { d as defineCustomElement$1 } from './icon.js';

const { state } = createStore({
  value: new Map(),
});
const setValue = val => {
  const { id, dataSource } = val;
  const data = {
    id: snq(() => id, Generate.UID),
    dataSource: dataSource && dataSource.length > 0 && recursive(dataSource),
    searchIsOnlyChilds: val.searchIsOnlyChilds,
  };
  state.value.set(data.id, data);
};
const recursive = (data, parentId) => {
  let recData = [];
  if (data) {
    recData = data.map(item => (Object.assign(Object.assign(Object.assign({}, item), { parentId: snq(() => parentId, null) }), (item.items && item.items.length > 0 && { items: snq(() => recursive(item.items, item.id), []) }))));
  }
  return recData;
};
const removeValue = _id => {
  state.value.delete(_id);
};
const getValue = id => snq(() => state.value.get(id), {});
const getDataPagination = (id, limit = 3) => {
  let data = [];
  const value = getValue(id);
  if (value) {
    const { dataSource } = value;
    data = dataSource.slice(0, limit);
  }
  return data;
};
// //search treeview data by text
// const searchData = (id, search, items?) => {
//     let dataSource
//     if (items && items.length > 0) {
//         dataSource = items
//     } else {
//         const value = getValue(id);
//         dataSource = value.dataSource;
//     }
//     let result = []
//     if (dataSource) {
//         dataSource.forEach(child => {
//             if (child.text.toLowerCase().includes(search.toLowerCase())) {
//                 result.push(child)
//             }
//             result = result.concat(searchData(id, search, child.items))
//         })
//     };
//     return result
// };
//nested search
const searchData = (id, searchText, items) => {
  let dataSource;
  if (items && items.length > 0) {
    dataSource = items;
  }
  else {
    const value = getValue(id);
    dataSource = value.dataSource;
  }
  let results = [];
  if (dataSource) {
    results.push(nestedSearch(id, dataSource, searchText));
    //return with their parents
    // dataSource.forEach(child => {
    //     if (child.text.toLowerCase().includes(searchText.toLowerCase())) {
    //         results.push(child)
    //     } else if (child.items && child.items.length > 0) {
    //         child.items = nestedSearch(child.items, searchText)
    //         if (child.items.length > 0) {
    //             results.push(child)
    //         }
    //     }
    // })
  }
  return {
    result: snq(() => Generate.flatArray(results[0].result), []),
    matching: snq(() => results[0].matching, []),
    parentIndex: snq(() => results[0].parentIndex, []),
    nestedResult: snq(() => results[0].nestedResult, []),
  };
};
const nestedSearch = (id, items, searchText) => {
  let matching = [];
  let parentIndex = [];
  let result = [];
  let nestedResult = [];
  const { searchIsOnlyChilds } = getValue(id);
  if (items && items.length > 0) {
    items.forEach(child => {
      let isChild = searchIsOnlyChilds && child.parentId !== null;
      let isMatching = (child.text && StringHelper.slugify(child.text).includes(StringHelper.slugify(searchText))) ||
        (child.keywords && StringHelper.slugify(child.keywords).includes(StringHelper.slugify(searchText)));
      if (isMatching && (!searchIsOnlyChilds || (searchIsOnlyChilds && isChild))) {
        result.push(child.id);
        matching.push(child.id);
        parentIndex.push(child.id);
        nestedResult.push({ id: child.id, text: child.text, parentId: child.parentId });
        if (child.items && child.items.length > 0) {
          const found = nestedSearch(id, child.items, searchText);
          if (found.matching.length > 0) {
            found.matching.forEach(chi => {
              matching.push(chi);
            });
          }
        }
      }
      else if (child.items && child.items.length > 0) {
        const found = nestedSearch(id, child.items, searchText);
        if (found.result.length > 0) {
          found.result.forEach(chi => {
            result.push(chi);
          });
          result.push(child.id);
          parentIndex.push(child.id);
          nestedResult.push({ id: child.id, text: child.text, parentId: child.parentId, items: found.nestedResult });
        }
        if (found.matching.length > 0) {
          matching.push(...found.matching);
        }
      }
    });
  }
  return { result, matching, parentIndex, nestedResult };
};
const searchState = {
  setValue,
  removeValue,
  getValue,
  getDataPagination,
  searchData,
};

const searchCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");@import url(\"https://fonts.googleapis.com/css2?family=Inter:wght@200;400;500;600&display=swap\"); .size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}.bcm-input[no-margin]{margin:0}.bcm-textarea[no-margin]{margin:0}.bcm-switch[no-margin]{margin:0}.bcm-checkbox[no-margin]{margin:0}.bcm-radio-group[no-margin]{margin:0}.bcm-checkbox-group[no-margin]{margin:0}.bcm-range[no-margin]{margin:0}.bcm-form[no-margin]{margin:0}.bcm-form-group[no-margin]{margin:0}.bcm-list[no-margin]{margin:0}.bcm-colorpicker[no-margin]{margin:0}.bcm-date-picker[no-margin]{margin:0}.bcm-time-picker[no-margin]{margin:0}.bcm-datetime-picker[no-margin]{margin:0}.bcm-select[no-margin]{margin:0}.bcm-listbox[no-margin]{margin:0}*{box-sizing:border-box}.bcm-input{display:inline-flex;flex-direction:column;width:100%;max-width:256px;margin-bottom:8px}.bcm-input.full-width{max-width:100%}.bcm-input.hidden{display:none}.bcm-input__container{font-family:\"Inter\", sans-serif;font-size:14px;line-height:calc(14px + 8px);font-weight:400;display:flex;align-items:stretch;flex-direction:row;background-color:var(--bcm-new-ds-color-white);border:1px solid var(--bcm-new-ds-color-slate-300);border-radius:4px;box-sizing:border-box;cursor:text;padding:0 0 0 4px;position:relative;width:100%}.bcm-input__container.disabled{background-color:var(--bcm-new-ds-color-slate-50);cursor:not-allowed}.bcm-input__container.disabled .bcm-input-element{color:var(--bcm-new-ds-color-slate-500)}.bcm-input__container.readonly{background-color:var(--bcm-new-ds-color-slate-50);cursor:default}.bcm-input__container.readonly .bcm-input-element{color:var(--bcm-new-ds-color-slate-600)}.bcm-input__container:not(.disabled):hover .caret-container,.bcm-input__container:not(.disabled).focused .caret-container{opacity:1}.bcm-input__container:not(.disabled):hover .input-clear-button,.bcm-input__container:not(.disabled).focused .input-clear-button{opacity:1}.bcm-input__container:not(.disabled).focused{background-color:var(--bcm-new-ds-color-slate-50)}.bcm-input__container:not(.disabled):hover{border-color:var(--bcm-color-prime-blue-6)}.bcm-input__container.empty .input-clear-button{display:none}.bcm-input__container.invalid,.bcm-input__container.error{border-color:var(--bcm-new-ds-color-red-500)}.bcm-input__container .bcm-input-element{width:100%;color:var(--bcm-new-ds-color-slate-600);flex:1;border:none;background:transparent;box-shadow:unset;padding:0 4px;font-family:\"Inter\", sans-serif;cursor:inherit;-webkit-appearance:none;-moz-appearance:none;appearance:none;outline:none;text-overflow:ellipsis}.bcm-input__container .bcm-input-element::placeholder{font-family:\"Inter\", sans-serif;color:var(--bcm-new-ds-color-slate-300);user-select:none}.bcm-input__container .bcm-input-element::-webkit-search-decoration,.bcm-input__container .bcm-input-element::-webkit-search-cancel-button,.bcm-input__container .bcm-input-element::-webkit-search-results-button,.bcm-input__container .bcm-input-element::-webkit-search-results-decoration,.bcm-input__container .bcm-input-element::-webkit-calendar-picker-indicator,.bcm-input__container .bcm-input-element::-webkit-inner-spin-button,.bcm-input__container .bcm-input-element::-webkit-outer-spin-button{-webkit-appearance:none}.bcm-input__container .caret-container{width:30px;display:flex;flex-direction:column;justify-content:center;align-items:stretch;border-left:1px solid #D9D9D9;transition:0.2s opacity;user-select:none;box-sizing:border-box}.bcm-input__container .caret-container .bcm-input-caret{display:flex;align-items:center;justify-content:center;color:var(--bcm-color-grey-7);cursor:pointer;flex:1}.bcm-input__container .caret-container .bcm-input-caret bcm-icon{line-height:1}.bcm-input__container .caret-container .bcm-input-caret:hover{color:var(--bcm-color-prime-blue-6)}.bcm-input__container .caret-container .bcm-input-caret:active{background-color:var(--bcm-color-grey-3)}.bcm-input__container .caret-container .bcm-input-caret:last-child{border-top:1px solid #D9D9D9}.bcm-input__container .caret-container.disabled{background-color:var(--bcm-color-grey-3)}.bcm-input__container .caret-container.disabled .bcm-input-caret{cursor:not-allowed}.bcm-input__container .input-suffix-content{display:flex;align-items:center;justify-content:center}.bcm-input__container .input-prefix:not(:empty){padding-left:4px}.bcm-input__container .input-suffix:not(:empty){min-width:24px;padding-right:4px}.bcm-input__container .input-prefix:not(:empty),.bcm-input__container .input-suffix:not(:empty){display:flex;align-items:center;justify-content:center;cursor:default;color:#64748a}.bcm-input__container .input-prefix:not(:empty) .unit-prefix,.bcm-input__container .input-suffix:not(:empty) .unit-prefix{color:#64748a}.bcm-input__container .input-prefix-button:not(:empty),.bcm-input__container .input-suffix-button:not(:empty){cursor:pointer}.bcm-input__container .caret-container,.bcm-input__container .input-clear-button{opacity:0}.bcm-input__container--small{font-size:12px}.bcm-input__container--medium{font-size:14px}.bcm-input__container--large{font-size:16px}.bcm-input__textarea.clearable .bcm-input-element{padding-right:24px}.bcm-input__textarea .input-clear-button{position:absolute;top:4px;right:0}";

const BcmSearch = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.bcmOnSearch = createEvent(this, "bcm-on-search", 7);
    this.bcmOnSearchClear = createEvent(this, "bcm-on-search-clear", 7);
    this.debounceHandler = Generate.debounceInput((event) => this.onSearchValue(event, this), 300, false, () => {
      var _a, _b;
      this.isActiveClearable = ((_b = (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.length) > 0 && this.clearable;
    });
    this._id = Generate.UID();
    this.placeholder = "Search";
    this.disabled = false;
    this.readonly = false;
    this.clearable = true;
    this.size = Bcm.Size.medium;
    this.limit = 10;
    this.searchIsOnlyChilds = false;
    this.minSearchLength = 2;
    this.data = [];
    this.tick = {};
    this.isActiveClearable = false;
  }
  async componentDidLoad() {
    await this.initState();
  }
  async dataChanged(newValue, oldValue) {
    // console.log('data changed', newValue, oldValue)
    if (newValue !== oldValue) {
      await this.initState();
    }
  }
  async initState() {
    await searchState.setValue({
      id: this._id + "-input",
      dataSource: this.data,
      searchIsOnlyChilds: this.searchIsOnlyChilds,
    });
    await this.markForCheck();
  }
  markForCheck() {
    this.tick = Generate.UID();
  }
  handleInput() {
    // console.log("aheyya")
  }
  async onBcmFocus() {
    this.inputElement.focus();
  }
  async onBcmBlur() {
    this.inputElement.blur();
  }
  async onBcmSelect() {
    this.inputElement.select();
  }
  async setValue(newValue) {
    this.inputElement.value = newValue;
  }
  handleFocus() { }
  handleBlur() { }
  handleChange() { }
  async handleClear(eventEmit = true) {
    this.inputElement.value = '';
    const { value, id } = this.inputElement;
    searchState.searchData(id, value);
    this.isActiveClearable = false;
    this.bcmOnSearch.emit({ result: null, value: null, matching: null, parentIndex: null });
    this.bcmOnSearchClear.emit(eventEmit);
    await delay(10);
    return Promise.resolve();
  }
  onSearchValue(event, instance) {
    const { value, id } = event.target;
    const searchResult = (value === null || value === void 0 ? void 0 : value.length) >= this.minSearchLength ? searchState.searchData(id, value, null) : null;
    const { bcmOnSearch, inputElement } = instance;
    bcmOnSearch.emit({ result: searchResult === null || searchResult === void 0 ? void 0 : searchResult.result, value: inputElement === null || inputElement === void 0 ? void 0 : inputElement.value, matching: searchResult === null || searchResult === void 0 ? void 0 : searchResult.matching, parentIndex: searchResult === null || searchResult === void 0 ? void 0 : searchResult.parentIndex, nestedResult: searchResult === null || searchResult === void 0 ? void 0 : searchResult.nestedResult });
  }
  render() {
    const { _id, placeholder, disabled, readonly } = this;
    return h(Host, { onClick: () => { this.onBcmFocus(); this.onBcmSelect(); } }, h(InputTemplate, { size: this.size, id: _id, type: "search", placeholder: placeholder, disabled: disabled, "full-width": true, readonly: readonly, clearable: this.isActiveClearable, handleFocus: () => this.handleFocus(), handleBlur: () => this.handleBlur(), handleChange: () => this.handleChange(), handleClear: () => this.handleClear(), handleInput: this.debounceHandler, ref: (el) => this.inputElement = el }));
  }
  get el() { return this; }
  static get watchers() { return {
    "data": ["dataChanged"]
  }; }
  static get style() { return searchCss; }
}, [4, "bcm-search", {
    "_id": [1537, "id"],
    "placeholder": [1025],
    "disabled": [1028],
    "readonly": [1028],
    "clearable": [1028],
    "size": [1025],
    "limit": [1026],
    "searchIsOnlyChilds": [1028, "search-is-only-childs"],
    "minSearchLength": [2, "min-search-length"],
    "data": [1032],
    "tick": [32],
    "isActiveClearable": [32],
    "onBcmFocus": [64],
    "onBcmBlur": [64],
    "onBcmSelect": [64],
    "setValue": [64],
    "handleClear": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-search", "bcm-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-search":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmSearch);
      }
      break;
    case "bcm-icon":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { BcmSearch as B, defineCustomElement as d };
