/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import './element-dragger.js';
import './bcm.js';
import { c as ce } from './index5.js';

const skeletonStyle = ce({
  slots: {
    host: "tw-flex tw-flex-row tw-items-center tw-justify-center tw-flex-shrink-0 tw-relative tw-overflow-hidden tw-bg-gray-200",
  },
  variants: {
    variant: {
      rect: {
        host: "tw-rounded",
      },
      circle: {
        host: "tw-rounded-full",
      },
    },
    effect: {
      pulse: {
        host: "tw-animate-pulse",
      },
      ping: {
        host: "tw-animate-ping",
      },
      none: {
        host: "tw-animate-none",
      },
    },
    hidden: {
      true: {
        host: "tw-hidden",
      },
      false: {
        host: "tw-flex",
      },
    }
  },
  defaultVariants: {
    variant: "rect",
    effect: "pulse",
    hidden: false,
  },
});

const BcmSkeleton = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this._id = Generate.UID();
    this.effect = "pulse";
    this.variant = "rect";
    this.hidden = false;
    this.width = "100%";
    this.height = "16px";
    this.marginBottom = null;
    this.customStyles = {};
  }
  render() {
    const { variant, effect, width, height, marginBottom, customStyles, hidden } = this;
    const style = Object.assign(Object.assign(Object.assign(Object.assign({}, (width ? { width } : {})), (height ? { height } : {})), (marginBottom ? { marginBottom } : {})), (typeof customStyles === "string" ? JSON.parse(customStyles) : customStyles));
    const { host } = skeletonStyle({ variant, effect, hidden });
    return (h(Host, { class: host(), style: style }));
  }
}, [0, "bcm-skeleton", {
    "_id": [1537, "id"],
    "effect": [513],
    "variant": [513],
    "hidden": [516],
    "width": [1],
    "height": [1],
    "marginBottom": [1, "margin-bottom"],
    "customStyles": [1, "custom-styles"]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-skeleton"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-skeleton":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmSkeleton);
      }
      break;
  } });
}

export { BcmSkeleton as B, defineCustomElement as d };
