/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { forceUpdate, proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as classnames } from './index2.js';
import { G as Generate, s as snq } from './generate.js';
import { c as createStore } from './index4.js';

const { state } = createStore({
  value: new Map(),
  setData: {},
});
const setValue = val => {
  const data = Object.assign(Object.assign({}, val), { steps: val.steps &&
      val.steps.length > 0 &&
      val.steps
        .filter(data => !data.hidden)
        .map((item, index) => ({
        id: item.id || Generate.UID(),
        title: item.title || item._title,
        _title: item.title || item._title || 'Step ' + (index + 1),
        description: item.description || '',
        status: item.status || 'default',
        disabled: val.linear ? (index == 0 ? false : true) : item.disabled || false,
        icon: item.icon || undefined,
        valid: item.valid || true,
        loading: item.loading || false,
        index: index + 1,
        active: val.linear ? (index == 0 ? true : false) : val.active === index + 1,
        isLast: index === val.steps.length - 1,
        statusIcon: val.icon,
        headerHidden: item.hidden,
      })) });
  state.value.set(val.id, data);
};
const removeState = _id => {
  state.value.delete(_id);
};
const changeActiveStep = (id, index) => {
  const value = state.value.get(id);
  if (value) {
    value.active = index;
    value.steps = value.steps.map((step, i) => (Object.assign(Object.assign({}, step), { active: i === index - 1 })));
  }
};
const getActiveIndex = id => snq(() => state.value.get(id).active, 0);
const changeStepStatus = (id, stepId, status) => {
  const value = state.value.get(id);
  if (value && value.steps) {
    value.steps = value.steps.map(step => (Object.assign(Object.assign({}, step), { status: step.id === stepId ? status : step.status })));
    forceUpdate(value.el);
  }
};
const changeStepDisabled = (id, stepId, disabled) => {
  const value = state.value.get(id);
  if (value && value.steps) {
    value.steps = value.steps.map(step => (Object.assign(Object.assign({}, step), { disabled: step.id === stepId ? disabled : step.disabled })));
    forceUpdate(value.el);
  }
};
const setActiveIndex = (id, index) => state.value.get(id).active = index;
const stepperReset = id => {
  const value = state.value.get(id);
  if (value) {
    value.active = 1;
    value.steps = value.steps.map((step, i) => (Object.assign(Object.assign({}, step), { active: i === 0, status: 'default', disabled: i !== 0, loading: false })));
    forceUpdate(value.el);
  }
};
const getValue = id => snq(() => state.value.get(id), {});
const stepperStates = {
  removeState,
  getValue,
  setValue,
  changeActiveStep,
  getActiveIndex,
  changeStepStatus,
  changeStepDisabled,
  stepperReset,
  setActiveIndex
};

const BcmStep = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.dataIsLoaded = createEvent(this, "data-is-loaded", 7);
    this.bcmChangeHidden = createEvent(this, "bcm-change-hidden", 7);
    this.hidden = false;
    this._id = Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = 'default';
    this.icon = undefined;
    this.loading = false;
    this.valid = true;
    this.index = undefined;
    this.active = false;
    this.htmlTemplate = undefined;
    this.isLast = false;
    this.disabled = false;
  }
  componentDidLoad() {
    if (this.isLast) {
      this.dataIsLoaded.emit(true);
    }
  }
  getStepper() {
    const stepper = this.el.closest('bcm-stepper');
    return stepper || null;
  }
  activeChanged() {
    if (this.active) {
      const stepper = this.getStepper();
      if (stepper) {
        stepperStates.changeStepDisabled(stepper.stepperId, this._id, false);
      }
    }
  }
  statusChange() {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepStatus(stepper.stepperId, this._id, this.status);
    }
  }
  hiddenChange(newValue, oldValue) {
    const stepper = this.getStepper();
    if (newValue !== oldValue && stepper && !this.active) {
      this.bcmChangeHidden.emit(newValue);
    }
  }
  disabledChange(newValue, _) {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepDisabled(stepper.stepperId, this._id, newValue);
    }
  }
  render() {
    const { hidden, valid, active } = this;
    const hostClasses = classnames('bcm-stepper__step', valid ? 'bcm-stepper__step-valid' : 'bcm-stepper__step-invalid', {
      'bcm-stepper__step-active': active,
      hidden,
    });
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  get el() { return this; }
  static get watchers() { return {
    "active": ["activeChanged"],
    "status": ["statusChange"],
    "hidden": ["hiddenChange"],
    "disabled": ["disabledChange"]
  }; }
}, [4, "bcm-step", {
    "hidden": [1028],
    "_id": [1537, "id"],
    "_title": [1025],
    "description": [1025],
    "status": [1025],
    "icon": [1025],
    "loading": [1028],
    "valid": [1028],
    "index": [1026],
    "active": [1540],
    "htmlTemplate": [1025, "html-template"],
    "isLast": [1028, "is-last"],
    "disabled": [1028]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-step"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-step":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmStep);
      }
      break;
  } });
}

export { BcmStep as B, defineCustomElement as d, stepperStates as s };
