/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import './color-helper.js';
import './validators.js';
import { G as Generate } from './generate.js';
import './number-helper.js';
import './string-helper.js';
import { T as Tooltip } from './tooltip-helper.js';
import './element-dragger.js';
import { B as Bcm } from './bcm.js';

const BcmTooltip = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.bcmTooltipShow = createEvent(this, "bcmTooltipShow", 7);
    this.bcmTooltipHide = createEvent(this, "bcmTooltipHide", 7);
    this._id = Generate.UID();
    this.message = "";
    this.placement = Bcm.Placement.top;
    this.trigger = Bcm.TriggerType.click;
    this.open = false;
    this.color = "slate";
    this.size = Bcm.Size.medium;
    this.openDelay = 0;
    this.closeDelay = 0;
    this.autoHide = false;
    this.targetId = undefined;
    this.tooltip = undefined;
  }
  emitShow() {
    this.bcmTooltipShow.emit();
  }
  emitHide() {
    this.bcmTooltipHide.emit();
  }
  connectedCallback() {
    this.tooltip = new Tooltip({
      targetId: this.targetId || this._id,
      message: this.message,
      placement: this.placement,
      color: this.color,
      trigger: this.trigger,
      size: this.size,
      openDelay: this.openDelay,
      closeDelay: this.closeDelay,
      autoHide: this.autoHide,
      emitShow: () => this.emitShow(),
      emitHide: () => this.emitHide(),
    });
  }
  componentDidLoad() {
    this.tooltip.listener();
  }
  componentDidRender() {
    this.tooltip.message = this.message;
    this.tooltip.placement = this.placement;
    this.tooltip.color = this.color;
    this.tooltip.trigger = this.trigger;
    this.tooltip.size = this.size;
    this.tooltip.openDelay = this.openDelay;
    this.tooltip.closeDelay = this.closeDelay;
    this.tooltip.autoHide = this.autoHide;
    this.tooltip.update();
  }
  disconnectedCallback() {
    this.tooltip.destroy();
  }
  async show() {
    this.tooltip.show();
  }
  async hide() {
    this.tooltip.hide();
  }
  async toggle() {
    this.tooltip.toggle();
  }
  render() {
    return (h(Host, { class: "tw-inline-flex tw-h-fit tw-w-fit", role: "tooltip" }, h("slot", null)));
  }
  get el() { return this; }
}, [4, "bcm-tooltip", {
    "_id": [513, "id"],
    "message": [1],
    "placement": [1],
    "trigger": [1],
    "open": [1028],
    "color": [1],
    "size": [1],
    "openDelay": [2, "open-delay"],
    "closeDelay": [2, "close-delay"],
    "autoHide": [4, "auto-hide"],
    "targetId": [1, "target-id"],
    "tooltip": [32],
    "show": [64],
    "hide": [64],
    "toggle": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bcm-tooltip"];
  components.forEach(tagName => { switch (tagName) {
    case "bcm-tooltip":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, BcmTooltip);
      }
      break;
  } });
}

export { BcmTooltip as B, defineCustomElement as d };
