/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import './color-helper-74614f24.js';
import './datetime-helper-df25808a.js';
import { G as Generate } from './generate-d009624a.js';
import './number-helper-b7a7a318.js';
import './string-helper-3592bdc2.js';
import './validators-5519219e.js';
import './element-dragger-89bb52fd.js';
import { d as delay } from './utils-57652744.js';
import { F as FloatingUI } from './floating-ui-3f4394da.js';
import { B as Bcm } from './bcm-a50d55ba.js';
import { c as ce } from './index-3bc8c6e3.js';
import './colors-1d7854ea.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';

const attendanceActionCommentStyle = ce({
  slots: {
    host: [
      // "tw-flex tw-h-fit tw-w-full tw-min-w-[288px] tw-flex-col tw-items-start tw-rounded tw-p-4 tw-shadow-2",
      "tw-w-fit",
      "tw-absolute tw-left-0 tw-right-0 tw-z-tooltip tw-opacity-100 tw-transition-opacity tw-duration-300",
      "tw-rounded tw-bg-white tw-shadow",
      "bcm-ui-element bcm-attendance-actions tw-flex tw-flex-col tw-items-start tw-justify-start tw-p-4 tw-font-medium tw-text-slate-700",
    ],
    footer: ["tw-flex tw-w-full tw-flex-row tw-items-center tw-justify-end tw-gap-2 "],
  },
  variants: {
    size: {
      small: {},
      medium: {},
      large: {},
    },
  },
  defaultVariants: {
    size: "medium",
  },
});

const BcmAttendanceActionsComment = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.change = createEvent(this, "bcm-attendance-actions-comment-change", 3);
    this.lateMinutes = null;
    this.comment = null;
    this._id = Generate.UID();
    this.size = Bcm.Size.medium;
    this.disabled = false;
    this.selectedKey = null;
    this.targetId = null;
    this.isShow = false;
    this.isLate = false;
    this.selectedItem = null;
  }
  handleSave(lateMinutes, comment) {
    this.change.emit({ lateMinutes, comment });
    this.hide();
  }
  async show() {
    const targetElement = document.getElementById(this.targetId);
    FloatingUI({
      ref: targetElement,
      floating: this.el,
      placement: "bottom-end",
      offset: 8,
      arrow: true,
      arrowColor: "tw-bg-white",
    });
    this.isShow = true;
    await delay(100);
    if (this.isLate) {
      this.focusInput();
    }
    else {
      this.focusTextarea();
    }
  }
  focusInput() {
    const input = this.el.querySelector("bcm-input");
    input === null || input === void 0 ? void 0 : input.onBcmSelect();
  }
  focusTextarea() {
    const textarea = this.el.querySelector("bcm-textarea");
    textarea === null || textarea === void 0 ? void 0 : textarea.onBcmSelect();
  }
  async hide() {
    if (this.isShow) {
      this.el.remove();
    }
  }
  handleClickOutside(event) {
    if (!Generate.findEventPath(event, this.el)) {
      this.hide();
    }
  }
  async formSubmit() {
    var _a, _b, _c;
    const { lateMinutes, comment, isLate } = this;
    if (isLate) {
      if (lateMinutes) {
        if (!(lateMinutes >= 0 && lateMinutes <= 200)) {
          return;
        }
      }
    }
    const { isValid } = await ((_a = this.el.querySelector("bcm-form")) === null || _a === void 0 ? void 0 : _a.submit());
    if (!isValid)
      return;
    this.handleSave(lateMinutes || ((_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.lateMin), comment || ((_c = this.selectedItem) === null || _c === void 0 ? void 0 : _c.comment));
  }
  render() {
    var _a, _b;
    const { size } = this;
    const { host, footer } = attendanceActionCommentStyle({
      size,
    });
    return (h(Host, { class: host(), onKeyDown: e => {
        if (e.key === "Escape")
          this.hide();
      } }, h("bcm-form", { validation: true, style: { width: "256px" } }, this.isLate ? (h("bcm-input", { label: "Minus Late", required: true, name: "lateMin", id: this._id + "-input-dropdown-late-content-input", type: "number", size: this.size, autocomplete: false, "no-margin": true, min: 0, max: 200, value: (_a = this.selectedItem) === null || _a === void 0 ? void 0 : _a.lateMin, placeholder: "Enter Minus Late", "onBcm-change": ({ detail }) => (this.lateMinutes = detail), onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }, h("i", { slot: "prefix", class: "far fa-clock" }))) : null, h("bcm-textarea", { name: "comment", value: (_b = this.selectedItem) === null || _b === void 0 ? void 0 : _b.comment, size: this.size, "onBcm-change": ({ detail }) => (this.comment = detail), label: "Description", "no-caption": true, rows: 4, onKeyDown: e => {
        if (e.key === "Enter")
          this.formSubmit();
      } }), h("footer", { class: footer() }, h("bcm-button", { class: "tw-order-2", icon: "fas fa-save", "icon-position": "prefix", size: this.size, onClick: () => this.formSubmit() }, "Save"), h("bcm-button", { class: "tw-order-1", icon: "fas fa-times", kind: "ghost", size: this.size, "icon-position": "prefix", onClick: () => this.hide() }, "Close")))));
  }
  get el() { return getElement(this); }
};

export { BcmAttendanceActionsComment as bcm_attendance_actions_comment };
