/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { c as classnames } from './index-f549811f.js';
import './_commonjsHelpers-82aa9212.js';

const autocompleteCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}:host{display:inline-block;width:fit-content;height:fit-content}:host(.hidden){display:none}:host(.full-width){width:100%;height:100%}:host(.full-width) .bcm-autocomplete{max-height:100%;max-width:100%}.bcm-autocomplete{background-color:var(--bcm-color-grey-1);display:flex;flex-direction:column;height:100%;width:100%;min-width:256px;max-width:256px;position:relative}.bcm-autocomplete-result{position:relative}.bcm-autocomplete-result .viewport{position:absolute;overflow:hidden;height:0;transition:height 0.3s ease;background-color:#ffffff;width:100%;z-index:10000}.bcm-autocomplete-result.open .viewport{height:256px}";

const BcmAutocomplete = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.autocompleteItemSelected = createEvent(this, "bcm-autocomplete-item-selected", 7);
    this.objectMapping = {
      'id': "id",
      'text': "text",
      'value': "value",
    };
    this.searchFields = ['text', 'secondary_text', 'tooltip_text'];
    this._items = [];
    this.fullWidth = false;
    this.hidden = false;
    this.searchText = null;
    this.returnField = "id";
    this.searchSub = "items";
    this.searchPlaceholder = "Search";
    this.inputLength = 0;
    this.isOpen = false;
    this.resultFound = false;
    this.searchData = undefined;
    this.searchResults = [];
    this.items = [];
  }
  async config(_config) {
    await customElements.whenDefined("bcm-listbox");
    this.bcmListbox = this.el.shadowRoot.querySelector("bcm-listbox");
    await this.bcmListbox.config(_config);
    if (_config['searchFields']) {
      if (Array.isArray(_config['searchFields'])) {
        this.searchFields = _config['searchFields'];
      }
    }
    if (_config['searchSub']) {
      typeof _config['searchSub'] == 'string'
        ? this.searchSub = _config['searchSub']
        : this.searchSub = null;
    }
    if (_config['fullWidth']) {
      typeof _config['fullWidth'] == 'boolean'
        ? this.fullWidth = _config['fullWidth']
        : this.fullWidth = false;
    }
    if (_config['searchPlaceholder']) {
      typeof _config['searchPlaceholder'] == 'string'
        ? this.searchPlaceholder = _config['searchPlaceholder']
        : this.searchPlaceholder = null;
    }
    if (_config['mapping']) {
      if (_config['mapping']['id']) {
        this.objectMapping['id'] = _config['mapping']['id'];
        this.returnField = this.objectMapping['id'];
      }
      if (_config['mapping']['text']) {
        this.objectMapping['text'] = _config['mapping']['text'];
      }
      if (_config['mapping']['value']) {
        this.objectMapping['value'] = _config['mapping']['value'];
      }
      if (_config['mapping']['items']) {
        this.objectMapping['items'] = _config['mapping']['items'];
        this.searchSub = this.objectMapping['items'];
      }
    }
  }
  async setData(_data = []) {
    await customElements.whenDefined("bcm-listbox");
    this.bcmListbox = this.el.shadowRoot.querySelector("bcm-listbox");
    await this.bcmListbox.setData(_data);
    this.searchData = JSON.parse(_data);
  }
  /**
   * @desc
   */
  async handleListboxClearSelected(e) {
    const optionValue = e.detail[this.objectMapping['text']];
    const search = this.el.shadowRoot.querySelector("bcm-old-search");
    this.autocompleteItemSelected.emit(e.detail);
    await search.setValue(optionValue);
    this.isOpen = false;
  }
  listenSearchValue(e) {
    this.inputLength = e.detail.length;
    if (this.inputLength > 0) {
      this.isOpen = true;
    }
    else {
      this.isOpen = false;
    }
  }
  async listenSearchResult(e) {
    this.searchResults = e.detail;
    const bcmListboxx = this.el.shadowRoot.querySelector("bcm-listbox");
    await bcmListboxx.searchResultMethod(this.searchResults);
  }
  handleClickOutside(e) {
    if (!this.el.contains(e.target)) {
      if (this.isOpen)
        this.isOpen = false;
    }
    else {
      if (this.inputLength > 0) {
        this.isOpen = true;
      }
      else {
        this.isOpen = false;
      }
    }
  }
  render() {
    const { searchText, searchData, searchFields, searchSub, returnField, isOpen, fullWidth } = this;
    const classes = classnames('bcm-autocomplete', {
      'open': isOpen
    });
    const classesSearch = classnames('bcm-autocomplete-search');
    const classesResult = classnames('bcm-autocomplete-result', {
      'open': isOpen
    });
    const hostClasses = classnames(this.hidden ? 'hidden' : null, {
      'full-width': fullWidth
    });
    return (h(Host, { class: hostClasses }, h("div", { class: classes }, h("div", { class: classesSearch }, h("bcm-old-search", { searchPlaceholder: this.searchPlaceholder, id: "autocomplete", searchValue: searchText, data: searchData, returnField: returnField, searchFields: searchFields, searchSub: searchSub, clearable: true })), h("div", { class: classesResult }, h("div", { class: "viewport" }, h("bcm-listbox", { searchText: searchText, _flex: "true" }))))));
  }
  get el() { return getElement(this); }
};
BcmAutocomplete.style = autocompleteCss;

export { BcmAutocomplete as bcm_autocomplete };
