/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { C as ColorHelper } from './color-helper-74614f24.js';
import './datetime-helper-df25808a.js';
import { G as Generate } from './generate-d009624a.js';
import './number-helper-b7a7a318.js';
import './string-helper-3592bdc2.js';
import './validators-5519219e.js';
import './element-dragger-89bb52fd.js';
import { B as Bcm } from './bcm-a50d55ba.js';
import { c as ce } from './index-3bc8c6e3.js';
import './colors-1d7854ea.js';
import './utils-57652744.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';

const avatarStyle = ce({
  slots: {
    base: "bcm-avatar bcm-ui-element bcm-ui-content-display tw-uppercase tw-inline-flex tw-items-center tw-justify-center tw-bg-[--bcm-avatar-bg] tw-text-[--bcm-avatar-text]",
    image: "tw-w-full tw-h-full",
    badge: "",
  },
  variants: {
    size: {
      small: "bcm-avatar__size-small tw-text-1 tw-w-6 tw-h-6",
      medium: "bcm-avatar__size-medium tw-text-2 tw-w-8 tw-h-8",
      large: "bcm-avatar__size-large tw-text-3 tw-w-10 tw-h-10",
    },
    shape: {
      ellipse: {
        base: "bcm-avatar__shape-ellipse tw-rounded-full",
        image: "tw-rounded-full",
        badge: "tw-top-0 tw-right-0",
      },
      square: {
        base: "bcm-avatar__shape-square tw-rounded",
        image: "tw-rounded",
        badge: "tw--top-0.5 tw--right-0.5",
      },
    },
    status: {
      true: {
        base: "bcm-avatar__status tw-relative",
        badge: "tw-absolute",
      },
    },
  },
  defaultVariants: {
    size: "medium",
    shape: "ellipse",
    status: false,
  },
});

const BcmAvatar = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this._id = Generate.UID();
    this.image = undefined;
    this.name = undefined;
    this.alt = undefined;
    this.shape = Bcm.Shape.ellipse;
    this.size = Bcm.Size.medium;
    this.color = undefined;
    this.icon = "fas fa-user";
    this.status = undefined;
    this.blink = false;
    this.isFallback = false;
  }
  getFirstLetters(name) {
    const words = name.split(" ");
    const initials = words.map(word => word.charAt(0).toUpperCase()).join("");
    return initials.substring(0, 2);
  }
  renderDefaultIcon() {
    return h("bcm-icon", { icon: this.icon });
  }
  render() {
    const { shape, size, status, color, blink } = this;
    const { base, badge, image } = avatarStyle({ size, shape, status: !!status });
    const dc = Generate.getTwColor({
      color: color,
      tones: {
        200: ["bg"],
        700: ["text"],
      },
      variable: "--bcm-avatar",
    });
    const customSize = !Object.keys(Bcm.BaseSize).includes(size)
      ? {
        width: `${size}px`,
        height: `${size}px`,
        fontSize: `${+size / 3}px`,
      }
      : {};
    const RenderContent = () => {
      if (this.isFallback) {
        if (this.name) {
          return this.getFirstLetters(this.name);
        }
        else {
          return this.renderDefaultIcon();
        }
      }
      else if (this.image) {
        return h("img", { class: image(), onError: () => (this.isFallback = true), src: this.image, alt: this.alt, "aria-label": this.alt || this.name || "Avatar" });
      }
      else if (this.name) {
        return this.getFirstLetters(this.name);
      }
      else {
        return this.renderDefaultIcon();
      }
    };
    return (h(Host, { role: "img", "aria-label": this.alt || this.name || "Avatar", class: base(), style: Object.assign(Object.assign({}, dc), customSize) }, h(RenderContent, null), status && h("bcm-badge", { class: badge(), status: status, blink: blink, "aria-hidden": "true" }), h("slot", null)));
  }
  get el() { return getElement(this); }
};

const badgeStyle = ce({
  slots: {
    host: "bcm-badge bcm-ui-element bcm-ui-content-display tw-inline-flex tw-items-center tw-justify-center tw-font-normal ",
    base: [
      "bcm-badge__dot tw-z-10",
      "tw-flex tw-items-center tw-justify-center tw-flex-row",
      "tw-rounded-full tw-text-center",
      "tw-bg-[--bcm-badge-bg] tw-text-[--bcm-badge-text]",
      "tw-whitespace-nowrap",
    ],
    dotText: "empty:tw-hidden",
  },
  variants: {
    size: {
      small: {
        host: "tw-text-2 tw-gap-1",
        base: "bcm-badge__dot-size-small tw-text-1 tw-min-w-1.5 tw-min-h-1.5",
        dotText: "tw-px-1.5 tw-min-w-4",
      },
      medium: {
        host: "tw-text-3 tw-gap-1.5",
        base: "bcm-badge__dot-size-medium tw-text-2 tw-min-w-2 tw-min-h-2",
        dotText: "tw-px-1.5 tw-min-w-5",
      },
      large: {
        host: "tw-text-4 tw-gap-2",
        base: "bcm-badge__dot-size-large tw-text-3 tw-min-w-2.5 tw-min-h-2.5",
        dotText: "tw-px-2 tw-min-w-6",
      },
    },
    blink: {
      true: {
        base: [
          "bcm-badge__dot-blink tw-relative",
          "after:tw-content-[''] after:tw-absolute after:tw-top-0 after:tw-left-0 after:tw-z-[-1]",
          "after:tw-w-full after:tw-h-full after:tw-bg-[--bcm-badge-bg] after:tw-opacity-75 after:tw-rounded-full",
          "after:tw-animate-blink after:tw-pointer-events-none",
        ],
      },
    },
  },
  defaultVariants: {
    size: "medium",
    blink: false,
  },
});

const BcmBadge = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.statusColors = {
      info: "blue",
      error: "red",
      warning: "amber",
      success: "emerald",
    };
    this._id = Generate.UID();
    this.blink = false;
    this.value = undefined;
    this.color = undefined;
    this.size = Bcm.Size.medium;
    this.status = undefined;
  }
  render() {
    const { blink, value, size, color, status } = this;
    const _color = this.statusColors[status] || color;
    const { tone, colorName } = ColorHelper.parseColor(_color, 700);
    const dc = Generate.getTwColor({
      color: colorName,
      tones: {
        [tone]: ["bg"],
      },
      variable: "--bcm-badge",
    });
    const style = Object.assign(Object.assign({}, dc), (dc["--bcm-badge-bg"] && { "--bcm-badge-text": ColorHelper.reverseColor(dc["--bcm-badge-bg"]) }));
    const { base, dotText, host } = badgeStyle({ size, blink });
    return (h(Host, { class: host(), role: "status", "aria-live": "polite" }, h("div", { class: base(), style: style }, h("span", { class: dotText(), role: "presentation", "aria-hidden": "true" }, value), h("span", { class: "tw-sr-only" }, value)), h("slot", null)));
  }
  get el() { return getElement(this); }
};

export { BcmAvatar as bcm_avatar, BcmBadge as bcm_badge };
