/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { c as classnames } from './index-f549811f.js';
import { C as ColorHelper } from './color-helper-74614f24.js';
import { G as Generate } from './generate-d009624a.js';
import { S as StringHelper } from './string-helper-3592bdc2.js';
import { S as SlotTemplate } from './slot-template-2a19b2db.js';
import './_commonjsHelpers-82aa9212.js';
import './colors-1d7854ea.js';
import './utils-57652744.js';
import './colors-dd598eea.js';
import './package-7549581c.js';
import './bcm-a50d55ba.js';

const cardCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");@import url(\"https://fonts.googleapis.com/css2?family=Inter:wght@200;400;500;600&display=swap\"); .size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}*{box-sizing:border-box}.bcm-card{font-family:\"Inter\", sans-serif;font-size:14px;line-height:calc(14px + 8px);font-weight:400;display:block;width:100%;background-color:var(--bcm-new-ds-color-white);border-radius:8px;box-shadow:0 1px 3px 0 rgba(17, 24, 38, 0.1);box-sizing:border-box}.bcm-card__header,.bcm-card__body,.bcm-card__footer{display:block;width:100%}.bcm-card__header{border-radius:8px 8px 0 0;padding:12px 24px;color:var(--bcm-new-ds-color-slate-700);font-weight:600}.bcm-card__body{padding:12px 24px}.bcm-card__footer{padding:12px 24px;border-radius:0 0 8px 8px}";

const BcmCard = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.modalSlots = {};
    this._id = Generate.UID();
    this.hidden = false;
    this.color = "slate";
    this.header = undefined;
    this.footer = undefined;
  }
  connectedCallback() {
    const bcmSlots = this.el.querySelectorAll('bcm-card-header, bcm-card-footer');
    if (bcmSlots) {
      bcmSlots.forEach(el => {
        let slotName = StringHelper.lowercase(StringHelper.split(el.nodeName, '-').pop());
        el.slot = slotName;
        if (slotName === 'header') {
          let color = el.getAttribute('color');
          if (color) {
            color = color == 'white' ? color : color + '-50';
            el.setAttribute('style', `background-color: ${ColorHelper.dsColor(color)}`);
          }
        }
        el.classList.add(`bcm-card__${slotName}`);
        this.modalSlots[slotName] = el;
      });
    }
  }
  render() {
    const hostClasses = classnames('bcm-card', this.hidden ? 'hidden' : null);
    const FooterTemplate = () => {
      return (this.modalSlots['footer'] && (h("slot", { name: "footer" })));
    };
    const HeaderTemplate = () => {
      return (this.modalSlots['header'] && (h("slot", { name: "header" })));
    };
    return (h(Host, { class: hostClasses }, h(HeaderTemplate, null), h("div", { class: "bcm-card__body" }, h(SlotTemplate, null)), h(FooterTemplate, null)));
  }
  get el() { return getElement(this); }
};
BcmCard.style = cardCss;

export { BcmCard as bcm_card };
