/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import './color-helper-74614f24.js';
import './datetime-helper-df25808a.js';
import { G as Generate } from './generate-d009624a.js';
import './number-helper-b7a7a318.js';
import './string-helper-3592bdc2.js';
import './validators-5519219e.js';
import './element-dragger-89bb52fd.js';
import { d as delay } from './utils-57652744.js';
import { B as Bcm } from './bcm-a50d55ba.js';
import { c as classnames } from './index-f549811f.js';
import './colors-1d7854ea.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';

const BcmModal2 = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.modalClose = createEvent(this, "bcm-modal-close", 7);
    this.modalOpen = createEvent(this, "bcm-modal-open", 7);
    this.modalBeforeClose = createEvent(this, "bcm-modal-before-close", 7);
    this.modalBeforeOpen = createEvent(this, "bcm-modal-before-open", 7);
    this._id = Generate.UID();
    this.isShow = false;
    this.center = false;
    this.zIndex = "10400";
    this.size = Bcm.Size.medium;
    this.backdrop = true;
    this.fullWidth = false;
    this.fullScreen = false;
    this.overflow = true;
    this.isShowModal = false;
    this.shake = false;
  }
  // Listen to the click event on the component and its children
  handleClick(e) {
    // Check if the click event is triggered on a child element with "data-dismiss" attribute
    const dismiss = Generate.findEventPathHasAttribute(e, "data-dismiss");
    // Check if the click event is triggered on the overlay element and backdrop is "static"
    const target = e.target;
    const isClickOverlay = target.attributes.getNamedItem("is-click-overlay");
    if (isClickOverlay && this.backdrop === "static") {
      // Shake the modal if the overlay is clicked and backdrop is "static"
      this.shakeModal();
    }
    else if (dismiss || isClickOverlay) {
      // Hide the modal if the "data-dismiss" element is clicked
      this.hide();
      e.preventDefault();
      e.stopPropagation();
    }
  }
  // Listen to the keydown event on the document element
  handleKeyDown(ev) {
    // Get the top modal in the DOM with "is-show" attribute
    const topModal = Array.from(document.querySelectorAll("bcm-modal-2[is-show]")).pop();
    if (topModal !== this.el) {
      return;
    }
    if (ev.key === "Escape") {
      if (this.backdrop === "static") {
        // Shake the modal if the backdrop is "static" and Escape key is pressed
        this.shakeModal();
      }
      else {
        // Hide the modal if Escape key is pressed
        this.hide();
      }
      ev.preventDefault();
      ev.stopPropagation();
      ev.stopImmediatePropagation();
    }
  }
  // Shake the modal by toggling "shake" state
  shakeModal() {
    this.shake = true;
    setTimeout(() => (this.shake = false), 300);
  }
  // Hide the modal by emitting "bcm-modal-before-close" event
  async hide() {
    this.modalBeforeClose.emit();
  }
  // Listen to "bcm-modal-before-close" event and hide the modal if defaultPrevented is not set
  async hideModal(e) {
    if (e.defaultPrevented) {
      return;
    }
    if (e.target === this.el) {
      this.isShowModal = false;
      await delay(150);
      this.isShow = false;
      this.modalClose.emit();
    }
  }
  // Show the modal by emitting "bcm-modal-before-open" event
  async show() {
    this.modalBeforeOpen.emit();
  }
  // Listen to "bcm-modal-before-open" event and show the modal if defaultPrevented is not set
  async showModal(e) {
    if (e.defaultPrevented) {
      return;
    }
    if (e.target === this.el) {
      this.isShow = true;
      await delay(50);
      this.isShowModal = true;
      document.body.appendChild(this.el);
      this.modalOpen.emit();
    }
  }
  // Get the width of the modal based on the size
  getWidth(size) {
    const sizes = {
      small: "576px",
      medium: "768px",
      large: "992px",
      xlarge: "1024px",
      xxlarge: "1200px",
      xxxlarge: "1700px",
    };
    return sizes[size] || size;
  }
  // Get the modal size
  setSize() {
    const { fullScreen, fullWidth, size, overflow } = this;
    let minHeight = "auto";
    let height = "auto";
    let width = fullWidth ? "100%" : this.getWidth(size);
    if (fullScreen) {
      height = overflow ? "100vh" : "auto";
      width = "100vw";
      minHeight = "100vh";
    }
    return { width, minHeight, height };
  }
  render() {
    return (h(Host, { class: { "tw-hidden": !this.isShow } }, h("bcm-overlay", { "z-index": +this.zIndex - 1, opacity: this.isShowModal ? "0.5" : "0" }), h("div", { class: classnames("tw-fixed tw-inset-0", "tw-flex tw-h-full tw-w-full tw-items-start tw-justify-center", "tw-overflow-y-scroll tw-overscroll-contain tw-font-sans", this.center ? "tw-items-center" : ""), style: {
        zIndex: this.zIndex,
      }, "is-click-overlay": true }, h("div", { class: classnames("tw-flex tw-w-full tw-flex-col", "tw-relative tw-z-modal tw-bg-white", "tw--translate-y-[20px] tw-transform tw-opacity-0 tw-transition tw-duration-300 tw-ease-in-out", this.fullScreen ? "" : `tw-m-4 tw-rounded-lg ${this.overflow ? "tw-max-h-[90vh]" : ""} `, this.shake ? "tw-animate-shake" : ""), style: Object.assign(Object.assign({}, this.setSize()), (this.isShowModal
        ? {
          transform: "translateY(0)",
          opacity: "1",
        }
        : {})) }, h("slot", { name: "header" }), h("bcm-scroll-area", { "is-disabled": !this.overflow }, h("slot", null)), h("slot", { name: "footer" })))));
  }
  get el() { return getElement(this); }
};

export { BcmModal2 as bcm_modal_2 };
