/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import './color-helper-74614f24.js';
import './datetime-helper-df25808a.js';
import { G as Generate } from './generate-d009624a.js';
import './number-helper-b7a7a318.js';
import { S as StringHelper } from './string-helper-3592bdc2.js';
import './validators-5519219e.js';
import './element-dragger-89bb52fd.js';
import { d as delay } from './utils-57652744.js';
import { B as Bcm } from './bcm-a50d55ba.js';
import { C as CaptionTemplate } from './caption-template-d7953d13.js';
import { c as classnames } from './index-f549811f.js';
import './types-6f06fa1a.js';
import './types-9493a7ed.js';
import { i as inputStyle } from './input.style-e855c0f3.js';
import './colors-1d7854ea.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';
import './index-3bc8c6e3.js';

const NumberInput = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.focus = createEvent(this, "bcm-focus", 3);
    this.blur = createEvent(this, "bcm-blur", 3);
    this.clear = createEvent(this, "bcm-clear", 3);
    this.change = createEvent(this, "bcm-change", 7);
    this.input = createEvent(this, "bcm-input", 3);
    this.beforeClear = createEvent(this, "bcm-input-before-clear", 3);
    this._onChange = createEvent(this, "bcm-onchange", 3);
    this.inValid = false;
    this.onInput = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("input", (e) => {
        this.input.emit(e);
      });
    };
    this.onChange = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("change", (e) => {
        this.change.emit(e);
      });
    };
    this.onFocus = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("focus", e => {
        this.isFocus = true;
        this.focus.emit(e);
      });
    };
    this.onBlur = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("blur", e => {
        this.isFocus = false;
        this.blur.emit(e);
      });
    };
    this.onKeydown = () => {
      var _a;
      (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener("keydown", (e) => {
        if (e.key === "ArrowUp") {
          e.preventDefault();
          this.up();
        }
        if (e.key === "ArrowDown") {
          e.preventDefault();
          this.down();
        }
      });
    };
    this.listener = () => {
      this.onInput();
      this.onChange();
      this.onFocus();
      this.onBlur();
      this.onKeydown();
    };
    this._id = Generate.UID();
    this.value = "";
    this.defaultValue = undefined;
    this._prefix = undefined;
    this._suffix = undefined;
    this.size = "medium";
    this.min = undefined;
    this.max = undefined;
    this.step = 1;
    this.label = undefined;
    this.name = undefined;
    this.fullWidth = false;
    this.clearable = false;
    this.readonly = false;
    this.disabled = false;
    this.required = false;
    this.mode = "none";
    this.placeholder = undefined;
    this.thousandSeparator = false;
    this.decimalSeparator = undefined;
    this.decimalScale = undefined;
    this.fixedDecimalScale = false;
    this.allowNegative = true;
    this.allowLeadingZeros = false;
    this.unitPrefix = undefined;
    this.unitSuffix = undefined;
    this.caption = undefined;
    this.noCaption = false;
    this.captionType = Bcm.Status.default;
    this.captionError = undefined;
    this.customErrorMessage = undefined;
    this.isFocus = false;
    this.captionCache = undefined;
    this.captionTypeCache = undefined;
    this.valueCache = "";
  }
  connectedCallback() {
    if (this.defaultValue)
      this.value = +this.defaultValue;
    this.valueCache = this.value;
    this.captionCache = this.caption;
    this.captionTypeCache = this.captionType;
  }
  componentDidLoad() {
    this.listener();
  }
  up() {
    if (this.disabled)
      return;
    if (!this.value)
      this.value = Number(0);
    this.value = +this.value + +this.step;
    if (this.mode === "strict") {
      if (this.max && +this.value > +this.max) {
        this.value = this.max;
      }
    }
  }
  down() {
    if (this.disabled)
      return;
    if (!this.value)
      this.value = Number(0);
    this.value = +this.value - +this.step;
    if (this.mode === "strict") {
      if (this.min && +this.value < +this.min) {
        this.value = this.min;
      }
    }
  }
  handleClear(e) {
    if (this.disabled)
      return;
    const event = this.beforeClear.emit(e);
    if (event.defaultPrevented)
      return;
    this.value = "";
    this.clear.emit(e);
  }
  checkLength() {
    const { min, max, value } = this;
    const minLength = min;
    const maxLength = max;
    let valuePoint = value === null || value === void 0 ? void 0 : value.toString();
    if (min && max) {
      if (+valuePoint < +min || +valuePoint > +max) {
        this.setCaption(StringHelper.getMessage("betweenlength", [minLength, maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (minLength) {
      if (+valuePoint < +minLength) {
        this.setCaption(StringHelper.getMessage("minlength", [minLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
    else if (maxLength) {
      if (valuePoint > maxLength) {
        this.setCaption(StringHelper.getMessage("maxlength", [maxLength]), "error");
        this.inValid = true;
      }
      else {
        this.setCaption();
        this.inValid = false;
      }
    }
  }
  watchValue() {
    this.checkLength();
  }
  async onBcmFocus() {
    this.inputElement.focus();
  }
  async onBcmBlur() {
    this.inputElement.blur();
  }
  async onBcmSelect() {
    this.inputElement.select();
  }
  async set(data) {
    await this.setValue(data);
  }
  async get() {
    return this.value || null;
  }
  async setValue(value) {
    this.value = value;
  }
  async setClear() {
    this.value = null;
    await delay(10);
    return Promise.resolve();
  }
  async resetCaption() {
    await delay(10);
    this.caption = this.captionCache;
    this.captionType = this.captionTypeCache;
    this.captionError = null;
    this.inValid = false;
    await delay(10);
    return Promise.resolve();
  }
  setCaption(caption = null, captionType = null) {
    this.caption = caption || this.captionCache;
    this.captionType = captionType || this.captionTypeCache;
    this.captionError = captionType == "error" ? caption : null;
    return Promise.resolve();
  }
  disconnectedCallback() {
    var _a, _b, _c, _d, _e;
    (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.removeEventListener("input", this.onInput.bind(this));
    (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.removeEventListener("change", this.onChange.bind(this));
    (_c = this.inputElement) === null || _c === void 0 ? void 0 : _c.removeEventListener("focus", this.onFocus.bind(this));
    (_d = this.inputElement) === null || _d === void 0 ? void 0 : _d.removeEventListener("blur", this.onBlur.bind(this));
    (_e = this.inputElement) === null || _e === void 0 ? void 0 : _e.removeEventListener("keydown", this.onKeydown.bind(this));
  }
  render() {
    const { _prefix, _suffix, noCaption, captionType, captionError, caption, required, _id, label, size, fullWidth, clearable, value, placeholder, thousandSeparator, decimalSeparator, decimalScale, fixedDecimalScale, allowNegative, allowLeadingZeros, unitPrefix, unitSuffix, min, max, step, defaultValue, disabled, mode, } = this;
    const isActiveClearable = (value === null || value === void 0 ? void 0 : value.toString().length) > 0 && clearable;
    const isInvalid = this.captionType == "error" || this.inValid;
    const { host, inputContainer, slotClass, clearButton, controls, controlsBtn, controlsBtnDown, controlsBtnUp, controlsDivider, input } = inputStyle({
      size,
      fullWidth,
      clearable: isActiveClearable,
      disabled,
      invalid: isInvalid,
    });
    const numericProps = {
      placeholder,
      thousandSeparator,
      decimalSeparator,
      decimalScale,
      fixedDecimalScale,
      allowNegative,
      allowLeadingZeros,
      prefix: unitPrefix,
      suffix: unitSuffix,
    };
    /* input  props */
    const inputProps = {
      min,
      max,
      step,
      value,
      defaultValue,
    };
    const generalProps = Object.assign(Object.assign(Object.assign({ "id": _id + "-base-input", "name": this.name, disabled,
      mode, "aria-label": label, "aria-labelledby": _id + "-label", "getInputRef": el => {
        this.inputElement = el;
      }, "className": input() }, numericProps), inputProps), { "onValueChange": (e) => {
        const { floatValue } = e;
        this.value = floatValue;
        this._onChange.emit(e);
      } });
    return (h(Host, { class: host() }, label && h("bcm-label", { size: size, type: captionType, value: label, required: required, htmlFor: _id + "-base-input", id: _id + "-label" }), h("div", { class: inputContainer() }, h("div", { class: slotClass() }, h("slot", { name: "prefix" }, _prefix && _prefix.length > 0 && h("span", { innerHTML: _prefix }))), h("bcm-react-number", { disabled: disabled, class: "tw-flex-1", options: generalProps }), h("div", { class: slotClass() }, !disabled && isActiveClearable && (h("button", { class: clearButton(), onClick: event => this.handleClear(event) }, h("i", { class: "far fa-times" }))), h("slot", { name: "suffix" }, _suffix && _suffix.length > 0 && h("span", { innerHTML: _suffix })), h("div", { class: controls() }, h("button", { type: "button", disabled: disabled, class: classnames(controlsBtn(), controlsBtnUp()), onClick: this.up.bind(this), "aria-label": "Increase value" }, h("i", { class: "fas fa-caret-up" })), h("div", { class: controlsDivider() }), h("button", { type: "button", disabled: disabled, class: classnames(controlsBtn(), controlsBtnDown()), onClick: this.down.bind(this), "aria-label": "Decrease value" }, h("i", { class: "fas fa-caret-down" }))))), h(CaptionTemplate, { noCaption: noCaption, captionType: captionType, captionError: captionError, caption: caption, customErrorMessage: this.customErrorMessage })));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "value": ["watchValue"]
  }; }
};

export { NumberInput as bcm_number_input };
