/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { c as classnames } from './index-f549811f.js';
import { G as Generate } from './generate-d009624a.js';
import { s as stepperStates } from './stepper-states-ec0b9c40.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';
import './colors-dd598eea.js';
import './index-7e71bbc8.js';

const BcmStep = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.dataIsLoaded = createEvent(this, "data-is-loaded", 7);
    this.bcmChangeHidden = createEvent(this, "bcm-change-hidden", 7);
    this.hidden = false;
    this._id = Generate.UID();
    this._title = undefined;
    this.description = undefined;
    this.status = 'default';
    this.icon = undefined;
    this.loading = false;
    this.valid = true;
    this.index = undefined;
    this.active = false;
    this.htmlTemplate = undefined;
    this.isLast = false;
    this.disabled = false;
  }
  componentDidLoad() {
    if (this.isLast) {
      this.dataIsLoaded.emit(true);
    }
  }
  getStepper() {
    const stepper = this.el.closest('bcm-stepper');
    return stepper || null;
  }
  activeChanged() {
    if (this.active) {
      const stepper = this.getStepper();
      if (stepper) {
        stepperStates.changeStepDisabled(stepper.stepperId, this._id, false);
      }
    }
  }
  statusChange() {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepStatus(stepper.stepperId, this._id, this.status);
    }
  }
  hiddenChange(newValue, oldValue) {
    const stepper = this.getStepper();
    if (newValue !== oldValue && stepper && !this.active) {
      this.bcmChangeHidden.emit(newValue);
    }
  }
  disabledChange(newValue, _) {
    const stepper = this.getStepper();
    if (stepper) {
      stepperStates.changeStepDisabled(stepper.stepperId, this._id, newValue);
    }
  }
  render() {
    const { hidden, valid, active } = this;
    const hostClasses = classnames('bcm-stepper__step', valid ? 'bcm-stepper__step-valid' : 'bcm-stepper__step-invalid', {
      'bcm-stepper__step-active': active,
      hidden,
    });
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "active": ["activeChanged"],
    "status": ["statusChange"],
    "hidden": ["hiddenChange"],
    "disabled": ["disabledChange"]
  }; }
};

export { BcmStep as bcm_step };
