/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, f as forceUpdate, h, H as Host, g as getElement } from './index-0615c2b8.js';
import './color-helper-74614f24.js';
import './datetime-helper-df25808a.js';
import { G as Generate } from './generate-d009624a.js';
import './number-helper-b7a7a318.js';
import './string-helper-3592bdc2.js';
import './validators-5519219e.js';
import './element-dragger-89bb52fd.js';
import { d as delay } from './utils-57652744.js';
import { B as Bcm } from './bcm-a50d55ba.js';
import './colors-1d7854ea.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';

const BcmTabGroup = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.tabChange = createEvent(this, "bcm-tab-change", 7);
    this._id = Generate.UID();
    this.active = "1";
    this.color = "blue";
    this.size = Bcm.Size.medium;
  }
  get tabItems() {
    return this.el.querySelectorAll("bcm-tab-item");
  }
  get tabItemHeaders() {
    return this.el.querySelectorAll("bcm-tab-item-header");
  }
  setTabAttributes() {
    this.tabItems.forEach((tab, index) => {
      const itemActive = tab.hasAttribute("active") ? true : false;
      const itemDisabled = tab.hasAttribute("disabled") ? true : false;
      const itemIndex = index + 1;
      tab.setAttribute("index", `${itemIndex}`);
      tab.setAttribute("size", this.size);
      tab.setAttribute("color", this.color);
      tab.removeAttribute("active");
      tab.removeAttribute("disabled");
      if (itemActive) {
        tab.setAttribute("active", "");
        this.active = itemIndex.toString();
      }
      if (itemDisabled) {
        tab.setAttribute("disabled", "");
      }
      // const headerElement = tab.querySelector("bcm-tab-item-header");
      // if (headerElement) {
      //     this.setHeaderTabAttributes(headerElement, tab);
      // }
    });
    const isHasActiveTab = Array.from(this.tabItems).some((tab) => tab.hasAttribute("active"));
    if (!isHasActiveTab) {
      this.tabItems[0].setAttribute("active", "");
    }
  }
  componentWillLoad() {
    this.setTabAttributes();
  }
  handleTabClick(index) {
    var _a, _b;
    if (index) {
      this.active = index.toString();
    }
    if ((_a = this.tabItems) === null || _a === void 0 ? void 0 : _a.length) {
      this.tabItems.forEach(tab => {
        tab.removeAttribute("active");
        tab.setAttribute("size", this.size);
        tab.setAttribute("color", this.color);
        if (tab.getAttribute("index") == this.active) {
          tab.setAttribute("active", "");
        }
      });
    }
    if ((_b = this.tabItemHeaders) === null || _b === void 0 ? void 0 : _b.length) {
      this.tabItemHeaders.forEach(tabHeader => {
        tabHeader.removeAttribute("active");
        tabHeader.setAttribute("size", this.size);
        tabHeader.setAttribute("color", this.color);
        if (tabHeader.getAttribute("index") == this.active) {
          tabHeader.setAttribute("active", "");
          // tabHeader.scrollIntoView({
          //     behavior: "smooth",
          //     block: "start",
          //     inline: "center",
          // });
        }
      });
    }
    this.tabChange.emit(this.active);
  }
  componentDidRender() {
    this.inkBarControl();
    this.handleTabClick();
  }
  componentDidLoad() {
    this.inkBarControl();
    this.observeTabItem();
  }
  async setActive(index) {
    if (index > 0 && index <= this.tabItems.length && index != +this.active) {
      this.handleTabClick(index);
    }
  }
  async refresh() {
    await delay(100);
    forceUpdate(this);
  }
  inkBarControl() {
    var _a;
    const inkBar = this.el.querySelector("#ink-bar");
    if (!inkBar || !((_a = this.tabItemHeaders) === null || _a === void 0 ? void 0 : _a.length))
      return;
    for (let i = 0; i < this.tabItemHeaders.length; i++) {
      const tab = this.tabItemHeaders[i];
      const tabIndex = tab.getAttribute("index");
      if (tabIndex === this.active) {
        inkBar.style.width = `${tab.offsetWidth}px`;
        // inkBar.style.top = `${tab.offsetHeight}px`;
        inkBar.style.transform = `translateX(${tab.offsetLeft}px)`;
      }
    }
  }
  activeChanged(newVal, _oldVal) {
    var _a;
    if (!((_a = this.tabItems) === null || _a === void 0 ? void 0 : _a.length))
      return;
    const isHasActiveTab = Array.from(this.tabItems).some(tab => tab.getAttribute("index") == newVal);
    if (!isHasActiveTab) {
      this.active = "1";
    }
  }
  observeTabItem() {
    const callback = (mutationsList, _observer) => {
      for (const mutation of mutationsList) {
        if (mutation.type === "childList") {
          // for (const node of mutation.addedNodes) {
          //     if (node.nodeName === "BCM-TAB-ITEM") {
          //         // Do something when a bcm-tab-item is added
          //     }
          // }
          for (const node of mutation.removedNodes) {
            if (node.nodeName === "BCM-TAB-ITEM") {
              // Do something when a bcm-tab-item is removed
              // console.log("A bcm-tab-item was removed.", node);
              if (node === null || node === void 0 ? void 0 : node.active) {
                this.active = "1";
              }
              this.tabProcess();
            }
          }
        }
      }
    };
    this.observer = new MutationObserver(callback);
    this.observer.observe(this.el, { childList: true });
  }
  tabProcess() {
    this.updateTabHeaders();
    this.regenerateIndex();
    this.refresh();
    this.inkBarControl();
  }
  updateTabHeaders() {
    const tabIds = Array.from(this.tabItems).map(tab => tab.getAttribute("id"));
    const tabHeaderElements = Array.from(this.el.querySelector("#tab-header").querySelectorAll("bcm-tab-item-header"));
    if (tabIds.length !== tabHeaderElements.length) {
      tabHeaderElements.forEach(tabHeader => {
        const tabId = tabHeader.getAttribute("tab-id");
        if (!tabIds.includes(tabId)) {
          tabHeader.remove();
        }
      });
    }
  }
  regenerateIndex() {
    this.tabItems.forEach((tab, index) => {
      const itemIndex = index + 1;
      tab.setAttribute("index", `${itemIndex}`);
      const headerElement = this.el.querySelector(`bcm-tab-item-header[tab-id="${tab.getAttribute("id")}"]`);
      if (headerElement) {
        headerElement.setAttribute("index", `${itemIndex}`);
        headerElement.style.order = `${itemIndex}`;
      }
    });
  }
  render() {
    const { color } = this;
    const dc = Generate.getTwColor({
      color: color,
      tones: [700]
    });
    return (h(Host, { class: "tw-flex tw-w-full tw-flex-col tw-items-center tw-font-sans" }, h("div", { role: "tablist", "aria-label": "Tab Group", class: `tw-relative tw-flex tw-w-full tw-flex-row tw-items-end tw-justify-between`, id: "tab-header", style: dc }, h("div", { id: "ink-bar", class: `tw-bg-default-700 tw-absolute tw-bottom-0 tw-left-0 tw-mt-[-2px] tw-h-[2px] tw-rounded-sm tw-transition-transform` })), h("slot", null)));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "active": ["activeChanged"]
  }; }
};

export { BcmTabGroup as bcm_tab_group };
