/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, f as forceUpdate, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { S as SizeProps, T as TypeProps, D as DirectionProps } from './types-b211a5eb.js';
import { d as delay, c as getChilds } from './utils-57652744.js';
import { c as classnames } from './index-f549811f.js';
import './_commonjsHelpers-82aa9212.js';

const tabsCss = "@import url(\"https://fonts.googleapis.com/css?family=Roboto:400,500\");.size-1{font-size:12px;line-height:20px}.size-2{font-size:14px;line-height:22px}.size-3{font-size:16px;line-height:24px}.size-4{font-size:20px;line-height:28px}.size-5{font-size:24px;line-height:32px}.size-6{font-size:30px;line-height:38px}.size-7{font-size:38px;line-height:46px}.size-8{font-size:46px;line-height:54px}.size-9{font-size:56px;line-height:64px}.weight-regular{font-weight:400}.weight-semibold{font-weight:500}.input-size-large{height:40px}.input-size-medium{height:32px}.input-size-small{height:24px}.resize-none textarea{resize:none}.resize-vertical textarea{resize:vertical}.resize-auto textarea{height:auto;resize:none}.textarea-size-large{padding:4px 0 0 4px;min-height:40px}.textarea-size-large .bcm-input-element{min-height:calc((40px - 8px) + 2px)}.textarea-size-large .input-clear-button{height:calc( 40px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-medium{padding:4px 0 0 4px;min-height:32px}.textarea-size-medium .bcm-input-element{min-height:calc((32px - 8px) + 2px)}.textarea-size-medium .input-clear-button{height:calc( 32px - 16px );top:8px;padding:0;margin-right:8px}.textarea-size-small{padding:0px 0 0 4px;min-height:24px}.textarea-size-small .bcm-input-element{min-height:calc((24px - 8px) + 2px)}.textarea-size-small .input-clear-button{height:calc( 24px - 16px );top:8px;padding:0;margin-right:8px}:host{display:block}:host(.hidden){display:none}::slotted(bcm-tab:not(:last-child)){margin-right:32px}.tabs{position:relative;display:flex;flex-wrap:wrap}.tabs.large{--bcm-tab-vertical-space:16px;--bcm-tab-font-size:16px;--bcm-tab-line-height:24px}.tabs.medium{--bcm-tab-vertical-space:11px;--bcm-tab-font-size:14px;--bcm-tab-line-height:22px}.tabs.small{--bcm-tab-vertical-space:7px;--bcm-tab-font-size:14px;--bcm-tab-line-height:22px}.tabs.pane{--bcm-tab-vertical-space:9px;--bcm-tab-font-size:14px;--bcm-tab-line-height:22px}.tabs.full-width{width:100%;display:flex;align-items:center;justify-content:center}.tabs.full-width ::slotted(bcm-tab){flex:1}.tabs.underline:after{content:\"\";margin-top:0;z-index:0;left:0;width:100%;height:2px;background-color:var(--bcm-color-grey-4)}.tabs.animatable:before{transition:all 0.4s ease}.tabs.pane ::slotted(bcm-tab:not(:last-child)){margin-right:2px}.tabs.pane:before{display:none}.tabs.vertical{flex-direction:column;width:fit-content}.tabs.vertical:before{content:\"\";position:absolute;top:var(--bcm-tabs-line-top);left:var(--bcm-tabs-line-left);right:var(--bcm-tabs-line-right);width:var(--bcm-tabs-line-width);height:var(--bcm-tabs-line-height);background-color:var(--bcm-color-prime-blue-6)}.tabs.vertical ::slotted(bcm-tab){margin-right:0}";

const BcmTabs = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.click = createEvent(this, "bcm-click", 7);
    this.change = createEvent(this, "bcm-change", 7);
    this.bcmTabsActiveItem = createEvent(this, "bcm-tabs-active-item", 7);
    /**
     * Component Private variables
     */
    this.tabs = [];
    this.firstInit = true;
    this.count = 0;
    this.size = SizeProps.large;
    this.type = TypeProps.normal;
    this.direction = DirectionProps.horizontal;
    this.hidden = false;
    this.fullWidth = false;
    this.underline = true;
    this.activeChild = 1;
    this.lineOpts = { top: '0px', left: '0px', right: '0px', width: '0px', height: '0px' };
    this.prevButton = null;
    this.nextButton = null;
    this.completeButton = null;
    this.prevButtonState = 'disabled';
    this.nextButtonState = 'disabled';
    this.completeButtonState = 'disabled';
  }
  async config(_config) {
    if (_config['prevButton']) {
      if (_config['prevButton']['id']) {
        var prevButtonId = "#" + _config['prevButton']['id'];
        this.prevButton = document.querySelector(prevButtonId);
      }
      if (_config['prevButton']['state']) {
        this.prevButtonState = _config['prevButton']['state'];
      }
    }
    if (_config['nextButton']) {
      if (_config['nextButton']['id']) {
        var nextButtonId = "#" + _config['nextButton']['id'];
        this.nextButton = document.querySelector(nextButtonId);
      }
      if (_config['nextButton']['state']) {
        this.nextButtonState = _config['nextButton']['state'];
      }
    }
    if (_config['completeButton']) {
      if (_config['completeButton']['id']) {
        var completeButtonId = "#" + _config['completeButton']['id'];
        this.completeButton = document.querySelector(completeButtonId);
      }
      if (_config['completeButton']['state']) {
        this.completeButtonState = _config['completeButton']['state'];
      }
    }
    this.buttonControl();
    return true;
  }
  async next() {
    var getFormId = "#" + this.tabs[this.getActive() - 1]["formControl"];
    var getForm = document.querySelector(getFormId);
    var formData = null;
    if (getForm) {
      formData = await getForm.submit();
    }
    if ((getForm && formData.isValid) || !getForm) {
      this.activeChild = this.getActive() + 1;
      this.active(this.tabs[this.getActive() - 1]["tabId"]);
    }
    return formData;
  }
  async prev() {
    this.activeChild = this.getActive() - 1;
    this.active(this.tabs[this.getActive() - 1]["tabId"]);
    return this.activeChild;
  }
  async complete() {
    const tabStatuses = [];
    var isValid = true;
    this.tabs.forEach(element => {
      if (element['formControl']) {
        const getFormId = "#" + element["formControl"];
        const getForm = document.querySelector(getFormId);
        if (getForm) {
          tabStatuses.push(getForm.submit());
        }
      }
    });
    const allStatuses = await Promise.all(tabStatuses);
    allStatuses.forEach(item => {
      if (item.isValid === false) {
        isValid = false;
        if (item.formName) {
          const formId = this.tabs.filter((input) => input['formControl'] == item.formName)[0]['tabId'];
          this.active(formId);
        }
      }
    });
    return { isValid: isValid, Forms: allStatuses };
  }
  async active(id) {
    var i = 0;
    this.activeChild = -1,
      await delay(100);
    getChilds(this.el, 'bcm-tab').forEach(items => {
      i++,
        items.tabId == id && (items.disabled = false,
          items.active = true,
          this.bcmTabsActiveItem.emit(items.tabId),
          this.activeChild = i);
      items.tabId != id && (items.active = false);
    });
  }
  async getActiveTab() {
    return this.tabs[this.getActive() - 1]["tabId"];
  }
  async getActiveIndex() {
    return this.getActive();
  }
  async error(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "error");
    });
  }
  async info(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "processing");
    });
  }
  async warning(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "warning");
    });
  }
  async success(id, blink = false) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = true,
        items.blink = blink,
        items.status = "success");
    });
  }
  async statusReset(id) {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.tabId == id && (items.badge = false,
        items.blink = false,
        items.status = "default");
    });
  }
  buttonControl() {
    if (this.prevButton) {
      if (this.activeChild == 1) {
        this.prevButton.setAttribute(this.prevButtonState, "true");
      }
      else {
        this.prevButton.removeAttribute(this.prevButtonState);
      }
    }
    if (this.nextButton) {
      if (this.getActive() == this.count) {
        this.nextButton.setAttribute(this.nextButtonState, "true");
      }
      else {
        this.nextButton.removeAttribute(this.nextButtonState);
      }
    }
    if (this.completeButton) {
      if (this.getActive() == this.count) {
        this.completeButton.removeAttribute(this.completeButtonState);
      }
      else {
        this.completeButton.setAttribute(this.completeButtonState, "true");
      }
    }
  }
  /**
   * @ComponentMethod
   */
  componentDidRender() {
    if (!this.firstInit)
      return;
    let slotElements;
    slotElements = this.el.shadowRoot.querySelector('slot').assignedElements();
    this.tabs = [];
    this.activeChild = this.getActive();
    var isActive = false;
    var firstId = null;
    var i = 0;
    // Get only bcm-tab elements from 
    // slot childs
    slotElements.map((element) => {
      i++;
      this.type == "stepper" && (element.disabled = true);
      if (!firstId) {
        firstId = element.tabId;
      }
      if (String(element.tagName).toLowerCase() == 'bcm-tab') {
        element.type = this.type;
        // Activate default tab
        // #
        if (this.tabs.push(element) - 1 == this.activeChild) {
          this.activateChild(this.activeChild);
          isActive = true;
        }
      }
      this.count = i;
    });
    if (!isActive) {
      this.active(firstId);
    }
    setTimeout(() => {
      this.el.shadowRoot.querySelector('.tabs').classList.add('animatable');
    }, 500);
  }
  componentDidLoad() {
    getChilds(this.el, 'bcm-tab').forEach(items => {
      items.active && (this.bcmTabsActiveItem.emit(items.tabId), items.disabled = false);
    });
  }
  getActive() {
    const childs = getChilds(this.el, 'bcm-tab');
    const childIndex = childs.findIndex(child => child.active);
    return childIndex >= 0 ? childIndex + 1 : 1;
  }
  /**
   * @desc
   */
  getChildIdx(child) {
    let i = 1;
    while (child.previousElementSibling) {
      child = child.previousElementSibling;
      i++;
    }
    return i;
  }
  /**
   * @desc
   * @param idx
   */
  activateChild(idx) {
    idx = idx - 1;
    const child = this.tabs[idx];
    if (child) {
      this.tabs.forEach((tab) => tab.active = false);
      child.active = true;
      child.disabled = false;
      this.transformLine(idx);
      this.firstInit && forceUpdate(this.el);
      this.firstInit = false;
    }
  }
  /**
   * @desc
   * @param idx
   */
  transformLine(idx) {
    const tab = this.tabs[idx];
    const rect = tab.getBoundingClientRect();
    const top = (rect.top + window.pageYOffset);
    const left = (rect.left + window.pageXOffset);
    const width = rect.width;
    const height = rect.height;
    if (this.direction == 'horizontal') {
      // this.lineOpts.top = (tab.offsetTop + tab.offsetHeight) + 'px'
      this.lineOpts.top = top + height + 'px';
      // this.lineOpts.left = tab.offsetLeft + 'px'
      this.lineOpts.left = left + 'px';
      // this.lineOpts.width = tab.offsetWidth + 'px'
      this.lineOpts.width = width + 'px';
      this.lineOpts.right = 'unset';
      this.lineOpts.height = '2px';
    }
    else {
      // this.lineOpts.top = tab.offsetTop + 'px'
      this.lineOpts.top = top + 'px';
      this.lineOpts.left = 'unset';
      this.lineOpts.right = '0';
      this.lineOpts.width = '2px';
      // this.lineOpts.height = tab.offsetHeight + 'px'
      this.lineOpts.height = height + 'px';
    }
  }
  /**
   * @desc
   */
  getLineTransform() {
    return {
      '--bcm-tabs-line-top': this.lineOpts.top,
      '--bcm-tabs-line-left': this.lineOpts.left,
      '--bcm-tabs-line-width': this.lineOpts.width,
      '--bcm-tabs-line-right': this.lineOpts.right,
      '--bcm-tabs-line-height': this.lineOpts.height
    };
  }
  /**
   * @desc
   * @param event
   */
  handleTabClick(event) {
    const targetElement = event.target;
    const childIdx = this.getChildIdx(targetElement);
    this.activeChild = null;
    this.activeChild = childIdx;
  }
  /**
   * @desc
   * @param event
   */
  handleTabClose(event) {
    const targetElement = event.target;
    const childIdx = this.getChildIdx(targetElement);
    // Remove element
    // #
    targetElement.remove();
    this.tabs.splice(childIdx - 1, 1);
    if (childIdx === this.activeChild) {
      this.activeChild = 1;
      this.activateChild(1);
    }
  }
  /**
   * @desc
   * @param newValue
   */
  activeChildChange(newValue) {
    this.activateChild(newValue);
    this.change.emit(newValue);
    this.buttonControl();
  }
  /**
   * @desc
   */
  windowResize() {
    this.transformLine(this.activeChild - 1);
    forceUpdate(this.el);
  }
  render() {
    const classes = classnames('tabs', [this.direction], [this.size], [this.type], this.fullWidth ? 'full-width' : null, this.underline ? 'underline' : null);
    const hostClasses = classnames(this.hidden ? 'hidden' : null);
    return (h(Host, { class: hostClasses }, h("div", { class: classes, style: this.getLineTransform(), "on-bcm-click": (event) => this.handleTabClick(event), "on-bcm-close": (event) => this.handleTabClose(event) }, h("slot", null))));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "activeChild": ["activeChildChange"]
  }; }
};
BcmTabs.style = tabsCss;

export { BcmTabs as bcm_tabs };
