/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { h, r as registerInstance, c as createEvent, g as getElement } from './index-0615c2b8.js';
import { G as Generate } from './generate-d009624a.js';
import { a as Bcm } from './bcm-a50d55ba.js';
import { c as classnames } from './index-f549811f.js';
import { N as NumberHelper } from './number-helper-b7a7a318.js';
import { e as extractColor } from './utils-57652744.js';
import { C as ColorPalette } from './colors-1d7854ea.js';
import { C as ColorProps } from './types-9493a7ed.js';
import './package-7549581c.js';
import './colors-dd598eea.js';
import './_commonjsHelpers-82aa9212.js';

const TagTemplate = ({ _id, color, customStyle, size, checked, hidden, value, type, onDismiss, onSelect }) => {
  !size && (size = Bcm.FontSize['size-1']);
  const getColor = (color) => {
    if ((type === Bcm.TagType.basic || type == Bcm.TagType.dismissable) && ColorProps[color])
      return {
        color: extractColor(ColorPalette, ColorProps[color] + '-6'),
        borderColor: extractColor(ColorPalette, ColorProps[color] + '-3'),
        backgroundColor: extractColor(ColorPalette, ColorProps[color] + '-1')
      };
  };
  const getSize = (size, returnNumber = false) => {
    if (NumberHelper.isNumber(size)) {
      size = Bcm.FontSize["size-" + size] || Bcm.FontSize["size-4"];
    }
    if (returnNumber) {
      return +(Bcm.FontSizeValue[size] || Bcm.FontSizeValue['size-4']) + 2;
    }
    return size;
  };
  const classes = classnames(getSize(size), 'bcm-tag__content', type);
  const hostClasses = classnames('bcm-tag', {
    hidden
  });
  return (h("div", { id: _id, class: classnames(hostClasses, { checked }) },
    h("span", { hidden: true },
      h("slot", null)),
    h("span", { class: classes, style: getColor(color), onClick: (e) => onSelect(e) },
      type === Bcm.TagType.add && (h("span", { class: "left" },
        h("bcm-icon", { icon: "far fa-plus" }))),
      type === Bcm.TagType.checkable && h("input", { type: "checkbox", checked: checked }),
      h("span", { class: "value", style: customStyle }, value),
      type === Bcm.TagType.dismissable && (h("span", { class: "right close-button", onClick: (e) => onDismiss(e) },
        h("bcm-icon", { icon: "fal fa-times" }))))));
};

const BcmTag = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.change = createEvent(this, "bcm-change", 7);
    this.tagDismissable = createEvent(this, "bcm-tag-dismissable", 7);
    this.bcmTagSelect = createEvent(this, "bcm-tag-select", 7);
    this._id = Generate.UID();
    this._internal_id = Generate.UID();
    this.hidden = false;
    this.value = '';
    this.type = Bcm.TagType.basic;
    this.checked = false;
    this.color = undefined;
    this.customStyle = undefined;
    this.itemObject = undefined;
    this.size = Bcm.FontSize['size-1'];
    this.closeFunc = (e) => e;
    this.selectFunc = (e) => e;
  }
  handleClick() {
    this.type === Bcm.TagType.checkable && this.handleChecked();
  }
  handleClose() {
    this.tagDismissable.emit(this.el);
  }
  handleChecked() {
    this.checked = !this.checked;
    this.change.emit(this.checked);
  }
  render() {
    const { type, color, value, customStyle, size, hidden, checked } = this;
    return (h(TagTemplate, { size: size, hidden: hidden, color: color, customStyle: customStyle, checked: checked, value: value, type: Bcm.TagType[type], onSelect: (e) => { this.bcmTagSelect.emit(e); this.selectFunc({ event: e, id: this._id }); }, onDismiss: (e) => { this.handleClose(); this.closeFunc(e); } }));
  }
  get el() { return getElement(this); }
};

export { BcmTag as bcm_tag };
