/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-0615c2b8.js';
import { c as classnames } from './index-f549811f.js';
import { G as Generate } from './generate-d009624a.js';
import { S as StringHelper } from './string-helper-3592bdc2.js';
import { a as Bcm } from './bcm-a50d55ba.js';
import { d as delay } from './utils-57652744.js';
import './_commonjsHelpers-82aa9212.js';
import './package-7549581c.js';
import './colors-dd598eea.js';

const BcmToast = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.bcmToastClose = createEvent(this, "bcm-toast-close", 7);
    this.bcmToastOpen = createEvent(this, "bcm-toast-open", 7);
    this.bcmToastView = createEvent(this, "bcm-toast-view", 7);
    this._id = Generate.UID();
    this.status = undefined;
    this.message = "";
    this.description = "";
    this.duration = 3000;
    this.position = Bcm.Placement.bottom;
    this.messageIcon = undefined;
    this.viewButton = false;
    this.closeButton = true;
    this.removeAfterClose = true;
    this.visible = false;
    this.visibleContainer = false;
  }
  async show() {
    const toast = document.getElementById(this._id + "-container");
    if (toast) {
      const container = this.createContainer(this.position);
      container.appendChild(toast);
    }
    this.visible = true;
    await delay(50);
    this.visibleContainer = true;
    this.bcmToastOpen.emit();
    if (this.timer) {
      clearTimeout(this.timer);
    }
    this.timer = setTimeout(() => this.hide(), this.duration);
    return this.el;
  }
  async hide() {
    this.visibleContainer = false;
    clearTimeout(this.timer);
    this.bcmToastClose.emit();
    await delay(100);
    this.visible = false;
    if (this.removeAfterClose) {
      await delay(100);
      this.el.remove();
    }
  }
  handleMouseEnter() {
    clearTimeout(this.timer);
  }
  handleMouseLeave() {
    if (this.visible) {
      this.timer = setTimeout(() => this.hide(), 1000);
    }
  }
  createContainer(position) {
    const containerId = "bcm-toast-container-" + position;
    let container = document.getElementById(containerId);
    if (!container) {
      container = document.createElement("div");
      container.id = containerId;
      const classlist = classnames("tw-pointer-events-none tw-overflow-hidden tw-z-50", "tw-fixed tw-flex", "tw-p-4 tw-gap-4", {
        "tw-flex-col-reverse tw-inset-x-0 tw-top-4 tw-mx-auto": this.position === Bcm.Placement.top,
        "tw-flex-col tw-inset-x-0 tw-bottom-4 tw-mx-auto": this.position === Bcm.Placement.bottom,
        "tw-flex-col-reverse tw-top-4 tw-left-4": this.position === Bcm.Placement["top-start"],
        "tw-flex-col-reverse tw-top-4 tw-right-4": this.position === Bcm.Placement["top-end"],
        "tw-flex-col tw-bottom-4 tw-left-4": this.position === Bcm.Placement["bottom-start"],
        "tw-flex-col tw-bottom-4 tw-right-4": this.position === Bcm.Placement["bottom-end"],
      });
      container.classList.add(...classlist.split(" "));
      document.body.appendChild(container);
    }
    return container;
  }
  render() {
    return (h(Host, { class: "tw-hidden" }, h("div", { class: classnames({ "tw-hidden": !this.visible }), id: this._id + "-container", onMouseEnter: () => this.handleMouseEnter(), onMouseLeave: () => this.handleMouseLeave() }, h("div", { class: classnames("tw-pointer-events-auto tw-transform tw-transition tw-duration-300 tw-ease-in-out", "tw-flex tw-min-w-[256px] tw-max-w-[420px] tw-flex-col tw-gap-2 tw-rounded-lg tw-border tw-font-sans tw-font-normal tw-shadow-2", "tw-border-gray-300 tw-bg-white tw-text-slate-600", {
        "tw-p-4": !this.description,
        "tw-p-6": this.description,
      }, "tw-transform tw-transition tw-duration-300 tw-ease-in-out ", {
        "tw-translate-y-0 tw-opacity-100 ": this.visibleContainer,
        "tw-opacity-0": !this.visibleContainer,
        "tw-translate-y-[-100%]": !this.visibleContainer && this.position.split("-")[0] === "top",
        "tw-translate-y-[100%]": !this.visibleContainer && this.position.split("-")[0] === "bottom",
      }) }, h("div", { class: "tw-flex tw-items-center tw-justify-between tw-gap-5" }, h("div", { class: "tw-flex tw-items-center tw-gap-3 " }, this.status && (h("i", { class: classnames("tw-text-6", this.messageIcon ? this.messageIcon : "far", {
        "tw-text-emerald-500": this.status === "success",
        "tw-text-red-500": this.status === "error",
        "tw-text-amber-500": this.status === "warning",
        "tw-text-blue-500": this.status === "info",
      }, {
        "fa-check-circle": !this.messageIcon && this.status === "success",
        "fa-exclamation-circle": !this.messageIcon && this.status === "error",
        "fa-exclamation-triangle": !this.messageIcon && this.status === "warning",
        "fa-info-circle": !this.messageIcon && this.status === "info",
      }) })), h("span", { class: classnames({
        "tw-text-4": !this.description,
        "tw-text-6 tw-font-medium": this.description,
      }) }, !this.description && this.status && h("span", null, StringHelper.capitalize(this.status), ": "), this.message)), h("div", { class: "tw-flex tw-items-baseline tw-gap-2" }, this.viewButton && (h("button", { onClick: () => this.bcmToastView.emit(), title: "View details", class: classnames("tw-cursor-pointer tw-rounded tw-bg-transparent tw-p-0 tw-px-1 tw-text-4 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200", "tw-transition tw-duration-300 tw-ease-in-out", {
        "tw-font-medium": this.description,
      }) }, "View")), this.closeButton && (h("button", { title: "Close", onClick: () => this.hide(), class: classnames("tw-rounded tw-bg-transparent tw-p-0", "tw-flex tw-h-6 tw-w-6 tw-cursor-pointer tw-items-center tw-justify-center tw-text-3 tw-text-inherit", "hover:tw-bg-gray-100 hover:tw-text-slate-800", "active:tw-bg-gray-200") }, h("i", { class: "fal fa-times" }))))), this.description && h("div", { class: "tw-text-3 " }, this.description), h("slot", null)))));
  }
  get el() { return getElement(this); }
};

export { BcmToast as bcm_toast };
