/*!
 * Built with Stencil
 * Copyright (c) Bromcom.
 */
import { v as version } from './package-7549581c.js';
import { c as colors } from './colors-dd598eea.js';

function snq(callback, defaultValue) {
    try {
        var result = callback();
        return typeof result === 'undefined' ? defaultValue : result;
    }
    catch (err) {
        if (err instanceof TypeError) {
            return defaultValue;
        }
        throw err;
    }
}

/**
 * 'status' prop predefined values
 */
var StatusProps;
(function (StatusProps) {
  StatusProps["info"] = "info";
  StatusProps["error"] = "error";
  StatusProps["warning"] = "warning";
  StatusProps["success"] = "success";
  StatusProps["default"] = "default";
})(StatusProps || (StatusProps = {}));
/**
 * 'type' prop predefined values
 */
var TypeProps;
(function (TypeProps) {
  TypeProps["banner"] = "banner";
  TypeProps["basic"] = "basic";
})(TypeProps || (TypeProps = {}));

const allowedDomains = ["localhost", "beta-regression.bromcomcloud.com", "beta-scrum.bromcomcloud.com", "beta-kanban.bromcomcloud.com"];
const isAllowedDomain = () => {
  const hostname = window.location.hostname;
  return allowedDomains.includes(hostname);
};
if (isAllowedDomain()) {
  console.log(`%cbromcom ui%c version: ${version}-${"p"} `, `color: white; background:#1890FF; font-weight: 300;  font-size:10px;  padding:2px 6px;  border-radius: 5px`, `font-size:10px;`);
}
(function checkTheme() {
  const body = document.querySelector("body");
  let theme = localStorage.getItem("bcm-theme");
  if (!body.classList.contains("bcm-theme-*") && theme) {
    body.classList.add(`bcm-theme-${theme}`);
  }
})();
(function checkOverlayContainer() {
  const body = document.querySelector("body");
  let overlayContainer = document.querySelector("#bcm-overlay-container");
  if (!overlayContainer) {
    overlayContainer = document.createElement("div");
    overlayContainer.id = "bcm-overlay-container";
    overlayContainer.classList.add("bcm-overlay-container");
    body.appendChild(overlayContainer);
  }
})();
class Generate {
  static prettify(input) {
    const prettifiedObject = {};
    for (const key in input) {
      if (input.hasOwnProperty(key)) {
        prettifiedObject[key] = input[key];
      }
    }
    return prettifiedObject;
  }
  /**
      const dc = Generate.getTwColor({
          color: "blue",
          tones: {
              200: ["text"],
              700: ["bg", "border"],
              800: ["bg-hover", "border-hover"],
          },
          variable: "--bcm-avatar",
      });

      return {
          --bcm-avatar-text: --tw-blue-200;
          --bcm-avatar-bg: --tw-blue-700;
          --bcm-avatar-bg-hover: --tw-blue-800;
          --bcm-avatar-border: --tw-blue-700;
          --bcm-avatar-border-hover: --tw-blue-800;
      }
      
   */
  static getTwColor(args) {
    const { color, variable, tones } = (args = Object.assign({
      variable: "--bcm-color-default",
      defaultTone: "600",
      tones: {
        50: {},
        100: {},
        200: {},
        300: {},
        400: {},
        500: {},
        600: {},
        700: {},
        800: {},
        900: {},
        950: {},
      },
    }, args));
    if (!color || !colors[color]) {
      return false;
    }
    const result = {};
    if (typeof colors[color] === "string") {
      result[variable] = colors[color];
      return result;
    }
    const toneList = Array.isArray(tones) ? tones : Object.keys(tones);
    toneList.forEach(tone => {
      if (colors[color][tone]) {
        if (Array.isArray(tones[tone])) {
          tones[tone].forEach(item => {
            result[`${variable}-${item}`] = colors[color][tone];
          });
        }
        else {
          result[`${variable}-${tone}`] = colors[color][tone];
        }
      }
    });
    return result;
  }
}
Generate.UID = (prefix = "bcm") => `${prefix}${Math.random().toString(36).substr(2, 9)}`;
Generate.getIconAttrWithStatusType = (status, loading = false) => {
  if (loading)
    return { icon: "far fa-spinner-third fa-spin" };
  switch (status) {
    case StatusProps.info:
      return { icon: "far fa-info-circle" };
    case StatusProps.error:
      return { icon: "far fa-times-circle" };
    case StatusProps.warning:
      return { icon: "far fa-exclamation-circle" };
    case StatusProps.success:
      return { icon: "far fa-check-circle" };
  }
};
Generate.domParser = (html) => {
  const parser = new DOMParser();
  const doc = parser.parseFromString(html, "text/html");
  return doc.body.innerText;
};
Generate.createComponent = (el, props) => {
  let element = document.getElementById(props.id);
  if (!element) {
    element = document.createElement(el);
  }
  Object.keys(props).forEach(key => {
    element[key] = props[key];
  });
  document.getElementById("bcm-overlay-container").appendChild(element);
  return element;
};
Generate.createElement = (tag = "div", props) => {
  let element = document.getElementById(props.id);
  if (!element) {
    element = document.createElement(tag);
  }
  Object.keys(props).forEach(key => {
    if (key == "innerHTML") {
      element[key] = props[key];
    }
    else {
      element.setAttribute(key, props[key]);
    }
  });
  document.body.appendChild(element);
  return element;
};
Generate.debounceInput = (func, wait, immediate, callBack) => {
  let timeout;
  return function () {
    let context = this, args = arguments;
    if (callBack)
      callBack.apply(context, args);
    let later = function () {
      timeout = null;
      if (!immediate)
        func.apply(context, args);
    };
    let callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
    if (callNow)
      func.apply(context, args);
  };
};
Generate.flatArray = arr => {
  return arr.reduce((acc, val) => {
    return acc.concat(Array.isArray(val) ? Generate.flatArray(val) : val);
  }, []);
};
Generate.reverseNodeList = node_list => {
  let arr = [];
  for (let i = node_list.length - 1; i >= 0; i--) {
    arr.push(node_list[i]);
  }
  return arr;
};
Generate.findEventPathWithKey = (event, key, items) => {
  items = Array.isArray(items) ? items : [items];
  let path = event.composedPath() || event.path;
  if (path) {
    return path.filter((element) => items.includes(element[key]))[0] !== undefined;
  }
};
Generate.findEventPathHasAttribute = (event, key) => {
  let path = event.composedPath() || event.path;
  if (path) {
    return snq(() => !!path.find((element) => element.hasAttribute(key)), false);
  }
};
Generate.findEventTarget = (event, items) => {
  items = Array.isArray(items) ? items : [items];
  let target = event.target || event.srcElement;
  if (target) {
    return items.includes(target);
  }
};
Generate.findEventPathWithNodeName = (event, nodeName) => {
  let path = event.composedPath() || event.path;
  if (path) {
    return path.filter((element) => element.nodeName === nodeName)[0];
  }
};
Generate.findEventPath = (event, items) => {
  items = Array.isArray(items) ? items : [items];
  let path = event.composedPath() || event.path;
  if (path) {
    return path.filter((element) => items.includes(element))[0] !== undefined;
  }
};
Generate.objectIsEmpty = obj => Object.keys(obj).length === 0 && obj.constructor === Object;
Generate.convertIdsToNestedData = (ids, checklist) => {
  let result = [];
  let found = [];
  checklist.forEach(item => {
    if (ids && ids.indexOf(item.id.toString()) > -1) {
      result.push(item);
    }
    else {
      if (item.items) {
        found = Generate.convertIdsToNestedData(ids, item.items);
        if (found.length > 0) {
          item.items = found;
          result.push(item);
        }
      }
    }
  });
  return result;
};
//nodelist to array
Generate.nodeListToArray = nodeList => {
  return Array.prototype.slice.call(nodeList);
};
Generate.setAttributes = (item, obj) => {
  Object.keys(obj).forEach(key => {
    item.setAttribute(key, obj[key]);
  });
};
Generate.computedStyle = (element, property) => window.getComputedStyle(element, null).getPropertyValue(property);
Generate.getAncestor = (el, selector) => {
  while ((el = el.parentElement) && !el.matches(selector))
    ;
  return el;
};
Generate.findParentElement = (element, selector) => {
  const firstChar = selector.charAt(0);
  function getElementByClass(currentElement, className) {
    while (currentElement !== null) {
      if (currentElement.classList.contains(className)) {
        return currentElement;
      }
      currentElement = currentElement.parentElement;
    }
    return null;
  }
  function getElementById(currentElement, id) {
    while (currentElement !== null) {
      if (currentElement.id === id) {
        return currentElement;
      }
      currentElement = currentElement.parentElement;
    }
    return null;
  }
  function getElementByAttribute(currentElement, attr, value) {
    while (currentElement !== null) {
      if (currentElement.hasAttribute(attr) && (value === undefined || currentElement.getAttribute(attr) === value)) {
        return currentElement;
      }
      currentElement = currentElement.parentElement;
    }
    return null;
  }
  function getElementByTag(currentElement, tag) {
    while (currentElement !== null) {
      if (currentElement.tagName.toLowerCase() === tag.toLowerCase()) {
        return currentElement;
      }
      currentElement = currentElement.parentElement;
    }
    return null;
  }
  if (selector === "*") {
    // Tüm elementleri seç
    return element.parentElement;
  }
  else if (firstChar === ".") {
    // Sınıf adı seçicisi
    const className = selector.slice(1);
    return getElementByClass(element.parentElement, className);
  }
  else if (firstChar === "#") {
    // ID seçicisi
    const id = selector.slice(1);
    return getElementById(element.parentElement, id);
  }
  else if (firstChar === "[") {
    // Öznitelik seçicisi
    const end = selector.indexOf("]");
    const attr = selector.slice(1, end);
    const value = selector.slice(end + 2, -1);
    return getElementByAttribute(element.parentElement, attr, value);
  }
  else {
    // Etiket adı seçicisi
    const tag = selector.toLowerCase();
    return getElementByTag(element.parentElement, tag);
  }
};

export { Generate as G, StatusProps as S, TypeProps as T, snq as s };
