import { EventEmitter, ComponentInterface } from '../../../stencil-public-runtime';
import { ElementDragger, DragDirections } from '../../../utils/element-dragger';
import { Months, Type } from './types';
import { PlacementTypes } from '../../../models/bcm-types';
export declare class BcmDatetimePicker implements ComponentInterface {
  private inputId;
  private emptyState;
  private initialized;
  private disabledDatesList;
  private dayNumbers;
  private valueText;
  private currentDate;
  private inputElement;
  private timeStripElement;
  private yearsElement;
  private yearsWrapperElement;
  private yearsStripElement;
  private yearsDragger;
  private hoursElement;
  private hoursWrapperElement;
  private hoursDragger;
  private minutesElement;
  private minutesWrapperElement;
  private minutesDragger;
  private dataIsValid;
  el: HTMLElement;
  /**
   * Component Properties
   */
  _id: string;
  date: string | Date;
  endDate: string | Date;
  type: Type;
  disabled: boolean;
  disabledDates: any;
  value: any;
  _open: boolean;
  hidden: boolean;
  name: string;
  fullWidth: boolean;
  required: boolean;
  label: string;
  placeholder: string;
  minYear: number;
  maxYear: number;
  noCaption: boolean;
  noMargin: boolean;
  captionError: string;
  caption: string;
  captionType: 'primary' | 'success' | 'warning' | 'error' | 'default';
  alignment: any;
  placement: PlacementTypes;
  linkedComponent: string;
  clearable: boolean;
  /**
   * Component States
   */
  years: Array<number>;
  hours: Array<number>;
  minutes: Array<number>;
  hasDateModule: boolean;
  hasTimeModule: boolean;
  renderDate: Date;
  selectedDate: Date;
  selectedEndDate: Date;
  error: boolean;
  errorText: string;
  tick: {};
  private markForCheck;
  /**
   * @desc
   */
  load: EventEmitter;
  change: EventEmitter;
  bcmOpen: EventEmitter;
  openChanged(): void;
  /**
   * @ComponentMethod
   */
  componentWillLoad(): void;
  /**
   * @ComponentMethod
   */
  componentDidRender(): void;
  /**
   * @desc
   * @param newValue
   */
  renderDateChange(): void;
  /**
   * @desc
   * @param newValue
   */
  openChange(newValue: boolean): void;
  /**
   * @desc
   * @param newValue
   */
  valueChange(newValue: any, oldValue: any): void;
  /**
   * @desc
   * @param newValue -
   */
  disabledDatesChange(newValue: any): void;
  /**
   * @desc
   * @param date
   */
  stringToDate(date: string | Date): string | Date;
  /**
   * @desc
   * @param date
   */
  dateToString(date: Date): boolean;
  /**
  *
  * @param str
  */
  switchDDMM(str: string): string;
  /**
  * @desc
  */
  setData(): void;
  /**
   * @desc
   */
  prepareDisabledDates(): void;
  /**
   *
   * @param target
   * @param wheelTriggerElement
   * @param strip
   */
  createWheelChange(target: string, wheelTriggerElement: HTMLElement, strip: HTMLElement): void;
  /**
   * @desc
   */
  createDragSelection(target: string, element: HTMLElement, dragTriggerElement: HTMLElement, strip: HTMLElement, direction: DragDirections): void;
  /**
   * @desc Calculates which child in center of strip
   * for drag areas
   */
  getTargetChildByDragPosition({ y }: {
    y: number;
  }, strip: HTMLElement): number;
  /**
   * @esc
   * @param target
   * @param childIdx
   */
  selectTargetChildByDragIdx(childIdx: number, strip: HTMLElement, dragger: ElementDragger, anim?: boolean): void;
  /**
   * @desc
   * @param year
   */
  setDragSelection(target: string, value: number | string, scroll?: boolean, anim?: boolean): void;
  /**
   * @desc
   * @returns
   */
  getHours12(): number;
  /**
   * @desc
   * @param date
   * @param excludeTime -
   */
  createDateObject(date: Date, { excludeTime, exactMonthNumber }?: {
    excludeTime?: boolean;
    exactMonthNumber?: boolean;
  }): {
    minute: number;
    hour: number;
    year: number;
    day: number;
    month: number;
    dayOfWeek: number;
    monthFirstDay: number;
  };
  /**
   * @desc
   * @param max
   */
  generateDaysArrayFromMax(max: number): number[];
  /**
   * @desc
   * @param date
   */
  getMonthDayNumbers(date: Date): number[];
  /**
   * @desc
   * @param date
   * @param day
   */
  getTargetDayDate(date: Date, day: number): Date;
  /**
   * @desc
   * @param date
   * @param day
   */
  getDayIsWeekend(date: Date, day: number): boolean;
  /**
   * @desc
   * @param date
   */
  getClock(date?: Date): {
    hours: number;
    minutes: number;
    seconds: number;
    milis: number;
  };
  /**
   * @desc
   */
  setValue(reset?: boolean): void;
  /**
   * @desc
   * @param month
   */
  setMonth(month: number): void;
  /**
   * @desc
   * @param popupClosing -
   */
  checkRangeValidation({ popupClosing }?: {
    popupClosing?: Boolean;
  }): boolean;
  /**
   * @desc
   * @param value
   * @param max
   * @returns
   */
  pad(value: number, max: number): "" | "0";
  /**
   * @desc
   * @returns
   */
  isInputClearable(): boolean;
  /**
   * @desc
   * @param date
   */
  isDisabledDate(date: Date): boolean;
  /**
  * @desc
  * @param date
  */
  setDate(date: Date, isUserSelect?: boolean, notClosePicker?: boolean): void;
  setDataFromText(text: string, updateData?: boolean, openPicker?: boolean): void;
  /**
   * @desc
   */
  setDateToday(): void;
  HourContentDblClick(e: any): void;
  /**
  * @desc
  */
  setTime(hours?: string, minutes?: string): void;
  /**
   * @desc
   * @param state
   * @param error
   */
  setError(state?: boolean, error?: string): void;
  /**
   * @desc
   */
  getError(): {
    valid: boolean;
    error: boolean;
    errorText: string;
  };
  /**
   * @desc
   * @param month
   * @param year
   */
  getDaysInMonth(month: number, year: number): number;
  /**
   * @desc
   * @param date
   * @param day
   * @returns
   */
  getIsCurrentDate(date: Date, day: number): boolean;
  /**
   * @desc
   * @param date1
   * @param date2
   */
  isSameDate(date1: Date, date2: Date): boolean;
  /**
   * @desc
   * @param date
   * @returns
   */
  getSelected(date: Date, endDate?: boolean): boolean;
  /**
   * @desc
   * @param date
   */
  getDateIsInRange(date: Date): boolean;
  /**
   * @desc
   */
  getMonthText(): Months;
  /**
   * @desc
   */
  getCurrentDateText(): string;
  /**
  * @desc
  * @returns
  */
  getTimeText(): string;
  /**
   *
   */
  getValueText(): string;
  parseDate(input: any): Date | "invalid date";
  /**
   * @desc
   * @param dateStr
   */
  parse(dateStr: string): {
    date: Date;
    date2: Date;
    timeSections: string | string[];
    dateIsValid: boolean;
    date2IsValid: boolean;
    rangeIsValid: boolean;
    timeIsValid: boolean;
  };
  /**
   * @desc
   */
  getPrevMonth(): void;
  /**
   * @desc
   */
  getNextMonth(): void;
  /**
   * @desc
   * @param idx
   */
  yearClick(idx: number): void;
  /**
   * @desc
   * @param idx
   */
  hourClick(idx: number): void;
  /**
   * @desc
   * @param idx
   */
  minuteClick(idx: number): void;
  /**
   * @desc
   */
  openPicker(): void;
  /**
   * @desc
   * @param date
   */
  handleDayNumber(date: Date): void;
  /**
   * @desc
   * @param value
   */
  handleInputInput(value: string): void;
  /**
   * @desc
   * @param event -
   */
  handleInputChange(event: Event): void;
  /**
   * @desc
   */
  handleInputClear(): Promise<void>;
  /**
   * @desc
   * @param event
   */
  handleWheel(event: Event): void;
  /**
   * @desc
   * @param event
   */
  handleWindowClick(event: any): void;
  openToggle(): void;
  /**
   * @desc
   */
  open(): Promise<void>;
  /**
   * @desc
   */
  close(): Promise<void>;
  /**
   * @desc
   */
  getValue(): Promise<any>;
  /**
   * @param
   * @param disabled
   */
  setDisabled(disabled: boolean): Promise<void>;
  changeFormat(date: Date, reverseDay?: boolean): string;
  isTime(value: any): boolean;
  getHHMM(date: any): string;
  /**
   * @desc
   * @param dateOrTime
   */
  set(dateOrTime: any): Promise<void>;
  get(): Promise<any>;
  setClear(): Promise<void>;
  /**
   * @desc
   */
  getInput(): Promise<HTMLBcmOldInputElement>;
  /**
   * @desc
   * @returns
   */
  isValid(): Promise<{
    valid: boolean;
    error: boolean;
    errorText: string;
  }>;
  resetCaption(): Promise<void>;
  addZeroBefore(n: any): string;
  formatDate(date: any, format: any): any;
  handleResize(): void;
  handleWheels(event: WheelEvent): void;
  componentDidLoad(): void;
  disconnectedCallback(): void;
  private calculateLocation;
  handleKeyDown(ev: any): void;
  render(): any;
}
